#include "cal_svtdef.h"
#include "error.h"
#include "reglist.h"
#include "tk.h"
#include "opcodes.h"
#include "cal_global.h"
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "cal_functions.h"

vme_rc vmePmCommandAll(int, int, int, int*, int*, int, int, vme_vmetype*); 

/*****************************************************************/
/* Setup the daq card at the beginning                           */
/*****************************************************************/
char* cal_setup_pm(Tcl_Interp* interp)
{
static char c1[] = "vme_write daq_0(timeout_register) 0xfff";
static char c2[] = "vme_set daq_0(trailer_en)";
static char c3[] = "vme_set daq_0(fctrl_en)";
static char c33[] = "vme_clear daq_0(fctrl_en)";
static char c4[] = "vme_set daq_0(pm_id)";
static char c5[] = "vme_set daq_0(timeout_rst)";
static char c6[] = "vme_set daq_0(done_rst)";
int dummy;
static char return_code[100];


   dummy = Tcl_GlobalEval(interp,c1);
   if (dummy != TCL_OK) {
      strcpy(return_code,"Error writing to timeout register");
      return return_code;
  }
/* - -  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/*   dummy = Tcl_GlobalEval(interp,c2);                            */
/*   if (dummy != TCL_OK) {                                        */
/*      strcpy(return_code,"Error setting trailer enable bit");    */
/*      return return_code;                                        */
/*  }                                                              */
/* - -  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* comment out setting of fast-control enable  24 Nov 1996         */
/* (And add a clear fast-control enable)                           */
/*   dummy = Tcl_GlobalEval(interp,c3);                            */
/*   if (dummy != TCL_OK) {                                        */
/*      strcpy(return_code,"Error setting fast-control enable");   */
/*      return return_code;                                        */
/*  }                                                              */
/* - -  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
   dummy = Tcl_GlobalEval(interp,c33);                           
   if (dummy != TCL_OK) {                                       
      strcpy(return_code,"Error clearing fast-control enable");  
      return return_code;                                       
  }                                                             
   dummy = Tcl_GlobalEval(interp,c4);
   if (dummy != TCL_OK) {
      strcpy(return_code,"Error setting pm_id");
      return return_code;
  }
   dummy = Tcl_GlobalEval(interp,c5);
   if (dummy != TCL_OK) {
      strcpy(return_code,"Error resetting timeout");
      return return_code;
  }
   dummy = Tcl_GlobalEval(interp,c6);
   if (dummy != TCL_OK) {
      strcpy(return_code,"Error resetting done");
      return return_code;
  }
  strcpy(return_code,"OK");
  return return_code;
}


/*****************************************************************/
/*  Setup the hybrid at the beginning. Send CLEAR, SYNCH, then   */
/*  write control registers and masks; returns "OK" if okay      */
/*****************************************************************/
char* cal_setup_hybrid()
{
  int i1;
  vme_rc rc;
  char *return_code;
  static char temp[100];
  char chnum[20];
/*-------------------------------------------------------*/
/* zero the chip data                                    */
/*-------------------------------------------------------*/
  for (i1=0; i1<32; i1++) {
      chip_data[i1] = 0;
    }
/*-------------------------------------------------------------*/
/* Send CLEAR and SYNCH                                        */
/* NB This is in principle a very good thing to do.            */
/*    However, this causes the DAQ card to give us a           */
/*    timeout the first time through.  The reason is not       */
/*    understood.  So, for now we turn this feature OFF        */
/*  Well, now we put this back in.  The problem is still there */
/*  It is a hardware problem.  We can fake it by setting       */
/*  the direction to 1.  (Sep 19, 1996)                        */
/*  Send RESET in broadcast mode, 28-feb-1997                  */
/*             (because of bug, it used to go to addr=0 only)  */
/*-------------------------------------------------------------*/
    rc = vmePmCommand(CARD, CLEAR, 0,  0xf, 0, 1, chip_data);    
    if (rc != VME_OK) {                                          
       return_code = "Error sending CLEAR command";              
       return return_code;                                       
     }                                                           
    rc = vmePmCommand(CARD, SYNCH, 0,  0xf, 0, 1, chip_data);    
    if (rc != VME_OK) {                                          
       return_code = "Error sending SYNCH command";              
       return return_code;                                       
     }                                                           
    rc = vmePmCommand(CARD, RESET, 0xf,  0xf, 0, 1, chip_data);    
    if (rc != VME_OK) {                                          
       return_code = "Error sending SYNCH command";              
       return return_code;                                       
     }                                                           
/*-------------------------------------------------------------*/
/* set the DAC entries to the control registers to zero        */
/*-------------------------------------------------------------*/
   for (i1=0; i1<number_of_chips; i1++) {
      cal_thresh_dac[i1] = 0;
      cal_cal_dac[i1] = 0;
   }
/*-------------------------------------------------------*/
/* write the control registers                           */
/*-------------------------------------------------------*/
   for (i1=0; i1<number_of_chips; i1++) {
        build_creg(i1);
        rc = cal_write_control_registers(i1);
        if (rc != VME_OK) {
           return_code = "Error writing control register chip";
	   sprintf(chnum," %d",i1);
	   strcpy(temp,return_code);
	   strcat(temp,chnum);
	   return_code = temp;
           return return_code;
        }
   }
/*-------------------------------------------------------*/
/* write the calibration and channel masks               */
/*-------------------------------------------------------*/
   for (i1=0; i1<number_of_chips; i1++) {
       rc = cal_write_masks(i1);
       if (rc != VME_OK) {
          return_code = "Error writing calibration and channel masks";
	  sprintf(chnum," %d",i1);
	  strcpy(temp,return_code);
	  strcat(temp,chnum);
	  return_code = temp;
          return return_code;
       }
   }
/*-------------------------------------------------------*/
/* done                                                  */
/*-------------------------------------------------------*/
  strcpy(temp,"OK");
  return_code = temp;
  return return_code;
}

/*****************************************************************/
/* write control registers for chip number jchip                 */
/*****************************************************************/
vme_rc cal_write_control_registers(int jchip)
{
  vme_rc rc;
  int chip_address;
  int pm_chan;

  creg_into_data(jchip);
  cal_get_chip_address(jchip,&chip_address,&pm_chan);
/*---- To be sure, write on both channels    -----------------*/
  if (pm_chan <= 0x3) pm_chan=0x3;
  if (pm_chan >  0x3) pm_chan=0xc;
  rc = vmePmCommand(CARD, WRITECONTROL, chip_address,
		    (vme_vmetype) pm_chan, 4, 1, chip_data);

  return rc;
}

/*****************************************************************/
/* write cal and chan masks for chip number jj                   */
/* jj=-1 : write all of the masks                                */
/*****************************************************************/
vme_rc cal_write_masks(int jj)
{
  vme_rc rc;
  int chip_address;
  int pm_chan;
  int i, j;
  int itemp;
  int chip_address_array[MAX_NUM_CHIP];
  int pm_chan_array[MAX_NUM_CHIP];
  vme_vmetype chip_data_array[32*MAX_NUM_CHIP];

  if (jj != -1) {

    masks_into_data(jj);
    cal_get_chip_address(jj,&chip_address,&pm_chan);
/*---- To be sure, write on both channels    -----------------*/
    if (pm_chan <= 0x3) pm_chan=0x3;
    if (pm_chan > 0x3)  pm_chan=0xc;
    rc = vmePmCommand(CARD, WRITEMASK, chip_address,
			(vme_vmetype) pm_chan, 32, 1, chip_data);

/* store the written masks                    */
    for (i=0; i<8; i++) {
      written_calib_mask[jj][i]   = calib_mask[jj][i];
      written_channel_mask[jj][i] = channel_mask[jj][i]; 
    }
/* here for all of them */
  
  } else {

    for (i=0; i<number_of_chips; i++) {
      masks_into_data(i);
      cal_get_chip_address(i,&chip_address,&pm_chan); 
      if (pm_chan <= 0x3) pm_chan=0x3;
      if (pm_chan > 0x3)  pm_chan=0xc;
      chip_address_array[i] = chip_address;
      pm_chan_array[i]      = pm_chan;
      for (j=0; j<32; j++) chip_data_array[32*i+j] = chip_data[j];
    }

    rc = vmePmCommandAll(CARD, WRITEMASK, number_of_chips, 
         chip_address_array, pm_chan_array, 32, 1, chip_data_array);
    for(i=0; i<number_of_chips; i++) {
      for (j=0; j<8; j++) {
	written_calib_mask[i][j]   = calib_mask[i][j];
	written_channel_mask[i][j] = channel_mask[i][j]; 
      }
    }



  }

/* done                */
    return rc;

}

/*****************************************************************/
/* take the control register for chip "which_chip", turn it into */
/*   Armin's format                                              */
/*****************************************************************/
void creg_into_data(int which_chip)
{
  int i1;
  unsigned int byte[4];
  unsigned int mask[4];
  int temp;
/*-------------------------------------------------------*/
/*  first zero everything                                */
/*-------------------------------------------------------*/
  for (i1=0; i1<32; i1++) {
      chip_data[i1] = 0;
    }
/*-------------------------------------------------------*/
/* set up the masks which will be needed later           */
/*-------------------------------------------------------*/
  mask[0] = 0xff;
  mask[1] = 0xff00;
  mask[2] = 0xff0000;
  mask[3] = 0xff000000;
/*-------------------------------------------------------*/
/* extract the four bytes                                */
/*-------------------------------------------------------*/
  for (i1=0; i1<4; i1++) {
    byte[i1] = cal_creg[which_chip] & mask[i1];
/*debugdebug    printf("byte %d = %x \n",i1,byte[i1]);  */ 
  }
/*-------------------------------------------------------*/
/* shift the higher order bytes                          */
/*-------------------------------------------------------*/
  byte[1] = byte[1] >> 8;
  byte[2] = byte[2] >> 16;
  byte[3] = byte[3] >> 24;
/*-------------------------------------------------------*/
/* now switch the order and stick them into chip data    */
/*-------------------------------------------------------*/
  for (i1=0; i1<4; i1++) {
    temp = (unsigned int) byte[i1];
    temp = switch_bits(temp,8);
    chip_data[i1] = (vme_vmetype) temp;
  }

}

/***********************************************************************/
/*   build control registers for chip number "this_chip"               */
/*   NB: This is NOT the format that Armin's code                      */
/*   expects it into.  Will have to call "creg_into_data" to set it up */
/*   properly                                                          */
/***********************************************************************/
void build_creg(int this_chip)
{
      unsigned int temp;
      

      cal_creg[this_chip]= 0;

      temp = cal_thresh_dac[this_chip];
      cal_creg[this_chip] = cal_creg[this_chip] | temp;

      temp = cal_cal_dac[this_chip] << 6;

      cal_creg[this_chip] = cal_creg[this_chip] | temp;

      temp = jitter[this_chip] << 12;
      cal_creg[this_chip] = cal_creg[this_chip] | temp;

      temp = shaping_time[this_chip] << 17;
      cal_creg[this_chip] = cal_creg[this_chip] | temp;

      temp = sampling_rate[this_chip] << 19;
      cal_creg[this_chip] = cal_creg[this_chip] | temp;

      temp = skip[this_chip] << 21;
      cal_creg[this_chip] = cal_creg[this_chip] | temp;

      temp = readout_direction[this_chip] << 23;
      cal_creg[this_chip] = cal_creg[this_chip] | temp;

      temp = respond[this_chip] << 24;
      cal_creg[this_chip] = cal_creg[this_chip] | temp;

      temp = clock_through[this_chip] << 25;
      cal_creg[this_chip] = cal_creg[this_chip] | temp;

      temp = test_pattern[this_chip] << 26;
      cal_creg[this_chip] = cal_creg[this_chip] | temp;

      temp = Set_Clock_B[this_chip] << 30;
      cal_creg[this_chip] = cal_creg[this_chip] | temp;

/*debugdebug        printf("%x \n",cal_creg[this_chip]);    */  

}

/*****************************************************************/
/* stick calibration and channel mask into chip_data array       */
/*****************************************************************/
void masks_into_data(int this_chip) 
{
   int i1;
   unsigned int first_byte;
   unsigned int second_byte;
   int temp;
   int data_index;
   int temp2[32];
   

/*----------------------------------------------------------------*/
/*  first zero everything                                         */
/*----------------------------------------------------------------*/
  for (i1=0; i1<32; i1++) {
      chip_data[i1] = 0;
    }
/*------------------------------------------------------------------*/
/* loop over channel masks ... they go 0->127, one byte per word,   */
/*  in addition we have to switch the order for Armin's format      */
/*------------------------------------------------------------------*/
   for (i1=0; i1<8; i1++) {
      data_index = 2 * i1;
      first_byte  = channel_mask[this_chip][i1] & 0xff;
      second_byte = channel_mask[this_chip][i1] & 0xff00;
      second_byte = second_byte >> 8;
      temp        = first_byte;
      first_byte  = switch_bits(temp,8);
      temp        = second_byte;
      second_byte = switch_bits(temp,8);
      chip_data[data_index]   = (vme_vmetype) first_byte;
      chip_data[data_index+1] = (vme_vmetype) second_byte;
    }
/*--------------------------------------------------------------------*/
/* now loop over calibration masks ... they go 127->0, one byte       */
/* per word,  here we do not need to switch the order (aaarrrggghh)   */
/*--------------------------------------------------------------------*/
   for (i1=7; i1>=0; i1--) {
      data_index = 16 + 2 * (7-i1);
      first_byte  = calib_mask[this_chip][i1] & 0xff00;
      second_byte = calib_mask[this_chip][i1] & 0xff;
      first_byte = first_byte >> 8;
      chip_data[data_index]   = (vme_vmetype) first_byte;
      chip_data[data_index+1] = (vme_vmetype) second_byte;
    }
/*--------------------------------------------------------------------*/
/* Kludge because the order is not what it says in the document       */
/* This has calibration mask first, transmitted 127 --> 0             */
/* Then channel mask, transmitted 0 --> 127                           */  
/*--------------------------------------------------------------------*/
    for (i1=0; i1<32; i1++) {
	temp2[i1] = chip_data[i1];
    }
    for (i1=0; i1<16; i1++) {
	chip_data[i1] = temp2[16+i1];
    }
    for (i1=16; i1<32; i1++) {
	chip_data[i1] = temp2[i1-16];
    }
}


/**************************************************************************/
/* Read registers and masks for chip number j                             */
/* (look first on channel try_first)                                      */
/* Returns "OK" if everything is Okay                                     */
/* Also returns in variable chan_read the channel from which we read it   */
/**************************************************************************/
char* cal_read_reg_masks(int ichip, int try_first, Tcl_Interp* interp, 
                                                             int* chan_read)
{
  int dummy, chip_address, pm_chan;
  vme_rc rc;
  static char* command_result;
  static char return_message[200];
  vme_vmetype vram_data[80];
  int byte[4][40];
  int i,j,ifound;
  int ic,ichan,header,this_address,read_chan;
  unsigned int itemp;
  int jtemp, iwrite, bad_first;
  static unsigned int mask[4] = {0xff, 0xff00, 0xff0000, 0xff000000};
  static char c666[] = "vme_set daq_0(vram_dir)";
  static char c667[] = "vme_clear daq_0(vram_dir)";


    *chan_read = -1;
/*-----------------------------------------------------------------------*/
/* Toggle the VRAM direction                                             */
/*-----------------------------------------------------------------------*/
   dummy = Tcl_GlobalEval(interp,c666);
   if (dummy != TCL_OK) {
      strcpy(return_message,"Error setting vram_dir");
      return return_message;
  }
   dummy = Tcl_GlobalEval(interp,c667);
   if (dummy != TCL_OK) {
      strcpy(return_message,"Error clearing vram_dir");
      return return_message;
  }
/*-----------------------------------------------------------------------*/
/* Get the chip address and such                                         */
/*-----------------------------------------------------------------------*/
  cal_get_chip_address(ichip, &chip_address, &pm_chan);
/*-----------------------------------------------------------------------*/
/* Make sure that try_first makes sense                                  */
/*-----------------------------------------------------------------------*/
  itemp = 0;
  if (try_first < 0  || try_first > 3) itemp=1;
  if (pm_chan <= 0x3 && try_first > 1) itemp=1;
  if (pm_chan > 0x3  && try_first < 2) itemp=1;
  if (itemp != 0) {
    sprintf(return_message,"Illegal attempt to read registers \n on channel %d for chip %d",try_first,ichip);
    return return_message;
  }
/*-----------------------------------------------------------------------*/
/* We will check both possible channels                                  */
/* We will use the variable "try_first" to decide which channel          */
/* to look at first                                                      */
/*-----------------------------------------------------------------------*/
  bad_first = 0;
  for (iwrite=0; iwrite<2; iwrite++) {

    if (iwrite == 0) {
      if (try_first == 0) read_chan=0x1;
      if (try_first == 1) read_chan=0x2;
      if (try_first == 2) read_chan=0x4;
      if (try_first == 3) read_chan=0x8;
    } else {
      if (try_first == 0) read_chan=0x2;
      if (try_first == 1) read_chan=0x1;
      if (try_first == 2) read_chan=0x8;
      if (try_first == 3) read_chan=0x4;
    }

    command_result = EventSeq(CARD, READREG, 0, read_chan, 40,chip_address, 0);

/*sp      rc = vmePmCommand(CARD, READREG, chip_address,               sp*/
/*sp		    (vme_vmetype) read_chan, 40, 0, chip_data);        sp*/
/*sp      if (rc != VME_OK) {                                          sp*/
/*sp         strcpy(return_message,"Error issuing a READREG");         sp*/
/*sp         return return_message;                                    sp*/
/*sp      }                                                            sp*/
/*-----------------------------------------------------------------------*/
/* move data into VRAM                                                   */
/*-----------------------------------------------------------------------*/
/*sp      command_result = cal_move_into_vram(interp);                 sp*/
/*-----------------------------------------------------------------------*/
/* if it is the first try, and we have a timeout, try again              */
/*-----------------------------------------------------------------------*/
      if (strcmp("Timeout reading done_int",command_result) == 0 &&
	  iwrite == 0) bad_first=1;
      if (strcmp("timeout_int reports a timeout",command_result) == 0 &&
	  iwrite == 0) bad_first=1;
      if (strcmp("Fatal timeout reading done_int",command_result) == 0 &&
	  iwrite == 0) bad_first=1;
      if (bad_first == 1 && iwrite == 0) {
	  command_result = cal_setup_pm(interp);
          if (strcmp(command_result,"OK") != 0) {
             strcpy(return_message,"Error resetting personality module");
             return return_message;
	   }
      }
      if (bad_first == 1 && iwrite == 0) continue;
/*-----------------------------------------------------------------------*/
/* If it is a different error or if it is the second iteration, quit     */
/*-----------------------------------------------------------------------*/
      if (strcmp("OK",command_result) != 0) {
         sprintf(return_message,"Error from cal_move_into_vram: %s",command_result);
         return return_message;
      }
/*-----------------------------------------------------------------------*/
/* read 80 words from VRAM                                               */
/* 5/16/97 read only 40, we are now using vmevmebuffread2                */
/*-----------------------------------------------------------------------*/
      rc = cal_vram_read(0, 40, vram_data);
      if (rc != VME_OK) {
        strcpy(return_message,"Error from cal_vram_read");
        return return_message;
      }
/*-----------------------------------------------------------------------*/
/* Get the data into 4*40 bytes format                                   */
/*-----------------------------------------------------------------------*/
      for (i=0; i<4; i++) {
        for (j=0; j<40; j++) {
/***          itemp = (unsigned int) vram_data[2*j];    5/16/97  *******/
          itemp = (unsigned int) vram_data[j];
	  itemp = itemp & mask[i];
	  if (i != 0) itemp = itemp >> (8*i);
	  itemp = itemp & 0xff;
	  byte[i][j] = itemp;      
        }
      }


/*******      for (i=0; i<40; i++) {    
          printf("byte %d 0x%x \n",i,byte[0][i]);  
      }    **********/  
/*-----------------------------------------------------------------------*/
/* Switch bit ordering for control registers and channel masks           */
/*-----------------------------------------------------------------------*/
      for (i=0; i<4; i++) {
        for (j=2; j<6; j++) {
          jtemp = byte[i][j];
          jtemp = switch_bits(jtemp,8);
	  byte[i][j] = jtemp;
        }
        for (j=22; j<38; j++) {
	    jtemp = byte[i][j];
	    jtemp = switch_bits(jtemp,8);
	    byte[i][j] = jtemp;
        }
      }
/*-----------------------------------------------------------------------*/
/* see if can find a header on one of the two channels                   */
/* careful : the address is sent down in 5 bits, comes back in 4 bits    */
/* march 9, 1998  CC Accept trailers with 0x80 followed by 0x00          */
/* june  9, 1998  CC Look on channel = read_chan first                   */ 
/*-----------------------------------------------------------------------*/
      ifound = 0;
/*      ic = 2;                                  6 jun 98  */
/*      if (pm_chan <= 0x3) ic=0;                6 jun 98  */
/*      for (ichan=ic; ichan<(ic+2); ichan++) {  6 jun 98  */
      for (ic=0; ic<2; ic++) {
	if (ic==0) {
	  if (read_chan == 0x1) ichan = 0;
	  if (read_chan == 0x2) ichan = 1;
	  if (read_chan == 0x4) ichan = 2;
	  if (read_chan == 0x8) ichan = 3;
	} else {
	  if (read_chan == 0x1) ichan = 1;
	  if (read_chan == 0x2) ichan = 0;
	  if (read_chan == 0x4) ichan = 3;
	  if (read_chan == 0x8) ichan = 2;
	}

	  header = byte[ichan][0];
	  if ( (header & 0x80) != 0) {              /* found start bit */
	      this_address = header & 0x78;
	      this_address = this_address >> 3;
	      this_address = this_address << 1;
/*****        this_address = switch_bits(this_address,4);   **********/
	      if (this_address == chip_address) {   /* found correct address */
		  if ( (byte[ichan][38]==0 || byte[ichan][38]==0x80) && 
		                           byte[ichan][39]==0) { /* trailer */
		      ifound = 1;
		      cal_creg[ichip] = byte[ichan][2];
		      cal_creg[ichip] = cal_creg[ichip] | (byte[ichan][3]<<8);
		      cal_creg[ichip] = cal_creg[ichip] | (byte[ichan][4]<<16);
		      cal_creg[ichip] = cal_creg[ichip] | (byte[ichan][5]<<24);

		      for (i=0; i<8; i++) {
			  channel_mask[ichip][i] = byte[ichan][22+2*i] |
		                      (byte[ichan][23+2*i] << 8);
			  calib_mask[ichip][i]   = byte[ichan][21-2*i] |
		                      (byte[ichan][20-2*i] << 8);
			}
		      *chan_read = ichan;
		      goto found;
		    }
		}
	  }
      }
/*-----------------------------------------------------------------------*/
/* Did we find the data ?                                                */
/*-----------------------------------------------------------------------*/
found:
      if (ifound == 0) {
	  strcpy(return_message,"Cannot find data in VRAM");
	  return return_message;
      } else break;
/*-----------------------------------------------------------------------*/
/* Close the for loop over iwrite variable                               */
/*-----------------------------------------------------------------------*/
  }
/*-----------------------------------------------------------------------*/
/* If we got here, everything okay                                       */
/*-----------------------------------------------------------------------*/
  strcpy(return_message,"OK");
  return return_message;
}





