/*****************************************************************/
/* This builds up the data.  Fills a pointer to the              */ 
/* first byte of data (cal_data_point). Returns the length       */
/* (in bytes) of the data (nbytes)                               */
/* nev is the event number                                       */
/*                                                               */
/* The return code is "OK" or a string with an error message     */
/* If the return code is not "OK", then there was a problem.     */
/* If there was a problem, data_point = NULL                     */
/* If the problem is such that we should abort the run :         */
/*           nbytes = 0                                          */
/* If the problem is not such that we should abort the run :     */
/*           nbytes = -1                                         */
/*****************************************************************/

#include "cal_svtdef.h"
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "error.h"
#include "reglist.h"
#include "tk.h"
#include "opcodes.h"
#include "cal_global.h"
#include "cal_functions.h"

/*  This MUST be even   */
#define NWORD_READ   64

/* This MUST BE 4   */
#define NCHAN         4

#define MEMCHUNK   1024

char *cal_data(Tcl_Interp* interp, int* nbytes, int nev) {

static char return_message[200];
vme_vmetype vram_words[NWORD_READ];
int end_chip_flag[NCHAN];
int sh_chan[NCHAN];
char this_char[NCHAN];
static int chip_vram_code[MAX_NUM_CHIP];
static int chip_pm_chan[MAX_NUM_CHIP];
int chip_begin[MAX_NUM_CHIP];
int chip_end[MAX_NUM_CHIP];
char *channel_data[NCHAN];
int  bytes_allocated[NCHAN];
int  bytes_used[NCHAN];
int  chan_nchip[NCHAN];
int  chan_nchip_non_ignored[NCHAN];
int  chan_finished[NCHAN];
int  all_chan_finished;
int ichip;

char *temp;
char buff[4];
static int bit[32];
static int bytemask[4];
int sh_chpbegin, sh_chpend, sh_evlength, sh_data, sh_run;
int sh_event, sh_nchip, sh_contreg, sh_calmask, sh_chnmask;
int sh_xybegin, sh_xyend, sh_xydata, ix, iy;
int nbytes_read, new_chip, this_chip_addr;
int ndeleted;
char this_byte;
float x, y;
int current_chip[NCHAN];
int num_chips_finished, it_is_a_trailer;
char previous_byte, previous_byte2, second_byte, first_byte;
int  iprevious_byte, iprevious_byte2;
int i, i1, i2, i3, i4, ich, j, dummy, iw;
int i90, jchip;
int itemp, iby, add, pm_chan, shift, this_shift;
int this_word;
vme_rc status;
static int dummy_vram;
static char* command_result;
int bytes_this_chip;


/* printf("**** new event number %d ******** XS\n",nev);      */
/*------------------------------------------------------------*/
/* initialize                                                 */
/*------------------------------------------------------------*/
   cal_data_point = NULL;
   for (i1=0; i1<number_of_chips; i1++) {
     chip_begin[i1]     = -1;
     chip_end[i1]       = -1;
   }
/*====================================================================+*/
/*====================================================================+*/
/* save some time, a few things only need to be done on the 1st event  */
/*====================================================================+*/
/*====================================================================+*/
  if (nev == 1) {
/*------------------------------------------------------------*/
/* fake event ?????                                           */
/*------------------------------------------------------------*/
     dummy_vram     = 1;
     if (strcmp(C_fake,"1") == 0) dummy_vram = 0;
/*------------------------------------------------------------*/
/* bits and masks ........                                    */
/*------------------------------------------------------------*/
      bit[0] = 1;
      bit[1] = 2;
      for (i1=2; i1<32; i1++) {
	bit[i1] = 2 * bit[i1-1];
      }
      bytemask[0]  = 0xff;
      bytemask[1]  = 0xff00;
      bytemask[2]  = 0xff0000;
      bytemask[3]  = 0xff000000;
/*------------------------------------------------------------*/
/* channel --> chip correspondance                            */
/* careful : cal_get_chip_address return the chip address     */
/* in five bits and switched....                              */
/* VRAM has it in four bits switched                          */
/*------------------------------------------------------------*/
      for (i1=0; i1<number_of_chips; i1++) {
	cal_get_chip_address(i1,&add,&pm_chan);
	add = add >> 1;
	itemp = -1;
	for (i2=0; i2<NCHAN; i2++) {
	  if ( (pm_chan & bit[i2]) == bit[i2]) itemp=i2;
	}
	if (itemp == -1) {
	  cal_data_point = NULL;
	  *nbytes     = 0;
	  strcpy(return_message,"Illegal chip pm channel in cal_get_data");
	  return return_message;
	}
	chip_vram_code[i1] = add;
	chip_pm_chan[i1]   = itemp;
      }
/*=====================================================================*/
/*=====================================================================*/
/* End of block over things that need to be done only on the 1st event */
/*=====================================================================*/
/*=====================================================================*/
   }
/*------------------------------------------------------------*/
/* if so required, write dummy data to VME                    */
/*------------------------------------------------------------*/
   if (dummy_vram == 0) {
      status = cal_vram_write();
      if (status != VME_OK) {
         *nbytes = 0;
         strcpy(return_message,"Error writing dummy data to VME");
         return return_message;
       }
   } 
/*------------------------------------------------------------*/
/* Move data into VRAM                                        */
/* (ALL COMMENTED OUT WHEN FUNCTIONALITY MOVED FROM           */
/*  UNIX TO VXWORKS IN AN ATTEMPT TO GAIN SPEED)              */
/*------------------------------------------------------------*/
/*sp   if (dummy_vram != 0) {                               sp*/
/*sp      command_result = cal_move_into_vram(interp);      sp*/
/*sp      if (strcmp("OK",command_result) != 0) {           sp*/
/*sp	 *nbytes = -1;                                      sp*/
/*sp	 return command_result;                             sp*/
/*sp       }                                                sp*/
/*sp    }                                                   sp*/
/*------------------------------------------------------------*/


/*&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&*/
/* We are now ready to read the data                          */
/*&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&*/


/*------------------------------------------------------------*/
/* allocate NCHAN chunks of memory, one for each channel      */
/*------------------------------------------------------------*/
      for (i1=0; i1<NCHAN; i1++) {
	  channel_data[i1] = (char *) malloc(MEMCHUNK*sizeof(char));
	  if (channel_data[i1] == NULL) {
              strcpy(return_message,"Run out of UNIX memory");
              *nbytes = 0;
	      for (i2=i1-1; i2>=0; i2--) {
		free(channel_data[i2]);
	      }
              return return_message;
	    }
	  bytes_allocated[i1] = MEMCHUNK;
	  bytes_used[i1]      = 0;
      }
/*--------------------------------------------------------------*/
/* Count how many chips report to a given channel               */
/* Distinguish between all chips and all "non-ignored" chips    */
/*--------------------------------------------------------------*/
      for (ich=0; ich<NCHAN; ich++) {
	 chan_nchip[ich] = 0;
         chan_nchip_non_ignored[ich] = 0;
         for (i1=0; i1<number_of_chips; i1++) {
	    if (chip_pm_chan[i1] == ich) {
	       chan_nchip[ich] = chan_nchip[ich] + 1;
	       if (ignore[i1] == 0) 
                  chan_nchip_non_ignored[ich] = 
		    chan_nchip_non_ignored[ich] + 1; 
	    }
	 }
       }
/*--------------------------------------------------------------*/
/* set the flags for channel finished                           */
/* chan_finished[ch] = 1  -> ch finished                        */
/* all_chan_finished = 1  -> all channels are done              */
/*--------------------------------------------------------------*/
       all_chan_finished = 1;
       for (ich=0; ich<NCHAN; ich++) {
	 chan_finished[ich] = 1;
	 if (chan_nchip_non_ignored[ich] != 0) {
	   chan_finished[ich] = 0;
	   all_chan_finished  = 0;
	 }
       }

/*=========================================================*/
/*=========================================================*/
/* Read data from VRAM, NWORD_READ at a time               */
/*=========================================================*/
/*=========================================================*/

      num_chips_finished = 0;
/*----------------------------------------------------------*/
/* while loop until all the data is in                      */
/*----------------------------------------------------------*/
      shift = 0;
      while (all_chan_finished != 1) {
      status = cal_vram_read(shift, NWORD_READ, &vram_words[0]);
      if (status != VME_OK) {
	 for (i2=0; i2<NCHAN; i2++) {
		      free(channel_data[i2]);
	 }
         strcpy(return_message,"Error reading VRAM");
         *nbytes = -1;
	 return return_message;
       }
/*----------------------------------------------------------*/
/* Loop over channels                                       */
/*----------------------------------------------------------*/
          for (ich=0; ich<NCHAN; ich++) {
/*----------------------------------------------------------*/
/* If this channel is done, go to the next channel          */
/*----------------------------------------------------------*/
	     if (chan_finished[ich] == 1)goto next_channel;

/*----------------------------------------------------------*/
/*----------------------------------------------------------*/
/* If there isnt enough room, try to get more               */
/*----------------------------------------------------------*/
/*----------------------------------------------------------*/
	      if ( (bytes_used[ich]+NWORD_READ/2) > bytes_allocated[ich]) {
		 temp = channel_data[ich];
		 channel_data[ich] =  (char *)
		        malloc(MEMCHUNK*sizeof(char)+bytes_used[ich]);
/*----------------------------------------------------------*/
/* If we are out of memory, free it all, then quit          */
/*----------------------------------------------------------*/
		 if (channel_data[ich] == NULL) {
		    channel_data[ich] = temp;
		    for (i2=0; i2<NCHAN; i2++) {
		      free(channel_data[i2]);
		    }
                    strcpy(return_message,"Run out of UNIX memory");
                    *nbytes = -1;
		    return return_message;
		  }   
/*-----------------------------------------------------------------------*/
/* We got more memory, now we need to move the data into it              */
/*-----------------------------------------------------------------------*/
		  for (iby=0; iby<bytes_used[ich]; iby++) {
		       *(channel_data[ich]+iby) = *(temp+iby);
		  }
		  bytes_allocated[ich] = bytes_used[ich] + 
		               MEMCHUNK*sizeof(char);
/*----------------------------------------------------------*/
/* free up the old memory                                   */ 
/*----------------------------------------------------------*/
		 free(temp);
/*----------------------------------------------------------*/
/* end of if block for trying to get more memory            */
/*----------------------------------------------------------*/
	       }
/*----------------------------------------------------------*/
/* now loop over the words from the vram                    */
/*----------------------------------------------------------*/
/****		 for (iw=0; iw<NWORD_READ; iw=iw+2) {    ***/
		 for (iw=0; iw<NWORD_READ; iw++) {

		      j  = iw;
		      i2 = ich;

/***		  if (ich < 4) {                         ***/
/***		       j  = iw;                          ***/
/***		       i2 = ich;                         ***/
/***		  } else {                               ***/
/***		       j  = iw+1;                        ***/
/***		       i2 = ich - 4;                     ***/
/***		  }                                      ***/
/*----------------------------------------------------------*/
/* Get the byte for this channel                            */
/*----------------------------------------------------------*/
		  this_word = (int) vram_words[j];
		  this_word = this_word & bytemask[i2];
		  if (i2 != 0) {
		      this_word = this_word >> (8*i2);
		  }
		  this_byte = (char) this_word;
                  new_chip = 1;

/*  if (ich == 0) printf("channel %d byte 0x%x \n",ich, this_byte); */

/*----------------------------------------------------------*/
/* If it is the 1st byte, make sure it is not empty         */
/*----------------------------------------------------------*/
		  if (bytes_used[ich] == 0) {     /* 1st word */
		     new_chip = 0;
		     if (this_byte == 0) {
			   for (i3=0; i3<NCHAN; i3++) {
		               free(channel_data[i3]);
			   }
	 sprintf(return_message,"First byte of channel %d is empty",ich);
                           *nbytes = -1;
		           return return_message;
		      }
		  } 
/*--------------------------------------------------------------------*/
/* If not 1st byte, see if it is the last byte for a chip             */
/* If it is the last byte for a chip, and the number of bytes is odd  */
/* then report an error and get out of here                           */
/* .... Account for buggy trailers, a one followed by 15 zeros ...    */
/*--------------------------------------------------------------------*/
		  it_is_a_trailer = 0;
		  if (bytes_used[ich] != 0) {
		   previous_byte = *(channel_data[ich]+bytes_used[ich]-1);
		   bytes_this_chip = bytes_used[ich] - 
		                     chip_begin[current_chip[ich]] + 1;
		   if (previous_byte==0 && this_byte==0  && (bytes_this_chip > 3 )) 
                      it_is_a_trailer=1;
		   itemp = (int) previous_byte;
		   itemp = itemp & 0xff;
                   if (itemp==0x80 && this_byte==0 &&
                       bytes_this_chip%2 == 0 ) it_is_a_trailer = 1;

		   if (it_is_a_trailer == 1) {
		      num_chips_finished = num_chips_finished + 1;
 /*     printf("finished data for chip %d \n",current_chip[ich]);          */
		      chip_end[current_chip[ich]] = bytes_used[ich];

		      chan_nchip[ich] = chan_nchip[ich] - 1;

		      if (ignore[current_chip[ich]] == 0) 
			chan_nchip_non_ignored[ich] = 
			          chan_nchip_non_ignored[ich] - 1; 
		      
		      if (chan_nchip[ich]             == 0 || 
                          chan_nchip_non_ignored[ich] == 0) 
			                           chan_finished[ich]=1;

/*******************  bytes_this_chip = chip_end[current_chip[ich]] -  ***********/ 
/*******************                    chip_begin[current_chip[ich]] +1;  ******/

/*- - - - - - - - - - - - - Odd number of bytes  - - - - - - - - - - - - - */
		      if ((bytes_this_chip%2 != 0)     && 
                           (bytes_this_chip   > 3 ))  {                       
			for (i3=0; i3<NCHAN; i3++) free(channel_data[i3]);
			*nbytes = -1;                                    
			sprintf(return_message,                          
 "Odd number of bytes for data from chip %d : number of bytes = %d",     
			current_chip[ich],bytes_this_chip);                     
			return return_message;                             
		      }                                                 
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
                    }
		 }
/*------------------------------------------------------------------*/
/* if it is not the 1st or 2nd byte, see if it is a new chip        */ 
/*------------------------------------------------------------------*/
/*		 if (ich==0)printf("bytes used %d trailer %d \n",bytes_used[ich],it_is_a_trailer);   */
		 if (bytes_used[ich] >= 4 && it_is_a_trailer == 0
		     && bytes_used[ich]%2 == 0) {
		   previous_byte   = *(channel_data[ich]+bytes_used[ich]-1);
		   previous_byte2  = *(channel_data[ich]+bytes_used[ich]-2);
		   iprevious_byte  = (int) previous_byte;
		   iprevious_byte2 = (int) previous_byte2;
		   iprevious_byte  = iprevious_byte & 0xff;
		   iprevious_byte2 = iprevious_byte2 & 0xff;
/*		   if (ich==0) printf(" prec 0x%x 0x%x \n",iprevious_byte,iprevious_byte2);   */
		   if ( iprevious_byte == 0 && 
		       (iprevious_byte2==0 || iprevious_byte2==0x80)) {
		     new_chip = 0;
		   }
		 }
/*----------------------------------------------------------*/
/* If this is a new chip, make sure it belongs here         */
/* NB I am assuming that the chip address is four bits      */
/*----------------------------------------------------------*/
		 if (new_chip == 0) {
/*		     if (ich == 0) printf("new chip ");     */
		     this_chip_addr = this_word & 0x78;
                     this_chip_addr = this_chip_addr >> 3;

                     current_chip[ich] = -1;
		     for (i4=0; i4<number_of_chips; i4++) {
		       if (chip_pm_chan[i4] == ich) {
			 if (this_chip_addr == chip_vram_code[i4]) { 
		            current_chip[ich] = i4;
			  }
		       }
		     }
/*  if (ich ==0) printf("start data for chip %d \n",current_chip[ich]);   */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* Bad address on this channel !!!                                        */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
		     if (current_chip[ich] == -1) {
			   for (i3=0; i3<NCHAN; i3++) {
			     free(channel_data[i3]);
			   }
			   *nbytes = -1;
                           sprintf(return_message,
             "Invalid chip address %d on ch %d",this_chip_addr,ich);
			   return return_message;
		      }
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* We had this chip already !!!!                                          */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
		     if (chip_begin[current_chip[ich]] != -1) {
			   for (i3=0; i3<NCHAN; i3++) {
			     free(channel_data[i3]);
			   }
			   *nbytes = -1;
                           sprintf(return_message,
               "Chip %d appears twice on channel %d",current_chip[ich],ich);
			   return return_message;
		     }
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* Okay, this is a valid chip                                             */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
		     chip_begin[current_chip[ich]] = bytes_used[ich];
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* Close if (new_chip==0) block                                            */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
		   }
/*-------------------------------------------------------------------*/
/* okay, put the data in                                             */
/*-------------------------------------------------------------------*/
		  *(channel_data[ich] + bytes_used[ich]) = this_byte;
		  bytes_used[ich] = bytes_used[ich] + 1;
		  if (chan_finished[ich] == 1)goto next_channel;
/*-------------------------------------------------------------------*/
/* end of loop over NWORD_READ words from VRAM                       */
/*-------------------------------------------------------------------*/
		}

/*-------------------------------------------------------------------*/
/* end of loop over channels                                         */
/*-------------------------------------------------------------------*/
	  next_channel: {}
     }

/*--------------------------------------------------------------------*/
/* end of while loop                                                  */
/* 5/16/97   change the shift, since we are using vemvembuffread2 now */
/* see if all channels are done                                       */
/*--------------------------------------------------------------------*/
/****	  shift = shift + NWORD_READ;                            ******/
      shift = shift + 2*NWORD_READ;
      dummy = 1;
      for(i90=0; i90<NCHAN; i90++) {
	if (chan_finished[i90] == 0) dummy=0;
      }
      all_chan_finished = dummy;
    }

/*&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&*/
/* At this point we have to do something about ignored chips         */
/* If there is no data from a skipped chip we will add a             */
/* a header and a trailer.  If there is data with hits, we           */
/* remove the hits.... If there is data with no hits, we leave as is */
/*&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&*/

  for (ichip=0; ichip<number_of_chips; ichip++) {
    if (ignore[ichip] == 0) continue;   /* if not ignored get next one       */
    ich = chip_pm_chan[ichip];          /* this is the channel for this chip */
    if (chip_begin[ichip] != -1) {      /* there is data for this chip       */
      if ( (chip_end[ichip] - chip_begin[ichip]) == 3 )
	               continue;        /* there are no hits, so we are okay */

/*----                                                           */
/* here we "delete" the hits for the ignored chip                */
/*----                                                           */
/* this is the number of deleted bytes                           */
      ndeleted = chip_end[ichip] - chip_begin[ichip] - 3;
/* stick in the trailer                                          */
      *(channel_data[ich] + chip_begin[ichip] + 2) = 0x0;
      *(channel_data[ich] + chip_begin[ichip] + 3) = 0x0;
      chip_end[ichip] = chip_begin[ichip] + 3;
/* shift all of the data up by ndeleted bytes                     */
      for (i90=(chip_end[ichip]+1); i90<bytes_used[ich]; i90++) {
	*(channel_data[ich] + i90) = *(channel_data[ich] + i90 + ndeleted);
      }
/* fix the chip pointers for the chips that follow                 */
      for (i90=0; i90<number_of_chips; i90++) {
	if (i90 != ichip  && chip_pm_chan[i90] == ich  &&
	                   chip_begin[i90] > chip_begin[ichip]) {
	  chip_begin[i90] = chip_begin[i90] - ndeleted;
	  chip_end[i90]   = chip_end[i90]   - ndeleted;
	}
      }
/* fix the number of bytes that we are dealing with                  */
      bytes_used[ich] = bytes_used[ich] - ndeleted;


    } else {               /* no data from this chip, let's make it up */
      
/*----                                                           */
/* here we "add" entries for the ignored chip                    */
/*----                                                           */

/* is there enough memory allocated  ?????    */

      if ( (bytes_used[ich]+4) > bytes_allocated[ich]) {   /* need more room */
	temp = channel_data[ich];
	channel_data[ich] = (char *) malloc((bytes_used[ich]+4)*sizeof(char));

	if (channel_data[ich] == NULL) {     /* cannot get more memory */
	  channel_data[ich] = temp;
	  for (i2=0; i2<NCHAN; i2++) {
	    free(channel_data[i2]);
	  }
	  strcpy(return_message,"Run out of unix memory");
	  *nbytes = -1;
	  return return_message;

	} else {                                  /* got more memory */
	  
	  for (iby=0; iby<bytes_used[ich]; iby++) {
	    *(channel_data[ich] + iby) = *(temp+iby);
	  }
	  bytes_allocated[ich] = bytes_used[ich] + 4;
	  free(temp);
	}
      }

/*-- add the ignored chip entry at the end --------------------------------*/
/* NB in this system the trigger tag is always 1                           */
/*    To get the trigger time right, let's look at the second byte in the  */
/*    header of an other chip.  If we cannot find any other chip, which    */
/*    should not happen, we'll stick in a 0x1                              */

      first_byte = chip_vram_code[ichip] << 3;
      first_byte = first_byte | 0x86;      /* Start bit, data bit, trig=1 */
      second_byte = 0x1;

      for (i90=0; i90<NCHAN; i90++) {
	if (bytes_used[i90] == 0) continue;
	for (jchip=0; jchip<number_of_chips; jchip++) {
	  if (chip_pm_chan[jchip] != i90)     continue;
	  if (jchip == ichip)                 continue;
	  if (chip_begin[jchip] == -1)        continue;
	  second_byte = *(channel_data[i90] + chip_begin[jchip] + 1);
	  goto put_fake_data_in;
	}
      }

 put_fake_data_in: {}

      *(channel_data[ich] + bytes_used[ich])     = first_byte;  /*  header1  */
      *(channel_data[ich] + bytes_used[ich] + 1) = second_byte; /*  header2  */
      *(channel_data[ich] + bytes_used[ich] + 2) = 0x0;         /*  trailer1 */
      *(channel_data[ich] + bytes_used[ich] + 3) = 0x0;         /*  trailer2 */
      chip_begin[ichip] = bytes_used[ich];
      chip_end[ichip]   = bytes_used[ich] + 3;
      bytes_used[ich]   = bytes_used[ich] + 4;   
    
    }
  }


/*&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&*/
/* we now have all of the data in separate chunks of memory   */
/* Let us collect it into one chunk.                          */
/* First, get all of the shifts that we need                  */
/*&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&*/

/*-------------------------------------------------------------------*/
/* How many bytes in the event record ?                              */
/*-------------------------------------------------------------------*/
  *nbytes = 0;
/*-------------------------------------------------------------------*/
/* Four bytes with number of bytes in the event                      */
/*-------------------------------------------------------------------*/
  sh_evlength = 0;
  *nbytes     = *nbytes + 4;
/*-------------------------------------------------------------------*/
/* Four bytes of run number                                          */
/*-------------------------------------------------------------------*/
  sh_run        = *nbytes;
  *nbytes       = *nbytes + 4;
/*-------------------------------------------------------------------*/
/* Four bytes of event number                                        */
/*-------------------------------------------------------------------*/
  sh_event      = *nbytes;
  *nbytes       = *nbytes + 4;
/*-------------------------------------------------------------------*/
/* One byte with number of chips                                     */
/*-------------------------------------------------------------------*/
  sh_nchip      = *nbytes;
  *nbytes       = *nbytes + 1;
/*-------------------------------------------------------------------*/
/* Four bytes of control registers per chip                          */
/*-------------------------------------------------------------------*/
  sh_contreg    = *nbytes;
  *nbytes       = *nbytes + 4*(number_of_chips);                   
/*-------------------------------------------------------------------*/
/* Sixteen bytes of calibration mask per chip                        */
/*-------------------------------------------------------------------*/
  sh_calmask    = *nbytes;                                         
  *nbytes       = *nbytes + 16*(number_of_chips);                  
/*-------------------------------------------------------------------*/
/* Sixteen bytes of channel mask per chip                            */
/*-------------------------------------------------------------------*/
  sh_chnmask    = *nbytes;                                         
  *nbytes       = *nbytes + 16*(number_of_chips);                  
/*-------------------------------------------------------------------*/
/* Four bytes of begin data for each chip                            */
/*-------------------------------------------------------------------*/
  sh_chpbegin   = *nbytes;
  *nbytes       = *nbytes + 4*(number_of_chips);
/*-------------------------------------------------------------------*/
/* Four bytes of end data for each chip                              */
/*-------------------------------------------------------------------*/
  sh_chpend     = *nbytes;
  *nbytes       = *nbytes + 4*(number_of_chips);
/*-------------------------------------------------------------------*/
/* If this is an XY scan we also stick in the following :            */
/*         four bytes of begin XY data                               */
/*         four bytes of end   XY data                               */
/*         four bytes of begin X position                            */
/*         four bytes of begin Y position                            */
/*-------------------------------------------------------------------*/
  if (xy_scan == 1) {
     sh_xybegin = *nbytes;
     *nbytes    = *nbytes + 4;
     sh_xyend   = *nbytes;
     *nbytes    = *nbytes + 4;
     sh_xydata  = *nbytes;
     *nbytes    = *nbytes + 8;
   }
/*-------------------------------------------------------------------*/
/* How many bytes did we read from VRAM ???                          */
/*-------------------------------------------------------------------*/
  nbytes_read = 0;
  for (i1=0; i1<NCHAN; i1++) {
    nbytes_read = nbytes_read + bytes_used[i1];
  }
  sh_data = *nbytes;
  *nbytes = *nbytes + nbytes_read;

/*&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&*/
/* Now lets get some memory                                   */
/*&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&*/
  cal_data_point = (char *) malloc(*nbytes);
  if (cal_data_point == NULL) {
	     for (i2=0; i2<NCHAN; i2++) {
		      free(channel_data[i2]);
	     }
             strcpy(return_message,"Run out of UNIX memory");
             *nbytes = -1;
	     return return_message;
  }
/*------------------------------------------------------------*/
/* Stick in what we can right now...number of bytes           */
/*------------------------------------------------------------*/
  j = cal_extract(0,nbytes,buff);
  for (i=0; i<4; i++) {
    *(cal_data_point+sh_evlength+i) = buff[i];
  }
/*-------------------------------------------------------------------*/
/* run number                                                        */
/*-------------------------------------------------------------------*/
  itemp = run_number;
  j = cal_extract(0,&itemp,buff);
  for (i=0; i<4; i++) {
    *(cal_data_point + sh_run+i) = buff[i];
  }
/*-------------------------------------------------------------------*/
/* event number                                                      */
/*-------------------------------------------------------------------*/
  j = cal_extract(0,&nev,buff);
  for (i=0; i<4; i++) {
    *(cal_data_point + sh_event+i) = buff[i];
  }
/*-------------------------------------------------------------------*/
/* One byte with number of chips                                     */
/*-------------------------------------------------------------------*/
  *(cal_data_point + sh_nchip) = (char) number_of_chips;
/*-------------------------------------------------------------------*/
/* Control registers                                                 */
/*-------------------------------------------------------------------*/
  this_shift = sh_contreg;
  for (i=0; i<number_of_chips; i++) {
    itemp = cal_creg[i];
    j     = cal_extract(0,&itemp,buff);
    for (i3=0; i3<4; i3++) {
      *(cal_data_point + this_shift) = buff[i3];
      this_shift++;
    }
  }
/*-------------------------------------------------------------------*/
/* Calibration mask                                                  */
/*-------------------------------------------------------------------*/
  this_shift = sh_calmask;                                         
  for (i=0; i<number_of_chips; i++) {                              
    for (i2=0; i2<8; i2++) {                                       
/*     itemp = calib_mask[i][i2];   CHANGED TO WRITTEN MASKS CC 1/4/97  */
       itemp = written_calib_mask[i][i2];
      j     = cal_extract(0,&itemp,buff);                          
      for (i3=0; i3<2; i3++) {                                     
	*(cal_data_point + this_shift) = buff[i3];                 
	this_shift++;                                              
      }                                                            
    }                                                              
  }                                                                
/*-------------------------------------------------------------------*/
/* Channel mask                                                      */
/*-------------------------------------------------------------------*/
  this_shift = sh_chnmask;                                         
  for (i=0; i<number_of_chips; i++) {                              
    for (i2=0; i2<8; i2++) {                                       
/*    itemp = channel_mask[i][i2]; CHANGED TO WRITTEN MASKS CC 1/4/97  */
      itemp = written_channel_mask[i][i2];
      j     = cal_extract(0,&itemp,buff);   
      for (i3=0; i3<2; i3++) {                                     
	*(cal_data_point + this_shift) = buff[i3];                 
	this_shift++;                                              
      }                                                            
    }                                                              
  }                                                                
/*===================================================================*/
/* Now we stick in the actual data                                   */
/*===================================================================*/
  this_shift = sh_data;
  for (ich=0; ich<NCHAN; ich++) {
   if (bytes_used[ich] != 0) {
      sh_chan[ich] = this_shift;
      for (i2=0; i2<bytes_used[ich]; i2++) {
	*(cal_data_point + this_shift) = *(channel_data[ich] + i2);
	this_shift++;
      }
    }
  }
/*-----------------------------------------------------------------*/
/* Stick in the XY data and pointers                               */
/*-----------------------------------------------------------------*/
  if (xy_scan == 1) {
    x  = 1000. * coord[0];
    y  = 1000. * coord[1];
    ix = x;
    iy = y;
    itemp = sh_xydata;
    j     = cal_extract(0,&itemp,buff);
    for (i3=0; i3<4; i3++) {
      *(cal_data_point + sh_xybegin + i3) = buff[i3];
    }
    itemp = sh_xydata + 8;
    j    = cal_extract(0,&itemp,buff);
    for (i3=0; i3<4; i3++) {
      *(cal_data_point + sh_xyend + i3) = buff[i3];
    }
    j    = cal_extract(0,&ix,buff);
    for (i3=0; i3<4; i3++) {
      *(cal_data_point + sh_xydata + i3) = buff[i3];
    }
    j    = cal_extract(0,&iy,buff);
    for (i3=0; i3<4; i3++) {
      *(cal_data_point + sh_xydata + 4 + i3) = buff[i3];
    }
  }
/*-----------------------------------------------------------------*/
/* FINALLY, the chip pointers                                      */
/*-----------------------------------------------------------------*/
  for (i=0; i<number_of_chips; i++) {
    ich = chip_pm_chan[i];
    chip_begin[i] = chip_begin[i] + sh_chan[ich];
    chip_end[i]   = chip_end[i]   + sh_chan[ich];
    itemp = chip_begin[i];
    j     = cal_extract(0,&itemp,buff);
    for (i3=0; i3<4; i3++) {
        *(cal_data_point + sh_chpbegin + 4*i + i3) = buff[i3];
    }
    itemp = chip_end[i];
    j     = cal_extract(0,&itemp,buff);
    for (i3=0; i3<4; i3++) {
        *(cal_data_point + sh_chpend + 4*i + i3) = buff[i3];
    }
  }

/*-------------------------------------------------------------*/
/* Release our temporary memory                                */
/*-------------------------------------------------------------*/
  for (i=0; i<NCHAN; i++) {
    free(channel_data[i]);
  }
/*&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&*/
/* Done.  Release get out of here                             */
/*&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&*/
  strcpy(return_message,"OK");
  return return_message;
}

/******************************************************************/
/******************************************************************/
/******************************************************************/
/******************************************************************/
/* This builds the start of run record                            */
/* Fills a pointer with the first byte of data (cal_data_point)   *
/* Returns the length in bytes of the data (nnbytes)              *
/*                                                                */
/* The return code is "OK" or a string with an error message      */
/* In case of error, nnbytes = 0                                  */
/*                                                                */
/* 11 february 1997 :                                             */
/*        removed masks from begin-run record                     */
/*        added 128 user defined bytes                            */
/* 28 january 1998  :                                             */
/*        added number of chips top side                          */
/*  9 may 1998 :                                                  */
/*        added a bit mask of ignored chips                       */
/******************************************************************/

char *cal_start_run_record(int* nnbytes)
{
  int sh_reclength, sh_run, sh_nchip, sh_calmask, sh_chnmask, sh_user;
  int i, j, i2, i3, this_shift, itemp, bit;
  char buff[4];
  static char return_message[200];

/*----------------------------------------------------------*/
/* Initialize                                               */
/*----------------------------------------------------------*/
   cal_data_point = NULL;
/*-------------------------------------------------------------------*/
/* How many bytes in the event record ?                              */
/*-------------------------------------------------------------------*/
  *nnbytes = 0;
/*-------------------------------------------------------------------*/
/* Four bytes with number of bytes in the record                     */
/*-------------------------------------------------------------------*/
  sh_reclength = 0;
  *nnbytes     = *nnbytes + 4;
/*-------------------------------------------------------------------*/
/* Four bytes of run number                                          */
/*-------------------------------------------------------------------*/
  sh_run         = *nnbytes;
  *nnbytes       = *nnbytes + 4;
/*-------------------------------------------------------------------*/
/* One byte with number of chips                                     */
/*-------------------------------------------------------------------*/
  sh_nchip       = *nnbytes;
  *nnbytes       = *nnbytes + 1;
/*-------------------------------------------------------------------*/
/* Sixteen bytes of calibration mask per chip                        */
/*-------------------------------------------------------------------*/
/*  sh_calmask     = *nnbytes;                                       */
/*  *nnbytes       = *nnbytes + 16*(number_of_chips);                */  
/*-------------------------------------------------------------------*/
/* Sixteen bytes of channel mask per chip                            */
/*-------------------------------------------------------------------*/
/*  sh_chnmask     = *nnbytes;                                       */  
/*  *nnbytes       = *nnbytes + 16*(number_of_chips);                */
/*-------------------------------------------------------------------*/
/* user defined bytes                                                */
/*-------------------------------------------------------------------*/
  sh_user        = *nnbytes;
  *nnbytes       = *nnbytes + 128;
/*-------------------------------------------------------------------*/
/* Now lets get some memory                                          */
/*-------------------------------------------------------------------*/
  cal_data_point = (char *) malloc(*nnbytes);
  if (cal_data_point == NULL) {
             strcpy(return_message,"Run out of UNIX memory");
             *nnbytes = 0;
	     return return_message;
  }
/*------------------------------------------------------------*/
/* Stick in the number of bytes                               */
/*------------------------------------------------------------*/
  j = cal_extract(0,nnbytes,buff);
  for (i=0; i<4; i++) {
    *(cal_data_point+sh_reclength+i) = buff[i];
  }
/*-------------------------------------------------------------------*/
/* run number                                                        */
/*-------------------------------------------------------------------*/
  itemp = run_number;
  j = cal_extract(0,&itemp,buff);
  for (i=0; i<4; i++) {
    *(cal_data_point + sh_run+i) = buff[i];
  }
/*-------------------------------------------------------------------*/
/* One byte with number of chips                                     */
/*-------------------------------------------------------------------*/
  *(cal_data_point + sh_nchip) = (char) number_of_chips;
/*-------------------------------------------------------------------*/
/* Calibration mask                                                  */
/*-------------------------------------------------------------------*/
/*  this_shift = sh_calmask;                                         */
/*  for (i=0; i<number_of_chips; i++) {                              */
/*    for (i2=0; i2<8; i2++) {                                       */
/*      itemp = calib_mask[i][i2];                                   */
/*      j     = cal_extract(0,&itemp,buff);                          */
/*      for (i3=0; i3<2; i3++) {                                     */
/*	*(cal_data_point + this_shift) = buff[i3];                   */
/*	this_shift++;                                                */
/*      }                                                            */
/*    }                                                              */
/*  }                                                                */
/*-------------------------------------------------------------------*/
/* Channel mask                                                      */
/*-------------------------------------------------------------------*/
/*  this_shift = sh_chnmask;                                         */
/*  for (i=0; i<number_of_chips; i++) {                              */
/*    for (i2=0; i2<8; i2++) {                                       */
/*      itemp = channel_mask[i][i2];                                 */
/*      j     = cal_extract(0,&itemp,buff);                          */
/*      for (i3=0; i3<2; i3++) {                                     */
/*	*(cal_data_point + this_shift) = buff[i3];                   */
/*	this_shift++;                                                */
/*      }                                                            */
/*    }                                                              */
/*  }                                                                */
/*-------------------------------------------------------------------*/
/* 128 user defined bytes                                            */
/*-------------------------------------------------------------------*/
  this_shift = sh_user;
  for (i=0; i<128; i++) {
    itemp = 0;
    if (i==0 && compressed_format==1) itemp=0x2;
    if (i==0 && xy_scan == 1) itemp = itemp | 0x1;
    *(cal_data_point + this_shift) = (char) itemp;
    this_shift++;
  }
/*-------------------------------------------------------------------*/
/* Number of chips on the top side (added 28 jan 1998)               */
/*-------------------------------------------------------------------*/
  this_shift = sh_user + 1;
  *(cal_data_point + this_shift) = (char) number_of_chips_top;
/*-------------------------------------------------------------------*/
/* bit mask of ignored chips (added 9 may 1998)                      */
/*-------------------------------------------------------------------*/
  for (i=0; i<number_of_chips; i++) {
    if (ignore[i] == 1) {
      i2 = 1;
      j = i/8;
      this_shift = sh_user + 2 + j;
      bit = i - 8*j;
      if (bit == 0) {
	bit = i2;
      } else {
	bit = i2 <<  bit;
      }
      *(cal_data_point + this_shift) =
	*(cal_data_point + this_shift) | bit;
    }
  }
/*-------------------------------------------------------------------*/
/* Done                                                              */
/*-------------------------------------------------------------------*/
  strcpy(return_message,"OK");
  return return_message;
}



/**********************************************************************/
/* Move data into VRAM.  Return "OK" if okay.                         *
/**********************************************************************/
char *cal_move_into_vram(Tcl_Interp* interp)
{
/*----------------------------------------------------------*/
/* A bunch of commands to pass to Tcl_Eval                  */
/*----------------------------------------------------------*/
  static char cmd100[] = "vme_set daq_0(vram_dir)";
  static char cmd101[] = "vme_set daq_0(done_rst)"; 
  static char cmd102[] = "vme_read daq_0(done_int)";
  static char cmd103[] = "vme_read daq_0(timeout_int)";
  static char cmd301[] = "vme_set daq_0(done_rst)"; 
  static char cmd302[] = "vme_set daq_0(brd_reset)";
  static char cmd304[] = "vme_set daq_0(fctrl_en)";
  static char cmd305[] = "vme_clear daq_0(fctrl_en)";

int bit_value, dummy;
time_t now;
time_t then;
double elapsed_time;     /* in seconds  */
double timeout=1.0;     /* in seconds (changed from 3.0   Dec 7, 1996 */
static char return_message[200];
int fctrl_set;


     fctrl_set = 0;
/*------------------------------------------------------------*/
/* Keep checking done_int                                     */
/* Do it only for a maximum amount of time                    */
/*------------------------------------------------------------*/
     then = time(NULL);
     for (;;) {
        dummy = Tcl_GlobalEval(interp, cmd102);
        if (dummy != TCL_OK) {
            strcpy(return_message,"Error reading done_int");
	    return return_message;
	}
	dummy = Tcl_GetInt(interp,interp->result,&bit_value);
	if (dummy != TCL_OK) {
	    strcpy(return_message,"Error unpacking done_int");
            return return_message;
	}
        if (bit_value == 1) break;
	now = time(NULL);
	elapsed_time = difftime(now,then);
	if (elapsed_time > timeout) break;
      }
/*-------------------------------------------------------------*/
/* If the bit value is not one, try one more time              */
/*-------------------------------------------------------------*/
     if (bit_value != 1) {
	dummy = Tcl_GlobalEval(interp, cmd102);
	if (dummy != TCL_OK) {
           strcpy(return_message,"Error reading done_int");
	   return return_message;
        }
	dummy = Tcl_GetInt(interp,interp->result,&bit_value);
	if (dummy != TCL_OK) {
	    strcpy(return_message,"Error unpacking done_int");
            return return_message;
	}
      }
/*-------------------------------------------------------------*/
/* now set the timeout enable bit                              */
/*-------------------------------------------------------------*/
      if (bit_value != 1) {
         fctrl_set = 1;
	 dummy = Tcl_GlobalEval(interp, cmd304);

	 
/*------------------------------------------------------------*/
/* Keep checking done_int                                     */
/* Do it only for a maximum amount of time                    */
/*------------------------------------------------------------*/
         then = time(NULL);
	 for (;;) {
	   dummy = Tcl_GlobalEval(interp, cmd102);
	   if (dummy != TCL_OK) {
	     strcpy(return_message,"Error reading done_int");
	     return return_message;
	   }
	   dummy = Tcl_GetInt(interp,interp->result,&bit_value);
	   if (dummy != TCL_OK) {
	     strcpy(return_message,"Error unpacking done_int");
	     return return_message;
	   }
	   if (bit_value == 1) break;
	   now = time(NULL);
	   elapsed_time = difftime(now,then);
	   if (elapsed_time > timeout) break;
	 }
       }
/*-------------------------------------------------------------*/
/* If the bit value is not one, try one more time              */
/*-------------------------------------------------------------*/
     if (bit_value != 1) {
	dummy = Tcl_GlobalEval(interp, cmd102);
	if (dummy != TCL_OK) {
           strcpy(return_message,"Error reading done_int");
	   return return_message;
        }
	dummy = Tcl_GetInt(interp,interp->result,&bit_value);
	if (dummy != TCL_OK) {
	    strcpy(return_message,"Error unpacking done_int");
            return return_message;
	}
      }
/*------------------------------------------------------------*/
/*  clear the fctrl bit, if it had been set                   */
/*------------------------------------------------------------*/
      if (fctrl_set == 1) {
	 dummy = Tcl_GlobalEval(interp, cmd305);
       }
       if (bit_value != 1) {
        strcpy(return_message,"Fatal timeout reading done_int");
	return return_message;
       }
/*------------------------------------------------------------*/
/* Check for timeout_int                                      */
/* NB: PROBLEMS WITH THIS BIT AND done-rst ???                */
/*     MAYBE MOVE THIS AFTER DONE_RST ???????                 */
/*------------------------------------------------------------*/
      dummy = Tcl_GlobalEval(interp, cmd103);
      if (dummy != TCL_OK) {
            strcpy(return_message,"Error reading timeout_int");
	    return return_message;
      }
      dummy = Tcl_GetInt(interp,interp->result,&bit_value);
      if (dummy != TCL_OK) {
	    strcpy(return_message,"Error unpacking timeout_int");
            return return_message;
      }
      if (bit_value == 1) {
	  strcpy(return_message,"timeout_int reports a timeout");
	  return return_message;
      }
/*-------------------------------------------------------------*/
/* Set done_rst                                                */
/*-------------------------------------------------------------*/
/*  dummy = Tcl_GlobalEval(interp,"vme_read daq_0(timeout_int)");         */
/*  printf("timeout int before checking done_rst %s \n",interp->result);  */
     dummy = Tcl_GlobalEval(interp,cmd301);
     if (dummy != TCL_OK) {
        free(cal_data_point);
        strcpy(return_message,"Error setting done_rst");
	return return_message;
      }
/*  dummy = Tcl_GlobalEval(interp,"vme_read daq_0(timeout_int)");        */
/*  printf("timeout int after checking done_rst %s \n",interp->result);  */
/*-------------------------------------------------------------*/
/* xfer_done is okay ... change direction to move data to VRAM */
/* ARMIN SAYS DONT DO IT HERE                                  */
/*-------------------------------------------------------------*/
/*    dummy = Tcl_GlobalEval(interp,cmd100);                   */
/*     if (dummy != TCL_OK) {                                  */
/*       strcpy(return_message,"Error setting vram_dir");      */
/*	return return_message;                                 */
/*      }                                                      */ 
/*-------------------------------------------------------------*/
/* if we get here, everything is okay                          */
/*-------------------------------------------------------------*/
     strcpy(return_message,"OK");
     return return_message;
}

