#include "cal_svtdef.h"
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <histoscope.h> 
#include "tk.h"
#include "reglist.h"
#include "cal_global.h"
#include "cal_functions.h"

#define MAX_USER_HIST 100
#define NUM_CHIP_CHANNELS 128
#define NUM_HARDWIRED_HIST (MAX_NUM_CHIP+8)
#define TRUE 1
#define DEFINED_HIST_TYPES 6
/*-----------------------------------------------------------------*/
/* These are the user defined histograms                           */
/*-----------------------------------------------------------------*/
struct user_hist_struc {    /* for histograms booked on the fly */
  int min_chip;             
  int max_chip;
  int min_chan;
  int max_chan;
  int hist_type;
  int booked;               /* =1 booked */
  int user_hist_id;
};
int num_user_hist = 0;
struct user_hist_struc user_hist[MAX_USER_HIST];
void pass_histoscope_define(Tcl_Interp*);
void cal_book_user_hist();
void cal_fill_user_hist(int, int, int, int);
void cal_display_message(Tcl_Interp*, char*);
/*---------------------------------------------------------------------------*/
/* These are the hardwired histograms                                        */
/* Hbook_ID    Histoscope_ID       what                                      */
/* 100+i           i            hit map for chip i                           */
/* 100+nchip       nchip        hit map, all chips                           */
/* 500             nchip+1      threshold scan, all channels                 */
/* 102+nchip       nchip+2      ntuple, commented out                        */
/* 103+nchip       nchip+3      XY scan, 2D hist, commented out              */
/* 501             nchip+4      threshold scan, fired channels               */
/* 502             nchip+5      charge scan, all channels                    */
/* 503             nchips+6     charge scan, fired channels                  */
/* 200+nchip       nchips+7    all chip map, charge weighted (XY scans only) */
/*---------------------------------------------------------------------------*/
int hs_hmap_id[NUM_HARDWIRED_HIST];      /* these are the histoscope id */
int update = 0;
/*****************************************************************/
/* starts histoscope, books histogram                            */
/*****************************************************************/
int cal_start_histoscope()
{

   int i;
   char title[200];
   static char *names[5] = {  "channel#", "TOT", "T_stamp", "threshold",
    			     "injected charge"  };
   float coord_min, coord_max, coord_temp;
   int nc;
/*-------------------------------------------------------------*/
/*  zeroth order initializations                               */
/*-------------------------------------------------------------*/
   hs_initialize("Histoscope_stuff.c");
   hs_histoscope(TRUE);  
/*-------------------------------------------------------------*/
/* book histograms with hit maps                               *
/*-------------------------------------------------------------*/
   for (i=0; i<number_of_chips; i++) {
       sprintf(title,"Chip %d",i);
       strcat(title," map");
       hs_hmap_id[i] = hs_create_1d_hist(100+i, title, "", 
				 "Channel number",
			         "Number of hits", NUM_CHIP_CHANNELS, 0, 
			          NUM_CHIP_CHANNELS);
       if (hs_hmap_id[i] < 0) return 1;
  }
  hs_hmap_id[number_of_chips] = hs_create_1d_hist(100+number_of_chips, 
				     "All chips map", "",
				     "Channel number", "Number_of_hits",
				     number_of_chips*NUM_CHIP_CHANNELS, 0,
				     number_of_chips*NUM_CHIP_CHANNELS);
  if (hs_hmap_id[number_of_chips] < 0) return 1;
/*------------------------------------------------------------------------*/
/* These are Armin's histograms                                           */
/* Comment out his ntuple                                                 */
/*------------------------------------------------------------------------*/
  hs_hmap_id[number_of_chips+1] = hs_create_2d_hist(500, 
		"Threshold scan", "", "Threshold", "Channel number",
		"Number_of_hits",64, number_of_chips*NUM_CHIP_CHANNELS, 
		-0.5, 63.5, 0, number_of_chips*NUM_CHIP_CHANNELS);
  if (hs_hmap_id[number_of_chips+1] < 0) return 1;
/*------------------------------------------------------------------------*/
/*  hs_hmap_id[number_of_chips+2] = hs_create_ntuple(102+number_of_chips, */
/*		"Armin's SVT chip test ntuple", "", 5, names);            */
/*  if (hs_hmap_id[number_of_chips+2] < 0) return 1;                      */
/*------------------------------------------------------------------------*/

/*------------------------------------------------------------------------*/
/* XY scan...position vs channel number                                   */
/* ASSUMES that X is the top of the hybrid                                */
/* (commented out, histoscope_id set to -1)                               */
/*------------------------------------------------------------------------*/
     hs_hmap_id[number_of_chips+3] = -1;       
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/*   if (xy_scan == 1 && xy_nstep != 0) {                                 */
/*      coord_min = xy_x_initial;                                         */
/*      if (xy_which_axis == 2) coord_min = xy_y_initial;                 */
/*      coord_min = coord_min - 0.5 * xy_step_size;                       */
/*      coord_max = coord_min + (xy_nstep+1)*xy_step_size;                */
/*      if (xy_step_size < 0.) {                                          */
/*	coord_temp = coord_min;                                           */
/*	coord_min  = coord_max;                                           */
/*	coord_max  = coord_min;                                           */
/*      }                                                                 */
/*      if (number_of_sides == 1) {                                       */
/*	nc = number_of_chips;                                             */
/*      } else if (xy_which_axis == 1) {                                  */
/*	nc = number_of_chips_top;                                         */
/*      } else {                                                          */
/*	nc = number_of_chips - number_of_chips_top;                       */
/*      }                                                                 */
/*      hs_hmap_id[number_of_chips+3] =                                   */
/*                hs_create_2d_hist(103+number_of_chips,                  */
/*	  "XY Scan", "", "LED position", "Channel number",                */
/*        "Number of hits",                                               */
/*	  (xy_nstep+1), NUM_CHIP_CHANNELS*nc, coord_min,                  */
/*        coord_max, 0,  NUM_CHIP_CHANNELS*nc);                           */
/*      if (hs_hmap_id[number_of_chips+3] < 0) return 1;                  */
/*                                                                        */
/*    } else {                                                            */
/*      hs_hmap_id[number_of_chips+3] = -1;                               */
/*    }                                                                   */
/*------------------------------------------------------------------------*/
/* same as hist500, but only for channels that were fired                 */
/*------------------------------------------------------------------------*/
  hs_hmap_id[number_of_chips+4] = hs_create_2d_hist(501, 
		"Threshold scan-fired chan", "", "Threshold", "Channel number",
		"Number_of_hits",64, number_of_chips*NUM_CHIP_CHANNELS, 
		-0.5, 63.5, 0, number_of_chips*NUM_CHIP_CHANNELS);
  if (hs_hmap_id[number_of_chips+4] < 0) return 1;

/*-----------------------------------------------------------------------*/
/* add histograms of charge scan (29-may-1997)                           */
/*-----------------------------------------------------------------------*/
  hs_hmap_id[number_of_chips+5] = hs_create_2d_hist(502, 
		"Charge scan", "", "Charge", "Channel number",
		"Number_of_hits",64, number_of_chips*NUM_CHIP_CHANNELS, 
		-0.5, 63.5, 0, number_of_chips*NUM_CHIP_CHANNELS);
  if (hs_hmap_id[number_of_chips+5] < 0) return 1;
  hs_hmap_id[number_of_chips+6] = hs_create_2d_hist(503, 
		"Charge scan-fired channel", "", "Charge", "Channel number",
		"Number_of_hits",64, number_of_chips*NUM_CHIP_CHANNELS, 
		-0.5, 63.5, 0, number_of_chips*NUM_CHIP_CHANNELS);
  if (hs_hmap_id[number_of_chips+6] < 0) return 1;
/*------------------------------------------------------------------------*/
/* book histograms with hit maps - charge weighted                        */
/* DO IT FOR XY SCANS ONLY                                                */
/*------------------------------------------------------------------------*/
  if (xy_scan == 1) {
    hs_hmap_id[number_of_chips+7] = hs_create_1d_hist(200+number_of_chips, 
				     "All chips map - Charge weighted", "",
				     "Channel number", "Number_of_hits",
				     number_of_chips*NUM_CHIP_CHANNELS, 0,
				     number_of_chips*NUM_CHIP_CHANNELS);
    if (hs_hmap_id[number_of_chips+7] < 0) return 1;
  } else {
    hs_hmap_id[number_of_chips+7] = -1;
  }
/*------------------------------------------------------------------------*/
/* Book those histograms that were booked interactively                   */
/*------------------------------------------------------------------------*/
  cal_book_user_hist();
/*------------------------------------------------------------------------*/
/* done                                                                   */
/*------------------------------------------------------------------------*/
  return 0;
}

/**************************************************************************/
/* ends histoscope                                                        */
/**************************************************************************/
int cal_finish_histoscope(Tcl_Interp* interp) {

int num_items;
int dummy;
static char msg[] = ".run_menu.text insert end \"Saving histoscope output to file....";
static char msg2[] = ".run_menu.text yview -pickplace end";
static char msg3[] = "cal_close_histoscope_connection";
char his_out_file[200];
char temp[100];
char* temp2;
char tempmsg[300];

/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* Figure out the name of the output file                                */
/* e.g. something like /nfs/babar/data/part_name/histoscope_XXX.out      */
/*      where XXX is the runnumber.  It assumes that the path (up to     */
/*      but not including the part name) includes a trailing slash       */
/*      in the global tcl variable data_directory                        */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  temp2 = Tcl_GetVar(interp,"data_directory",0);
  strcpy(his_out_file,temp2);
  temp2 = Tcl_GetVar(interp,"part_name",0);
  strcat(his_out_file,temp2);
  if (strcmp(temp,"") != 0)strcat(his_out_file,"/"); 
  strcat(his_out_file,"histoscope_");
  strcat(his_out_file,C_run_number);
  strcat(his_out_file,".out");
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* Now a loop waiting for the user to close histoscope                   */
/* If she does not want to close histoscope, then update it instead      */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  strcpy(temp,"empty");
  while (strcmp(temp,"close") != 0) {
       hs_update();
       dummy = Tcl_Eval(interp,msg3);
       strcpy(temp,interp->result);
     }
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* Close histoscope                                                      */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  num_user_hist = 0;
  num_items = hs_save_file(his_out_file);
  strcpy(tempmsg,msg);
  strcat(tempmsg,his_out_file);
  strcat(tempmsg,"\n\"");
  dummy = Tcl_Eval(interp,tempmsg);    
  dummy = Tcl_Eval(interp,msg2);    
  hs_complete();  
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* Turn the histoscope file into an hbook file                           */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  sprintf(tempmsg,"histo_to_hbook %s .run_menu",his_out_file);
  dummy = Tcl_Eval(interp,tempmsg);
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* Done                                                                  */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  return 0;
}
/*******************************************************************/
/*  this fills histograms....note it updates histoscope every time */
/*    (Perhaps excessive updating rate ?????)                      */
/*******************************************************************/
void cal_fill_histograms(Tcl_Interp* interp) {


  float this_coord;
  int this_channel;
  int ishift, i, j, temp, dummy, nhits, k, dummy1, dummy2;
  int begin_data[MAX_NUM_CHIP];
  int end_data[MAX_NUM_CHIP];
  int chnum, tot, tstamp;
  float ntuple_val[5];
  float tot_charge;
  static char  *command_result;
/*--------------------------------------------------------*/
/* Get the begin and end of data for each chip            */
/*--------------------------------------------------------*/
  ishift = 13 + 4 * number_of_chips + 32 * number_of_chips;
  for (i=0; i<number_of_chips; i++) {
    dummy = cal_extract(1, &temp, (cal_data_point+ishift));
    begin_data[i] = temp;
    ishift = ishift + 4;
  }
  for (i=0; i<number_of_chips; i++) {
    dummy = cal_extract(1, &temp, (cal_data_point+ishift));
    end_data[i] = temp;
    ishift = ishift + 4;
  }
/*--------------------------------------------------------*/
/* Now loop over chips, NB if only four bytes ==> no hits */
/* First two bytes are header                             */
/* Last two bytes are trailer                             */
/* NB; make sure the histogram has not been deleted       */
/*--------------------------------------------------------*/
  for (i=0; i<number_of_chips; i++) {
    nhits = (end_data[i] - begin_data[i] - 3) / 2;
    for (j=1; j<=nhits; j++) {
      ishift = begin_data[i] + 2*j;
      cal_get_hit(cal_data_point, ishift, &chnum, &tot, &tstamp);
	ntuple_val[0] = (float) chnum;
	ntuple_val[1] = (float) tot;
	ntuple_val[2] = (float) tstamp;
	ntuple_val[3] = (float) cal_thresh_dac [0];
	ntuple_val[4] = (float) cal_cal_dac[0];
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* comment out the filling of the ntuple                                  */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/*        if (hs_hmap_id[number_of_chips+2] >= 0) {                       */
/*             hs_fill_ntuple(hs_hmap_id[number_of_chips+2], ntuple_val); */
/*        }                                                               */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

        if (hs_hmap_id[number_of_chips+1] >= 0) {
             hs_fill_2d_hist(hs_hmap_id[number_of_chips+1],
			   (float) cal_thresh_dac [i], 
			    (float) (i*NUM_CHIP_CHANNELS + chnum), 1.);
	}
        if (hs_hmap_id[number_of_chips+5] >= 0) {
             hs_fill_2d_hist(hs_hmap_id[number_of_chips+5],
			   (float) cal_cal_dac [i], 
			    (float) (i*NUM_CHIP_CHANNELS + chnum), 1.);
	}

        dummy1 = cal_mask_manipulate(i, chnum, 0, 0, 1); 
        dummy2 = cal_mask_manipulate(i, chnum, 1, 0, 1);

        if (dummy1 == 1 && dummy2 == 1) {
	  if (hs_hmap_id[number_of_chips+4] >= 0) 
                hs_fill_2d_hist(hs_hmap_id[number_of_chips+4],
			   (float) cal_thresh_dac [i], 
			    (float) (i*NUM_CHIP_CHANNELS + chnum), 1.);
	  if (hs_hmap_id[number_of_chips+6] >= 0) 
                hs_fill_2d_hist(hs_hmap_id[number_of_chips+6],
			   (float) cal_cal_dac [i], 
			    (float) (i*NUM_CHIP_CHANNELS + chnum), 1.);
	}



        if (hs_hmap_id[i] >= 0) {
             hs_fill_1d_hist(hs_hmap_id[i], (float) chnum, 1.);
	}
        if (hs_hmap_id[number_of_chips] >= 0) {
              hs_fill_1d_hist(hs_hmap_id[number_of_chips], 
		      (float) (i*NUM_CHIP_CHANNELS + chnum), 1.);
	}


        if (hs_hmap_id[number_of_chips+7] >= 0) {
	     command_result = cal_tot_convert(tot, i, chnum, &tot_charge);
	     if (strcmp("OK",command_result) == 0 ||
                 strcmp("CHANNEL DEAD.  USED 1st AVAILABLE CHANNEL",
			command_result) == 0) {
                       hs_fill_1d_hist(hs_hmap_id[number_of_chips+7], 
		      (float) (i*NUM_CHIP_CHANNELS + chnum), tot_charge);
	     } else {
	        cal_display_message(interp, command_result);
	     }
	   }
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* here we fill the XY scan scatter plot                      */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
        if (hs_hmap_id[number_of_chips+3] >= 0 && xy_scan == 1) {
	   this_coord = coord[0];
	   if (xy_which_axis == 2) this_coord = coord[1];
	   if (number_of_sides == 1) {
	     this_channel = i*NUM_CHIP_CHANNELS + chnum;
	   } else if (i < number_of_chips_top  && xy_which_axis == 1) {
	     this_channel = i*NUM_CHIP_CHANNELS + chnum;
	   } else if (i >= number_of_chips_top && xy_which_axis == 2) {
	     this_channel = (i-number_of_chips_top)*NUM_CHIP_CHANNELS + chnum;
	   } else {
	     this_channel = -999;
	   }
	   if (this_channel != -999) {
	     hs_fill_2d_hist(hs_hmap_id[number_of_chips+3], 
			     this_coord, (float) this_channel, 1.);
	   }
	 }
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* here we fill interactive histograms                        */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
      cal_fill_user_hist(i, chnum, tot, tstamp);
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* Close loops over hits and over chips                       */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
    }
  }
/*--------------------------------------------------------*/
/* Update histoscope                                      */
/*--------------------------------------------------------*/
/*  if (update == 99) {    */
/*	hs_update();       */      
/*	update = 0;        */
/*    }                    */
/*    update ++;           */
hs_update();

return;
}

/*************************************************************************/
/* This is the C code for zeroing/deleting/booking histograms on the fly */
/* seven calling parameters :                                            */
/* par 0 :  1=zero hist  2=delete hist 3=book hist                       */
/* par 1 :  histogram number to delete or zero.  Ignored if booking      */
/*      All subsequent parameters are used for booking only              */
/* par 2 : minimum chip number                                           */
/* par 3 : maximum chip number                                           */
/* par 4 : minimum channel number                                        */
/* par 5 : maximum channel number                                        */
/* par 6 : histogram type                                                */
/*************************************************************************/
static int pass_histoscopeCmd(ClientData clientData, Tcl_Interp* interp,
                        int argc, char* argv[]) 
{
 int i, dummy, num_match, id, ideleted, j;
 int par[7];
 int id_list[1000];
 static char error_message[200];
/*-----------------------------------------------------*/
/* Make sure call has right number of arguments        */
/*-----------------------------------------------------*/
 if (argc != 8) {
   interp->result = "wrong number of arguments to pass_histoscope";
   return TCL_OK;
 }
/*-----------------------------------------------------*/
/* unpack the parameters into an array                 */
/*-----------------------------------------------------*/
 for (i=1; i<8; i++) {
  if (Tcl_GetInt(interp,argv[i],&dummy) != TCL_OK) {
    interp->result = "bad parameters to pass_histoscope";
    return TCL_OK;
  }
  par[i-1] = dummy;
 }
/*--------------------------------------------------------*/
/* Zero or delete histograms if so required               */
/* Make sure to note deletion of user defined histograms  */
/*--------------------------------------------------------*/
 if (par[0] == 1 || par[0] == 2) {


   if (par[1] == 0) {
     num_match = hs_list_items("", "", id_list, 1000, 1);
     if (num_match > 1000) num_match=1000;
     for (i=0; i<num_match; i++) {
         if (par[0] == 1) hs_reset(id_list[i]);
         if (par[0] == 2) {
	    hs_delete(id_list[i]);
	    num_user_hist = 0;
            for (j=0; j<NUM_HARDWIRED_HIST; j++) {
	          hs_hmap_id[j] = -1;
	    }
          printf(
            " You may now get an error message from hs_update..dont worry \n");
	  }
      }
      hs_update();
      interp->result = "OK";
      return TCL_OK;
   }



   id = hs_id(par[1],"");
   if (id == -1) {
      sprintf(error_message,"Histogram %d does not exist",par[1]);
      interp->result = error_message;
      return TCL_OK;
   }
   if (par[0] == 1) hs_reset(id);
   if (par[0] == 2) {
     printf(" You may now get an error message from hs_update..dont worry \n");
     hs_delete(id);
/* if this was a user histogram, get rid of its structure  */
     ideleted = -1;
     for (i=0; i<num_user_hist; i++) {
       if (id == user_hist[i].user_hist_id) ideleted=i;
     }
     if (ideleted != -1) {
       for (i=(ideleted+1); i<num_user_hist; i++) {
           user_hist[i-1] = user_hist[i];
       }
       num_user_hist = num_user_hist - 1;
     }
/* if this was a hardwired histogram, mark it as deleted  */
     for (j=0; j<NUM_HARDWIRED_HIST; j++) {
	if( hs_hmap_id[j] == id) hs_hmap_id[j] -1;
     }

   }  /* close block if par[0]==2  */



   hs_update();
   interp->result = "OK";
   return TCL_OK;
 }
/*-----------------------------------------------------*/
/* fill the structure with new histogram               */
/* If we are in a suspended state, also book them      */
/*-----------------------------------------------------*/
 if (par[0] == 3) {
   num_user_hist++;
   if (num_user_hist > MAX_USER_HIST) {
     interp->result = "Too many user histograms";
     return TCL_OK;
   }
   if (par[6] < 0 || par[6] >= DEFINED_HIST_TYPES) {
      sprintf(error_message,"Illegal histogram type no. %d",par[6]);
      interp->result = error_message;
      return TCL_ERROR;
    }
   user_hist[num_user_hist-1].min_chip  = par[2];
   user_hist[num_user_hist-1].max_chip  = par[3];
   user_hist[num_user_hist-1].min_chan  = par[4];
   user_hist[num_user_hist-1].max_chan  = par[5];
   user_hist[num_user_hist-1].hist_type = par[6];
   user_hist[num_user_hist-1].booked    = 0;
   C_run = Tcl_GetVar(interp,"Cal_run",0);
   if (strcmp("suspend",C_run) == 0) {
      cal_book_user_hist();
      hs_update();
    }
 }
/*-----------------------------------------------------*/
/* done                                                */
/*-----------------------------------------------------*/
  interp->result = "OK";
  return TCL_OK;
}


/*********************************************************************/
/* Here we actually book the user histograms                         */
/*********************************************************************/
void cal_book_user_hist() {
   int i, id, itype, cp1, cp2, ch1, ch2;
   char title[200];
   static int how_many_channels_x[DEFINED_HIST_TYPES] = {64,64,16,64,64,64};
   static float low_channel_x[DEFINED_HIST_TYPES]     = 
                                  {-0.5, -0.5, -0.5 , -0.5 , -0.5 , -0.5};
   static float high_channel_x[DEFINED_HIST_TYPES] = 
                                  {63.5 ,63.5 ,15.5 ,63.5 ,63.5 ,63.5};
   static int how_many_channels_y[DEFINED_HIST_TYPES] = {0,0,0,0,16,16};
   static float low_channel_y[DEFINED_HIST_TYPES]     = 
                                  {0., 0., 0., 0., -0.5, -0.5};
   static float high_channel_y[DEFINED_HIST_TYPES] = 
                                  {0., 0., 0., 0., 15.5 ,15.5};
   static char *tit[DEFINED_HIST_TYPES]   = { "hit freq vs threshold",
			                      "hit freq vs q inj",
			                      "TOT dist",
			                      "Tstamp dist", 
					      "TOT vs Q",
					      "TOT vs Thresh" };
   static char *xaxis[DEFINED_HIST_TYPES] = { "threshold",
					      "charge injection",
					      "Time over threshold",
					      "Time stamp",
					      "charge injection",
					      "threshold" };
   static char *yaxis[DEFINED_HIST_TYPES] = { "number of hits",
					      "number of hits",
					      "number of hits",
					      "number of hits", 
					      "TOT", 
					      "TOT" };
/*----------------------------------------------------------------------*/
/* Loop over histograms, book the one that have not been already booked */
/*----------------------------------------------------------------------*/
   for (i=0; i<num_user_hist; i++) {
     if (user_hist[i].booked != 1) {
        itype = user_hist[i].hist_type;
        cp1 = user_hist[i].min_chip;
        cp2 = user_hist[i].max_chip;
        ch1 = user_hist[i].min_chan;
        ch2 = user_hist[i].max_chan;
        if (cp1 == cp2 && ch1 == ch2) {
	  sprintf(title,"%s chip %d chan %d", tit[itype], cp1, ch1);
	} else if (cp1 == cp2) {
	  sprintf(title,"%s chip %d chan %d-%d", tit[itype], cp1, ch1, ch2);
	} else if (ch1 == ch2) {
	  sprintf(title,"%s chip %d-%d chan %d", tit[itype], cp1, cp2, ch1);
	} else {
	  sprintf(title,"%s chip %d-%d chan %d-d", tit[itype], cp1, cp2, ch1,
		                                              ch2);
	}
	id = 1000;
        printf(" You may now get an error message from hs_id..dont worry \n");
	while (hs_id(id,"") != -1) {
	  id++;
	}
	if (how_many_channels_y[itype] == 0) {
	  user_hist[i].user_hist_id = hs_create_1d_hist(id, title, "", 
			xaxis[itype],
			yaxis[itype],
			how_many_channels_x[itype],
			low_channel_x[itype], high_channel_x[itype]);
	} else {
	  user_hist[i].user_hist_id = hs_create_2d_hist(id, title, "", 
			xaxis[itype],
			yaxis[itype], "",
			how_many_channels_x[itype],
			how_many_channels_y[itype],
			low_channel_x[itype], high_channel_x[itype],
			low_channel_y[itype], high_channel_y[itype]);

	}
        user_hist[i].booked == 1;
      }
   }
/*----------------------------------------------------------------------*/
/* Done                                                                 */
/*----------------------------------------------------------------------*/
 }


/*********************************************************************/
/* Here we fill user histograms                                      */
/*********************************************************************/
 void cal_fill_user_hist(int ichip, int ichan, int itot, int itstamp) {
 int i, type, id;

 for (i=0; i<num_user_hist; i++) {
   if ( ichip >= user_hist[i].min_chip &&
        ichip <= user_hist[i].max_chip &&
        ichan >= user_hist[i].min_chan &&
        ichan <= user_hist[i].max_chan ) {
     type = user_hist[i].hist_type;
     id   = user_hist[i].user_hist_id;
     if (type == 0) hs_fill_1d_hist(id, (float) cal_thresh_dac[ichip], 1.);
     if (type == 1) hs_fill_1d_hist(id, (float) cal_cal_dac[ichip],    1.);
     if (type == 2) hs_fill_1d_hist(id, (float) itot,                  1.);
     if (type == 3) hs_fill_1d_hist(id, (float) itstamp,               1.);
     if (type == 4) hs_fill_2d_hist(id, (float) cal_cal_dac[ichip],
				        (float) itot, 1.);
     if (type == 5) hs_fill_2d_hist(id, (float) cal_thresh_dac[ichip],
				        (float) itot, 1.);
   }
 }
}
     

/*********************************************************************/
/* definition of the command "pass_histoscope"                       */
/*********************************************************************/
void pass_histoscope_define(Tcl_Interp* interp){
  Tcl_CreateCommand(interp, "pass_histoscope", pass_histoscopeCmd, 
		    (ClientData) NULL,
                    (Tcl_CmdDeleteProc *) NULL);
}



