#include "cal_svtdef.h"
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "error.h"
#include "reglist.h"
#include "tk.h"
#include "opcodes.h"
#include "cal_global.h"
#include "cal_functions.h"
#include "cal_format_check.h"

int cal_error_handler(Tcl_Interp*, char*);
void cal_display_message(Tcl_Interp*, char*);
void pass_histoscope_define(Tcl_Interp*);
void loop_test_define(Tcl_Interp*);
void shorts_test_define(Tcl_Interp*);
void misc_tests_define(Tcl_Interp*);
void rw_atom_registers_define(Tcl_Interp*);
char* set_single(int, int, int*);
void get_list_of_channels(int, int*, int*);

#define EVENT_REPORT_FREQ 100

/****************************************************************/
/* This the "master function" for the calibration runs          */
/****************************************************************/
static int Calibrate_Loop(ClientData clientData, Tcl_Interp* interp, int argc,
                    char* argv[])
{

/*==========================================================*/
/* NB: all lines commented out with a /*sp were commented   */
/*     out in an attempt to speed up the code by moving     */
/*     functionality from UNIX to VXWORKS                   */
/*==========================================================*/

/*----------------------------------------------------------*/
/* A bunch of commands to pass to Tcl_Eval                  */
/*----------------------------------------------------------*/
  static char cmd1[]   = "update";
  static char cmd2[]   = ".run_menu.text insert end \" Starting the run \n\"";
  static char cmd3[]   = ".run_menu.text insert end \" Resuming the run \n\"";
  static char cmd4[]   = ".run_menu.text insert end \" Suspending the run \n\"";
  static char cmd5[]   = ".run_menu.text insert end \" Ending the run \n\"";
  static char cmd6[]   = ".run_menu.text insert end \" RUN HAS BEEN ABORTED \n\"";
  static char cmd7[]   = ".run_menu.text insert end \" RUN HAS COMPLETED \n\"";
  static char cmd8[]   = "set Cal_run stop";
  static char cmd9[]   = "grab release .run_menu";
  static char cmd10[]  = "vme_clear daq_0(led)";
  static char cmd11[]  = "vme_set daq_0(led)";
  static char cmd20[]  = ".run_menu.text insert end \"..skip event and continue \n\"";
  static char cmd21[]  = ".run_menu.text insert end \" Too many errors reading VRAM..abort \n\"";
  static char cmd22[]  = ".run_menu.text insert end \" Error building the start run record..abort \n\"";
  static char cmd30[]  = ".run_menu.text yview -pickplace end";
  static char cmd31[]  = ".run_menu.text insert end \" Last event processed event no. ";  
  static char cmd32[]  = ".run_menu.text insert end \" Resuming at event no. ";  static char cmd33[]  = ".run_menu.text insert end \" Processing event no. ";
  static char cmd34[]  = ".run_menu.text insert end \" Reporting every ";
  static char cmd35[]  = ".run_menu.text insert end \"Error setting up the daq card";
  static char cmd40[]  = ".run_menu.text insert end \"Error reading VRAM ";
/*----------------------------------------------------------*/
/* Variable declarations                                    */
/*----------------------------------------------------------*/ 
  static FILE *output_file = NULL;
  static int  all_the_same;
  static char        *command_result;
  static char        *eventseq_result;
  static char        *sanity_check;
  static int         list_of_channels[128][128];
  static int         niter;
  char        *dataptr;
  int         data_len;
  int         dummy;
  static int  event_number;
  int         first_charge;
  static int  first_chip;
  int         first_event;
  int         first_thr;
  int         i, crap, ntoggle, sanity_flag;
  int         i1;
  int         i2, i3;
  int         icharge;
  int         ichip;
  int         ievent;
  int         ithr;
  int         j, k;
  static int  last_chip;
  int         resuming;
  vme_rc      rc;
  float this_coord;
  int         status;
  int nxyscan;
  int ixyscan;
  int first_ixyscan;
  static int  suspend_ixyscan;
  static int  suspend_icharge;
  static int  suspend_ievent;
  static int  suspend_ithr;
  static int  suspend_isingle;
  static char temp_string[2000];
  static char temp_string2[2000];
  static char okeydokey[100];
  static int  error_count;         
  char        event_message[200];
  static int pm_ch_mask;
  int pm_chan, chip_address, i87, cont;
  static int nstep_charge[MAX_NUM_CHIP];
  static int nstep_threshold[MAX_NUM_CHIP];
  static int written_cal_cal_dac[MAX_NUM_CHIP];
  static int written_cal_thresh_dac[MAX_NUM_CHIP];
  int isingle, nsingle, first_isingle;
  static FILE* dump_file = NULL;
/*-------------------------------------------------*/
/* Misc. initializations                           */
/*-------------------------------------------------*/
  resuming = 0;       /* set to one if we are resuming a run */
  strcpy(okeydokey, "OK");
/*--------------------------------------------------------*/
/*  Make sure the call has the right number of arguments  */
/*--------------------------------------------------------*/
  if (argc != 1) {
      interp->result = "wrong number of arguments";
      return TCL_ERROR;
    }
/*--------------------------------------------------------*/
/* Unpack the Tcl variables                               */
/*--------------------------------------------------------*/
   Cal_unpack_variables(interp);
/*--------------------------------------------------------*/
/* If we are here, we are stopping the run                */
/*--------------------------------------------------------*/
   if (strcmp("stop",C_run) == 0) {      
        dummy = Tcl_Eval(interp,cmd5);
        dummy = Tcl_Eval(interp,cmd9);
        sprintf(event_message,"%s %d \n\"",cmd31,event_number);      
        dummy = Tcl_Eval(interp,event_message);
        dummy = Tcl_Eval(interp,cmd30);
	status = cal_stop_run(interp,&output_file,dump_file); 
	if (status !=0) {
	   interp->result = "error trying to end the run";
	   return TCL_ERROR;
	 }
	   return TCL_OK;
      }
/*--------------------------------------------------------*/
/* If we are here, we are starting the run                */
/*--------------------------------------------------------*/
   else if (strcmp("start",C_run) == 0) {    
       dummy = Tcl_Eval(interp,cmd2);
       dummy = Tcl_Eval(interp,cmd30);
       error_count  = 0;
       event_number = 0;

/* flag the masks as unwritten.  NB: only lowest 16 bits are used,   */
/* so setting the higher order 16 bits is a kludgy way to flag       */
/* them as not-written                                               */
       for (i=0; i<number_of_chips; i++) {
	 for (i1=0; i1<8; i1++) {
	   written_calib_mask[i][i1] = 0xffffffff;
	 }
       }


/* fill the list of channels to be fired together     */
/* BUT WE DO NOT NEED ANY OF THIS FOR XY SCANS        */
     if (xy_scan != 1) {
       get_list_of_channels(one_at_a_time, &niter, &list_of_channels[0][0]);

/* if the number of channels to be fired is unconventional, warn the user */
       if (one_at_a_time != 1 && one_at_a_time != 2 && one_at_a_time !=4
	                      && one_at_a_time != 8 && one_at_a_time !=16
	                      && one_at_a_time !=32 && one_at_a_time !=64
	                      && one_at_a_time !=128) {
	   sprintf(temp_string,"The number of channels to be fired");
           cal_display_message(interp,temp_string);
	   sprintf(temp_string,"together is not a power of two...");
           cal_display_message(interp,temp_string);
	   sprintf(temp_string,"Here is how we will deal with it:");
           cal_display_message(interp,temp_string);
	   sprintf(temp_string,"Number of iterations = %d",niter);
	   for (j=0; j<niter; j++) {
	     sprintf(temp_string,"Iteration number %d...Channels fired:",
		     (j+1));
             cal_display_message(interp,temp_string);
	     sprintf(temp_string," ");
	     for (k=0; list_of_channels[j][k] !=-1; k++) {
	       sprintf(temp_string2,"%d ",list_of_channels[j][k]);
	       strcat(temp_string,temp_string2);
	     }
             cal_display_message(interp,temp_string);
	   }
	 }
     } else {      /* xy scans here   */
       niter = 1;
     }

/* see if the number of steps for charge and thresholds are the same */
       all_the_same = cal_chip_by_chip();
       if (all_the_same == 0) {
	  first_chip = 0;
	  last_chip  = 0;
	} else {
	  first_chip = 0;
	  last_chip  = number_of_chips - 1;
	}

/*- if it is an XY scan, do necessary initializations      */
       if (xy_scan == 1) {
	 for (i=1; i<=2; i++) {
	   command_result = cal_xy_init_axis(interp, i);
           if (strcmp("OK",command_result) != 0) {
	     cal_display_message(interp,command_result);
             return TCL_OK;
	   }
	 }
         for (i=1; i<=2; i++) {
	   command_result = cal_xy_get_coord(interp, i, &this_coord);
           if (strcmp("OK",command_result) != 0) {
	     cal_display_message(interp,command_result);
             return TCL_OK;
	   }
	   coord[i-1] = this_coord;
	 }
	 if (xy_start_flag == 1) {
           xy_x_initial = coord[0];
           xy_y_initial = coord[1];
	 } else {
	   command_result = cal_xy_move_to(interp, 1, xy_x_initial);
           if (strcmp("OK",command_result) != 0) {
	     cal_display_message(interp,command_result);
	     interp->result = "error trying to go to initial X position";
             return TCL_ERROR;
	   }
	   cal_xy_move_to(interp, 2, xy_y_initial);
           if (strcmp("OK",command_result) != 0) {
	     cal_display_message(interp,command_result);
	     interp->result = "error trying to go to initial Y position";
             return TCL_ERROR;
	   }
	 }
       }


/*- get the number of steps explicitely        */
       for (i=0; i<number_of_chips; i++) {
	 nstep_threshold[i] = cal_get_nstep(i,1);
	 nstep_charge[i]    = cal_get_nstep(i,2);
         if (nstep_threshold[i] == -999) {
	   sprintf(temp_string,"Bad charge inj parameters chip %d..abort",i);
           cal_display_message(interp,temp_string);
	   return TCL_OK;
	 }
         if (nstep_charge[i] == -999) {
	   sprintf(temp_string,"Bad threshold parameters chip %d..abort",i);
           cal_display_message(interp,temp_string);
	   return TCL_OK;
	 }
       }
     
/*- debug printout                                                     */
/*       printf("all_the_same = %d \n",all_the_same);                  */
/*       for (i=0; i<number_of_chips; i++) {                           */
/*	 printf("chip %d threshold steps = %d charge_steps = %d \n",i, */
/*		nstep_threshold[i],nstep_charge[i]);                   */
/*       }                                                             */

/*- setup personality module */
       command_result = cal_setup_pm(interp);
       if (strcmp(command_result,"OK") != 0) {
	   dummy = Tcl_Eval(interp,cmd35);
	   dummy = Tcl_Eval(interp,cmd30);
	   interp->result = command_result;
	   return TCL_ERROR;
       }
	   
/*- open file for write, if needed   */
       if (strcmp("1",C_write_file) == 0) {
	 status = cal_open_file(&output_file);
         if (status != 0) {
	      cal_abort_run(interp,&output_file,event_number,dump_file);
	      interp->result = "error trying to open the file";
	      return TCL_ERROR;
	    }
       }

/*-start histoscope if needed       */
       if (strcmp("1",C_histoscope) == 0) {
	   status = cal_start_histoscope();
	   if (status != 0) {
	      cal_abort_run(interp,&output_file,event_number,dump_file);
	      interp->result = "error trying to fire up histoscope";
	      return TCL_ERROR;
	    }
	 }

/* fill the start run record, write it out if needed      */
/* Also, open the dump file, write the begin record       */
/* Then, free the memory                                  */

         command_result = cal_start_run_record(&data_len);
	 if (strcmp("OK",command_result) != 0) {
              dummy = Tcl_Eval(interp,cmd22);      
              dummy = Tcl_Eval(interp,cmd30);      
	      cal_abort_run(interp,&output_file,event_number,dump_file);
	      interp->result = command_result;
	      return TCL_ERROR;
	 }

         if (strcmp("1",C_write_file) == 0) {
	    dummy = cal_write_file(&output_file);
	    if (dummy != 0) {
               sprintf(event_message,"%s %d \n\"",cmd31,event_number);  
               dummy = Tcl_Eval(interp,event_message);
               dummy  = Tcl_Eval(interp,cmd30);
               free(cal_data_point);
	       cal_abort_run(interp,&output_file,event_number,dump_file);
	       interp->result = "error writing output file";
	       return TCL_ERROR;
            }            
	 }

         dump_file = fopen("check.txt","w");
          if (dump_file == NULL) {
	    sprintf(event_message,"unable to open file for event dump");
            cal_display_message(interp,event_message);
	 } else {
            dummy = cal_dump(0,dump_file);
	 }
         free(cal_data_point);
	
/*--------------------------------------------------------*/
/* If we are here, we are resuming the run                */
/* (for xy_scans, get the coordinates again               */
/*--------------------------------------------------------*/
  } else if (strcmp("resume",C_run) == 0) { 
       resuming = 1;
       dummy = Tcl_Eval(interp,cmd3);      	                   
       sprintf(event_message,"%s %d \n\"",cmd32,event_number); 
       dummy = Tcl_Eval(interp,event_message);
       dummy = Tcl_Eval(interp,cmd30);
       Tcl_SetVar(interp,"Cal_run","start",TCL_GLOBAL_ONLY);
       if (xy_scan == 1) {
         for (i=1; i<=2; i++) {
	   command_result = cal_xy_get_coord(interp, i, &this_coord);
           if (strcmp("OK",command_result) != 0) {
	     cal_display_message(interp,command_result);
             return TCL_OK;
	   }
	   coord[i-1] = this_coord;
	 }
       }
/*--------------------------------------------------------*/
/* If we are here, something is wrong                     */
/*--------------------------------------------------------*/
  } else {
      interp->result = "Bad value for Cal_run variable";
      return TCL_ERROR;
    }
/*--------------------------------------------------------*/
/* make sure the led bit is off                           */
/*--------------------------------------------------------*/
   dummy = Tcl_GlobalEval(interp,cmd10);
   if (dummy != TCL_OK) {
      cal_abort_run(interp,&output_file,event_number,dump_file);
      interp->result = "Error trying to turn off the led bit";
      return TCL_ERROR;
    }
/*--------------------------------------------------------*/
/* if this is an XY scan, zero all the calibration masks  */
/*--------------------------------------------------------*/
  if (xy_scan == 1) {
    for (i=0; i<number_of_chips; i++) {
      for (j=0; j<8; j++) {
	calib_mask[i][j] = 0;
      }
    }
  }
/*--------------------------------------------------------*/
/* setup the hybrid at the beginning                      */
/* NB: this will also write the masks for you             */
/*--------------------------------------------------------*/
   command_result = cal_setup_hybrid();
   if (strcmp("OK",command_result) != 0) {
      cal_abort_run(interp,&output_file,event_number,dump_file);
      interp->result = command_result;
      return TCL_ERROR;
    }       
/*-----------------------------------------------------------*/
/* Careful now.  cal_setup_hybrid writes control registers   */
/* with all dacs set to zero.  To avoid confusion, let us    */
/* start out from well defined settings.  Also, if we are    */
/* resuming, restore the settings that we had before         */
/*-----------------------------------------------------------*/
  for (i=0; i<number_of_chips; i++) {
    if (resuming == 1) {
      cal_cal_dac[i]    = written_cal_cal_dac[i];
      cal_thresh_dac[i] = written_cal_thresh_dac[i];
    } else {
      cal_cal_dac[i]    = min_charge[i];
      cal_thresh_dac[i] = min_threshold[i];
    }
    build_creg(i);
    creg_into_data(i);
    rc = cal_write_control_registers(i);
    if (rc != VME_OK) {
	     sprintf(temp_string,
		     "Error writing control register for chip %d event %d ",
		     i1, event_number);
             cal_display_message(interp,temp_string);
	     cal_abort_run(interp,&output_file,event_number,dump_file);
             interp->result = temp_string;
             return TCL_ERROR;
    }
    written_cal_cal_dac[i]    = cal_cal_dac[i];
    written_cal_thresh_dac[i] = cal_thresh_dac[i];
  }
/* &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&*/
/* Here is the body of the calibration loop               */
/* &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&*/
/*--------------------------------------------------------*/
/* Temporarily, an infinite loop for debugging .......     */
/*--------------------------------------------------------*/
/* debug debug debug  for (;;) {         debug debug debug*/
/*--------------------------------------------------------*/
/* Loop over chips...if sanity prevails and everything    */
/* is the same for all chips, this will be run only once  */
/*--------------------------------------------------------*/
  for (ichip=first_chip; ichip <= last_chip; ichip++) {
   first_event = 1;

/*--------------------------------------------------------*/
/* Loop over different threshold settings                 */
/*--------------------------------------------------------*/
    first_thr = 1;
    if (resuming == 1) first_thr = suspend_ithr;
    for (ithr=first_thr; ithr<=nstep_threshold[ichip]; ithr++) { 

/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/*       At this stage we may as well set the thresholds  */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - */
     if (all_the_same == 0) {
           for (i=0; i<number_of_chips; i++) {
	        cal_thresh_dac[i] = min_threshold[i] + 
		                    (ithr-1)*step_threshold[i];
           } 
      } else {
	    cal_thresh_dac[ichip] = min_threshold[ichip] + 
		                    (ithr-1)*step_threshold[ichip];
      }
/*--------------------------------------------------------*/
/* display a message for the one at a time case           */
/*--------------------------------------------------------*/
       if (niter != 1 && resuming == 0) {
	  if(all_the_same == 0) {
	    sprintf(temp_string,"not all channels at a time: thresh step = %d ",ithr);
	  } else {
	    sprintf(temp_string,"not all channels at a time: chip %d thresh step = %d ",
		    ichip, ithr);
	  }
	  cal_display_message(interp,temp_string);
       }
/*--------------------------------------------------------*/
/* Loop over different charge inj settings                */
/*--------------------------------------------------------*/
      first_charge = 1;
      if (resuming == 1) first_charge = suspend_icharge;
      for (icharge=first_charge; icharge<=nstep_charge[ichip]; icharge++) {

/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/*       Now we may finally set the charge injections     */
/*       We will also write the needed control reg.       */
/*       Abort the run if we have trouble with it         */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - */
      if (all_the_same == 0) {
         i1 = 0;
         i2 = number_of_chips;
      } else {
         i1 = ichip;
         i2 = ichip + 1;
      }
      for (i=i1; i<i2; i++) {
	  cal_cal_dac[i] = min_charge[i] + (icharge-1)*step_charge[i];
	  build_creg(i);
	  creg_into_data(i);
	  rc = cal_write_control_registers(i);
	  if (rc != VME_OK) {
	     sprintf(temp_string,
		     "Error writing control register for chip %d event %d ",
		     i1, event_number);
             cal_display_message(interp,temp_string);
	     cal_abort_run(interp,&output_file,event_number,dump_file);
             interp->result = temp_string;
             return TCL_ERROR;
          }
	  written_cal_cal_dac[i]    = cal_cal_dac[i];
	  written_cal_thresh_dac[i] = cal_thresh_dac[i];
      }  
/*--------------------------------------------------------*/
/* Now send a clear readout...should not be needed  but   */
/* there may be problems with reflections and such (?)    */
/* NB : the direction in principle should be zero, but    */
/*      there is some sort of a bug                       */
/*--------------------------------------------------------*/
     rc = vmePmCommand(CARD, CLEAR, 0,  0xf, 0, 1, chip_data);
     if (rc != VME_OK) {
	     sprintf(temp_string,"Error sending CLEAR event %d ",event_number);
             cal_display_message(interp,temp_string);
	     cal_abort_run(interp,&output_file,event_number,dump_file);
	     interp->result = temp_string;
             return TCL_ERROR;
     }  
/*--------------------------------------------------------*/
/* figure out what channels we expect to receive data on  */
/*--------------------------------------------------------*/
      pm_ch_mask = 0;
      for (i87=0; i87<number_of_chips; i87++) {
	  cal_get_chip_address(i87,&chip_address,&pm_chan);
	  if (ignore[i87] == 0) pm_ch_mask = pm_ch_mask | pm_chan;
      }
/*--------------------------------------------------------*/
/*  XY scan loop                                          */
/*--------------------------------------------------------*/
      if (xy_scan != 1) {
	nxyscan       = 0;
	first_ixyscan = 0;
      } else {
	nxyscan       = xy_nstep;
        first_ixyscan = 0;
	if (resuming == 1) first_ixyscan = suspend_ixyscan;
      }
      for (ixyscan=first_ixyscan; ixyscan<=nxyscan; ixyscan++) {
	if (xy_scan == 1) {
	   if (ixyscan != 0 && strcmp("stop",C_run) != 0) {
	      this_coord = xy_x_initial;
	      if (xy_which_axis == 2) this_coord = xy_y_initial;
	      this_coord = this_coord + ixyscan*xy_step_size;
move_it:
	      command_result = cal_xy_move_to(interp,xy_which_axis,this_coord);
              if (strcmp("OK",command_result) != 0) {
		error_count++;
	        sprintf(temp_string,"ERROR %s ..event %d",
			   command_result,event_number);
	        cal_display_message(interp,temp_string);
		if (error_count < error_limit) goto move_it;
		cont = 0;
		cont = cal_error_handler(interp,temp_string);
		if (cont == 0) goto move_it;  
	        cal_abort_run(interp,&output_file,event_number+1,dump_file);
		sprintf(temp_string,"Cannot move axis %d to coordinate %f",
			xy_which_axis,this_coord);
	        interp->result = temp_string;
	        return TCL_ERROR;
	      }

read_it:
	      command_result = cal_xy_get_coord(interp,
						xy_which_axis,&this_coord);
              if (strcmp("OK",command_result) != 0) {
		error_count++;
	        sprintf(temp_string,"ERROR %s ..event %d",
			   command_result,event_number);
	        cal_display_message(interp,temp_string);
		if (error_count < error_limit) goto move_it;
		cont = 0;
		cont = cal_error_handler(interp,temp_string);
		if (cont == 0) goto read_it;  
	        cal_abort_run(interp,&output_file,event_number+1,dump_file);
		sprintf(temp_string,"Cannot read coordinate for axis %d",
			xy_which_axis);
	        interp->result = temp_string;
	        return TCL_ERROR;
	      }
	      coord[xy_which_axis-1] = this_coord;
	    }
	 }
/*--------------------------------------------------------*/
/* One channel at a time loop                             */
/*--------------------------------------------------------*/
	first_isingle = 0;
	if (resuming == 1) first_isingle=suspend_isingle;
	for (isingle=first_isingle; isingle<niter; isingle++) {
	  if (xy_scan != 1) {
            command_result = 
	      set_single(all_the_same,ichip,&list_of_channels[isingle][0]);
              if (strcmp("NO",command_result)==0) goto next_single;
              if (strcmp("OK",command_result)!=0) {
                 sprintf(temp_string,"%s  Event %d",
                         command_result,event_number);
                 cal_abort_run(interp, &output_file, event_number,dump_file);
                 interp->result = temp_string;
                 return TCL_ERROR;
	       }
	  }
/*--------------------------------------------------------*/
/* if we are using a compressed format, write out         */
/* the parameter record if needed; then release memory    */
/*--------------------------------------------------------*/
     if (compressed_format == 1 && strcmp("1",C_write_file) == 0) {
       dummy = cal_parameter_record();
       if (dummy != 0) {
	 sprintf(temp_string,"Error building the parameter record event %d",
		 event_number);
	 interp->result = temp_string;
	 return TCL_ERROR;
       }
       dummy = cal_write_file(&output_file);
       if (dummy != 0) {
	  cal_abort_run(interp,&output_file,event_number,dump_file);
	  free(cal_data_point);
	  interp->result = "error writing file";
	  return TCL_ERROR;
       }
       free(cal_data_point);
      }            
/*--------------------------------------------------------*/
/* Repeat a number of times with identical settings       */
/*--------------------------------------------------------*/
      first_event = 1;
      if (resuming == 1) first_event = suspend_ievent;
       for (ievent=first_event; ievent<= number_of_events; ievent++) {
try_again:

/*send noop to toggle VRAM direction. bug fix atempt 10/30/96 Armin*/
/*sp  rc = vmePmCommand(CARD, NOOP, 0, pm_ch_mask, 0, 1, chip_data);  sp*/

/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* send update command to Tcl                             */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - */
       dummy = Tcl_Eval(interp,cmd1); 
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* check the variable from the Tcl script.......          */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - */
       C_run   = Tcl_GetVar(interp,"Cal_run",0);
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* suspended.... remember where we are                    */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - */
       if (strcmp("suspend",C_run) == 0) {
	      first_chip      = ichip;
	      suspend_ithr    = ithr;
	      suspend_icharge = icharge;
	      suspend_ixyscan = ixyscan;
	      suspend_ievent  = ievent;
	      suspend_isingle = isingle;
              dummy = Tcl_Eval(interp,cmd4);      
              sprintf(event_message,"%s %d \n\"",cmd31,event_number); 
              dummy = Tcl_Eval(interp,event_message);
              dummy  = Tcl_Eval(interp,cmd30);
              return TCL_OK;
       }
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* end run ..... close everything ..........              */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - */
       if (strcmp("stop",C_run) == 0) {
             dummy = Tcl_Eval(interp,cmd5);
             dummy = Tcl_Eval(interp,cmd9);
             sprintf(event_message,"%s %d \n\"",cmd31,event_number);
             dummy = Tcl_Eval(interp,event_message);
             dummy = Tcl_Eval(interp,cmd30);    
  	     status = cal_stop_run(interp,&output_file,dump_file); 
	     if (status !=0) {
	        interp->result = "error trying to end the run";
	        return TCL_ERROR;
	     }
	     return TCL_OK;
       }
/* * * * * * * * * * * * * * * * * * * * * * * * *  * * * * * * */
/* lets do it now                                               */
/* * * * * * * * * * * * * * * * * * * * * * * * *  * * * * * * */
       resuming = 0;                           /* running, have resumed  */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* Toggle the led bit                                                        */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - -  - - - - - - - - - */
/*sp       for (i3=0; i3<number_of_buffers; i3++) {                        sp*/
/*sp        dummy = Tcl_GlobalEval(interp,cmd11);                          sp*/
/*sp	if (dummy != TCL_OK) {                                             sp*/
/*sp	     sprintf(temp_string,                                          sp*/
/*sp		     "Error trying to turn on the LED bit.. event %d ",    sp*/
/*sp		     event_number);                                        sp*/
/*sp             cal_display_message(interp,temp_string);                  sp*/
/*sp	     cal_abort_run(interp,&output_file,event_number,dump_file);    sp*/
/*sp	     interp->result = temp_string;                                 sp*/
/*sp             return TCL_ERROR;                                         sp*/
/*sp          }                                                            sp*/
/*sp        dummy = Tcl_GlobalEval(interp,cmd10);                          sp*/
/*sp	if (dummy != TCL_OK) {                                             sp*/
/*sp	     sprintf(temp_string,                                          sp*/
/*sp		     "Error trying to turn off the LED bit.. event %d ",   sp*/
/*sp		     event_number);                                        sp*/
/*sp             cal_display_message(interp,temp_string);                  sp*/
/*sp	     cal_abort_run(interp,&output_file,event_number,dump_file);    sp*/
/*sp	     interp->result = temp_string;                                 sp*/
/*sp             return TCL_ERROR;                                         sp*/
/*sp          }                                                            sp*/
/*sp      }                                                                sp*/
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* Send out a READEVENT command to the hybrid                                */
/*  ---- For now set the wordcount at 0xff. It doesnt work with more         */
/*  ---- Oct 22, 1996.  Try to change it to 0xfff, timeout                   */
/*       ---> make it longer than 0xff but shorter than 0xfff                */
/*  24 Nov 1996 :  Make it 0x800 so says Armin (fast ctrl disabled)          */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
       for (i3=0; i3<number_of_buffers; i3++) {


/*sp      rc = vmePmCommand(                                               sp*/
/*sp                 CARD, READEVENT, 0, pm_ch_mask, 0x800, 0, chip_data); sp*/
/*sp	  if (rc != VME_OK) {                                              sp*/
/*sp	     sprintf(temp_string,"Error issuing READ command %d ",         sp*/
/*sp		     event_number);                                        sp*/
/*sp             cal_display_message(interp,temp_string);                  sp*/
/*sp	     cal_abort_run(interp,&output_file,event_number,dump_file);    sp*/
/*sp	     interp->result = temp_string;                                 sp*/
/*sp             return TCL_ERROR;                                         sp*/
/*sp          }                                                            sp*/
/*- - - - -  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* output event number                                                       */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
         event_number++;                         /* bump the event number  */
         dummy = (event_number - 1) / EVENT_REPORT_FREQ;
         if ( (event_number-1) == EVENT_REPORT_FREQ * dummy) { 
           sprintf(event_message,"%s %d \n\"",cmd33,event_number);
           dummy = Tcl_Eval(interp,event_message);
           dummy = Tcl_Eval(interp,cmd30);
	 }
         if (event_number == 1) {
	   sprintf(event_message,"%s %d events\n\"",
		   cmd34,EVENT_REPORT_FREQ);
           dummy = Tcl_Eval(interp,event_message);
	  dummy = Tcl_Eval(interp,cmd30);
	 }
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* Move functionality to VXWORKS.  Toggle LED, send out a READEVENT          */
/* (but do not do it for fake events).  The LED gets toggled                 */
/* n=number_of_buffers times, but only the first time through                */
/* For the case of a fake event, toggle the LED by hand                      */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
         ntoggle =  0;
	 if (i3 == 0) ntoggle = number_of_buffers;
         if (strcmp(C_fake,"1") != 0) {
	   eventseq_result = EventSeq(CARD, READEVENT, ntoggle, 
				      pm_ch_mask,0x800, 0, 1);
	 } else {
	   dummy = Tcl_GlobalEval(interp,cmd11);                          
	   if (dummy != TCL_OK) {                                             
	     sprintf(temp_string,                                          
		     "Error trying to turn on the LED bit.. event %d ",    
		     event_number);                                        
             cal_display_message(interp,temp_string);                  
	     cal_abort_run(interp,&output_file,event_number,dump_file);    
	     interp->result = temp_string;                                 
             return TCL_ERROR;                                         
           }                                                            
	   dummy = Tcl_GlobalEval(interp,cmd10);                          
	   if (dummy != TCL_OK) {                                             
	     sprintf(temp_string,                                          
		     "Error trying to turn off the LED bit.. event %d ",   
		     event_number);                                        
             cal_display_message(interp,temp_string);                  
	     cal_abort_run(interp,&output_file,event_number,dump_file);    
	     interp->result = temp_string;                                 
             return TCL_ERROR;                                        
	   }
           eventseq_result = okeydokey;
	 }
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* Now build the event data                                         */
/* also do the sanity check of checking the data again              */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
	if (strcmp(eventseq_result,"OK") == 0) {
           command_result = cal_data(interp,&data_len,event_number);
	} else {
	  strcpy(command_result,eventseq_result);
	  data_len = -1;
	}
	if (strcmp("OK",command_result) == 0) {

/*	  sanity_check = cal_format_check(cal_data_point, CHECK_ALL);   */
	  sanity_flag = format_check_flag | (NUMBER_OF_BYTES | CHIP_ADDRESS);
	  sanity_check = cal_format_check(cal_data_point, sanity_flag);
	}
	if (strcmp("OK",command_result) != 0 && data_len != -1) {
             sprintf(temp_string,"ERROR %s ..event %d",
		   command_result,event_number);
             cal_display_message(interp,temp_string);
	     cal_abort_run(interp,&output_file,event_number,dump_file);
	     interp->result = temp_string;
             return TCL_ERROR;
	} else if (strcmp("OK",command_result)!=0 ||
		   (  strcmp("OK",command_result)==0 && 
		      strcmp("OK",sanity_check)!=0 )  ) {
             error_count++;
             if (strcmp("OK",command_result) != 0) {
	           sprintf(temp_string,"ERROR %s ..event %d",
			   command_result,event_number);
	     } else {
	           sprintf(temp_string,"ERROR %s ..event %d",
			   sanity_check,event_number);
	     }
             cal_display_message(interp,temp_string);
             cont = 0;
             if (error_count > error_limit) {
               cont = cal_error_handler(interp,temp_string);
	     }
             if (cont == 0) {
	         sprintf(temp_string,"Skip event and continue...");
	         cal_display_message(interp,temp_string);
                 dummy = Tcl_Eval(interp,cmd1);
/* reset personality module in case of error                      */
                 command_result = cal_setup_pm(interp);
                 if (strcmp(command_result,"OK") != 0) {
	            dummy = Tcl_Eval(interp,cmd35);
	            dummy = Tcl_Eval(interp,cmd30);
	            interp->result = command_result;
	            return TCL_ERROR;
                 }
/* better also send a clear event to the hybrid in case of error  */
                rc = vmePmCommand(CARD, CLEAR, 0,  0xf, 0, 1, chip_data);
	        if (rc != VME_OK) {
	            sprintf(temp_string,
			    "Error sending CLEAR event %d ",event_number);
                    cal_display_message(interp,temp_string);
	            cal_abort_run(interp,&output_file,event_number,dump_file);
	            interp->result = temp_string;
                    return TCL_ERROR;
                }
/* better also send a SYNCH to the hybrid in case of error  */
                rc = vmePmCommand(CARD, SYNCH, 0,  0xf, 0, 1, chip_data);
	        if (rc != VME_OK) {
	            sprintf(temp_string,
			    "Error sending SYNCH event %d ",event_number);
                    cal_display_message(interp,temp_string);
	            cal_abort_run(interp,&output_file,event_number,dump_file);
	            interp->result = temp_string;
                    return TCL_ERROR;
                }
/* okay, now try again                                            */  
/* If the number of buffers is one, we will try this event again  */
/* Otherwise, to avoid confusion, we go to the next event         */
		 if (number_of_buffers != 1) {
	            goto next_ev;
		  } else {
		    event_number--;
		    goto try_again;
		  }
	     }
/* if we get here, we have to abort the run                        */
             if (cont == 2) {
		 sprintf(temp_string,"Unable to prompt for action..abort");
		 cal_display_message(interp,temp_string);
	     }
	     cal_abort_run(interp,&output_file,event_number,dump_file);
             return TCL_OK;
	}
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* Dump first event to a file                                       */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
       if (event_number == 1) {
	 dummy = cal_dump(1, dump_file);
       }      
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* Fill histograms                                                      */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
        if (strcmp("1",C_histoscope) == 0) cal_fill_histograms(interp);
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* Output file.  If the format is compressed, compress it first         */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
        if (strcmp("1",C_write_file) == 0) {
	    if (compressed_format == 1) {
	      dummy = cal_compress_event_record();
	      if (dummy != 0) {
		cal_abort_run(interp,&output_file,event_number,dump_file);
		free(cal_data_point);
		interp->result = "error compressing event record";
		return TCL_ERROR;
	      }
	    }
	    dummy = cal_write_file(&output_file);
	    if (dummy != 0) {
	       cal_abort_run(interp,&output_file,event_number,dump_file);
	       free(cal_data_point);
	       interp->result = "error writing file";
	       return TCL_ERROR;
	     }
         }            
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* Release the memory that we allocated                             */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
       free(cal_data_point);
      }    /* end of loop over number of buffers   */
/* * * * * * * * * * * * * * * * * * * * * * * * *  * * * * * * * * */
/* Done with this event, go to the next
/* * * * * * * * * * * * * * * * * * * * * * * * *  * * * * * * * * */
       next_ev: {}
         }            /* end of loop over events               */
	next_single: {}
        }             /* end of loop over single channel scan  */
       }              /* end of loop over xy scans             */ 
      }               /* end of loop over charge inj. settings */
     }                /* end of loop over threshold settings   */
    }                 /* end of loop over chips                */


 /*    }            end of temporary infinite loop        */
/*--------------------------------------------------------*/
/* okay, we are done                                      */ 
/*--------------------------------------------------------*/
  dummy  = Tcl_Eval(interp,cmd7);
  dummy  = Tcl_Eval(interp,cmd9);
  dummy  = Tcl_Eval(interp,cmd8);
  sprintf(event_message,"%s %d \n\"",cmd31,event_number);      
  dummy  = Tcl_Eval(interp,event_message);
  dummy  = Tcl_Eval(interp,cmd30);
  status = cal_stop_run(interp,&output_file,dump_file); 
  if (status !=0) {
      interp->result = "error trying to end the run";
      return TCL_ERROR;
  }
  return TCL_OK;
}



/***************************************************************************/
/* Error handling procedure                                                */
/* Will ask the user if we should continue or abort                        */
/* returns 0  if we should continue                                        */
/* returns 1  if we should abort                                           */
/* returns 2  if there were problems                                       */
/***************************************************************************/
int cal_error_handler(Tcl_Interp* interp, char* message)
{
  static char command[200];
  int dummy;

  strcpy(command,"cal_error_handler \"");
  strcat(command,message);
  strcat(command,"\"");
  dummy = Tcl_Eval(interp,command);
  if (dummy != TCL_OK) return 2;
  if (strcmp(interp->result,"continue") == 0) return 0;
  return 1;
}

/***************************************************************************/
/* Display message                                                         */
/***************************************************************************/
void cal_display_message(Tcl_Interp* interp, char* message)
{
  static char c1[]   = "update";
  static char command[200];
  int dummy;

  strcpy(command,".run_menu.text insert end \"");
  strcat(command,message);
  strcat(command,"\n\"");
  dummy = Tcl_Eval(interp,command);
  strcpy(command,".run_menu.text yview -pickplace end");
  dummy = Tcl_Eval(interp,command);
  dummy = Tcl_Eval(interp,c1);
}
/***************************************************************************/
/* this is the function that sets the masks when we are doing calibration  */
/* a number of  channels at a time                                         */
/* channels is a list of channels terminated by a -1                       */
/* Returns "NO" if we have to skip this channel, returns "OK" if           */
/* everything is okay, otherwise returns an error message                  */
/***************************************************************************/
char* set_single(int all_the_same, int this_chip, int* channels)
{
  int i,j, k;
  int calib_mask_store[MAX_NUM_CHIP][8];
  int channel_mask_store[MAX_NUM_CHIP][8];
  int reset_masks, this_channel;
  vme_rc rc;
  static char return_code[200];
  int chip_setup[MAX_NUM_CHIP];
  int nsetup, dummy;
  int calib_bit[MAX_NUM_CHIP][128], channel_bit[MAX_NUM_CHIP][128];

/*---------------------------------------------------------------------*/
/* get a list of all chips that we should setup                        */
/*---------------------------------------------------------------------*/
   nsetup = 0;
   for (i=0; i<number_of_chips; i++) {
     chip_setup[i] = 0;
     if (all_the_same == 0 || i == this_chip) {
       for (k=0; *(channels+k) !=-1; k++) {
	 this_channel = *(channels+k);
	 calib_bit[i][this_channel] = 
	                     cal_mask_manipulate(i, this_channel, 1, 0, 0);
	 channel_bit[i][this_channel] = 
	                     cal_mask_manipulate(i, this_channel, 0, 0, 0);
	 if (calib_bit[i][this_channel]   == 1 || 
             channel_bit[i][this_channel] == 1) {
	   nsetup = nsetup + 1;
	   chip_setup[i] = 1;
	 }
       }
     }
   }
/*---------------------------------------------------------------------*/
/* if there are no chips to setup, return                              */
/*---------------------------------------------------------------------*/
  if (nsetup == 0) {
     sprintf(return_code,"NO");
     return return_code;
   }
/*------------------------------------------------------------------------*/
/* Now loop over all chips                                                */
/* If we are not setting up this chip, let's make sure that               */
/* all masks are zero                                                     */
/*------------------------------------------------------------------------*/
/*sp  for (i=0; i<number_of_chips; i++) {                               *sp/
/*sp                                                                    *sp/
/*sp    if (chip_setup[i] == 0) {                                       *sp/
/*sp       reset_masks = 0;                                             *sp/
/*sp       for (j=0; j<8; j++) {                                        *sp/
/*sp	 if (written_calib_mask[i][j]   != 0) reset_masks = 1;          *sp/
/*sp	 if (written_channel_mask[i][j] != 0) reset_masks = 1;          *sp/
/*sp       }                                                            *sp/
/*sp       if (reset_masks == 1) {                                      *sp/
/*sp	    for(j=0; j<8; j++) {                                        *sp/
/*sp	      calib_mask_store[i][j]   = calib_mask[i][j];              *sp/
/*sp	      channel_mask_store[i][j] = channel_mask[i][j];            *sp/
/*sp	      calib_mask[i][j]         = 0;                             *sp/
/*sp	      channel_mask[i][j]       = 0;                             *sp/
/*sp	    }                                                           *sp/
/*sp	    rc = cal_write_masks(i);                                    *sp/
/*sp	    for(j=0; j<8; j++) {                                        *sp/
/*sp	      calib_mask[i][j]   = calib_mask_store[i][j];              *sp/
/*sp	      channel_mask[i][j] = channel_mask_store[i][j];            *sp/
/*sp	    }                                                           *sp/
/*sp            if (rc != VME_OK) {                                     *sp/
/*sp	      sprintf(return_code,"Error writing masks to chip %d",i);  *sp/
/*sp	      return return_code;                                       *sp/
/*sp	    }                                                           *sp/
/*sp	  }                                                             *sp/
/*sp                                                                    *sp/
/*sp     } else {             * chip_setup[i] != 0  *                   *sp/
/*sp       for (j=0; j<8; j++) {                                        *sp/
/*sp	 calib_mask_store[i][j]   = calib_mask[i][j];                   *sp/
/*sp	 channel_mask_store[i][j] = channel_mask[i][j];                 *sp/
/*sp	 calib_mask[i][j]         = 0;                                  *sp/
/*sp	 channel_mask[i][j]       = 0;                                  *sp/
/*sp       }                                                            *sp/
/*sp       dummy = cal_mask_manipulate(i, this_channel, 0, 1, 0);       *sp/
/*sp       dummy = cal_mask_manipulate(i, this_channel, 1, 1, 0);       *sp/
/*sp       rc = cal_write_masks(i);                                     *sp/
/*sp       for(j=0; j<8; j++) {                                         *sp/
/*sp	  calib_mask[i][j]   = calib_mask_store[i][j];                  *sp/
/*sp	  channel_mask[i][j] = channel_mask_store[i][j];                *sp/
/*sp       }                                                            *sp/
/*sp       if (rc != VME_OK) {                                          *sp/
/*sp	      sprintf(return_code,"Error writing masks to chip %d",i);  *sp/
/*sp	      return return_code;                                       *sp/
/*sp       }                                                            *sp/
/*sp     }                                                              *sp/
/*sp  }                                                                 *sp/

/*========================================================================*/
/* new way of doing it, hopefully faster.  write masks to all chips       */
/* with one single RPC call                                               */
/*========================================================================*/

/*----------------------------------------------*/
/* first store all the masks, then zero them    */
/*--------------------------------------------- */
  for (i=0; i<number_of_chips; i++) {
    for (j=0; j<8; j++) {
      calib_mask_store[i][j]   = calib_mask[i][j];
      channel_mask_store[i][j] = channel_mask[i][j];
      calib_mask[i][j]         = 0;
      channel_mask[i][j]       = 0;
    }
  }
/*----------------------------------------------*/
/* set the single bits, for the right chips     */
/*----------------------------------------------*/
  for (i=0; i<number_of_chips; i++) {
    if (chip_setup[i] == 1) {
       for (k=0; *(channels+k) !=-1; k++) {
	 this_channel = *(channels+k);
         if (channel_bit[i][this_channel] == 1)	 dummy = 
	                 cal_mask_manipulate(i, this_channel, 0, 1, 0);
         if (calib_bit[i][this_channel] == 1)	 dummy = 
	                 cal_mask_manipulate(i, this_channel, 1, 1, 0);
       }
    }
  }
/*----------------------------------------------*/
/* write all of the masks at once               */
/*----------------------------------------------*/
  rc = cal_write_masks(-1);
/*----------------------------------------------*/
/* restore all the masks                        */
/*--------------------------------------------- */
  for (i=0; i<number_of_chips; i++) {
    for (j=0; j<8; j++) {
      calib_mask[i][j]   = calib_mask_store[i][j];
      channel_mask[i][j] = channel_mask_store[i][j];
    }
  }
/*----------------------------------*/       
/* done                             */
/*----------------------------------*/
  sprintf(return_code,"OK");
  return return_code;
}

/***************************************************************************/
/* This is the definition of the Tcl command "cal_loop"                    */
/* It also calls the definition of other custom Tcl commands               */ 
/***************************************************************************/
int My_Procedure_Init(Tcl_Interp* interp)
{

  Tcl_CreateCommand(interp, "cal_loop", Calibrate_Loop, (ClientData) NULL,
                    (Tcl_CmdDeleteProc *) NULL);
  loop_test_define(interp);
  rw_atom_register_define(interp);
  pass_histoscope_define(interp);
  shorts_test_define(interp);
  misc_tests_define(interp);
  return 0;
}



/***************************************************************************/
/* Input:  number of channels to be fired together (num_together)          */
/* Output: number of iterations  (niter)                                   */
/*         list of channels to be fired (-1 to terminate)                  */
/***************************************************************************/
void get_list_of_channels(int num_together, int* niter, 
int* list_pointer) {


  int i;
  int j;
  int list_of_channels_local[128][128];

  *niter = 128 / num_together;
  if (128  != num_together * (*niter)) *niter = (*niter)+1;
  for (i=0; i<128; i++) {
    for (j=0; j<128; j++) list_of_channels_local[i][j] = -1;
  }

  for (i=0; i<*niter; i++) {
    for (j=0; j<num_together; j++) {
      if ( (i+(*niter)*j) < 128) {
	list_of_channels_local[i][j] = i + (*niter)*j;
      }
    }
  } 
  for (i=0; i<128; i++) {
    for (j=0; j<128; j++) {
      *(list_pointer+128*i+j) = list_of_channels_local[i][j];
    }
  }
  
}
