#include "cal_svtdef.h"
#include "tk.h"
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "error.h"
#include "reglist.h"
#include "cal_global.h"
#include "cal_functions.h"  
#include "opcodes.h"

/*****************************************************************/
/* dir = 0 takes num, extracts its four bytes, puts it in bytes  */
/* dir = 1 the other way around                                  */
/* returns zero on success                                       */
/*****************************************************************/
int cal_extract(int dir, int *num, char* bytes) {

  int shft[4] = {0, 8, 16, 24};
  int dummy;
  int i;

  if (dir !=0 && dir!=1) {
    return 1;
  }

  if (dir == 0) {
    *bytes     = (char) *num & 0xff;
    *(bytes+1) = (char) ((*num & 0xff00)     >> 8);
    *(bytes+2) = (char) ((*num & 0xff0000)   >> 16);
    *(bytes+3) = (char) ((*num & 0xff000000) >> 24);
  } else {
    *num = 0;
    for (i=0; i<4; i++) {
      dummy = (int) *(bytes+i);
      dummy = dummy & 0xff;
      if (shft[i] != 0) { 
         dummy = dummy << shft[i];
       }
      *num = *num | dummy;
    }

  }

  return 0;
}

/*****************************************************************/
/*   takes "value", looks at the first "how_many"                */
/*   bits, ignores all higher order bits, and                    */
/*   returns a "value" with the bit order switched               */
/*****************************************************************/
int switch_bits(int value, int how_many)
{
    int mask;
    int i;
    int this_bit_number;
    int return_value;
    int temp;
    int reversed_bit_number;


    if (how_many < 0) {
      how_many = 0;
    }
    if (how_many > 32) {
      how_many = 32;
    }


    mask = 0;
    for (i=1; i<=how_many; i++) {
      this_bit_number=i-1;
      mask = mask | (1 << this_bit_number);
    }

    temp = value & mask;

    return_value = 0;
    for (i=1; i<=how_many; i++) {
         this_bit_number=i-1;
         reversed_bit_number = how_many - this_bit_number -1;
	 mask = 1 << this_bit_number;
	 if ( (temp & mask) != 0) {
	    return_value = return_value | (1 << reversed_bit_number);
	  }
    }

    return (return_value);
}


/*************************************************************************/
/* This is some string manipulations needed by Cal_unpack_variables      */
/*************************************************************************/
char *Cal_unpack_array(Tcl_Interp* interp, char *this_array, char* this_index)
{
  char *temp;
  char temp2[100];
  strcpy(temp2,this_array);
  strcat(temp2,this_index);
  temp = Tcl_GetVar(interp,temp2,0);  
  return (temp);
}

/*************************************************************************/
/* Unpack the Tcl variables into C variables                             */
/*************************************************************************/
void Cal_unpack_variables(Tcl_Interp* interp)
{
   int i1;
   int j1;
   char index[100];
   int dummy;
   double dummy_double;

/*-----------------------------------------------------------*/
/* First translate Tcl variables into C-character strings    */
/*-----------------------------------------------------------*/
   C_run = Tcl_GetVar(interp,"Cal_run",0);  
   C_histoscope = Tcl_GetVar(interp,"Cal_histoscope",0);
   C_error_limit = Tcl_GetVar(interp,"Cal_error_limit",0);
   C_fake = Tcl_GetVar(interp,"Cal_fake",0);
   C_write_file = Tcl_GetVar(interp,"Cal_write_file",0);
   C_output_file = Tcl_GetVar(interp,"Cal_output_file",0);
   C_run_number = Tcl_GetVar(interp,"Cal_run_number",0); 
   C_number_of_events =  Tcl_GetVar(interp,"Cal_number_of_events",0);
   C_Side_Hybrid = Tcl_GetVar(interp,"Cal_Side_Hybrid",0);
   C_number_of_chips =  Tcl_GetVar(interp,"Cal_number_of_chips",0);
   C_number_of_chips_top =  Tcl_GetVar(interp,"Cal_number_of_chips_top",0);
   C_number_of_buffers =  Tcl_GetVar(interp,"Cal_number_of_buffers",0);
   C_root_file =  Tcl_GetVar(interp,"Cal_root_file",0);
   C_format_check_flag = Tcl_GetVar(interp,"Cal_format_check",0);

   for (i1=0; i1<MAX_NUM_CHIP; i1++) 
     {
       sprintf(index,"(%d)",i1);
       C_min_threshold[i1] = Cal_unpack_array(interp,"Cal_min_threshold",index);
       C_max_threshold[i1] = Cal_unpack_array(interp,"Cal_max_threshold",index);
       C_step_threshold[i1] = Cal_unpack_array(interp,"Cal_step_threshold",index);
       C_min_charge[i1] = Cal_unpack_array(interp,"Cal_min_charge",index);
       C_max_charge[i1] = Cal_unpack_array(interp,"Cal_max_charge",index);
       C_step_charge[i1] = Cal_unpack_array(interp,"Cal_step_charge",index);
       C_jitter[i1] = Cal_unpack_array(interp,"Cal_jitter",index);
       C_shaping_time[i1] = Cal_unpack_array(interp,"Cal_shaping_time",index);
       C_sampling_rate[i1] = Cal_unpack_array(interp,"Cal_sampling_rate",index);
       C_skip[i1] = Cal_unpack_array(interp,"Cal_skip",index);
       C_readout_direction[i1] =  Cal_unpack_array(interp,"Cal_readout_direction",index);
       C_ignore[i1] = Cal_unpack_array(interp,"Cal_ignore",index);
       C_respond[i1] =  Cal_unpack_array(interp,"Cal_respond",index);
       C_clock_through[i1] =  Cal_unpack_array(interp,"Cal_clock_through",index);
       C_test_pattern[i1] = Cal_unpack_array(interp,"Cal_test_pattern",index);
       C_Set_Clock_B[i1] = Cal_unpack_array(interp,"Cal_Set_Clock_B",index); 
       for (j1=0; j1<8; j1++)
	 {
	    sprintf(index,"(%d,%d)",i1,j1);
	    C_calib_mask[i1][j1] = Cal_unpack_array(interp,"Cal_calib_mask",index);
	    C_channel_mask[i1][j1] = Cal_unpack_array(interp,"Cal_channel_mask",index);   
	
	  }
     }
  C_xy_scan           = Tcl_GetVar(interp,"xy_scan",0);
  C_xy_which_axis     = Tcl_GetVar(interp,"xy_which_axis",0);
  C_xy_x_initial      = Tcl_GetVar(interp,"xy_x_initial",0);
  C_xy_y_initial      = Tcl_GetVar(interp,"xy_y_initial",0);
  C_xy_step_size      = Tcl_GetVar(interp,"xy_step_size",0);
  C_xy_nstep          = Tcl_GetVar(interp,"xy_nstep",0);
  C_xy_start_flag     = Tcl_GetVar(interp,"xy_start_flag",0);
  C_one_at_a_time     = Tcl_GetVar(interp,"Cal_one_at_a_time",0);
  C_compressed_format = Tcl_GetVar(interp,"Cal_compressed_format",0);
/*---------------------------------------------------------------*/
/* Now translate C-character strings into numbers, when needed   */
/* Replace the ugly sscanf method with Tcl_GetInt                */
/*---------------------------------------------------------------*/
   dummy = Tcl_GetInt(interp,C_compressed_format, &compressed_format);
   dummy = Tcl_GetInt(interp,C_one_at_a_time,     &one_at_a_time);
   dummy = Tcl_GetInt(interp,C_xy_scan,           &xy_scan);
   dummy = Tcl_GetInt(interp,C_xy_which_axis,     &xy_which_axis);
   dummy = Tcl_GetInt(interp,C_xy_nstep,          &xy_nstep);
   dummy = Tcl_GetInt(interp,C_xy_start_flag,     &xy_start_flag);
   dummy = Tcl_GetDouble(interp,C_xy_x_initial,   &dummy_double);
   xy_x_initial = (float) dummy_double;
   dummy = Tcl_GetDouble(interp,C_xy_y_initial,   &dummy_double);
   xy_y_initial = (float) dummy_double;
   dummy = Tcl_GetDouble(interp,C_xy_step_size,   &dummy_double);
   xy_step_size = (float) dummy_double;
   dummy = Tcl_GetInt(interp,C_error_limit,       &error_limit);
   dummy = Tcl_GetInt(interp,C_run_number,        &run_number);
   dummy = Tcl_GetInt(interp,C_number_of_events,  &number_of_events);
   dummy = Tcl_GetInt(interp,C_number_of_chips,   &number_of_chips);
   dummy = Tcl_GetInt(interp,C_number_of_chips_top,   &number_of_chips_top);
   dummy = Tcl_GetInt(interp,C_number_of_buffers, &number_of_buffers);
   dummy = Tcl_GetInt(interp,C_Side_Hybrid,       &number_of_sides);
   dummy = Tcl_GetInt(interp,C_format_check_flag, &format_check_flag); 

   for (i1=0; i1<MAX_NUM_CHIP; i1++)
   {
    dummy = Tcl_GetInt(interp,C_min_threshold[i1],     &min_threshold[i1]);
    dummy = Tcl_GetInt(interp,C_max_threshold[i1],     &max_threshold[i1]);
    dummy = Tcl_GetInt(interp,C_step_threshold[i1],    &step_threshold[i1]);
    dummy = Tcl_GetInt(interp,C_min_charge[i1],        &min_charge[i1]);
    dummy = Tcl_GetInt(interp,C_max_charge[i1],        &max_charge[i1]);
    dummy = Tcl_GetInt(interp,C_step_charge[i1],       &step_charge[i1]);
    dummy = Tcl_GetInt(interp,C_jitter[i1],            &jitter[i1]);
    jitter[i1] = Bin_to_gc(jitter[i1]);
    dummy = Tcl_GetInt(interp,C_shaping_time[i1],      &shaping_time[i1]);
    dummy = Tcl_GetInt(interp,C_sampling_rate[i1],     &sampling_rate[i1]);
    dummy = Tcl_GetInt(interp,C_skip[i1],              &skip[i1]);
    dummy = Tcl_GetInt(interp,C_readout_direction[i1], &readout_direction[i1]);
    dummy = Tcl_GetInt(interp,C_ignore[i1],            &ignore[i1]);
    dummy = Tcl_GetInt(interp,C_respond[i1],           &respond[i1]);
    dummy = Tcl_GetInt(interp,C_clock_through[i1],     &clock_through[i1]);
    dummy = Tcl_GetInt(interp,C_test_pattern[i1],      &test_pattern[i1]);
    dummy = Tcl_GetInt(interp,C_Set_Clock_B[i1],       &Set_Clock_B[i1]);
     for (j1=0; j1<8; j1++)
      {
       dummy = Tcl_GetInt(interp,C_calib_mask[i1][j1],  &calib_mask[i1][j1]);
       dummy = Tcl_GetInt(interp,C_channel_mask[i1][j1],&channel_mask[i1][j1]);
      }  
     }
/* ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~*/
/*  sscanf(C_error_limit,"%d",&error_limit);                        */
/*  sscanf(C_run_number,"%d",&run_number);                          */
/*  sscanf(C_number_of_events,"%d",&number_of_events);              */
/*  sscanf(C_number_of_chips,"%d%",&number_of_chips);               */
/*  sscanf(C_number_of_chips_top,"%d%",&number_of_chips_top);       */
/*  sscanf(C_number_of_buffers,"%d%",&number_of_buffers);           */
/*  sscanf(C_Side_Hybrid,"%d%",&number_of_sides);                   */
/*   for (i1=0; i1<MAX_NUM_CHIP; i1++)                              */
/*   {                                                              */
/*    sscanf(C_min_threshold[i1],"%d",&min_threshold[i1]);          */
/*    sscanf(C_max_threshold[i1],"%d",&max_threshold[i1]);          */
/*    sscanf(C_step_threshold[i1],"%d",&step_threshold[i1]);        */
/*    sscanf(C_min_charge[i1],"%d",&min_charge[i1]);                */
/*    sscanf(C_max_charge[i1],"%d",&max_charge[i1]);                */
/*    sscanf(C_step_charge[i1],"%d",&step_charge[i1]);              */
/*    sscanf(C_jitter[i1],"%d",&jitter[i1]);                        */
/*    jitter[i1] = Bin_to_gc(jitter[i1]);                           */
/*    sscanf(C_shaping_time[i1],"%d",&shaping_time[i1]);            */
/*    sscanf(C_sampling_rate[i1],"%d",&sampling_rate[i1]);          */
/*    sscanf(C_skip[i1],"%d",&skip[i1]);                            */
/*    sscanf(C_readout_direction[i1],"%d",&readout_direction[i1]);  */
/*    sscanf(C_respond[i1],"%d",&respond[i1]);                      */
/*    sscanf(C_clock_through[i1],"%d",&clock_through[i1]);          */
/*    sscanf(C_test_pattern[i1],"%d",&test_pattern[i1]);            */
/*    sscanf(C_Set_Clock_B[i1],"%d",&Set_Clock_B[i1]);              */
/*    for (j1=0; j1<8; j1++)                                        */
/*     {                                                            */
/*      sscanf(C_calib_mask[i1][j1],"0x%x",&calib_mask[i1][j1]);    */
/*      sscanf(C_channel_mask[i1][j1],"0x%x",&channel_mask[i1][j1]);*/ 
/*     }                                                            */
/*    }                                                             */
/* ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~*/
/*------------------------------------------------------------------*/
/* debug printout : Did we get it right ?????????                   */
/*------------------------------------------------------------------*/
/*   printf("C_run = %s \n",C_run);
   printf("C_histoscope = %s \n",C_histoscope);
   printf("C_write_file = %s \n",C_write_file);
   printf("C_output_file = %s \n",C_output_file);
   printf("run_number = %d \n",run_number);
   printf("number_of_events = %d \n",number_of_events);
   printf("number_of_chips = %d \n",number_of_chips);
   for (i1=0; i1<number_of_chips; i1++)
     {
       printf("min_threshold %d = %d \n", i1,min_threshold[i1]);
       printf("max_threshold %d = %d \n", i1,max_threshold[i1]);
       printf("step_threshold %d = %d \n", i1,step_threshold[i1]);
       printf("min_charge %d = %d \n", i1,min_charge[i1]);
       printf("max_charge %d = %d \n", i1,max_charge[i1]);
       printf("step_charge %d = %d \n", i1,step_charge[i1]);
       printf("jitter %d = %d \n", i1,jitter[i1]);
       printf("shaping_time %d = %d \n", i1,shaping_time[i1]);
       printf("sampling_rate %d = %d \n", i1,sampling_rate[i1]);
       printf("skip %d = %d \n", i1,skip[i1]);
       printf("readout_direction %d = %d \n", i1,readout_direction[i1]);
       printf("respond %d = %d \n", i1,respond[i1]);
       printf("clock_through %d = %d \n", i1,clock_through[i1]);
       printf("test_pattern %d = %d \n", i1,test_pattern[i1]);
       printf("Set_Clock_B %d = %d \n", i1,Set_Clock_B[i1]);
       for (j1=0; j1<8; j1++)
         {
	   printf("calib_mask %d %d = 0x%x \n",i1,j1,calib_mask[i1][j1]);
	   printf("channel_mask %d %d = 0x%x \n",i1,j1,channel_mask[i1][j1]);
	 }
     }
        end of debug printout  ***********************/
/*-----------------------------------------------------------*/
 }


/*************************************************************************/
/* This is the end run procedure.  Close file if file is open, close     */
/* histoscope if histoscope is going                                     */ 
/*************************************************************************/
int cal_stop_run(Tcl_Interp* interp, FILE** fp, FILE* dump_file)
{
  int status1;
  int status2;
  int status3;
  int status4;
  char* return_code;
        status1 = 0;
        status2 = 0;
        status3 = 0;
        status4 = 0;
        if (strcmp("1",C_histoscope) == 0) {
	       status1 = cal_finish_histoscope(interp);
	  }

        if (strcmp("1",C_write_file) == 0) {
	       status2 = cal_close_file(fp);
	}
        status3 = cal_dump(10, dump_file);
        if (xy_scan == 1) {
	  return_code = cal_xy_end_axis(interp, 1);
	  if (strcmp("OK",return_code) != 0) status4=999;
	  return_code = cal_xy_end_axis(interp, 2);
	  if (strcmp("OK",return_code) != 0) status4=999;
	}
	return (((status3 | status1) | status2) | status4);
}
/**************************************************************/
/* Abort run procedure                                        */
/**************************************************************/
void cal_abort_run(Tcl_Interp* interp, FILE** fp, int event_number, 
FILE* dump_file)
{
  int dummy;
  int status;

  static char c1[] = ".run_menu.text insert end \" RUN HAS BEEN ABORTED \n\"";
  static char c2[] = "set Cal_run stop";
  static char c3[] = "grab release .run_menu";
  static char c4[] = "cal_bump_runnumber";
  static char c5[] = ".run_menu.text yview -pickplace end";
  static char c6[] = ".run_menu.text insert end ";
  static char msg[200];
 
  sprintf(msg,"%s \"Last event processed was event no. %d \n\"",c6,
event_number);
  dummy = Tcl_Eval(interp,c1);      
  dummy = Tcl_Eval(interp,msg);
  dummy = Tcl_Eval(interp,c3);
  dummy = Tcl_Eval(interp,c2);
  dummy = Tcl_Eval(interp,c5);

/*  Removed the next three lines of code.  CC 4-apr-1997   */ 
/*  if (strcmp("1",C_write_file) == 0) {                   */
/*    dummy = Tcl_Eval(interp,c4);                         */
/*  }                                                      */

  status = cal_stop_run(interp,fp,dump_file); 
}


/*****************************************************************/
/* open the output file                                          */
/*****************************************************************/
int cal_open_file(FILE **fp)
{
  if (*fp != NULL) return 1;
  *fp = fopen(C_output_file,"w");
  if (*fp == NULL) return 1;
  return 0;
}
/*****************************************************************/
/* write to the output file                                      */
/*****************************************************************/
int cal_write_file(FILE **fp)
{
  int i, nwritten, nbytes, dummy;

  if (*fp == NULL) return 1;  
  dummy    = cal_extract(1,&nbytes,cal_data_point);
  nwritten = fwrite(cal_data_point, 1, nbytes, *fp);
  if (nwritten != nbytes) return 1;


/***  for (i=0; i<nbytes; i++) {                         ***/
/***    dummy = fputc(*(cal_data_point+i), *fp);         ***/
/***    if (*(cal_data_point+i) != EOF) {                ***/
/***      if (dummy == EOF) return 1;                    ***/
/***    }                                                ***/
/***  }                                                  ***/

   return 0;
}
/*****************************************************************/
/* close the output file                                          */
/*****************************************************************/
int cal_close_file(FILE **fp)
{
  if (*fp == NULL) return 1;
  fclose(*fp);
  *fp = NULL;
  return 0;
}


/*****************************************************************/
/*  Get channel and address for this chip                        */
/*****************************************************************/
void cal_get_chip_address(int chip_number, int* add, int* chan)
{
   int temp;

      if (number_of_sides == 1) {
	 *add = chip_number + 1;
	 *chan = 0x1;
	 if (readout_direction[chip_number] == 1) *chan = 0x2;
       }
      else if (chip_number < (number_of_chips_top)) {
	 *add = chip_number + 1;
	 *chan = 0x1;
	 if (readout_direction[chip_number] == 1) *chan = 0x2;
       }
      else {
	 *add = chip_number - (number_of_chips_top) + 1;
	 *chan = 0x4;
	 if (readout_direction[chip_number] == 1) *chan = 0x8;
       }

/* Nov 26 1996: subtract one from address, so that they start at zero  */
       *add = *add - 1;

       temp = *add;
       temp = switch_bits(temp,5);
       *add = temp;

 }
/*****************************************************************/
/* returns zero if the number of steps for thresholds and charge */
/* injection are the same for all chips                          */
/****************************************************************/ 
int  cal_chip_by_chip(void)
{
  int i1; 
  int nt0;
  int nc0;
  int nt;
  int nc;

/*- get the number of steps for chip 0                     */
  nt0 = cal_get_nstep(0,1);
  if (nt0 == -999) return 1;
  nc0 = cal_get_nstep(0,2);
  if (nc0 == -999) return 1;

/*- now compare with all other chips                       */
  for (i1=1; i1<number_of_chips; i1++) {
    nt = cal_get_nstep(i1,1);
    if (nt != nt0) return 1;
    nc = cal_get_nstep(i1,2);
    if (nc != nc0) return 1;
  }

/*- done                                                  */
  return 0;
}


/************************************************************************/
/* Returns number of steps for threshold or charge injection for a chip */
/* flag=1 for threshold, flag=2 for charge.  Returns -999 for error     */  
/************************************************************************/
int cal_get_nstep(int chip_number, int flag)
{
    int nstep;
    int this_min;
    int this_max;
    int this_step;
    int d;

/*- check for a valid chip number                                      */
    if (chip_number < 0)                return -999;
    if (chip_number >= number_of_chips) return -999;

/*- get min, max, step....also check for illegal flag                  */
    if (flag == 1) {   
      this_min  = min_threshold[chip_number];
      this_max  = max_threshold[chip_number];
      this_step = step_threshold[chip_number];
    } else if (flag==2) {
      this_min  = min_charge[chip_number];
      this_max  = max_charge[chip_number];
      this_step = step_charge[chip_number];
    } else {
      return -999;
    }

/*- make sure that the step size is not zero                           */
    if (this_step == 0) return -999;

/*- now get the number of steps                                        */
    d     = this_max - this_min;
    nstep = d/this_step;
    nstep++;
    return nstep;
}
/**************************************************************************/
/* Get or set or zero bit from channel or calibration mask                */
/* Inputs:  chip      = chip number    (0 - number_of_chips-1)            */
/*          channel   = channel number (0 - 127)                          */
/*          flag      = 0 (channel mask) 1 (calibration mask)             */
/*          action    = 0 (get the bit) 1 (set the bit)  2 (zero the bit) */
/*          variable  = 0 (calib_mask or channel_mask)                    */
/*                      1 (written_calib_mask or written_channel_mask)    */
/*                      (This last option is valid only for action=0)     */
/* Return:  999  if error                                                 */
/*          0 or 1 (bit ON or OFF, only meaningful if action=0)           */
/**************************************************************************/
int cal_mask_manipulate(int chip, int channel, int flag, int action, 
                        int variable) {

  int index, bit;
  int mask_value;
  int temp;
  int ivalue;

/*- check for valid inputs                                             */
  if (flag != 1 && flag != 0)       return 999;
  if (chip < 0)                     return 999;
  if (chip > (number_of_chips-1))   return 999;
  if (channel < 0)                  return 999;
  if (channel > 127)                return 999;
  if (action < 0)                   return 999;
  if (action > 2)                   return 999;
  if (variable < 0)                 return 999;
  if (variable > 1)                 return 999;
  if (action != 0 && variable == 1) return 999;

/*- get the index and bit number within the array                      */
  index = channel/16;
  bit   = channel - 16*index;

/* setup a variable with all bits zero except the one we care about    */
  temp = 1;
  if (bit > 0) temp = temp<<bit;

/* get the current value of the mask                                   */
  if (flag == 0) mask_value = channel_mask[chip][index];
  if (flag == 1) mask_value = calib_mask[chip][index];
  if (variable == 1 && flag == 0) 
    mask_value = written_channel_mask[chip][index];
  if (variable == 1 && flag == 1) 
    mask_value = written_calib_mask[chip][index];


/* what do we want to do with it ?                                     */
  if (action == 0) {
     ivalue = mask_value & temp;
     if (ivalue != 0) ivalue=1;
     return ivalue;
  } else if (action == 1) {
     mask_value = mask_value | temp;
  } else if (action == 2) {
     temp       = temp ^ 0xffff;
     mask_value = mask_value & temp;
  } 
  if (action == 1 || action == 2) {
     if (flag == 0) channel_mask[chip][index] = mask_value;
     if (flag == 1) calib_mask[chip][index]   = mask_value;
   }
  return 0;
}




/************************************************************************/
/*  Binary to gray code conversion  (5 bits)                            */
/************************************************************************/
int Bin_to_gc(a)
int a;
{
   switch(a){
      case 0: return(0);
      case 1: return(1);
      case 2: return(3);
      case 3: return(2);
      case 4: return(6);
      case 5: return(7);
      case 6: return(5);
      case 7: return(4);
      case 8: return(12);
      case 9: return(13);
      case 10: return(15);
      case 11: return(14);
      case 12: return(10);
      case 13: return(11);
      case 14: return(9);
      case 15: return(8);
      case 16: return(24);
      case 17: return(25);
      case 18: return(27);
      case 19: return(26);
      case 20: return(30);
      case 21: return(31);
      case 22: return(29);
      case 23: return(28);
      case 24: return(20);
      case 25: return(21);
      case 26: return(23);
      case 27: return(22);
      case 28: return(18);
      case 29: return(19);
      case 30: return(17);
      case 31: return(16);
      default : return(-1);
    }
}
/************************************************************************/
/* compress event record, return zero if OK                             */
/* take as input the uncompressed event record from "cal_data_point"    */
/************************************************************************/
int cal_compress_event_record() {
  int dummy, old_nbytes, i, new_nbytes, new_ishift, old_ishift;
  int ishift, dummy2;
  char buff[4];
  char* temp;
  int begin_data[MAX_NUM_CHIP];
  int end_data[MAX_NUM_CHIP];
/*----------------------------------------------------------------------*/
/* find the number of bytes in uncompressed record, save it into temp   */
/*----------------------------------------------------------------------*/
  dummy  = cal_extract(1, &old_nbytes, cal_data_point);
  temp   = (char *) malloc(old_nbytes);
  if (temp == NULL) return -1;
  for (i=0; i<old_nbytes; i++) {
    *(temp+i) = *(cal_data_point+i);
  }
  free(cal_data_point);
/*----------------------------------------------------------------------*/
/* find the number of bytes in the compressed record                    */
/* Remove run number (4 bytes), cont reg (4*nchips bytes),              */
/* masks (32*nchips bytes), chip pointers (8*nchips bytes),             */
/* xy_scan pointers (8 bytes).                                          */
/*----------------------------------------------------------------------*/
  new_nbytes = old_nbytes - 4 - 44*number_of_chips;
  if (xy_scan == 1) new_nbytes = new_nbytes - 8;
/*----------------------------------------------------------------------*/
/* get some space                                                       */
/*----------------------------------------------------------------------*/
  cal_data_point = (char *) malloc(new_nbytes);
  if (cal_data_point == NULL) {
    free(temp);
    return -1;
  }
/*----------------------------------------------------------------------*/
/* put in number of bytes                                               */
/*----------------------------------------------------------------------*/
 dummy = cal_extract(0, &new_nbytes, buff);
 for (i=0; i<4; i++) {
   *(cal_data_point + i) = buff[i];
 }
/*----------------------------------------------------------------------*/
/* put in record type flag                                              */
/*----------------------------------------------------------------------*/
  *(cal_data_point + 4) = 0x2; 
/*----------------------------------------------------------------------*/
/* put in event number                                                  */
/*----------------------------------------------------------------------*/
  for (i=0; i<4; i++) {
    *(cal_data_point + 5 + i) = *(temp + 8 +i); 
  }
/*----------------------------------------------------------------------*/
/* put in everything else                                               */
/* BUG IN FOLLOWING CODE WHEN TRYING TO READ FROM THE RIGHT 28-JAN-98   */   
/*----------------------------------------------------------------------*/
/*  old_ishift = old_nbytes - 1;                                        */
/*  for (new_ishift = new_nbytes-1; new_ishift>8; new_ishift--) {       */
/*    *(cal_data_point + new_ishift) = *(temp + old_ishift);            */
/*    old_ishift--;                                                     */
/*  }                                                                   */
/*----------------------------------------------------------------------*/
/* FIX THE ABOVE, 28 JAN 1998                                           */
/*----------------------------------------------------------------------*/
/* - - - - - - - - - - - - - - - - -- - - - - - - - - - - */
/* put in the xy scan information                         */
/* - - - - - - - - - - - - - - - - -- - - - - - - - - - - */
  new_ishift = 9;
  if (xy_scan == 1) {
    old_ishift = 13 + 44*number_of_chips + 8;
    for (i=0; i<8; i++) {
      *(cal_data_point + new_ishift) = *(temp + old_ishift);
      new_ishift++;
      old_ishift++;
    }
  }
/* - - - - - - - - - - - - - - - - -- - - - - - - - - - - */
/* Find the old chip data pointers                        */
/* - - - - - - - - - - - - - - - - -- - - - - - - - - - - */
  ishift = 13 + 36 * number_of_chips;
  for (i=0; i<number_of_chips; i++) {
    dummy2 = cal_extract(1,&dummy,(temp+ishift));
    begin_data[i] = dummy;
    ishift = ishift + 4;
  }
  for (i=0; i<number_of_chips; i++) {
    dummy2 = cal_extract(1,&dummy,(temp+ishift));
    end_data[i] = dummy;
    ishift = ishift + 4;
  }
/* - - - - - - - - - - - - - - - - -- - - - - - - - - - - */
/* put the chip data in using the chip data pointers      */
/* - - - - - - - - - - - - - - - - -- - - - - - - - - - - */
  for (i=0; i<number_of_chips; i++) {
    for (ishift=begin_data[i]; ishift<=end_data[i]; ishift++) {
      *(cal_data_point + new_ishift) = *(temp + ishift);
      new_ishift++;
    }
  }  
/*----------------------------------------------------------------------*/
/* Done                                                                 */
/*----------------------------------------------------------------------*/
  free(temp);
  return 0;
}


/************************************************************************/
/* build the parameter record....return zero if OK                      */
/************************************************************************/
int cal_parameter_record() {
  int dummy, nbytes, i, j, itemp, i3, i2, this_shift;
  char buff[4];
/*---------------------------------------------------------------*/
/* get number of bytes, memory                                   */
/*---------------------------------------------------------------*/
  nbytes = 4 + 1 + 36*number_of_chips;
  cal_data_point = (char *) malloc(nbytes);
  if (cal_data_point == NULL) return -1;
/*---------------------------------------------------------------*/
/* put in number of bytes, record type flag                      */
/*---------------------------------------------------------------*/
  dummy = cal_extract(0, &nbytes, buff);
  for (i=0; i<4; i++) {
    *(cal_data_point + i) = buff[i];
  }
  *(cal_data_point+4) = 0x1;
  this_shift = 5;
/*-------------------------------------------------------------------*/
/* Control registers                                                 */
/*-------------------------------------------------------------------*/
  for (i=0; i<number_of_chips; i++) {
    itemp = cal_creg[i];
    j     = cal_extract(0,&itemp,buff);
    for (i3=0; i3<4; i3++) {
      *(cal_data_point + this_shift) = buff[i3];
      this_shift++;
    }
  }
/*-------------------------------------------------------------------*/
/* Calibration mask                                                  */
/*-------------------------------------------------------------------*/
  for (i=0; i<number_of_chips; i++) {                              
    for (i2=0; i2<8; i2++) {                                       
/*    itemp = calib_mask[i][i2];   CHANGED TO WRITTEN MASKS CC 1/4/97  */
      itemp = written_calib_mask[i][i2];
      j     = cal_extract(0,&itemp,buff);  
      for (i3=0; i3<2; i3++) {                                     
        *(cal_data_point + this_shift) = buff[i3];                 
        this_shift++;                                              
      }                                                            
    }                                                              
  }                                                                
/*-------------------------------------------------------------------*/
/* Channel mask                                                      */
/*-------------------------------------------------------------------*/
  for (i=0; i<number_of_chips; i++) {                              
    for (i2=0; i2<8; i2++) {                                       
/*    itemp = channel_mask[i][i2]; CHANGED TO WRITTEN MASKS CC 1/4/97  */
      itemp = written_channel_mask[i][i2];
      j     = cal_extract(0,&itemp,buff);                          
      for (i3=0; i3<2; i3++) {                                     
        *(cal_data_point + this_shift) = buff[i3];                 
        this_shift++;                                              
      }                                                            
    }                                                              
  }                                                                
/*-------------------------------------------------------------------*/
/* Done                                                              */
/*-------------------------------------------------------------------*/
  return 0;
}
