#include "cal_svtdef.h"
#include "tk.h"
#include <stdio.h>
#include <string.h>
#include "error.h"
#include "reglist.h"
#include "cal_global.h"
#include "cal_functions.h"
#include "opcodes.h"
#include "cardlist.h"

vme_rc vmeVmeWrite(vme_vmetype*, int, vme_vmetype);
vme_rc vmeVmeRead(vme_vmetype*, int);
vme_rc vmeVmeBuffRead(vme_vmetype*, int, vme_vmetype*, int);
vme_rc vmeVmeBuffRead2(vme_vmetype*, int, vme_vmetype*, int);

#define VRAM_KLUDGE_SIZE 2048
#define VRAM_SIZE 1999999

vme_vmetype vram_kludge[VRAM_KLUDGE_SIZE];


vme_rc cal_vram_read(int shift, int nword, vme_vmetype* address) 
{
 int i, j;
 vme_vmetype base;
 vme_rc rc;
 vme_rc value; 
 vme_vmetype mask[4] = {0xff, 0xff00, 0xff0000, 0xff000000};
 vme_vmetype odd_word;
 vme_vmetype byte[4];

static int base_store;
static first = 0;
char this_classname[100], this_instname[100];


/*-----------------------------------------------------------*/
/* find the base address of daq_0 (do it only once)          */
/* Then add to it the appropriate shift to get to VRAM       */
/* the final +0x4 is according to Armin                      */
/* (in principle it should not be there, but it is)          */
/*-----------------------------------------------------------*/
  if (first == 0) {
     first = 1;
     strcpy(this_classname,"daq");
     strcpy(this_instname,"daq_0");
     base_store = cal_get_base(this_classname, this_instname);
     if (base_store == -999) {
	 first = 0;
	 return VME_ILLEGAL_CARD;
     }
     base_store = base_store + 0x200000 + 0x4;
  }
/*----------------------------------------------------------------*/
/* This reads one word at a time                                  */
/*----------------------------------------------------------------*/
/*    if ( (shift+nword) > VRAM_SIZE) {                           */
/*      return VME_ILLEGAL_PARAMETER;                             */
/*    }                                                           */
/*    for (i=0; i<nword; i++) {                                   */
/*       base = base_store;                                       */
/*       rc = vmeVmeRead( (vme_vmetype *)base, (shift+i));        */
/*       *(address+i) = (vme_vmetype) rc;                         */
/*    }                                                           */
/*    return VME_OK;                                              */
/*----------------------------------------------------------------*/
/* This reads a bunch of words at a time (less RPC overhead ?)    */
/* 5/16/97 : switched from vmeVmeBuffRead to vmeVmeBuffRead2      */
/*----------------------------------------------------------------*/
      if ( (shift+2*nword) > VRAM_SIZE) return VME_ILLEGAL_PARAMETER;
      base = base_store;
      rc   = vmeVmeBuffRead2( (vme_vmetype *)base, shift, address, nword);
      if (rc != nword) return VME_ERROR;
/*----------------------------------------------------------------*/
/* Kludge because of problem in prototype                         */
/* Actually, not needed !!!                                       */
/*----------------------------------------------------------------*/
/*    for (i=0; i<nword; i=i+2) {                                 */
/*      odd_word  = *(address+i+1);                               */
/*      for (j=0; j<4; j++) {                                     */
/*	  byte[j] = odd_word & mask[j];                           */
/*      }                                                         */
/*      byte[0] = (byte[0] << 24) & 0xff000000;                   */
/*      byte[1] = (byte[1] << 8)  & 0xff0000;                     */
/*      byte[2] = (byte[2] >> 8)  & 0xff00;                       */
/*      byte[3] = (byte[3] >> 24) & 0xff;                         */
/*      *(address+i) = 0;                                         */
/*      for (j=0; j<4; j++) *(address+i)= *(address+i) | byte[j]; */
/*    }                                                           */
/*----------------------------------------------------------------*/
/* Done                                                           */
/*----------------------------------------------------------------*/
      return VME_OK;
}

/*************************************************************/
/* Write event with one hit per chip                         */
/* We'll set all trigger counts to 3                         */
/* All time stamps to 1                                      */
/* The hit channel on chip i will be channel 10+i            */
/* The time over threshold will by always 1                  */
/*************************************************************/ 
vme_rc cal_vram_write() {

int i, j, ichan, ichip;
int this_address, this_channel;
int ishift;

int nbytes;
int trigger_count = 3;
int time_stamp = 1;
int temp;
vme_vmetype header16;
vme_vmetype header1;
vme_vmetype header2;
vme_vmetype hit16;
vme_vmetype hit1;
vme_vmetype hit2;
vme_vmetype mask[4]  = {0xffffff00, 0xffff00, 0xff00, 0xff};
vme_vmetype mask2[4] = {0xff000000, 0xff0000, 0xff00, 0xff};
int bit_shift[4] = {0, 8, 16, 24};
vme_vmetype bits[4] = {0x1, 0x2, 0x4, 0x8};
vme_vmetype byte[4];
int chip_channel;
int tot = 1;
int max_word;
vme_rc rc;
vme_vmetype value;
vme_vmetype base;
static int base_store;
static first = 0;
char this_classname[100], this_instname[100];


/*-----------------------------------------------------------*/
/* find the base address of daq_0 (do it only once)          */
/* Then add to it the appropriate shift to get to VRAM       */
/*-----------------------------------------------------------*/
  if (first == 0) {
     first = 1;
     strcpy(this_classname,"daq");
     strcpy(this_instname,"daq_0");
     base_store = cal_get_base(this_classname, this_instname);
     if (base_store == -999) {
	 first = 0;
	 return VME_ILLEGAL_CARD;
     }
     base_store = base_store + 0x200000 + 0x4;
  }
/*-----------------------------------------------------------*/
/* Loop over channels                                        */
/*-----------------------------------------------------------*/
  max_word = 0;
  for (ichan=0; ichan<4; ichan++) {
    nbytes = 0;
/*-----------------------------------------------------------*/
/* Loop over chips                                           */
/* careful cal_get_chip_address return address of chip in    */
/* five bits with the order reversed...we need four bits     */
/* with the order reversed ....................              */
/*-----------------------------------------------------------*/
    for (ichip=0; ichip<number_of_chips; ichip++) {
      cal_get_chip_address(ichip,&this_address,&this_channel);
      if (bits[ichan] != this_channel) {
	goto next_chip;
      }
      this_address = this_address >> 1;
      if ( (nbytes+6) > VRAM_KLUDGE_SIZE/2) {
	return 999;
      }
/*-----------------------------------------------------------*/
/* get the header                                            */
/*-----------------------------------------------------------*/
      header16 = 0;
      header16 = header16 | 0x8000;       /* start bit */
      header16 = header16 | (this_address << 11);
      header16 = header16 | 0x400;         /* event data */
      temp   = switch_bits(trigger_count,5);
      header16 = header16 | (temp << 5);
      temp   = switch_bits(time_stamp,5);
      header16 = header16 | temp;

      header1  = (header16 & 0xff00) >> 8;
      header2  =  header16 & 0xff;

      if (bit_shift[ichan] != 0) {
         header1  = header1 << bit_shift[ichan];
         header2  = header2 << bit_shift[ichan];
      }      
      vram_kludge[nbytes]   = vram_kludge[nbytes]   | header1;
      vram_kludge[nbytes+2] = vram_kludge[nbytes+2] | header2;
      nbytes = nbytes + 4;
/*-----------------------------------------------------------*/
/* get the hit                                               */
/*-----------------------------------------------------------*/
      hit16 = 0;
      chip_channel = 10 + ichip;
      temp = switch_bits(chip_channel,7);
      hit16 = hit16 | (vme_vmetype) temp;
      temp = switch_bits(tot,4);
      temp = temp << 7;
      hit16 = hit16 | (vme_vmetype) temp;
      temp = switch_bits(time_stamp,5);
      temp = temp << 11;
      hit16 = hit16 | (vme_vmetype) temp;


      hit1  = (hit16 & 0xff00) >> 8;
      hit2  =  hit16 & 0xff;


      if (bit_shift[ichan] != 0) {
         hit1  = hit1 << bit_shift[ichan];
         hit2  = hit2 << bit_shift[ichan];
      }
      vram_kludge[nbytes]   = vram_kludge[nbytes]   | hit1;
      vram_kludge[nbytes+2] = vram_kludge[nbytes+2] | hit2;
      nbytes = nbytes + 4;

/*-----------------------------------------------------------*/
/* get the trailer                                           */
/*-----------------------------------------------------------*/
      vram_kludge[nbytes]   = vram_kludge[nbytes]   & mask[ichan];
      vram_kludge[nbytes+2] = vram_kludge[nbytes+2] & mask[ichan];
      if ( (nbytes+2) > max_word) max_word = nbytes+2;
      nbytes = nbytes + 4;
/*-----------------------------------------------------------*/
/* Done this chip, goto the next                             */
/*-----------------------------------------------------------*/
    next_chip: {}

    }
  }

/*-----------------------------------------------------------------------*/
/* debug                                                                 */
/*-----------------------------------------------------------------------*/
/*  for (i=0; i<64; i++) {                                               */
/*    printf("i %d vram_kludge %x \n",i,vram_kludge[i]);                 */
/*  }                                                                    */   
/*-----------------------------------------------------------------------*/
/* Now kludge.  Data is in odd words and byte order is switched          */
/* This is because of a problem with prototype boards                    */
/* NO.  THIS IS NOT TRUE                                                 */
/*-----------------------------------------------------------------------*/
/*  for (i=0; i<=max_word; i=i+2) {                                      */
/*    for (j=0; j<4; j++) {                                              */
/*	  byte[j] = vram_kludge[i] & mask2[j];                           */
/*    }                                                                  */
/*    byte[0] = (byte[0] >> 24) & 0xff;                                  */
/*    byte[1] = (byte[1] >> 8)  & 0xff00;                                */
/*    byte[2] = (byte[2] << 8)  & 0xff0000;                              */
/*    byte[3] = (byte[3] << 24) & 0xff000000;                            */
/*    vram_kludge[i]   = 0;                                              */
/*    vram_kludge[i+1] = 0;                                              */
/*    for (j=0; j<4; j++) vram_kludge[i+1] = vram_kludge[i+1] | byte[j]; */
/* }                                                                     */
/*-----------------------------------------------------------------------*/
/* Now write it to VME one word at a time                                */
/*-----------------------------------------------------------------------*/
  for (i=0; i<=max_word; i++) {
    base = base_store;
    value = vram_kludge[i];
    value = value & 0xffffffff;
    rc = vmeVmeWrite((vme_vmetype *)base, i, value);
    if (rc != VME_OK) return rc;
  }
/*-----------------------------------------------------------*/
/* return                                                    */
/*-----------------------------------------------------------*/
  return VME_OK;
}

/*******************************************************************/
/* Get base address of card with this_classname and this_instname  */
/* return -999 if failure                                          */
/*******************************************************************/
int cal_get_base(char* this_classname, char* this_instname)
{
  int i, j;
  int this_base;

      this_base = -999;
      for (i=0; VME_Cards[i] != NULL; i++)  {
          if (strcmp(VME_Cards[i]->classname,this_classname)==0) {
             for (j=0; j<VME_Cards[i]->bases; j++) {
		 if (strcmp(VME_Cards[i]->instname[j],this_instname) == 0) {
		     this_base = (int) VME_Cards[i]->base[j];
		 }
	     }
	 }
      }
      return this_base;
}



