#include "cal_svtdef.h"
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "error.h"
#include "reglist.h"
#include "tk.h"
#include "opcodes.h"
#include "cal_global.h"
#include "cal_functions.h"

char* Cal_unpack_array(Tcl_Interp*, char*, char*);

static int loop_testCmd(ClientData clientData, Tcl_Interp* interp, 
			int argc, char* argv[]) {

int read_chan, pm_chan, chip_address;
int which_chip, check_registers, check_masks, max_error, ignore_last_mask;
int dummy, ichip, error_count, ibit, jbit, itest, i, j, itemp, itemp2;
int imask, iflag, i1, i2, imin, imax;
static char *command_result;
char event_message[200];
vme_rc rc;
unsigned int creg_store;
int calib_mask_store[8], channel_mask_store[8];
int try_first;
int found_data[MAX_NUM_CHIP];

static char in[]    = ".pltw.text insert end \"";
static char cmd10[] = ".pltw.text insert end \"Done...\n\"";
static char cmd11[] = "update";
static char cmd0[]  = ".pltw.text yview -pickplace end";
static char cmd1[]  = "make_pltw_widget";
static char cmd2[]  = ".pltw.text insert end \"Aborting \n\"";
static char cmd3[]  = ".pltw.text insert end \" \n \n ************ Begin testing *********** \n\"";
static char cmd4[]  = ".pltw.text insert end \" Now setting up personality module and hybrid.....\n\"";
static char cmd5[]  = ".pltw.text insert end \"Too many errors... \n\"";
 error_count = 0;
/*-----------------------------------------------------*/
/* Make sure call has right number of arguments        */
/*-----------------------------------------------------*/
 if (argc != 6) {
   interp->result = "wrong number of arguments";
   return TCL_ERROR;
 }
/*-----------------------------------------------------*/
/* Get the call parameters                             */
/*-----------------------------------------------------*/
  if (Tcl_GetInt(interp,argv[1],&which_chip) != TCL_OK) {
    interp->result = "bad chip number";
    return TCL_ERROR;
  }
  if (Tcl_GetInt(interp,argv[2],&check_registers) != TCL_OK) {
    interp->result = "bad check registers flags";
    return TCL_ERROR;
  }
  if (Tcl_GetInt(interp,argv[3],&check_masks) != TCL_OK) {
    interp->result = "bad check masks flags";
    return TCL_ERROR;
  }
  if (Tcl_GetInt(interp,argv[4],&max_error) != TCL_OK) {
    interp->result = "bad maximum error count";
    return TCL_ERROR;
  }
  if (Tcl_GetInt(interp,argv[5],&ignore_last_mask) != TCL_OK) {
    interp->result = "bad ignore_last_mask";
    return TCL_ERROR;
  }

/*---------------------------------------------------------*/
/* make sure that the text widget exists, if not make it   */
/*---------------------------------------------------------*/
  dummy = Tcl_Eval(interp,cmd1);
/*---------------------------------------------------------*/
/* Display begin mesage                                    */
/*---------------------------------------------------------*/
  dummy = Tcl_Eval(interp,cmd3);
  dummy = Tcl_Eval(interp,cmd0);
  dummy = Tcl_Eval(interp,cmd11);
/*---------------------------------------------------------*/
/* unpack some variables that we may need                  */
/*---------------------------------------------------------*/
  Cal_unpack_variables(interp);
/*---------------------------------------------------------*/
/* setup the DAQ card at the beginning                     */
/*---------------------------------------------------------*/
  dummy = Tcl_Eval(interp,cmd4);
  dummy = Tcl_Eval(interp,cmd0);
  dummy = Tcl_Eval(interp,cmd11);
  command_result = cal_setup_pm(interp);
  if (strcmp(command_result,"OK") != 0) {
     sprintf(event_message,"%s %s \n\"",in,command_result);
     dummy = Tcl_Eval(interp,event_message);
     dummy = Tcl_Eval(interp,cmd2);
     dummy = Tcl_Eval(interp,cmd0);
     dummy = Tcl_Eval(interp,cmd11);
     interp->result = "Error setting up the DAQ card";
     return TCL_OK;
   }
/*--------------------------------------------------------------*/
/* write control registers to all chips on the side under test  */
/* -1   : test all chips                                        */
/* -2   : test only the chips on the top side                   */
/* -3   : test all  the chips on the bottom side                */
/* else : test only one chip                                    */
/*--------------------------------------------------------------*/
   if (which_chip == -1) {
     imin = 0;
     imax = number_of_chips;
   } else if (which_chip == -2) {
     imin = 0;
     imax = number_of_chips_top;
   } else if (which_chip == -3) {
     imin = number_of_chips_top;
     imax = number_of_chips;
   } else if (which_chip < number_of_chips_top) {
     imin = 0;
     imax = number_of_chips_top;
   } else {
     imin = number_of_chips_top;
     imax = number_of_chips;
   }
   for (i=imin; i<imax; i++) {
	build_creg(i);
        rc = cal_write_control_registers(i);
        if (rc != VME_OK) {
	   sprintf(event_message,
	      "%s error setting up control registers chip %d\n\"",in,i);
	   dummy = Tcl_Eval(interp,event_message);
	   dummy = Tcl_Eval(interp,cmd2);
	   dummy = Tcl_Eval(interp,cmd0);
	   dummy = Tcl_Eval(interp,cmd11);
	   interp->result = "error setting up the control registers";
	   return TCL_OK;
	 }
   }
/*---------------------------------------------------------*/
/* Loop over chips for tests                               */
/*---------------------------------------------------------*/
  for (ichip=0; ichip<number_of_chips; ichip++) {

/*- skip chip if it is supposed to be ignored   */
    if (ignore[ichip] == 1) {
      sprintf(event_message,"%s Chip %d ignored.... \n\"",in,ichip);
      dummy = Tcl_Eval(interp,event_message);
      dummy = Tcl_Eval(interp,cmd0);
      dummy = Tcl_Eval(interp,cmd11);
      continue;
    }

    found_data[ichip] = 0;
    if (which_chip == -2 && ichip >= number_of_chips_top) goto next_chip;
    if (which_chip == -3 && ichip <  number_of_chips_top) goto next_chip;
    if (which_chip >=  0 && ichip != which_chip)          goto next_chip;
/*    if (which_chip != -1 && which_chip != ichip) goto next_chip;  */
    sprintf(event_message,"%s Testing chip %d.... \n\"",in,ichip);
    dummy = Tcl_Eval(interp,event_message);
    dummy = Tcl_Eval(interp,cmd0);
    dummy = Tcl_Eval(interp,cmd11);

/*-------------------------------------------------------------*/
/* the channel on which to try to read the data the first time */
/*-------------------------------------------------------------*/
/*    if (number_of_sides == 1) {                      */
/*       try_first = 0;                                */
/*     } else if (ichip < number_of_chips_top) {       */
/*       try_first = 0;                                */
/*     } else {                                        */
/*       try_first = 2;                                */
/*     }                                               */

    cal_get_chip_address(ichip,&chip_address,&pm_chan);
    if (pm_chan == 1) {
      try_first=0;
    } else if (pm_chan == 2) {
      try_first=1;
    } else if (pm_chan == 4) {
      try_first=2;
    } else if (pm_chan == 8) {
      try_first=3;
    }
    

/*----------------------------------                           */
/*-- two tests : walking zeros, and walking ones               */
/*-- bit   0 -->  31 : control registers                       */
/*-- bit  32 --> 159 : channel masks                           */
/*-- bit 160 --> 287 : calibration mask                        */
/*----------------------------------                           */
    for (itest=0; itest<2; itest++) {
     for (ibit=0; ibit<288; ibit++) {

     if (ibit < 32 && check_registers==0) goto next_bit;
     if (ibit > 31 && check_masks==0)     goto next_bit;
/*----------------------------------                          */
/*-- setup control registers                                  */
/*----------------------------------                          */
       if (ibit < 32) {
	 cal_creg[ichip] = 0xffffffff;
	 if (itest == 0) cal_creg[ichip] = 0;
	 itemp = 1;
	 if (ibit != 0) itemp = itemp << ibit;
	 cal_creg[ichip] = cal_creg[ichip] ^ itemp;
       }
/*----------------------------------                          */
/*-- setup masks                                              */
/*----------------------------------                          */
     if (ibit > 31) {
       imask = 0xffff;
       if (itest == 0) imask = 0;
       for (i=0; i<8; i++) {
	 calib_mask[ichip][i]   = imask;
	 channel_mask[ichip][i] = imask;
       }
       jbit = ibit - 160;
       if (ibit < 160) jbit = ibit - 32;
       i     = jbit/16;
       jbit  = jbit - 16*i;	
       itemp = 1;
       if (jbit !=0) itemp = itemp << jbit;
       if (ibit < 160) {
	 channel_mask[ichip][i] = channel_mask[ichip][i] ^ itemp;
       } else {
	 calib_mask[ichip][i]   = calib_mask[ichip][i] ^ itemp;
       }
     }
/*----------------------------------                          */
/*-- write the control registers                              */
/*----------------------------------                          */
      if (ibit < 32) {
/*-- never have SetClockB and ClockThrough set                */ 
/*-- fix bug 9 jun 98                                         */
	if (Set_Clock_B[ichip] == 0) {
	  cal_creg[ichip] = cal_creg[ichip] & 0xbdffffff;
	} else {
/* 	  cal_creg[ichip] = cal_creg[ichip] & 0xfdffffff;  bug 9 jun 98 */
	  cal_creg[ichip] = cal_creg[ichip] | 0xc0000000;
	  cal_creg[ichip] = cal_creg[ichip] & 0xfdffffff;
	}
/*-- never toggle readout direction or respond to read        */
        if (respond[ichip] == 1) {
	   cal_creg[ichip] = cal_creg[ichip] | 0x1000000;
	} else {
	   cal_creg[ichip] = cal_creg[ichip] & 0xfeffffff;
	}
        if (readout_direction[ichip] == 1) {
	   cal_creg[ichip] = cal_creg[ichip] | 0x800000;
	} else {
	   cal_creg[ichip] = cal_creg[ichip] & 0xff7fffff;
	}
        creg_store = cal_creg[ichip];
/*        printf("read dir %d ibit = %d   register = 0x%x \n",
             readout_direction[ichip],ibit,creg_store);   */
        rc = cal_write_control_registers(ichip);
        if (rc != VME_OK) {
         sprintf(event_message,"%s Error sending WRITECTRL command \n\"",in);
         dummy = Tcl_Eval(interp,event_message);
         dummy = Tcl_Eval(interp,cmd2);
         dummy = Tcl_Eval(interp,cmd0);
         dummy = Tcl_Eval(interp,cmd11);
         interp->result = "Error sending WRITECTRL command";
         return TCL_OK;
        } 
      }
/*----------------------------------                          */
/*-- write the masks                                          */
/*----------------------------------                          */
       if (ibit > 31) {
	 for (i=0; i<8; i++) {
	   calib_mask_store[i]   = calib_mask[ichip][i];
	   channel_mask_store[i] = channel_mask[ichip][i];
	 }
	 rc = cal_write_masks(ichip);
         if (rc != VME_OK) {
         sprintf(event_message,"%s Error sending WRITEMASK command \n\"",in);
         dummy = Tcl_Eval(interp,event_message);
         dummy = Tcl_Eval(interp,cmd2);
         dummy = Tcl_Eval(interp,cmd0);
         dummy = Tcl_Eval(interp,cmd11);
         interp->result = "Error sending WRITEMASK command";
         return TCL_OK;
         } 
      }
/*----------------------------------                          */
/*-- read them back                                           */
/*----------------------------------                          */
      command_result = cal_read_reg_masks(ichip,try_first,interp,&read_chan);
      if (strcmp(command_result,"OK") != 0) {
        sprintf(event_message,"%s %s \n\"",in,command_result);
        dummy = Tcl_Eval(interp,event_message);
        dummy = Tcl_Eval(interp,cmd2);
        dummy = Tcl_Eval(interp,cmd0);
        dummy = Tcl_Eval(interp,cmd11);
	interp->result = "Error reading masks";
        return TCL_OK;
      }
/*------------------------------------------------------------------*/
/* The next time we try to read the data, we will look at the       */
/* channel where we found the data first.  This is more efficient.  */
/*------------------------------------------------------------------*/
      if (try_first != read_chan) try_first=read_chan;
/*------------------------------------------------------------------*/
/* Let the user know where we found the data.                       */
/* But do it only once per chip                                     */
/*------------------------------------------------------------------*/
      if (found_data[ichip] == 0) {
	sprintf(event_message,
		"%s Found data on PM channel %d for chip %d.... \n\"",
		in,read_chan,ichip);
	dummy = Tcl_Eval(interp,event_message);
	dummy = Tcl_Eval(interp,cmd0);
	dummy = Tcl_Eval(interp,cmd11);
        found_data[ichip] = 1;
      }
/*----------------------------------                          */
/*-- zero the read-only bit                                   */
/*----------------------------------                          */
      if (ibit < 32) {
         creg_store        = creg_store      & 0x7fffffff;
         cal_creg[ichip]   = cal_creg[ichip] & 0x7fffffff;
       }
/*----------------------------------                         */
/*-- compare the control registers                           */
/*----------------------------------                         */
      if (creg_store != cal_creg[ichip] && ibit < 32) {
	error_count++;
	sprintf(event_message,"%s Error: chip %d, CR wrote 0x%x read 0x%x \n\""
               ,in,ichip,creg_store,cal_creg[ichip]);
        dummy = Tcl_Eval(interp,event_message);
        dummy = Tcl_Eval(interp,cmd0);
        dummy = Tcl_Eval(interp,cmd11);
        if (error_count >= max_error) {
           dummy = Tcl_Eval(interp,cmd5);
           dummy = Tcl_Eval(interp,cmd2);
           dummy = Tcl_Eval(interp,cmd0);
           dummy = Tcl_Eval(interp,cmd11);
	   interp->result = "Too many errors";
           return TCL_OK;
        }
      }
/*----------------------------------                         */
/*-- compare the masks                                       */
/*   Special code to ignore last bit                         */
/*----------------------------------                         */
       if (ibit > 31) {
	 iflag = 0;
	 if (ignore_last_mask != 1) {
	   for (i=0; i<8; i++) {
	     if (calib_mask[ichip][i]   != calib_mask_store[i] || 
		 channel_mask[ichip][i] != channel_mask_store[i]) iflag=1;
	   }
	 } else {
	   for (i=0; i<7; i++) {
	     if (calib_mask[ichip][i]   != calib_mask_store[i] || 
		 channel_mask[ichip][i] != channel_mask_store[i]) iflag=1;
	   }
	   if (calib_mask[ichip][7] != calib_mask_store[7]) iflag==1;
           itemp  = channel_mask[ichip][7] & 0x7fff;
           itemp2 = channel_mask_store[7]  & 0x7fff;
	   if (itemp != itemp2) iflag=1;
	 }
	 
	 if (iflag == 1) {
	    error_count++;
	    sprintf(event_message,"%s Error: chip %d \n\"",in,ichip);
	    dummy = Tcl_Eval(interp,event_message);

	    sprintf(event_message,"%s Calibration mask : \n\"",in);
            dummy = Tcl_Eval(interp,event_message);
	    for (i=0; i<8; i++) {
	      i1 = 16*i;
	      i2 = i1 + 15;
	      sprintf(event_message,"%s Bits %d to %d: wrote %#04x, read %#04x \n\"",in,i1,i2,calib_mask_store[i],calib_mask[ichip][i]);
	      dummy = Tcl_Eval(interp,event_message);
	    }

	    sprintf(event_message,"%s Channel mask : \n\"",in);
            dummy = Tcl_Eval(interp,event_message);
	    for (i=0; i<8; i++) {
	      i1 = 16*i;
	      i2 = i1 + 15;
	      sprintf(event_message,"%s Bits %d to %d: wrote %#04x, read %#04x \n\"",in,i1,i2,channel_mask_store[i],channel_mask[ichip][i]);
	      dummy = Tcl_Eval(interp,event_message);
	    }


	    sprintf(event_message,"%s ************************ \n\"",in);
	    dummy = Tcl_Eval(interp,event_message);
	    dummy = Tcl_Eval(interp,cmd0);
            dummy = Tcl_Eval(interp,cmd11);
            if (error_count >= max_error) {
               dummy = Tcl_Eval(interp,cmd5);
               dummy = Tcl_Eval(interp,cmd2);
               dummy = Tcl_Eval(interp,cmd0);
               dummy = Tcl_Eval(interp,cmd11);
	       interp->result = "Too many errors";
               return TCL_OK;
            }
	  }
       }
/*----------------------------------                       */
/* give the user some feedback                             */
/*----------------------------------                       */
       if (ibit == 31 && itest==0) {
	 sprintf(event_message,"%s Completed walking ones CR test chip %d \n\"",in,ichip);
       } else if (ibit == 31) {
	 sprintf(event_message,"%s Completed walking zeros CR test chip %d \n\"",in,ichip);
       }
       if (ibit == 287 && itest==0) {
	 sprintf(event_message,"%s Completed walking ones MSK test chip %d \n\"",in,ichip);
       } else if (ibit == 287) {
	 sprintf(event_message,"%s Completed walking zeros MSK test chip %d \n\"",in,ichip);
       }
       if (ibit == 31 || ibit == 287) {
	 dummy = Tcl_Eval(interp,event_message);
	 dummy = Tcl_Eval(interp,cmd0);
         dummy = Tcl_Eval(interp,cmd11);
       }

     next_bit: {}
     }
   }
  next_chip: {}
  }
/*----------------------------------------------------------------*/
/* Done                                                           */
/*----------------------------------------------------------------*/
  sprintf(event_message,
"%s \n \n Test completed. Number of detected errors = %d \n\"",in,error_count);
  dummy = Tcl_Eval(interp,event_message);
  dummy = Tcl_Eval(interp,cmd0);
  dummy = Tcl_Eval(interp,cmd11);
  interp->result = "OK";
  return TCL_OK;
}





/*********************************************************************/
/* read write control registers to a given chip                      */
/*********************************************************************/
static int rw_atom_registersCmd(ClientData clientData, Tcl_Interp* interp, 
			int argc, char* argv[]) {

int read_chan;
int i, dummy, flag, which_chip, try_first;
char event_message[200];
static char *command_result;
vme_rc rc;
char *C_creg_write;
char *C_chamsk_write[4];
char *C_calmsk_write[4];
char index[100];
int creg_write;
int chamsk_write[4];
int calmsk_write[4];
static char var_name[100];
static char var_value[100];
int tcl_flags, itemp;
char *set_result;
/*-----------------------------------------------------*/
/* Make sure call has right number of arguments        */
/*-----------------------------------------------------*/
 if (argc != 3) {
   interp->result = "wrong number of arguments";
   return TCL_ERROR;
 }
/*-----------------------------------------------------*/
/* Get the call parameters                             */
/*-----------------------------------------------------*/
  if (Tcl_GetInt(interp,argv[1],&flag) != TCL_OK) {
    interp->result = "bad flag";
    return TCL_ERROR;
  }
  if (flag < 1 || flag > 5) {
    interp->result = "bad flag";
    return TCL_ERROR;
  }
  if (Tcl_GetInt(interp,argv[2],&which_chip) != TCL_OK) {
    interp->result = "bad chip number";
    return TCL_ERROR;
  }
/*---------------------------------------------------------*/
/* unpack some variables that we may need                  */
/*---------------------------------------------------------*/
  Cal_unpack_variables(interp);
/*---------------------------------------------------------*/
/* setup the DAQ card at the beginning                     */
/*---------------------------------------------------------*/
  command_result = cal_setup_pm(interp);
  if (strcmp(command_result,"OK") != 0) {
     interp->result = command_result;
     return TCL_ERROR;
   }   
/*--------------------------------------------------------*/
/* Remember :::::::                                       */
/* flag = 1 : write control registers                     */
/* flag = 2 : write masks                                 */
/* flag = 3 : read everything                             */
/* flag = 4 : write default control registers all chips   */
/* flag = 5 : write default masks all chips               */
/*--------------------------------------------------------*/
/*--------------------------------------------------------*/
/* unpack needed variables                                */
/*--------------------------------------------------------*/
  if (flag == 1) {
     C_creg_write = Tcl_GetVar(interp,"creg_write",0);
     dummy = Tcl_GetInt(interp, C_creg_write, &creg_write);
/*     printf("C_creg_write = %s \n",C_creg_write);                 */
/*     printf("creg_write = 0%x \n",creg_write);                    */
  }
  if (flag == 2) {
    for (i=0; i<=3; i++) {
      sprintf(index,"%d",i);
      C_chamsk_write[i] = Tcl_GetVar2(interp,"chamsk_write",index,0);
      C_calmsk_write[i] = Tcl_GetVar2(interp,"calmsk_write",index,0);
      dummy = Tcl_GetInt(interp, C_chamsk_write[i] ,&chamsk_write[i]);
      dummy = Tcl_GetInt(interp, C_calmsk_write[i] ,&calmsk_write[i]);      
/*      printf("C_chamsk_write %d = %s \n",i,C_chamsk_write[i]);    */
/*      printf("C_calmsk_write %d = %s \n",i,C_calmsk_write[i]);    */
/*      printf("chamsk_write %d = 0x%x \n",i,chamsk_write[i]);      */
/*      printf("calmsk_write %d = 0x%x \n",i,calmsk_write[i]);      */
    }
  }
/*-------------------------------------------------------------*/
/* write control registers                                     */
/*-------------------------------------------------------------*/
  if (flag == 1) {
     cal_creg[which_chip] = creg_write;
     rc = cal_write_control_registers(which_chip);
     if (rc |= VME_OK) {
       interp->result = "Error sending WRITECTRL command";
       return TCL_ERROR;
     }
   }  
/*-------------------------------------------------------------------*/
/* write masks                                                       */
/*-------------------------------------------------------------------*/
   if (flag == 2) {
       for (i=0; i<4; i++) {
	 calib_mask[which_chip][2*i]   = calmsk_write[i] & 0xffff;
	 calib_mask[which_chip][2*i+1] = calmsk_write[i] & 0xffff0000;
         calib_mask[which_chip][2*i+1] =
	   calib_mask[which_chip][2*i+1] >> 16;

	 channel_mask[which_chip][2*i]   = chamsk_write[i] & 0xffff;
	 channel_mask[which_chip][2*i+1] = chamsk_write[i] & 0xffff0000;
         channel_mask[which_chip][2*i+1] =
	   channel_mask[which_chip][2*i+1] >> 16;

       }

       rc = cal_write_masks(which_chip);
       if (rc != VME_OK) {
            interp->result = "Error sending WRITEMASK command";
	    return TCL_ERROR;
       }
     }    
/*-------------------------------------------------------------------*/
/* read all registers                                                */
/*-------------------------------------------------------------------*/
    if (flag == 3) {

      if (number_of_sides == 1) {
	try_first = 0;
      } else if (which_chip < number_of_chips_top) {
	try_first = 0;
      } else {
	try_first = 2;
      }
      command_result = cal_read_reg_masks(
				    which_chip,try_first,interp,&read_chan);
      if (strcmp(command_result,"OK") != 0) {
	 interp->result = command_result;
	 return TCL_ERROR;
       }      

      tcl_flags = TCL_GLOBAL_ONLY;
/*- set the channel read from variable  */
      strcpy(var_name,"from_channel");
      sprintf(var_value," %d",read_chan);
      set_result = Tcl_SetVar(interp, var_name, var_value, tcl_flags);
      if (set_result == NULL) {
	interp->result = "TCL error trying to set variable from_channel";
        return TCL_ERROR;
      }

/*- set the control register variable  */
      strcpy(var_name,"creg_read");
      sprintf(var_value,"0x%x",cal_creg[which_chip]);
      set_result = Tcl_SetVar(interp, var_name, var_value, tcl_flags);
      if (set_result == NULL) {
	interp->result = "TCL error trying to set variable creg_read";
        return TCL_ERROR;
      }
/*- now set the masks     */
      for (i=0; i<4; i++) {
	sprintf(var_name,"chamsk_read(%d)",i);
        itemp = channel_mask[which_chip][2*i];
	itemp = itemp | (channel_mask[which_chip][2*i+1] << 16);
	sprintf(var_value,"0x%x",itemp);
	set_result = Tcl_SetVar(interp, var_name, var_value, tcl_flags);
	if (set_result == NULL) {
	   sprintf(interp->result,"TCL error trying to set variable %s",var_name);
	   return TCL_ERROR;
	 }
	sprintf(var_name,"calmsk_read(%d)",i);
        itemp = calib_mask[which_chip][2*i];
	itemp = itemp | (calib_mask[which_chip][2*i+1] << 16);
	sprintf(var_value,"0x%x",itemp);
	set_result = Tcl_SetVar(interp, var_name, var_value, tcl_flags);
	if (set_result == NULL) {
	   sprintf(interp->result,"TCL error trying to set variable %s",var_name);
	   return TCL_ERROR;
	 }
      }
    }
/*------------------------------------------------------------*/
/* write control registers to all chips                       */
/*------------------------------------------------------------*/
   if (flag == 4) {
      for (i=0; i<number_of_chips; i++) {
	build_creg(i);
        rc = cal_write_control_registers(i);
        if (rc != VME_OK) {
	   sprintf(interp->result,"Error trying to write CR chip %d",i);
	   return TCL_ERROR;
	 }
      }
    }
/*------------------------------------------------------------*/
/* write masks to all chips                                   */
/*------------------------------------------------------------*/
   if (flag == 5) {
     for (i=0; i<number_of_chips; i++) {
	rc = cal_write_masks(i);
	if (rc != VME_OK) {
	  sprintf(interp->result,"Error trying to write MSK chip %d",i);
	  return TCL_ERROR;
	}
     }
   }
/*------------------------------------------------------------*/
/* done                                                       */
/*------------------------------------------------------------*/
    return TCL_OK;
}




/*********************************************************************/
/* definition of the command "loop_test"                              */
/*********************************************************************/
void loop_test_define(Tcl_Interp* interp) {

  Tcl_CreateCommand(interp, "loop_test", loop_testCmd, (ClientData) NULL,
                    (Tcl_CmdDeleteProc *) NULL);
}


/***********************************************************************/
/* definition of the command "rw_atom_registers"                       */    
/***********************************************************************/
void rw_atom_register_define(Tcl_Interp* interp) {
  Tcl_CreateCommand(interp, "rw_atom_registers", rw_atom_registersCmd, 
		    (ClientData) NULL,
                    (Tcl_CmdDeleteProc *) NULL);
}











