#include "cal_svtdef.h"
#include <math.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "error.h"
#include "reglist.h"
#include "tk.h"
#include "opcodes.h"
#include "cal_global.h"
#include "cal_functions.h"
#include "site.h"
#include <histoscope.h>
#include "cal_format_check.h"


#define NTESTS 5
#define MAX_TOT_SETTINGS 10

/* for the buffer tests, these are :                                      */
/*  - minimum number of channels firing for the buffer to be considered   */
/*    a buffer with the data (ideally, 8)                                 */
/*  - maximum number of channels firing for the buffer to be considered   */
/*    a buffer with no data (ideally 0)                                   */
#define MINCHFIRED 3
#define MAXCHFIRED 2



char* Cal_unpack_array(Tcl_Interp*, char*, char*);
void short_display_message(Tcl_Interp*, char*);
void misc_set_all_masks(int);
int misc_do_channel_mask_test(Tcl_Interp*, int, int);
int misc_do_buffer_test(Tcl_Interp*, int, int);
int misc_do_tstamp_or_tot_test(Tcl_Interp*, int, int, int, int, int*, int*);
char* misc_switch_time(Tcl_Interp*, int);

static int misc_testsCmd(ClientData clientData, Tcl_Interp* interp, 
			int argc, char* argv[]) {

int input_argument, status;
int this_channel, file_open;
int dummy, i, j, k, ichip, ibit, ichan, pm_ch_mask, i1, i2, itest;
int chip_address, pm_chan, event_number;
int min_chip, max_chip, i87;
char message[200];
char index[100];
char event_message[200];
static char* command_result;
static char command[200];
vme_rc rc;
static char*  eventseq_result;
static char cmd10[]  = "vme_clear daq_0(led)";
static char cmd11[]  = "vme_set daq_0(led)";
char* C_misc_nrep[NTESTS];
char* C_misc_thr_dac[NTESTS];
char* C_misc_cal_dac_top[NTESTS][MAX_TOT_SETTINGS];
char* C_misc_cal_dac_bottom[NTESTS][MAX_TOT_SETTINGS];
char* C_misc_ntot;
int misc_nrep[NTESTS];
int misc_thr_dac[NTESTS];
int misc_cal_dac_top[NTESTS][MAX_TOT_SETTINGS];
int misc_cal_dac_bottom[NTESTS][MAX_TOT_SETTINGS];
char* test_name[NTESTS];
int misc_ntot, i2max;



static char cmd1[]  = "make_pltw_widget";
static char cmd3[]  = ".pltw.text insert end \" \n \n ************ Begin testing *********** \n\"";
static char cmd4[]  = ".pltw.text insert end \" Now setting up personality module and hybrid.....\n\"";
static char abort[] = "Aborting.....";


 status = -1;
/*-----------------------------------------------------*/
/* test names                                          */
/*-----------------------------------------------------*/
 test_name[0] = "Channel Mask Test";
 test_name[1] = "On Chip Buffer Test";
 test_name[2] = "TOT Test";
 test_name[3] = "Time Stamp Test";
 test_name[4] = "Jitter Time Test";
/*-----------------------------------------------------*/
/* Make sure call has right number of arguments        */
/*-----------------------------------------------------*/
 if (argc != 2) {
   interp->result = "wrong number of arguments";
   return TCL_ERROR;
 }
/*--------------------------------------------------------------------*/
/* get the bit mask of tests to be performed from the input argument  */
/*--------------------------------------------------------------------*/
  if (Tcl_GetInt(interp,argv[1],&input_argument) != TCL_OK) {
    interp->result = "bad input argument";
    return TCL_ERROR;
  }
/*---------------------------------------------------------*/
/* make sure that the text widget exists, if not make it   */
/*---------------------------------------------------------*/
  dummy = Tcl_Eval(interp,cmd1);
/*--------------------------------------------------------------*/
/* unpack variables that we need                                */
/* Remember, test for shorts is special (sigh)                  */
/*--------------------------------------------------------------*/
  Cal_unpack_variables(interp);
  C_misc_ntot      = Tcl_GetVar(interp,"misc_ntot",0);
  dummy = Tcl_GetInt(interp, C_misc_ntot, &misc_ntot);
  if (misc_ntot > MAX_TOT_SETTINGS) {
     sprintf(event_message,"No. of settings for TOT tests = %d", misc_ntot);
     short_display_message(interp, event_message);
     sprintf(event_message,"....should be at most = MISC_TOT_SETTINGS");
     short_display_message(interp, event_message);
     short_display_message(interp, abort);
     goto end;
   }

  i2max = 1;
  if (misc_ntot > i2max) i2max=misc_ntot;
  for (i1=0; i1<NTESTS; i1++) {
    sprintf(index,"(%d)",i1);
    C_misc_nrep[i1]    = Cal_unpack_array(interp,"misc_nrep",index);
    C_misc_thr_dac[i1] = Cal_unpack_array(interp,"misc_thr_dac",index);
    dummy   = Tcl_GetInt(interp, C_misc_nrep[i1], &misc_nrep[i1]);
    dummy   = Tcl_GetInt(interp, C_misc_thr_dac[i1], &misc_thr_dac[i1]);

    for (i2=0; i2<i2max; i2++) {
      if (i2 == 0 || strcmp(test_name[i1],"TOT Test") == 0) {
	sprintf(index,"(%d,%d)",i1,i2);
	C_misc_cal_dac_top[i1][i2] = Cal_unpack_array(interp,
					     "misc_cal_dac_top",index);
	C_misc_cal_dac_bottom[i1][i2] = Cal_unpack_array(interp,
					     "misc_cal_dac_bottom",index);
	dummy   = Tcl_GetInt(interp, C_misc_cal_dac_top[i1][i2],
			               &misc_cal_dac_top[i1][i2]);
	dummy   = Tcl_GetInt(interp, C_misc_cal_dac_bottom[i1][i2],
			               &misc_cal_dac_bottom[i1][i2]);
      }
    }
  }
/*----------------------------------------------------------*/
/* loop over tests                                          */
/*----------------------------------------------------------*/
      for (itest=0; itest<NTESTS; itest++) {
/*----------------------------------------------------------*/
/* is this test enabled ?                                   */
/*----------------------------------------------------------*/
	dummy = 1;
	if (itest != 0) dummy = dummy << itest;
	if ( (dummy & input_argument) == 0) goto next_test;
/*----------------------------------------------------------*/
/* This test is not implemented yet                         */
/*----------------------------------------------------------*/
        if (itest == 4) {
	  sprintf(event_message,"^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^");
	  short_display_message(interp, event_message);
	  sprintf(event_message,"%s NOT IMPLEMENTED YET",test_name[itest]);
	  short_display_message(interp, event_message);
	  sprintf(event_message,"^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^\n");
	  short_display_message(interp, event_message);
	  status = 0;
          goto next_test;
	}
/*----------------------------------------------------------*/
/* Let the user know that we are performing this test now   */
/*----------------------------------------------------------*/
	sprintf(event_message,"^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^");
	short_display_message(interp, event_message);
	sprintf(event_message,"PERFORMING %s\n",test_name[itest]);
	short_display_message(interp, event_message);
	sprintf(event_message,"THE TEST WILL BE REPEATED %d TIMES",
		misc_nrep[itest]);
	short_display_message(interp, event_message);
	sprintf(event_message,"^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^\n");
	short_display_message(interp, event_message);
/*--------------------------------------*/
/* make sure that the LED bit is off    */
/*--------------------------------------*/
	dummy = Tcl_GlobalEval(interp,cmd10);
	if (dummy != TCL_OK) {
	  sprintf(event_message,"Error turning LED bit off");
	  short_display_message(interp, event_message);
	  short_display_message(interp, abort);
	  goto end;
	}
/*---------------------------------------------------------*/
/* setup the DAQ card at the beginning                     */
/*---------------------------------------------------------*/
	command_result = cal_setup_pm(interp);
	if (strcmp(command_result,"OK") != 0) {
	  short_display_message(interp, command_result);
	  short_display_message(interp, abort);
	  goto end;
	}
/*-----------------------------------------------------------*/
/* Setup the hybrid at the beginning                         */
/* NB: This does a bunch of "good" things, but it also       */
/*     writes control registers and masks.  That's OK, we'll */
/*     then rewrite these with what we REALLY want           */
/*-----------------------------------------------------------*/
	command_result = cal_setup_hybrid();
	if (strcmp(command_result,"OK") != 0) {
	  short_display_message(interp, command_result);
	  short_display_message(interp, abort);
	  goto end;
	}
/*--------------------------------------------------------*/
/* figure out what channels we expect to receive data on  */
/* Careful about ignored chips............                */
/*--------------------------------------------------------*/
	pm_ch_mask = 0;
	for (i87=0; i87<number_of_chips; i87++) {
          cal_get_chip_address(i87,&chip_address,&pm_chan);
          if (ignore[i87] == 0) pm_ch_mask = pm_ch_mask | pm_chan;
	}
/*----------------------------------------------------------*/
/* Now actually write the "correct" control registers       */
/*----------------------------------------------------------*/
	for (ichip=0; ichip<number_of_chips; ichip++) {
	  cal_thresh_dac[ichip] = misc_thr_dac[itest];
	  if (number_of_sides == 1 || ichip < (number_of_chips/2)) {
	    cal_cal_dac[ichip]    = misc_cal_dac_top[itest][0];
	  } else {
	    cal_cal_dac[ichip]    = misc_cal_dac_bottom[itest][0];
	  }
	  build_creg(ichip);
	  creg_into_data(ichip);
	  rc = cal_write_control_registers(ichip);
	  if (rc != VME_OK) {
	    sprintf(event_message,"Error sending WRITECTRL command");
	    short_display_message(interp, event_message);
	    short_display_message(interp, abort);
	    goto end;
	  } 
	}
/*------------------------------------------------------------------------*/
/* Open the output file                                                   */
/*------------------------------------------------------------------------*/
	sprintf(command,"misc_file_handler %d 0",itest);
	dummy = Tcl_GlobalEval(interp,command);
	if (strcmp(interp->result,"OK") != 0) {
	  sprintf(event_message,"%s",interp->result);
	  short_display_message(interp,event_message);
	  short_display_message(interp,abort);
	  goto end;
	}
/*------------------------------------------------------------------------*/
/* now perform the actual tests                                           */
/*------------------------------------------------------------------------*/
	status = -1;
	if (itest == 0) status = 
	                misc_do_channel_mask_test(interp, misc_nrep[0], 
			pm_ch_mask);
	if (itest == 1) status = 
	                misc_do_buffer_test(interp, misc_nrep[1],
			pm_ch_mask);
	if (itest == 2 || itest == 3) status =
	                misc_do_tstamp_or_tot_test(interp, misc_nrep[itest],
			pm_ch_mask, itest, misc_ntot, 
			&misc_cal_dac_top[0][0], 
			&misc_cal_dac_bottom[0][0]);
/*------------------------------------------------------------------------*/
/* if the return flag from this test was 0 (succesful), write the         */
/* parameters to the output file                                          */
/* If it fails, set status=-1 and continue because we want to             */
/* eventually get to the piece of code where the file is closed           */
/*------------------------------------------------------------------------*/
	if (status == 0) {
	  sprintf(command,"misc_file_handler %d 2",itest);
	  dummy = Tcl_GlobalEval(interp,command);
	  if (strcmp(interp->result,"OK") != 0) {
	    sprintf(event_message,"%s",interp->result);
	    short_display_message(interp,event_message);
	    short_display_message(interp,abort);
	    status = -1;
	  }
	}
/*------------------------------------------------------------------------*/
/* if the return flag from this test was 0 (succesful), write the result  */
/* of the tests to the output file                                        */
/* If it fails, set status=-1 and continue because we want to             */
/* eventually get to the piece of code where the file is closed           */
/*------------------------------------------------------------------------*/
	if (status == 0) {
	  sprintf(command,"misc_file_handler %d 5",itest);
	  dummy = Tcl_GlobalEval(interp,command);
	  if (strcmp(interp->result,"OK") != 0) {
	    sprintf(event_message,"%s",interp->result);
	    short_display_message(interp,event_message);
	    short_display_message(interp,abort);
	    status = -1;
	  }
	}
/*------------------------------------------------------------------------*/
/* Close the output file                                                  */
/*------------------------------------------------------------------------*/
	sprintf(command,"misc_file_handler %d 1",itest);
	dummy = Tcl_GlobalEval(interp,command);
	if (strcmp(interp->result,"OK") != 0) {
	  sprintf(event_message,"%s",interp->result);
	  short_display_message(interp,event_message);
	  short_display_message(interp,abort);
	  goto end;
	}
/*------------------------------------------------------------------------*/
/* if the return flag from this test was !=0, quit                        */
/*------------------------------------------------------------------------*/
	if (status != 0) goto end;
/*---------------------------------------------------*/
/* close loop over tests                             */
/*---------------------------------------------------*/
next_test: {} 
      }
/*------------------------------------------------------------*/
/* done                                                       */
/*------------------------------------------------------------*/
end: {}
    if (status == 0) {
      interp->result = "OK";
    } else {
      interp->result = "ERROR";
    }
    return TCL_OK;
}



/*********************************************************************/
/*********************************************************************/
/*********************************************************************/
/*********************************************************************/
/* TimeStamp/TOT test.  Returns zero on succesful completion         */
/*********************************************************************/
/*********************************************************************/
/*********************************************************************/
/*********************************************************************/
int misc_do_tstamp_or_tot_test
      (Tcl_Interp* interp, int nrep, int pm_ch_mask, int test, int misc_ntot,
       int* dac_top, int* dac_bottom) {

static char *sanity_check;
int sanity_flag;
int ichip, i, j, k, data_len, irep, ishift, i87, itemp, this_chip, ihit;
int nhits, chnum, tot, tstamp, this_channel, bad;
vme_rc rc;
char index[100];
static char command[200];
char event_message[200];
static char* command_result;
static char*  eventseq_result;
static char abort[] = "Aborting.....";
int event_number, dummy, isetting, igroup, ichan;
char* C_misc_tstamp_shift;
char* C_misc_tstamp_toll;
char* C_misc_tstamp_dead;
char* C_misc_ntot;
char* C_misc_tot_dead;
char* C_misc_tot_toll[MAX_TOT_SETTINGS];
char* C_misc_tot[MAX_TOT_SETTINGS];
char* C_misc_tot_shaping_time[MAX_TOT_SETTINGS];
char* C_misc_tot_skip[MAX_TOT_SETTINGS];
char* C_misc_tot_test_it[MAX_TOT_SETTINGS];
int misc_tot_shaping_time[MAX_TOT_SETTINGS];
int misc_tot_skip[MAX_TOT_SETTINGS];
int misc_tot_test_it[MAX_TOT_SETTINGS];
double misc_tot_toll[MAX_TOT_SETTINGS];
double misc_tot[MAX_TOT_SETTINGS];
int tot_dac_top[MAX_TOT_SETTINGS];
int tot_dac_bottom[MAX_TOT_SETTINGS];
double misc_tstamp_shift,  misc_tstamp_toll; 
int    misc_tstamp_dead, misc_tot_dead;
int begin_data[MAX_NUM_CHIP], end_data[MAX_NUM_CHIP];
double CumulativeValue[MAX_NUM_CHIP][128][MAX_TOT_SETTINGS];
int    NumberOfHits[MAX_NUM_CHIP][128][MAX_TOT_SETTINGS];
double av[2],delta,avtot;
int delay_setting[2] = {2, 1};
int nsettings;

/*--------------------------------------------------*/
/* make sure that the calling parameters are OK     */
/*--------------------------------------------------*/
  if (test != 2 && test != 3) {
    sprintf(event_message,
      "Function misc_do_tstamp_or_tot_test called with test = %d",test);
    short_display_message(interp, event_message);
    return -1;
  }
/*-----------------------------------------*/
/* Unpack variables specific to this test  */
/*-----------------------------------------*/
  if (test == 3) {
    C_misc_tstamp_shift = Tcl_GetVar(interp,"misc_tstamp_shift",0);
    C_misc_tstamp_toll  = Tcl_GetVar(interp,"misc_tstamp_toll",0);
    C_misc_tstamp_dead  = Tcl_GetVar(interp,"misc_tstamp_dead",0);
    dummy = Tcl_GetDouble(interp, C_misc_tstamp_shift, &misc_tstamp_shift);
    if (dummy != TCL_OK) {
      sprintf(event_message, "%s", interp->result);
      short_display_message(interp, event_message);
      return -1;
    }
    dummy = Tcl_GetDouble(interp, C_misc_tstamp_toll, &misc_tstamp_toll);
    if (dummy != TCL_OK) {
      sprintf(event_message, "%s", interp->result);
      short_display_message(interp, event_message);
      return -1;
    }
    dummy = Tcl_GetInt(interp, C_misc_tstamp_dead, &misc_tstamp_dead);
    if (dummy != TCL_OK) {
      sprintf(event_message, "%s", interp->result);
      short_display_message(interp, event_message);
      return -1;
    }
  } else {
    C_misc_tot_dead  = Tcl_GetVar(interp,"misc_tot_dead",0);
    dummy = Tcl_GetInt(interp, C_misc_tot_dead, &misc_tot_dead);
    if (dummy != TCL_OK) {
      sprintf(event_message, "%s", interp->result);
      short_display_message(interp, event_message);
      return -1;
    }
    for (i=0; i<misc_ntot; i++) {
      sprintf(index,"(%d)",i);

      C_misc_tot_toll[i] = Cal_unpack_array(interp,"misc_tot_toll",index);
      dummy = Tcl_GetDouble(interp, C_misc_tot_toll[i], &misc_tot_toll[i]);
      if (dummy != TCL_OK) {
	sprintf(event_message, "%s", interp->result);
	short_display_message(interp, event_message);
	return -1;
      }

      C_misc_tot[i] = Cal_unpack_array(interp,"misc_tot",index);
      dummy = Tcl_GetDouble(interp, C_misc_tot[i], &misc_tot[i]);
      if (dummy != TCL_OK) {
	sprintf(event_message, "%s", interp->result);
	short_display_message(interp, event_message);
	return -1;
      }

      C_misc_tot_test_it[i] = Cal_unpack_array(interp,
					       "misc_tot_test_it",index);
      dummy = Tcl_GetInt(interp, C_misc_tot_test_it[i],
			             &misc_tot_test_it[i]);
      if (dummy != TCL_OK) {
	sprintf(event_message, "%s", interp->result);
	short_display_message(interp, event_message);
	return -1;
      }

      C_misc_tot_shaping_time[i] = Cal_unpack_array(interp,
					      "misc_tot_shaping_time",index);
      if        (strcmp(C_misc_tot_shaping_time[i], "100 nsec") == 0) {
	misc_tot_shaping_time[i] = 0;
      } else if (strcmp(C_misc_tot_shaping_time[i], "200 nsec") == 0) {
	misc_tot_shaping_time[i] = 1;
      } else if (strcmp(C_misc_tot_shaping_time[i], "300 nsec") == 0) {
	misc_tot_shaping_time[i] = 2;
      } else if (strcmp(C_misc_tot_shaping_time[i], "400 nsec") == 0) {
	misc_tot_shaping_time[i] = 3;
      } else {
	sprintf(event_message,
		"Illegal shaping time value %s for setting No. %d",
		C_misc_tot_shaping_time[i], i);
	short_display_message(interp, event_message);
	return -1;
      }

      C_misc_tot_skip[i] = Cal_unpack_array(interp,
					      "misc_tot_skip",index);
      if        (strcmp(C_misc_tot_skip[i], "skip 0") == 0) {
	misc_tot_skip[i] = 0;
      } else if (strcmp(C_misc_tot_skip[i], "skip 1") == 0) {
	misc_tot_skip[i] = 1;
      } else if (strcmp(C_misc_tot_skip[i], "skip 2") == 0) {
	misc_tot_skip[i] = 2;
      } else if (strcmp(C_misc_tot_skip[i], "skip 3") == 0) {
	misc_tot_skip[i] = 3;
      } else {
	sprintf(event_message,
		"Illegal skip %s for setting No. %d",
		C_misc_tot_skip[i], i);
	short_display_message(interp, event_message);
	return -1;
      }


      tot_dac_top[i]    = *(dac_top    + 2*MAX_TOT_SETTINGS + i);
      tot_dac_bottom[i] = *(dac_bottom + 2*MAX_TOT_SETTINGS + i);
    }
  }
/*--------------------------------------*/
/* zero our placeholders                */
/*--------------------------------------*/
  for (i=0; i<MAX_NUM_CHIP; i++) {
    for (j=0; j<128; j++) {
      for (k=0; k<MAX_TOT_SETTINGS; k++) {
	CumulativeValue[i][j][k] = 0;
	NumberOfHits[i][j][k] = 0;
      }
    }
  }
/*-------------------------------------------------------------*/
/* Loop over the two time delays or different TOT values       */
/*-------------------------------------------------------------*/
  if (test == 3) {
    nsettings = 2;
  } else {
    nsettings = misc_ntot;
  }
  event_number = 0;
  for (isetting=0; isetting<nsettings; isetting++) {
/*---------------------------------------------------------------------*/
/* if this is the TOT test, we may be skipping some setting            */
/*---------------------------------------------------------------------*/
    if (test == 2 && misc_tot_test_it[isetting] == 0) goto next_setting; 
/*---------------------------------------------------------------------*/
/* If this is is the TOT tests, reset CAL DAC and shaping time values  */
/*---------------------------------------------------------------------*/
    if (test == 2) {
	for (ichip=0; ichip<number_of_chips; ichip++) {
	  if (number_of_sides == 1 || ichip < (number_of_chips/2)) {
	    cal_cal_dac[ichip]    = tot_dac_top[isetting];
	  } else {
	    cal_cal_dac[ichip]    = tot_dac_bottom[isetting];
	  }
	  shaping_time[ichip] = misc_tot_shaping_time[isetting];
          skip[ichip]         = misc_tot_skip[isetting];
	  build_creg(ichip);
	  creg_into_data(ichip);
	  rc = cal_write_control_registers(ichip);
	  if (rc != VME_OK) {
	    sprintf(event_message,"Error sending WRITECTRL command");
	    short_display_message(interp, event_message);
	    short_display_message(interp, abort);
	    return -1;
	  } 
	}
      }
/*-----------------------------------------------------*/
/* loop over channels. we'll fire 8 channels at a time */
/*-----------------------------------------------------*/
    for (igroup=0; igroup<16; igroup++) {
      if (test == 3) {
	sprintf(event_message,"DELAY SETTING = %d   GROUP OF CHANNELS = %d",
	      (isetting+1), (igroup+1));
	short_display_message(interp, event_message);
      } else {
	sprintf(event_message,"TOT SETTING = %d   GROUP OF CHANNELS = %d",
	      isetting, (igroup+1));
	short_display_message(interp, event_message);
      }
/*-------------------------------------------*/
/* set the masks for this group of channels  */
/*-------------------------------------------*/
      for (i=0; i<number_of_chips; i++) {
	for (j=0; j<8; j++) {
	  calib_mask[i][j]   = 1;
	  channel_mask[i][j] = 1;
	  if (igroup != 0) {
	    calib_mask[i][j]   = calib_mask[i][j]   << igroup;
	    channel_mask[i][j] = channel_mask[i][j] << igroup;
	  }
	}
	rc = cal_write_masks(i);
	if (rc != VME_OK) {
	  sprintf(event_message,"Error sending WRITEMASK command");
	  short_display_message(interp, event_message);
          if (isetting == 1 && test == 3) {
	    command_result = misc_switch_time(interp, 1);
	    if (strcmp("OK",command_result) != 0) 
	      short_display_message(interp, command_result);
	  }
	  short_display_message(interp, abort);
	  return -1;
	}
      }
/*--------------------------------------*/
/* loop over number of repetitions      */
/*--------------------------------------*/
      for (irep=1; irep<=nrep; irep++) {
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* Send out a calibration pulse and a level 1 accept                     */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
	eventseq_result = EventSeq(CARD, READEVENT, 1, pm_ch_mask,
			       0x800, 0, 1);
	if (strcmp("OK",eventseq_result) != 0) {
	  short_display_message(interp, eventseq_result);
          if (isetting == 1 && test == 3) {
	    command_result = misc_switch_time(interp, 1);
	    if (strcmp("OK",command_result) != 0) 
	      short_display_message(interp, command_result);
	  }
	  short_display_message(interp, abort);
	  return -1;
	} 
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* Now build the event data                                              */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
	event_number++;
	command_result = cal_data(interp,&data_len,event_number);

        if (strcmp("OK", command_result) == 0) {
	  sanity_flag = format_check_flag | (NUMBER_OF_BYTES | CHIP_ADDRESS);
	  sanity_check = cal_format_check(cal_data_point, sanity_flag);
	  if (strcmp("OK", sanity_check) != 0) {
	    short_display_message(interp, sanity_check);
	    if (isetting == 1 && test == 3) {
	      command_result = misc_switch_time(interp, 1);
	      if (strcmp("OK",command_result) != 0) 
		short_display_message(interp, command_result);
	    }
	    short_display_message(interp, abort);
	    return -1;
	  }
	}

	if (strcmp("OK",command_result) != 0) {
	  short_display_message(interp, command_result);
          if (isetting == 1 && test == 3) {
	    command_result = misc_switch_time(interp, 1);
	    if (strcmp("OK",command_result) != 0) 
	      short_display_message(interp, command_result);
	  }
	  short_display_message(interp, abort);
	  return -1;
	} 
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* Find pointers to begin and end data for chips                    */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
	ishift = 13 + 36 * number_of_chips;
	for (i87=0; i87<number_of_chips; i87++) {
	  dummy = cal_extract(1, &itemp, (cal_data_point+ishift));
	  begin_data[i87] = itemp;
	  ishift = ishift + 4;
	}
	for (i87=0; i87<number_of_chips; i87++) {
	  dummy = cal_extract(1, &itemp, (cal_data_point+ishift));
	  end_data[i87] = itemp;
	  ishift = ishift + 4;
	}
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* Now loop over chips to get the hits                              */
/* NB : 4 bytes = no hits                                           */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
	for (this_chip=0; this_chip<number_of_chips; this_chip++) {
	  nhits = (end_data[this_chip] - begin_data[this_chip] - 3) / 2;
	  for (ihit=1; ihit<=nhits; ihit++) {
	    ishift = begin_data[this_chip] + 2*ihit;
	    cal_get_hit(cal_data_point, ishift, &chnum, &tot, &tstamp);
	    if (cal_mask_manipulate(this_chip, chnum, 1, 0, 0) == 1) {
	      if (test == 3) {
		CumulativeValue[this_chip][chnum][isetting] =
	             CumulativeValue[this_chip][chnum][isetting] +
		     ((double) tstamp);
	      } else {
		CumulativeValue[this_chip][chnum][isetting] =
	             CumulativeValue[this_chip][chnum][isetting] +
		     ((double) tot);
	      }
              NumberOfHits[this_chip][chnum][isetting]++;
	    }
	  }
	}
/*---------------------------------------*/
/* release the memory                    */
/*---------------------------------------*/
	free(cal_data_point);
/*---------------------------------------*/
/* close loop over number of repetitions */
/*---------------------------------------*/
      }
/*---------------------------------------*/
/* close loop over channels fired        */
/*---------------------------------------*/
    }
/*---------------------------------------*/
/* switch time delay setting             */
/*---------------------------------------*/
    if (test == 3) {
      sprintf(event_message,"SWITCHING TO DELAY setting %d",
	      delay_setting[isetting]);
      short_display_message(interp, event_message);
      command_result = misc_switch_time(interp, delay_setting[isetting]);
      if (strcmp("OK",command_result) != 0) {
	short_display_message(interp, command_result);
	short_display_message(interp, abort);
	return -1;
      }
    } 
/*---------------------------------------*/
/* close loop over settings              */
/*---------------------------------------*/
  next_setting: {}
  }
/*-----------------------------------------------------------*/
/* Display the results... Get index of where we are now      */
/*-----------------------------------------------------------*/
  sprintf(command,"misc_file_handler %d 3",test);
  dummy = Tcl_GlobalEval(interp,command);
  if (strcmp(interp->result,"OK") != 0) {
    sprintf(event_message,"%s",interp->result);
    short_display_message(interp,event_message);
    short_display_message(interp,abort);
    return -1;
  }
/*-----------------------------------------------------------*/
/* Time Stamp first                                          */
/*-----------------------------------------------------------*/
  if (test == 3) {
  sprintf(event_message,"\n\n\n\nLIST OF BAD CHANNELS \n");
  short_display_message(interp, event_message);
  sprintf(event_message,"N1 = number of firings on 1st setting");
  short_display_message(interp, event_message);
  sprintf(event_message,"N2 = number of firings on 2nd setting");
  short_display_message(interp, event_message);
  sprintf(event_message,"AvTstamp1 = average time stamp on 1st setting");
  short_display_message(interp, event_message);
  sprintf(event_message,"AvTstamp2 = average time stamp on 2nd setting");
  short_display_message(interp, event_message);
  sprintf(event_message,"AvDelta = AvTstamp2-AvTstamp1");
  short_display_message(interp, event_message);
  sprintf(event_message,
     "\n\nChip Chan  N1    N2    AvTstamp1  AvTstamp2  AvDelta");
  short_display_message(interp, event_message);
  sprintf(event_message,
     "-------------------------------------------------------");
  short_display_message(interp, event_message);

  for(ichip=0; ichip<number_of_chips; ichip++) {
    if (ignore[ichip] == 1) continue; 
    for (ichan=0; ichan<128; ichan++) {
      bad = 0;

      for (i=0; i<2; i++) {
	av[i] = 0.;
	if (NumberOfHits[ichip][ichan][i] != 0) {
	  av[i] = CumulativeValue[ichip][ichan][i] /
                  NumberOfHits[ichip][ichan][i];
	} else {
	  bad = 1;
	}
      }

      delta = av[1] - av[0];
      if (fabs(delta-misc_tstamp_shift) > misc_tstamp_toll)  bad=1;
      if (  (NumberOfHits[ichip][ichan][0] == 0 ||
             NumberOfHits[ichip][ichan][1] == 0) &&
          misc_tstamp_dead == 1) bad = 0;

      if (bad != 0)  {
	sprintf(event_message,
		"%-3d %-3d   %-5d  %-5d  %-3.2f      %-3.2f      %-3.2f",
              ichip, ichan, NumberOfHits[ichip][ichan][0],
	      NumberOfHits[ichip][ichan][1],av[0],av[1],delta);
	short_display_message(interp, event_message);
      }


    }
  }
}
/*-----------------------------------------------------------*/
/* Display the results... ToT now                            */
/* NB In this case the mean includes the zeros               */
/*-----------------------------------------------------------*/
  if (test == 2) {
    for (isetting=0; isetting<misc_ntot; isetting++) {
      if (misc_tot_test_it[isetting] == 0) goto next_setting_2;
      sprintf(event_message,"\n\n\n\nLIST OF BAD CHANNELS SETTING No. %d \n",
	      isetting);
      short_display_message(interp, event_message);
      sprintf(event_message,
     "\nChip Chan  Nfiring    AverageToT");
      short_display_message(interp, event_message);
      sprintf(event_message,
     "____________________________________________________");
      short_display_message(interp, event_message);
      for(ichip=0; ichip<number_of_chips; ichip++) {
        if (ignore[ichip] == 1) continue;
	for (ichan=0; ichan<128; ichan++) {
	  bad = 0;
	  avtot = CumulativeValue[ichip][ichan][isetting] / nrep;
	  if (avtot != 0) {
	    delta = fabs(avtot-misc_tot[isetting]);
	    if (delta > misc_tot_toll[isetting]) bad=1;
	  } else {
	    if (misc_tot_dead != 1) bad=1;
	  }

	  if (bad != 0) {
	    sprintf(event_message,"%-3d  %-3d     %-5d    %-3.2f",
		  ichip,ichan,NumberOfHits[ichip][ichan][isetting],avtot);
	    short_display_message(interp, event_message);
	  }
	
	}
      }
    next_setting_2: {}
    }
  }
/*---------------------------------------*/
/* DONE                                  */
/*---------------------------------------*/
  if (test == 3) {
    sprintf(event_message,"\n \n TIME STAMP TEST COMPLETED \n \n");
    short_display_message(interp, event_message);
  } else {
    sprintf(event_message,"\n \n TOT TEST COMPLETED \n \n");
    short_display_message(interp, event_message);
  }
/*---------------------------------------*/
/* Get the index of the end of this test */
/*---------------------------------------*/
  sprintf(command,"misc_file_handler %d 4",test);
  dummy = Tcl_GlobalEval(interp,command);
  if (strcmp(interp->result,"OK") != 0) {
    sprintf(event_message,"%s",interp->result);
    short_display_message(interp,event_message);
    short_display_message(interp,abort);
    return -1;
  }

  return 0;
}


/*********************************************************************/
/*********************************************************************/
/*********************************************************************/
/*********************************************************************/
/* Buffer test.  Returns zero on succesful completion                */
/*********************************************************************/
/*********************************************************************/
/*********************************************************************/
/*********************************************************************/
int misc_do_buffer_test(Tcl_Interp* interp, int nrep, int pm_ch_mask) {

static char *sanity_check;
int sanity_flag;
void misc_set_all_masks(int);
int ichip, i, j, data_len, irep, ishift, i87, itemp, this_chip, ihit;
int nhits, chnum, tot, tstamp, this_channel, bad;
vme_rc rc;
static char command[200];
char event_message[200];
static char* command_result;
static char*  eventseq_result;
static char abort[] = "Aborting.....";
int event_number, dummy;
int begin_data[MAX_NUM_CHIP], end_data[MAX_NUM_CHIP];
int nfound[MAX_NUM_CHIP][4];
int channels_fired[8] = {0, 16, 32, 48, 64, 80, 96, 112};
int buffer_used, position;
int temp_mask[4];  
static char cmd10[]  = "vme_clear daq_0(led)";
static char cmd11[]  = "vme_set daq_0(led)";

  event_number = 0;
/*-----------------------------------------------------*/
/* Keep track of first index                            */
/*-----------------------------------------------------*/
  sprintf(command,"misc_file_handler 1 3");
  dummy = Tcl_GlobalEval(interp,command);
  if (strcmp(interp->result,"OK") != 0) {
    sprintf(event_message,"%s",interp->result);
    short_display_message(interp,event_message);
    short_display_message(interp,abort);
    return -1;
  }
/*--------------------------------------*/
/* loop over number of repetitions      */
/*--------------------------------------*/
  for (irep=1; irep<=nrep; irep++) {
    sprintf(event_message,"REPETITION # %d \n", irep);
    short_display_message(interp, event_message);
/*--------------------------------------*/
/* use 1,2,3,4 buffers                  */
/*--------------------------------------*/
    for (buffer_used=1; buffer_used<5; buffer_used++) { 
/*----------------------------------------*/
/* loop over position of event with hits  */
/*----------------------------------------*/
      for (position=1; position<=buffer_used; position++) {
	for (i=0; i<4; i++) {
	  temp_mask[i] = 0;
	}
	temp_mask[position-1] = 1;
/*-------------------------------------------------------------------------*/
/* now fire the charge injection circuitry "buffer_used" number of times   */
/* with different masks                                                    */
/* REMEMBER TO SEND NOOP TO TOGGLE VRAM DIRECTION                          */
/*-------------------------------------------------------------------------*/
	for (i=1; i<=buffer_used; i++) {
	  misc_set_all_masks(temp_mask[i-1]);
	  for (ichip=0; ichip<number_of_chips; ichip++) {
	    rc = cal_write_masks(ichip);
	    if (rc != VME_OK) {
	      sprintf(event_message,"Error sending WRITEMASK command");
	      short_display_message(interp, event_message);
	      short_display_message(interp, abort);
	      return -1;
	    }
	  }
          rc = vmePmCommand(CARD, NOOP, 0, pm_ch_mask, 0, 1, chip_data);
	  dummy = Tcl_GlobalEval(interp,cmd11);
	  if (dummy != TCL_OK) {
	    sprintf(event_message,"Error turning LED bit on");
	    short_display_message(interp, event_message);
	    short_display_message(interp, abort);
	    return -1;
	  }
	  dummy = Tcl_GlobalEval(interp,cmd10);
	  if (dummy != TCL_OK) {
	    sprintf(event_message,"Error turning LED bit off");
	    short_display_message(interp, event_message);
	    short_display_message(interp, abort);
	    return -1;
	  }
	}
/*-----------------------------------------------*/
/* now read "buffer_used" number of events       */
/*-----------------------------------------------*/
	for (i=1; i<=buffer_used; i++) {
	  eventseq_result = EventSeq(CARD, READEVENT, 0, pm_ch_mask,
			       0x800, 0, 1);
	  if (strcmp("OK",eventseq_result) != 0) {
	    short_display_message(interp, eventseq_result);
	    short_display_message(interp, abort);
	    return -1;
	  }
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* Now build the event data                                              */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
	  event_number++;
	  command_result = cal_data(interp,&data_len,event_number);
	  if (strcmp("OK",command_result) != 0) {
	    short_display_message(interp, command_result);
	    short_display_message(interp, abort);
	    return -1;
          } 
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* format check                                                     */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
	  sanity_flag = format_check_flag | (NUMBER_OF_BYTES | CHIP_ADDRESS);
	  sanity_check = cal_format_check(cal_data_point, sanity_flag);
	  if (strcmp("OK", sanity_check) != 0) {
	    short_display_message(interp, sanity_check);
	    short_display_message(interp, abort);
	    return -1;
	  }
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* Find pointers to begin and end data for chips                    */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
	  ishift = 13 + 36 * number_of_chips;
	  for (i87=0; i87<number_of_chips; i87++) {
	    dummy = cal_extract(1, &itemp, (cal_data_point+ishift));
	    begin_data[i87] = itemp;
	    ishift = ishift + 4;
	  }
	  for (i87=0; i87<number_of_chips; i87++) {
	    dummy = cal_extract(1, &itemp, (cal_data_point+ishift));
	    end_data[i87] = itemp;
	    ishift = ishift + 4;
	  }
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* Now loop over chips to get the hits                              */
/* NB : 4 bytes = no hits                                           */
/* count how many of the expected hits we have                      */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
	  for (this_chip=0; this_chip<number_of_chips; this_chip++) {
	    nfound[this_chip][i-1] = 0;
	    nhits = (end_data[this_chip] - begin_data[this_chip] - 3) / 2;
	    for (ihit=1; ihit<=nhits; ihit++) {
	      ishift = begin_data[this_chip] + 2*ihit;
	      cal_get_hit(cal_data_point, ishift, &chnum, &tot, &tstamp);
	      this_channel = 128*this_chip + chnum;
	      for (j=0; j<8; j++) {
		if (chnum == channels_fired[j]) nfound[this_chip][i-1]++;
	      }
	    }
	  }
/*-------------------------------------------------------------------*/
/* close loop over read "buffer_used" number of events               */
/*-------------------------------------------------------------------*/
	  free(cal_data_point);
	}
/*-----------------------------------------------*/
/* now see if we have any errors                 */
/*-----------------------------------------------*/
	for (i=1; i<=buffer_used; i++) {
	  for (this_chip=0; this_chip<number_of_chips; this_chip++) {
	    bad = 0;
            if (ignore[this_chip] == 1) continue;
	    if (nfound[this_chip][i-1] > MAXCHFIRED
		&& temp_mask[i-1] == 0)              bad=1;
	    if (nfound[this_chip][i-1] < MINCHFIRED && 
		temp_mask[i-1] == 1)                 bad=1;
	    if (bad == 1) {
	      sprintf(event_message,
		      "ERROR Repetition = %d. Using %d buffers.",
		                                   irep,buffer_used);
	      short_display_message(interp, event_message);
              sprintf(event_message,
		      "Expect buffer with data to be buffer %d",position);
	      short_display_message(interp, event_message);
              sprintf(event_message,
                      "Buffer %d Chip %d Expect %d hits Find %d hits \n",
                      i,this_chip, 8*temp_mask[i-1], nfound[this_chip][i-1]);
	      short_display_message(interp, event_message);
	    }
	  }
	}
/*-----------------------------------------------*/
/* close loop over position of event with hits   */
/*-----------------------------------------------*/
	}
/*-----------------------------------------*/
/* close loop over number of buffers used  */
/*-----------------------------------------*/
    }
/*--------------------------------------*/
/* close loop over repetitions          */
/*--------------------------------------*/
  }
  sprintf(event_message,"\n \n BUFFER TEST COMPLETED \n \n");
  short_display_message(interp, event_message);
/*-----------------------------------------------------*/
/* Keep track of last index                            */
/*-----------------------------------------------------*/
  sprintf(command,"misc_file_handler 1 4");
  dummy = Tcl_GlobalEval(interp,command);
  if (strcmp(interp->result,"OK") != 0) {
    sprintf(event_message,"%s",interp->result);
    short_display_message(interp,event_message);
    short_display_message(interp,abort);
    return -1;
  }
/*--------------------------------------*/
/* done                                 */
/*--------------------------------------*/
  return 0;
}
/*********************************************************************/
/*********************************************************************/
/*********************************************************************/
/*********************************************************************/
/* Channel mask test                                                 */
/* returns zero on succesful completion                              */
/*********************************************************************/
/*********************************************************************/
/*********************************************************************/
/*********************************************************************/
int misc_do_channel_mask_test(Tcl_Interp* interp, int nrep, int pm_ch_mask) {

static char *sanity_check;
int sanity_flag;
int ichip, i, data_len, ishift, i87, itemp, this_chip, irep, ihit, dummy;
int nhits, chnum, tot, tstamp, this_channel, nbad;
vme_rc rc;
static char command[200];
char event_message[200];
static char* command_result;
static char*  eventseq_result;
static char abort[] = "Aborting.....";
int event_number;
int begin_data[MAX_NUM_CHIP], end_data[MAX_NUM_CHIP];
int hit_counter[MAX_NUM_CHIP][128];

/*------------------------------------------------------------------------*/
/* zero some stuff                                                        */
/*------------------------------------------------------------------------*/
    event_number = 0;
    for (ichip=0; ichip<number_of_chips; ichip++) {
      for (i=0; i<128; i++) {
	hit_counter[ichip][i] = 0;
      }
    }
/*------------------------------------------------------------------------*/
/* set all channel masks to zero, all calib masks to one                  */
/*------------------------------------------------------------------------*/
   for (ichip=0; ichip<number_of_chips; ichip++) {
    for (i=0; i<8; i++) {
      calib_mask[ichip][i]   = 0xffff;
      channel_mask[ichip][i] = 0;
    }
    rc = cal_write_masks(ichip);
    if (rc != VME_OK) {
      sprintf(event_message,"Error sending WRITEMASK command");
      short_display_message(interp, event_message);
      short_display_message(interp, abort);
      return -1;
    }
  }  
/*--------------------------------------------------------------------*/
/* Repeat the process n times                                         */
/*--------------------------------------------------------------------*/
  for (irep=1; irep<=nrep; irep++) {
    sprintf(event_message,"Repetition # %d", irep);
    short_display_message(interp, event_message);
    eventseq_result = EventSeq(CARD, READEVENT, 1, pm_ch_mask,
			       0x800, 0, 1);
    if (strcmp("OK",eventseq_result) != 0) {
      short_display_message(interp, eventseq_result);
      short_display_message(interp, abort);
      return -1;
    } 
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* Now build the event data                                              */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
    event_number++;
    command_result = cal_data(interp,&data_len,event_number);
    if (strcmp("OK",command_result) != 0) {
      short_display_message(interp, command_result);
      short_display_message(interp, abort);
      return -1;
    } 
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* format check                                                     */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
    sanity_flag = format_check_flag | (NUMBER_OF_BYTES | CHIP_ADDRESS);
    sanity_check = cal_format_check(cal_data_point, sanity_flag);
    if (strcmp("OK", sanity_check) != 0) {
      short_display_message(interp, sanity_check);
      short_display_message(interp, abort);
      return -1;
    }
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* Find pointers to begin and end data for chips                    */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
  ishift = 13 + 36 * number_of_chips;
  for (i87=0; i87<number_of_chips; i87++) {
    dummy = cal_extract(1, &itemp, (cal_data_point+ishift));
    begin_data[i87] = itemp;
    ishift = ishift + 4;
  }
  for (i87=0; i87<number_of_chips; i87++) {
    dummy = cal_extract(1, &itemp, (cal_data_point+ishift));
    end_data[i87] = itemp;
    ishift = ishift + 4;
  }
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* Now loop over chips to get the hits                              */
/* NB : 4 bytes = no hits                                           */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
  for (this_chip=0; this_chip<number_of_chips; this_chip++) {
    nhits = (end_data[this_chip] - begin_data[this_chip] - 3) / 2;
    for (ihit=1; ihit<=nhits; ihit++) {
        ishift = begin_data[this_chip] + 2*ihit;
        cal_get_hit(cal_data_point, ishift, &chnum, &tot, &tstamp);
	hit_counter[this_chip][chnum]++;
     }
  }
/*---------------------------------------------------*/
/* close loop over repetitions                       */
/*---------------------------------------------------*/
    free(cal_data_point);
  }
/*-----------------------------------------------------*/
/* Keep track of first index                           */
/*-----------------------------------------------------*/
  sprintf(command,"misc_file_handler 0 3");
  dummy = Tcl_GlobalEval(interp,command);
  if (strcmp(interp->result,"OK") != 0) {
    sprintf(event_message,"%s",interp->result);
    short_display_message(interp,event_message);
    short_display_message(interp,abort);
    return -1;
  }
/*---------------------------------------------------*/
/* output test results                               */
/*---------------------------------------------------*/
  nbad = 0;
  for (this_chip=0; this_chip<number_of_chips; this_chip++) {
    for (this_channel=0; this_channel<128; this_channel++) {
      if (this_chip==0 && this_channel==0) {
	sprintf(event_message,"LIST OF BAD CHANNELS");
        short_display_message(interp,event_message);
      }

      if (ignore[this_chip] == 1) continue;
      if (hit_counter[this_chip][this_channel] != 0) {
	  sprintf(event_message,"chip %d channel %d fired %d times",
	        this_chip,this_channel,hit_counter[this_chip][this_channel]);  
	  short_display_message(interp,event_message);
	  nbad++;
      }

/* close loops over chips and channels for output test results */
    }
  }
  sprintf(event_message,"THERE WERE %d BAD CHANNELS",nbad);
  short_display_message(interp,event_message);
  sprintf(event_message,"\n \n CHANNEL MASK TEST COMPLETED \n \n");
  short_display_message(interp, event_message);
/*-----------------------------------------------------*/
/* Keep track of last index                            */
/*-----------------------------------------------------*/
  sprintf(command,"misc_file_handler 0 4");
  dummy = Tcl_GlobalEval(interp,command);
  if (strcmp(interp->result,"OK") != 0) {
    sprintf(event_message,"%s",interp->result);
    short_display_message(interp,event_message);
    short_display_message(interp,abort);
    return -1;
  }
/*---------------------------------------------------*/
/* done                                              */
/*---------------------------------------------------*/
  return 0;
}


/*********************************************************************/
/*********************************************************************/
/*********************************************************************/
/*********************************************************************/
/* quick utility routine.  sets all the masks to the same value      */
/*********************************************************************/
/*********************************************************************/
/*********************************************************************/
/*********************************************************************/
void misc_set_all_masks(int value) {
int i,j;
    for (i=0; i<number_of_chips; i++) {
      for (j=0; j<8; j++) {
	calib_mask[i][j]   = value;
	channel_mask[i][j] = value;
      }
    }
}
/*********************************************************************/
/*********************************************************************/
/*********************************************************************/
/*********************************************************************/
/* definition of the command "misc_tests"                             */
/*********************************************************************/
/*********************************************************************/
/*********************************************************************/
void misc_tests_define(Tcl_Interp* interp) {
  Tcl_CreateCommand(interp, "misc_tests", misc_testsCmd, (ClientData) NULL,
                    (Tcl_CmdDeleteProc *) NULL);
}


/*********************************************************************/
/*********************************************************************/
/*********************************************************************/
/*********************************************************************/
/* This is the code to change the time delay                         */
/* Returns OK or an error message                                    */ 
/*              time_setting = 1 is the "normal" delay               */
/*              time_setting = 2 is the second delay setting         */
/*********************************************************************/
/*********************************************************************/
/*********************************************************************/
/*********************************************************************/
char* misc_switch_time(Tcl_Interp* interp, int time_setting) {

static char switch_message[200];
static char cmd[200];
int dummy; 
int stop_and_ask = 1;


/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* make sure that the passed parameter is OK                             */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  if (time_setting != 1 && time_setting != 2) {
    sprintf(switch_message,
      "ERROR: misc_switch_time called with illegal parameter time_setting = %d"
      , time_setting);
    return switch_message;
  }
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* This pops up a window asking the user to change the delay manually    */
/* Remember to reset the grab to the .pltw window when we are done       */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
#ifdef UCSB
 stop_and_ask = 0;
#endif
  if (stop_and_ask == 1) {
    sprintf(cmd,
     "cal_display_error_message \"\n\nPlease switch to delay setting %d\n\n\"",
	    time_setting);
     dummy = Tcl_GlobalEval(interp,cmd);
     if (dummy != TCL_OK) {
       sprintf(switch_message,
	       "ERROR PROMPTING FOR DELAY SWITCH : \n %s \n",interp->result);
     } else {
       sprintf(switch_message,"OK");
     }
     sprintf(cmd,"grab set .pltw");
     dummy = Tcl_GlobalEval(interp,cmd);
  }
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* This is the UCSB code                                                 */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
#ifdef UCSB
  stop_and_ask = 0;  
  if (time_setting == 1) {
    sprintf(cmd,"delay_select_off");
  } else {
    sprintf(cmd,"delay_select_on");
  }

  dummy = Tcl_GlobalEval(interp,cmd);
  if (strcmp(interp->result,"0") != 0) {
    sprintf(switch_message,
  "ERROR PROMPTING FOR DELAY SWITCH : \nreturn value is %s \n",interp->result);
  } else {
    sprintf(switch_message,"OK");
  }
#endif
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* return                                                                */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
  return switch_message;
}
