#include <tk.h>
#include <stdlib.h>
#include "cwish.h"
#include "tclutils.h"
#include "backrpc.h"
#include "backTypes.h"
#include "front.h"

static int streq(char* one, char* two)
{
  return !strcmp(one,two);
}

static int vme_ExecControlCmd(ClientData clientData, Tcl_Interp* interp, int argc, char* argv[])
{
  static struct vme_user_params params;
  static char server[256];
  static int firsttime = 1;
  int run = 0;
  int value;
  Tcl_DString dstring;
  char buffer[256];

  if (firsttime == 1) {
    firsttime = 0;
    strcpy(server,"gluon");
    params.server = server;
    params.events = 1;
  }

  if (argc == 1) /* no parameters --> report on state */
    {
      Tcl_DStringInit(&dstring); /* make sure Tcl_DStringFree is reached below */

      Tcl_DStringStartSublist(&dstring);
      Tcl_DStringAppendElement(&dstring,"events");
      sprintf(buffer,"%d",params.events);
      Tcl_DStringAppendElement(&dstring,buffer);
      Tcl_DStringEndSublist(&dstring);

      Tcl_DStringStartSublist(&dstring);
      Tcl_DStringAppendElement(&dstring,"server");
      Tcl_DStringAppendElement(&dstring,server);
      Tcl_DStringEndSublist(&dstring);

      Tcl_DStringResult(interp,&dstring);
      Tcl_DStringFree(&dstring);

      return TCL_OK;
    }
  else           /* respond to the given paramaters */
    {
      while (*++argv != NULL)
	{
	  if (streq(*argv,"events"))
	    {
	      if (Tcl_GetInt(interp,*++argv,&value) != TCL_OK) 
		{
		  return vme_TclReturn(interp,VME_BAD_NUMBER);
		}
	      else
		{
		  params.events = value;
		}
	    }
	  else if (streq(*argv,"server"))
	    {
	      if (*++argv == NULL)
		{
		  return vme_TclArgError(interp,"server",2);
		}
	      else
		{
		  strcpy(server,*argv);
		}
	    }
	  else if (streq(*argv,"run"))
	    {
	      run = 1;
	    }
	  else
	    {
	      return vme_TclReturn(interp,VME_BAD_ARGUMENT);
	    }
	}
      if (run == 1)
	{
	  return vme_TclReturn(interp,vmeExecControl(params));
	}
      else
	{
	  return vme_TclReturn(interp,VME_OK);
	}
    }
}

static CLIENT* eventClnt;

static char eventServer[256];

static int serverBad()
{
  static char oldserver[256];
  static int firsttime = 1;

  if (firsttime)
    {
      firsttime = 0;
      strcpy(oldserver,"");
    }
  
  if (!streq(eventServer,oldserver))   /* has the server name changed (from "" the first time) */
    {
      strcpy(oldserver,eventServer);
      if (eventClnt != NULL)           /* is this our first time through? */
	{
	  clnt_destroy(eventClnt);
	  eventClnt = NULL;
	}
    }

  if (eventClnt == NULL)
    {
      eventClnt = clnt_create(eventServer, BACKSERVER_PROGRAM, BACKSERVER_VERSION, "tcp");
      if (eventClnt == NULL)
	{
	  clnt_pcreateerror(eventServer);
	  vmeErrorReturn(VME_NO_CLIENT);
	  return VME_TRUE;
	}
    }
  return VME_FALSE;
}

static enum vme_userstate str2state(char* state)
{
  if (streq(state,"dne"))
    {
      return VME_DNE;
    }
  else if (streq(state,"dead"))
    {
      return VME_DEAD;
    }
  else if (streq(state,"active"))
    {
      return VME_ACTIVE;
    }
  else if (streq(state,"passive"))
    {
      return VME_PASSIVE;
    }
  else 
    {
      return VME_ERROR;
    }
}

static char* state2str(enum vme_userstate state)
{
  switch (state)
    {
    case VME_DNE : return "dne";
    case VME_DEAD : return "dead";
    case VME_ACTIVE : return "active";
    case VME_PASSIVE : return "passive";
    default: return "";
    }
}

static int vme_EventHandlerCmd(ClientData clientData, Tcl_Interp* interp, int argc, char* argv[])
{
  static int firsttime = 1;
  int id;
  Tcl_DString dstring;
  char strBuff[256];
  char** name;
  char** result;
  idArray* ids;
  int* statep;
  int state;
  char* tmpServerName;
  int i;
  
  if (firsttime)
    {
      strcpy(eventServer,"gluon.phys.unm.edu");
      firsttime = 0;
    }
  *++argv;
  if (*argv == NULL)
    {
      vme_TclReturn(interp,vmeErrorReturn(VME_BAD_ARGUMENT));
    }
  else if (streq(*argv,"server"))
    {
      if (*++argv == NULL)
	{
	  Tcl_DStringInit(&dstring); /* make sure Tcl_DStringFree is reached below */
	  Tcl_DStringAppendElement(&dstring,eventServer);
	  Tcl_DStringResult(interp,&dstring);
	  Tcl_DStringFree(&dstring);
	  return vme_TclReturn(interp,VME_OK);
	}
      else
	{
	  if (argv[1] != NULL)
	    {
	      return vme_TclReturn(interp,VME_BAD_ARGUMENT);
	    }
	  strcpy(eventServer,*argv);
	  return vme_TclReturn(interp,serverBad());
	}
    }
  else if (serverBad())
    {
      vme_TclReturn(interp,VME_ERROR);
    }
  else if (streq(*argv,"name"))
    {
      if (Tcl_GetInt(interp,*++argv,&id) != TCL_OK) 
	{
	  return vme_TclReturn(interp,VME_BAD_NUMBER);
	}
      else
	{
	  if (*++argv != NULL)
	    {
	      return vme_TclReturn(interp,VME_BAD_ARGUMENT);
	    }
	  name = vmegetusername_1(id,eventClnt);
	  if (name == NULL || *name == NULL)
	    {
	      return vme_TclReturn(interp,VME_RPC_FAILED);
	    }
	  else
	    {
	      strcpy(strBuff,*name);
	      xdr_free(xdr_string,name);
	      return vme_TclStrReturn(interp,strBuff);
	    }
	}
    }
  else if (streq(*argv,"send"))
    {
      if (Tcl_GetInt(interp,*++argv,&id) != TCL_OK) 
	{
	  if (argv[1] != NULL)
	    {
	      return vme_TclReturn(interp,VME_BAD_NUMBER);
	    }
	  else
	    {
	      --argv;
	      id = 0;
	    }
	}
      if (argv[1] == NULL || argv[2] != NULL)
	{
	  return vme_TclReturn(interp,VME_BAD_ARGUMENT);
	}
      result = vmesendcmd_1(id,*++argv,eventClnt);
      if (result == NULL || *result == NULL)
	{
	  return vme_TclReturn(interp,VME_RPC_FAILED);
	}
      else
	{
	  strcpy(strBuff,*result);
	  xdr_free(xdr_string,result);
	  return vme_TclStrReturn(interp,strBuff);
	}
    }
/*   else if (streq(*argv,"hello")) */
/*     { */
/*       if (*argv[1] == NULL || argv[2] != NULL) */
/* 	{ */
/* 	  return vme_TclReturn(interp,VME_BAD_ARGUMENT); */
/* 	} */
/*       if (vmehello_1(*++argv,eventClnt) == NULL) */
/* 	{ */
/* 	  return vme_TclReturn(interp,VME_RPC_FAILED); */
/* 	} */
/*       return vme_TclReturn(interp,VME_OK); */
/*     } */
  else if (streq(*argv,"users"))
    {
      if (*++argv != NULL)
	{
	  return vme_TclReturn(interp,VME_BAD_ARGUMENT);
	}
      ids = vmebackusers_1(eventClnt);
      if (ids == NULL)
	{
	  return vme_TclReturn(interp,VME_RPC_FAILED);
	}
      Tcl_DStringInit(&dstring); /* make sure Tcl_DStringFree is reached below */
      for (i = 0; i < ids->idArray_len; i++)
	{
	  Tcl_DStringAppendElement(&dstring,vme_Itoa(ids->idArray_val[i]));
	}
      Tcl_DStringResult(interp,&dstring);
      Tcl_DStringFree(&dstring);
      xdr_free(xdr_idArray,ids);
      return vme_TclReturn(interp,VME_OK);
    }
  else if (streq(*argv,"state"))
    {
      if (Tcl_GetInt(interp,*++argv,&id) != TCL_OK) 
	{
	  return vme_TclReturn(interp,VME_BAD_NUMBER);
	}
      else
	{
	  if (*++argv == NULL)
	    {
	      statep = vmegetuserstate_1(id,eventClnt);
	      if (statep == NULL)
		{
		  return vme_TclReturn(interp,VME_RPC_FAILED);
		}
	      return vme_TclStrReturn(interp,state2str(*statep));
	    }
	  else
	    {
	      state = str2state(*argv);
	      if (state == VME_ERROR)
		{
		  return vme_TclReturn(interp,VME_BAD_ARGUMENT);
		}		  
	      if (*++argv != NULL)
		{
		  return vme_TclReturn(interp,VME_BAD_ARGUMENT);
		}
	      vmesetuserstate_1(id,state,eventClnt);
	      return vme_TclReturn(interp,VME_OK);
	    }
	}
    }
  else
    {
      return vme_TclReturn(interp,VME_BAD_ARGUMENT);
    }
}

int VME_Handler_Mod_Init(Tcl_Interp* interp)
{
  Tcl_CreateCommand(interp, "vme_executor",vme_ExecControlCmd,(ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);
  Tcl_CreateCommand(interp, "vme_eventHandler",vme_EventHandlerCmd,(ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);
  return TCL_OK;
}
