/* 
 * VME Teststand Software
 *
 * Code revised by Armin Karcher (karcher@design.lbl.gov)
 *
 * Design and Template by:
 *   Eric Gottlieb  (egott@unm.edu)
 *   Tim Thomas     (thomas@hev1.phys.unm.edu)
 *   Micheal Gold   (gold@hepv1.phys.unm.edu)
 *
 */

#include <tk.h>
#include <string.h>
#include "regtrace.h"
#include "error.h"
#include "reglist.h"
#include "cardlist.h"
#include "low.h"



/* 
 * support for new (automatically generated) tcl variables
 *
 */

static char * traceread(ClientData clientData, Tcl_Interp *interp, char *name1, char *name2, int flags){
  int rc;
  vme_register reg;
  char tclrc[256];

  reg = (vme_register) clientData;

  printf("traceread\n");

  rc = vmeRead(0,reg);
  if (vmeError(rc)){
    /* Tcl_SetErrorCode(interp, "VME", vme_Itoa(-rc), vmeErrorString(rc), (char *) NULL);
     * doesn't seem to work??? */
    return vmeErrorString(rc);
  }
  sprintf(tclrc,"%d",rc);
  Tcl_SetVar2(interp,name1,name2,tclrc,TCL_GLOBAL_ONLY|TCL_LEAVE_ERR_MSG);
  return NULL;
}

static char * tracewrite(ClientData clientData, Tcl_Interp *interp, char *name1, char *name2, int flags){
  int val,rc;
  vme_register reg;
  char* tclval;
  reg = (vme_register) clientData;

  printf("tracewrite\n");

  tclval = Tcl_GetVar2(interp,name1,name2,TCL_GLOBAL_ONLY|TCL_LEAVE_ERR_MSG);
  Tcl_GetInt(interp,tclval,&val);
  rc = vmeWrite(0,reg,(vme_vmetype)val);
  if (vmeError(rc)){
    /* Tcl_SetErrorCode(interp, "VME", vme_Itoa(-rc), vmeErrorString(rc), (char *) NULL);
     * doesn't seem to work??? */
    return vmeErrorString(rc);
  }
  return NULL;
}

void vme_TclRegAdd(Tcl_Interp* interp, vme_register reg)
{
  int i = 0;
/*   printf("regtrace: regadd: reg = %s\n",reg->name); */
/*   printf("regtrace: regadd: card = %s\n",reg->card->classname); */
/*   printf("regtrace: regadd: bases = %d\n",reg->card->bases); */
  while (i < reg->card->bases)
    {
/*       printf("regtrace: regadd: inst = %s\n",reg->card->instname[i]); */
      Tcl_SetVar2(  interp, 
		    reg->card->instname[i], 
		    reg->name,"0",
		    TCL_GLOBAL_ONLY | TCL_LEAVE_ERR_MSG);
      Tcl_TraceVar2(interp, 
		    reg->card->instname[i], 
		    reg->name,
		    TCL_TRACE_WRITES | TCL_GLOBAL_ONLY,
		    tracewrite,
		    (ClientData) reg);
      Tcl_TraceVar2(interp, 
		    reg->card->instname[i], 
		    reg->name,
		    TCL_TRACE_READS | TCL_GLOBAL_ONLY,
		    traceread,
		    (ClientData) reg);
      i++;
    }
}

void vme_TclBaseAdd(Tcl_Interp* interp, vme_card card)
{
  int i;
  vme_register reg;
  int inst;

/*   printf("regtrace: baseadd: card  = %s\n",card->classname); */
/*   printf("                   bases = %d\n",card->bases); */

  if (card->bases >0)
    {
 
      inst = card->bases - 1;

      printf("inst = %d\n",inst);
      
      for (i = 0; i < VME_Reg.number; i++)
	{
	  reg = VME_Reg.list[i];
/* 	  printf("reg, reg card = %s, %s\n",reg->name,reg->card->classname); */
	  if (card == reg->card)
	    {
/* 	      printf("instname = %s\n", reg->card->instname[inst]); */
/* 	      printf("adding <%s(%s)>\n",reg->card->instname[inst],reg->name); */
	      Tcl_SetVar2(  interp, 
			    reg->card->instname[inst], 
			    reg->name,
			    "0",
			    TCL_GLOBAL_ONLY | TCL_LEAVE_ERR_MSG);
	      Tcl_TraceVar2(interp, 
			    reg->card->instname[inst], 
			    reg->name,
			    TCL_TRACE_WRITES | TCL_GLOBAL_ONLY, 
			    tracewrite,
			    (ClientData) reg);
	      Tcl_TraceVar2(interp, 
			    reg->card->instname[inst], 
			    reg->name,
			    TCL_TRACE_READS | TCL_GLOBAL_ONLY, 
			    traceread,
			    (ClientData) reg);
	    }
	}
    }
  else
    {
      vmeErrorReturn1("missing card instance");
    }
}

int vme_TclRegInit(Tcl_Interp *interp){
  int i;
  static int done = 0;
  if (!done) {
    for (i = 0; i < VME_Reg.number; i++) {
      vme_TclRegAdd(interp, VME_Reg.list[i]);
    }
    done = -1;
  }
  return TCL_OK;
}

vme_rc vme_TclGetRegister(Tcl_Interp* interp,char* name,vme_register* regptr, int* instptr){
  char message[256];
  char* where;
  char paren;

/*   printf("name = %s\n",name); */
  *regptr = (vme_register) Tcl_VarTraceInfo(interp,name,0,traceread,(ClientData) NULL);
  if (*regptr == NULL) 
    {
      sprintf(message,"illegal/unknown register: %s",name);
      return vmeErrorReturn1(message);
/*    return vmeErrorReturn(VME_ILLEGAL_REGISTER); */
    }
  else
    {
      /* register is good (it exists) ... now get card name */
      /* it will be all of name upto first ')' */

      /* hack alert :) */

      where = strpbrk(name,"( \t");   /* find the first break in the name */
      paren = *where;                 /* save it for a moment */
      *where = 0;                     /* replace it w/ a zero */
      *instptr = vmeName2Inst(name);  /* now we have the card name */
      *where = paren;                 /* restore the string just in case */

      return VME_OK;
    }
}
