/* 
 * VME  Teststand Software
 *
 * Code revised by Armin Karcher (karcher@design.lbl.gov)
 *
 * Design and Template by:
 *   Eric Gottlieb  (egott@unm.edu)
 *   Tim Thomas     (thomas@hev1.phys.unm.edu)
 *   Micheal Gold   (gold@hepv1.phys.unm.edu)
 *
 */

#include <tk.h>
#include "tclutils.h"
#include "error.h"

/* 
 * generic tcl support routines
 *
 */

char* vme_Itoa(int i){
  static char a[32];
  sprintf(a,"%d",i);
  return a;
}

char* vme_Htoa(int i){
  static char a[32];
  sprintf(a,"0x%x",i);
  return a;
}

int vme_TclError(Tcl_Interp* interp,char* s,vme_rc err)
{  
  Tcl_SetResult(interp,s,TCL_VOLATILE);
  Tcl_SetErrorCode(interp, "VME", vme_Itoa(-err), s, (char *) NULL);
  return TCL_ERROR;
}

int vme_TclReturn(Tcl_Interp* interp,vme_rc err)
{
  if (err != VME_OK && vmeErrorState() == VME_OK) 
    {
      vmeErrorReturn(err);
    }
  if (vmeErrorState() != VME_OK) 
    {
      return vme_TclError(interp,vmeErrorString(err),err);
    }
  return TCL_OK;
}

int vme_TclArgError(Tcl_Interp* interp, char* name, int number){
  char mess[256];
  sprintf(mess,"%s takes exactly %d arguments",name,--number);
  return vme_TclError(interp,mess,VME_WRONG_NUMBER_ARGS);
}

int vme_TclRcReturn(Tcl_Interp* interp,int rc){
  if (vmeError(rc)) 
    return vme_TclError(interp,vmeErrorString(rc),rc);
  Tcl_SetResult(interp,vme_Itoa(rc),TCL_VOLATILE);
  return TCL_OK;
}

int vme_TclStrReturn(Tcl_Interp* interp,char* str)
{
  enum vme_error err;
  if (vmeErrorState() != VME_OK)
    {
      err = vmeGetError();
      return vme_TclError(interp,vmeErrorString(err),err);
    }
  Tcl_SetResult(interp,str,TCL_VOLATILE);
  return TCL_OK;
}


