#include <stdio.h>
#include <sys/types.h>
#include <time.h>

#ifdef vxworks
#include <types/vxTypesOld.h>
#endif

#include "rpc.h"
#include "unmpre.h"
#include "front.h"
#include "low.h"
#include "error.h"

extern vme_card vme_pre;

extern vme_register daq_led;
extern vme_register pm_int_dne;
extern vme_register pm_clr_tranfer_dne;
extern vme_register pm_en_fast_ctl;
extern vme_register pm_int_timeout;

/*---------------------------------------------------------------*/
/* VxWorks routine to acquire one event (or to read registers)   */
/* Parameters :  c       = daq card number                       */
/*               opcode  = opcode for command to be sent. Should */
/*                         be READEVENT or READREG.  If it is    */
/*                         zero, then no command is sent out     */
/*               nled    = how many ON-OFF cycles of the LED bit */
/*                         to do.  Each cycle will result in     */
/*                         (possibly) a CALSTROBE and (always)   */
/*                         a L1ACCEPT.                           */
/*               chamsk  = channel mask for READEVENT or READREG */
/*               wrdcnt  = wordcount for READEVENT or READREG    */
/*               add     = address for READEVENT or READREG      */
/*               noop    = 1 to send a NOOP at the beginning     */
/*                                                               */
/* Returns null-terminated OK or error message                   */
/*---------------------------------------------------------------*/
char* eventseq(int c,      int opcode, int nled, int chamsk, 
               int wrdcnt, int add,    int noop) {

  vme_register local_timeout_int =   pm_int_timeout;
  vme_register local_daq_led     =   daq_led;
  vme_register local_done_int    =   pm_int_dne;
  vme_register local_done_rst    =   pm_clr_transfer_dne;
  vme_register local_fctrl_en    =   pm_en_fast_ctl;
  static char return_message[200];
  vme_rc dummy, bit_value;
  vme_vmetype data[32];
  int i;
  int j,n;
  time_t now, then;
  double elapsed_time;  /* in sec */
  double timeout=1.0;   /* in sec */
  int fctrl_set;



if (debugging) printf("eventseq: card %d opcode 0x%x nled %d chamsk 0x%x wrdcnt %d add 0x%x noop %d \n",
		      c,opcode,nled,chamsk,wrdcnt,add,noop);
  
/* check that the card number is legal   */
  if (c < 0 || c>= local_daq_led->card->bases) {
    if (debugging) printf("eventseq: Illegal card number = %d \n",c);
    sprintf(return_message,"eventseq: Illegal card number = %d",c);
    return return_message;
  }

/* send NOOP if required                  */
  if (noop == 1) {
    dummy = vmePmCommand(c, 0, 0, chamsk, 0, 1, data);
    if (debugging) printf("eventseq: NOOP returns %d \n",dummy);
  }
/* fire LED as many times as necessary    */
  for (i=0; i<nled; i++) {  

/* Add delay on the 2nd, 3rd, 4th firing of the LED.  This is because       */
/* otherwise the delay generator used for the L1Accept, which is fired      */
/* using the LED firing bit, will only generate one L1Accept                */
/* Claudio, 5 february 1998                                                 */
/* NB: maybe this explains why the LBL people need the delay below ???      */
/* This delay is approx equivalent to a 35 microsecond delay                */ 
    if (i != 0) {
      for (j=0; j<200; j++) {n=j*j;}
    }

    dummy = local_daq_led->set(local_daq_led->card->base[0],local_daq_led);
    if (debugging) printf("eventseq: set_led returns %d \n",dummy);

/* create a delay between cal_strobe, l1_accept and read event               */
/* This was found necessary at LBL by Armin Karcher and Marzio Pedrali_Noy   */
/* I (CC) don't find it that I need it, so at the moment it is commented out */
/*          for (j=0; j<700; j++) {n=j*j;}                                   */

    dummy = local_daq_led->clear(local_daq_led->card->base[0],local_daq_led);
    if (debugging) printf("eventseq: clear_led returns %d \n",dummy);
  }
/* send READEVENT or READREG              */
  if (opcode != 0) {
    dummy = vmePmCommand(c, opcode, add, chamsk, wrdcnt, 0, data);
    if (debugging) 
      printf("eventseq: READEVENT or READREG returns %d \n",dummy);
}

/* - - - - - - - - - - - - - - - - - - - - - - - */
/* now the contorsions to get the data in....    */
/* lifted from cal_move__into_vram               */
/* - - - - - - - - - - - - - - - - - - - - - - - */
  then = time(NULL);
  fctrl_set = 0;

/* check done_int for a maximum amount of time  */
  for (;;) {
    bit_value = local_done_int->read(local_done_int->card->base[c],
				     local_done_int);
    if (debugging) printf("eventseq: done_int = %d \n",bit_value);
    if (bit_value == 1) break;
    now = time(NULL);
    if (difftime(now,then) > timeout) break;
  }

/* if the bit value is not one, try one more time  */
  if (bit_value != 1) {
     bit_value = local_done_int->read(local_done_int->card->base[c],
				      local_done_int);
    if (debugging) printf("eventseq: done_int = %d \n",bit_value);
  }


/* The following piece of code adds a delay that has been found to */
/* be necessary by Alberto Lusiani.  I (Claudio C.) do not find it */
/* necessary, so for the moment it is commented out.  Should you   */
/* get errors that cause the ROM to be stuck, you may want to      */
/* uncomment the folowing piece of code......                      */
/* for (j=0; j<150; j++) {n=j*j;}                                  */

/* now set the timeout enable bit                   */
  if (bit_value != 1) {
    fctrl_set = 1;
    dummy = local_fctrl_en->set(local_fctrl_en->card->base[0],local_fctrl_en);
    if (debugging) printf("eventseq: set_fctrl_en returns %d \n",dummy);
  
/* keep checking done_int for a maximum amount of time   */
    then = time(NULL);
    for (;;) {
      bit_value = local_done_int->read(local_done_int->card->base[c],
				       local_done_int);
      if (debugging) printf("eventseq: done_int = %d \n",bit_value);
      if (bit_value == 1) break;
      now = time(NULL);
      if (difftime(now,then) > timeout) break;
    }
  }

/* if the bit value is not one, try one more time  */
  if (bit_value != 1) {
      bit_value = local_done_int->read(local_done_int->card->base[c],
				       local_done_int);
      if (debugging) printf("eventseq: done_int = %d \n",bit_value);
  }

/* clear the fctrl bit, if it had been set          */
  if (fctrl_set == 1) {
      dummy = local_fctrl_en->clear(local_fctrl_en->card->base[0],
				    local_fctrl_en);
      if (debugging) printf("eventseq: clear_fctrl_en returns %d \n",dummy);
  }


/* fatal timeout ????    */
  if (bit_value != 1) {
    if (debugging) printf("eventseq: Fatal timeout reading done int \n");
    strcpy(return_message, "Fatal timeout reading done_int");
    return return_message;
  }

/* check for timeout_int       */
  bit_value = local_timeout_int->read(local_timeout_int->card->base[c],
				      local_timeout_int);
  if (debugging) printf("eventseq: timeout_int = %d \n",bit_value);
  if (bit_value == 1) {
    if (debugging) printf("eventseq: timeout_int reports a timeout \n");
    strcpy(return_message,"timeout_int reports a timeout");
    return return_message;
  }

/* set done_rst                 */
    dummy = local_done_rst->set(local_done_rst->card->base[0],local_done_rst);
    if (debugging) printf("eventseq: set_done_rst returns %d \n",dummy);

/* done                         */
  if (debugging) printf("eventseq: OK\n"); 
  strcpy(return_message,"OK");
  return return_message;
}

