/* -*- C -*- */

/* 
 * VME Teststand Software
 *
 * Code revised by Armin Karcher (karcher@design.lbl.gov)
 *
 * Design and Template by:
 *   Eric Gottlieb  (egott@unm.edu)
 *   Tim Thomas     (thomas@hev1.phys.unm.edu)
 *   Micheal Gold   (gold@hepv1.phys.unm.edu)
 *
 */

#ifdef vxworks
#include <semLib.h>
#endif

#include <rpc/rpc.h>
#include "error.h"
#include "semaphore.h"
#include "backrpc.h"
#include "executor.h"

struct vme_execparams VME_ExecParams;

#ifdef vxworks

static void sleep(int seconds) 
{
      taskDelay(seconds*sysClkRateGet());
}

#endif /* vxworks */

#define MAXAMOUNT 100
  
static vme_data fakedata(int amount,int x)
{
  static struct vme_data_item items[MAXAMOUNT];
  vme_data data;
  int k;

  data.vme_data_len = amount;
  data.vme_data_val = items;

  for (k = 0; k < amount; k++)
    {
      items[k].counts = (k+1)*x;
      items[k].address = k+1;
    }
  return data;
}

void vmeExecutor()
{
  char* whoiam = "executor.c: vmeExecutor";  /* for fatal error messages */
  struct vme_execparams myParams;   /* for a local copy of the param structure */
  int events = 0;                   /* a counter */
  CLIENT* clnt = NULL;              /* the rpc client structure .... needed for doing rpc's */

  strcpy(myParams.server,"");       /* this will be caught by the server name check below */
#ifdef vxworks
  rpcTaskInit();                    /* this needs to be done under vxworks for any rpc using task */
#endif

  printf("Executor has begun\n");
  do
    {
				/************************************************************************/
      vmeSemLock(&VME_ExecParams.semaphore);    /**************** LOCK VME_ExecParams   *****************/
				/************************************************************************/
      if (VME_ExecParams.changed)
	{
	  if (strcmp(myParams.server,VME_ExecParams.server)) /* server name has changed */
	    {
	      if (clnt != NULL)                              /* have we already created a client? */
		{
		  clnt_destroy(clnt);
		}
	      strcpy(myParams.server,VME_ExecParams.server);
	      clnt = clnt_create(myParams.server, BACKSERVER_PROGRAM, BACKSERVER_VERSION, "tcp");
	      if (clnt == NULL)                              /* did we sucsesfuly create a client? */
		{
		  clnt_pcreateerror(myParams.server);
		  VME_ExecParams.running = 0;
		  VME_ExecParams.changed = 1;
		  vmeSemUnlock(&VME_ExecParams.semaphore);   /* since we are about to have a fatal error */
		  vmeFatalError(whoiam,VME_NO_CLIENT);
		}
	    }
	  myParams.events = VME_ExecParams.events;
	  VME_ExecParams.changed = 0;
	}
				/************************************************************************/
      vmeSemUnlock(&VME_ExecParams.semaphore);  /**************** UNLOCK VME_ExecParams *****************/
				/************************************************************************/

      /****************************************/
      /* now get data and send it on it's way */
      /****************************************/

      printf("Executor: events = %d\n",events);
/*      vmehello_1("hello from the executor\n",clnt); /* just say hello...be freindly like.... */
      vmesenddata_1(fakedata(10,events+1),clnt);    /* send some fake data */
      events++;
      sleep(3);
    }
  while (events < myParams.events);

  /**************************/
  /* clean up and shut down */
  /**************************/

				/********************************************************************/
  vmeSemLock(&VME_ExecParams.semaphore);    /**************** LOCK VME_ExecParams   *****************/
				/********************************************************************/
  VME_ExecParams.running = 0;
  VME_ExecParams.changed = 1;
				/********************************************************************/
  vmeSemUnlock(&VME_ExecParams.semaphore);  /**************** UNLOCK VME_ExecParams *****************/
				/********************************************************************/
  clnt_destroy(clnt);
  printf("Executor stoped after %d events\n",events);
}
  
