/* 
 * VME Teststand Software
 *
 * Code revised by Armin Karcher (karcher@design.lbl.gov)
 *
 * Design and Template by:
 *   Eric Gottlieb  (egott@unm.edu)
 *   Tim Thomas     (thomas@hev1.phys.unm.edu)
 *   Micheal Gold   (gold@hepv1.phys.unm.edu)
 * --------------------------------------------------------------------
 * Modified to include rpc support for motor control functions......
 *      Sanjay Hortikar (horti@physics.ucsb.edu)
 *______________________________________________________________________
 */


/* rpc.x : source file for rpcgen.new.
 *
 * CVS_ID = $Id: rpc.x,v 1.2 1995/05/10 15:52:04 egott Exp $
 *
 * generates rpc_svc.c, rpc_clnt.c, rpc_xdr.c, and rpc.h
 * 
 * because it is simpler to do so I am just going ahead and using
 * int's instead of vme_rc's, enum vme_resources, and the like.
 *
 */


#ifndef VME_SERVER_ID
#define VME_SERVER_ID 0x20002222
#endif

struct vme_rpc_return {
  int value;
  int error;
};

struct vme_rpc_chip_reg {
  int value;
  int error;
  unsigned long data<32>;
};

#define MAX_BUFF_LEN 8192

struct vme_rpc_buffer {
  int value;
  int error;
  unsigned long data<MAX_BUFF_LEN>;
};

struct int_vme_rpc_buffer {
  int value;
  int error;
  int data<MAX_BUFF_LEN>;
};


#define MAX_STATE_LEN 512

struct vme_rpc_state {
  int error;
  unsigned long values<MAX_STATE_LEN>;
  int reglist<MAX_STATE_LEN>;
};

struct vme_user_params
{
  int events;
  string server<256>;
};

struct md_ret {
	string reply<256>;
};


program TESTSTAND_PROGRAM {
  version TESTSAND_VERSION {
    vme_rpc_return VMEWRITE(int,int,int) = 2;
    vme_rpc_return VMEREAD(int,int) = 3;
    vme_rpc_return VMESET(int,int) = 4;
    vme_rpc_return VMECLEAR(int,int) = 5;
    vme_rpc_return VMETEST(int,int) = 6;
    vme_rpc_return VMEFILEWRITE(int,int,int,string) = 7;
    vme_rpc_return VMEFILEREAD(int,int,int,int,string) = 8;
    vme_rpc_return VMEBUFFWRITE(int,int,vme_rpc_buffer) = 9;
    vme_rpc_return VMELOCKRESOURCE(int,int) = 11;
    vme_rpc_return VMEUNLOCKRESOURCE(int,int) = 12;
    vme_rpc_return VMEFRONTDISPLAY(string) = 13;
    vme_rpc_return VMEADDCARD(int, string, int) = 15;
    vme_rpc_return VMEADDCARDTYPE(string) = 16;
    vme_rpc_return VMEWAIT(int,int,int,int) = 17;
    vme_rpc_return VMEVMEWRITE(int,int,int) = 18;
    vme_rpc_return VMEVMEREAD(int,int) = 19;
    vme_rpc_return VMEDEBUG(int) = 20;
    vme_rpc_return VMERESETFRONT(string) = 21;
    vme_rpc_return VMEPMCOMMAND(int,int,int,int,int,int,vme_rpc_chip_reg) = 22;
    int MEXEC(int,string) = 23;
    int MSTAT(int)=24;
    md_ret IPORT(string,int)= 25;
    int OPORT(string,string,int)=26;
    md_ret EVENTSEQ(int,int,int,int,int,int,int)=27;
    vme_rpc_return VMEPMCOMMANDALL(int,int,int,int_vme_rpc_buffer,int_vme_rpc_buffer,int,int,vme_rpc_buffer)=28;
    vme_rpc_buffer VMEBUFFREAD(int,int,int) = 100;
    vme_rpc_state VMEGETSTATE(int,int) = 101;
    vme_rpc_buffer VMEVMEBUFFREAD(int,int,int) = 102;
    vme_rpc_buffer VMEVMEBUFFREAD2(int,int,int) = 103;

    vme_rpc_return VMEEXECCONTROL(vme_user_params) = 150;

    string VMEFRONTERROR(int) = 200;

    vme_rpc_return VMEADDREGISTER(string, string, int, int, int, int, int, int, int) = 250;

  } = 1;
} = VME_SERVER_ID;

/* VME_SERVER_ID is a macro variable which is supplied by
 * the makefile.  While this software is under development it
 * should be a number between 0x20000000 and 0x3fffffff
 *
 */


