/*  
 * VME Teststand Software
 *
 * Code revised by Armin Karcher (karcher@design.lbl.gov)
 *
 * Design and Template by:
 *   Eric Gottlieb  (egott@unm.edu)
 *   Tim Thomas     (thomas@hev1.phys.unm.edu)
 *   Micheal Gold   (gold@hepv1.phys.unm.edu)
 *
 * --------------------------------------------------------------------
 * Modified to include rpc support for motor control functions......
 *      Sanjay Hortikar (horti@physics.ucsb.edu)
 *______________________________________________________________________
 */
#include "vxWorks.h"
#include "taskLib.h"
#include <ioLib.h>
#define IN 0x800
#define OUT 0x804
#define CR 13
#define DELAY (0.1)
#include <stdio.h>
#include "rpc.h"
#include "error.h"
#include "reglist.h"
#include "front.h"
#include "lock.h"
#include "server_date.h"
#include <ctype.h>
#include <string.h>

char* eventseq(int, int, int, int, int, int, int);

/*
 * _server.c
 * ---------
 *
 * These routines are automatically called by _svc.c code (which is generated by
 * rpcgen.new) in response to a call from the client code:
 *
 * vmeDoIt --> vmedoit_1 -------RPC-----> vmedoit_1_svc --> vmeDoIt
 * backend       _client.c   _clnt.c   _svc.c       _server.c      frontend
 *
 * file.x, file_client.c, and file_server.c must be maintained by hand.
 * file_clnt.c, file_svc.c, file_xdr.c, and file.h come from file.x and rpcgen.new.
 *
 * _server.c routines must return a pointer to a static object.  RPCGEN.NEW requires 
 * a pointer and a dynamic object would not retain it's value for long after the 
 * context in which it was created disapeared (ie: the procedure returned).
 *
 */

#ifdef vxworks
static int getpid () {
  return taskIdSelf();
}
#endif

static struct vme_rpc_return *passback(int value){
  static struct vme_rpc_return r;
  if (vmeErrorState() == 0)
    r.error = 0;
  else
    r.error = getpid();
  r.value = value;
  return &r;
}

vme_rpc_return *vmelockresource_1_svc(int resource, int resource_key, struct svc_req *rqstp){
  static int result;
  result = vmeLockResource(resource,resource_key);
  return passback(result);
}

vme_rpc_return *vmeunlockresource_1_svc(int resource, int resource_key, struct svc_req *rqstp){
  static int result;
  result = vmeUnlockResource(resource,resource_key);
  return passback(result);
}

vme_rpc_return *vmeresetfront_1_svc(char* server_creation_date, struct svc_req *rqstp){
  static int result;
  result = vmeResetFront();
  if (strcmp(server_creation_date,VME_ServerCompileDate))
    {
      if (debugging) 
	{ 
	  printf("server date (client) = %s\n",server_creation_date);
	  printf("server date (server) = %s\n",VME_ServerCompileDate);
	}
      vmeErrorReturn1("Server and Client are out of sync.  Proceed at own risk.");
    }
  return passback(result);
}

vme_rpc_return *vmewrite_1_svc(int c, int rindex, int v,  struct svc_req *rqstp){
  static int  result;
  result = vmeWrite(c,VME_Reg.list[rindex],(vme_vmetype)v);
  return passback(result);
}

vme_rpc_return *vmeread_1_svc(int c, int rindex,  struct svc_req *rqstp){
  static int  result;
  result = vmeRead(c,VME_Reg.list[rindex]);
  return passback(result);
}

vme_rpc_return *vmeset_1_svc(int c, int rindex,  struct svc_req *rqstp){
  static int  result;
  result = vmeSet(c,VME_Reg.list[rindex]);
  return passback(result);
}

vme_rpc_return *vmeclear_1_svc(int c, int rindex,  struct svc_req *rqstp){
  static int  result;
  result = vmeClear(c,VME_Reg.list[rindex]);
  return passback(result);
}

vme_rpc_return *vmetest_1_svc(int c, int rindex,  struct svc_req *rqstp){
  static int  result;
  result = vmeTest(c,VME_Reg.list[rindex]);
  return passback(result);
}

vme_rpc_return *vmefrontdisplay_1_svc(char* mess, struct svc_req *rqstp){
  static int result;
  result = vmeFrontDisplay(mess);
  return passback(result);
}

vme_rpc_return *vmefilewrite_1_svc(int c, int rindex,int add, char* fname, struct svc_req *rqstp){
  static int result;
  result = vmeFileWrite(c,VME_Reg.list[rindex],add,fname);
  return passback(result);
}

vme_rpc_return *vmefileread_1_svc(int c, int rindex,int add, int len, char* fname, struct svc_req *rqstp){
  static int result;
  result = vmeFileRead(c,VME_Reg.list[rindex],add,len,fname);
  return passback(result);
}

vme_rpc_return *vmepmcommand_1_svc(int c,int opcd,int addr,int chan,int cnt,int dir, vme_rpc_chip_reg chip_reg, struct svc_req *rqstp){
  static int result;
  result = vmePmCommand(c, opcd, addr, chan, cnt, dir, chip_reg.data.data_val);
  return passback(result);
}


vme_rpc_return *vmepmcommandall_1_svc(int c, int opcd, int nchips, int_vme_rpc_buffer addr_out, int_vme_rpc_buffer chan_out, int cnt, int dir, vme_rpc_buffer data_out, struct svc_req *rqstp) {
  static int result;
  result = vmePmCommandAll(c, opcd, nchips, addr_out.data.data_val, chan_out.data.data_val, cnt, dir, data_out.data.data_val);
  return passback(result);
}

vme_rpc_return *vmebuffwrite_1_svc(int c, int rindex, vme_rpc_buffer buff, struct svc_req *rqstp){
  static int result;
  result = vmeBuffWrite(c,VME_Reg.list[rindex],buff.data.data_val,buff.data.data_len);
  return passback(result);
}

/* *** cheap hack alert ***
 *
 * attempts to read buffers larger than maxtempbuff will result in fireworks.
 * ultimately a combination of length and some field in VME_Reg.list[rindex]
 * should be used to determine an appropriate maximum array size which then
 * should be allocated and assined to a static pointer.  Before allocating the
 * array though, the pointer should be checked (==NULL?)....if an array has
 * previously been allocated it should be deallocated and then the new one
 * created.
 *
 * alternativly maxtempbuff should be made sure to be big enough.
 *
 */

#define MAXTEMPBUFF 20000

vme_rpc_buffer *vmebuffread_1_svc(int c, int rindex, int length, struct svc_req *rqstp){
  static vme_rpc_buffer buff;
  static vme_vmetype tempbuff[MAXTEMPBUFF];
  buff.data.data_val = tempbuff;
  buff.value = vmeBuffRead(c,VME_Reg.list[rindex],buff.data.data_val,length);
  if (vmeErrorState() != 0) 
    {
      buff.data.data_len = 0;
      buff.error = getpid();
    } 
  else 
    {
      buff.error = 0;
      buff.data.data_len = buff.value;
    }
  return(&buff);
}

vme_rpc_buffer *vmevmebuffread_1_svc(int base, int offset, int length, struct svc_req *rqstp) {
    static vme_rpc_buffer buff;
    static vme_vmetype tempbuff[MAXTEMPBUFF];
    buff.data.data_val = tempbuff;
    buff.value = vmeVmeBuffRead((vme_vmetype*)base, offset, buff.data.data_val,length);
    if (vmeErrorState() != 0) {
	buff.data.data_len = 0;
	buff.error = getpid();
    } else {
	buff.error = 0;
	buff.data.data_len = buff.value;
    }
    return (&buff);
}


vme_rpc_buffer *vmevmebuffread2_1_svc(int base, int offset, int length, struct svc_req *rqstp) {
    static vme_rpc_buffer buff;
    static vme_vmetype tempbuff[MAXTEMPBUFF];
    buff.data.data_val = tempbuff;
    buff.value = vmeVmeBuffRead2((vme_vmetype*)base, offset, buff.data.data_val,length);
    if (vmeErrorState() != 0) {
	buff.data.data_len = 0;
	buff.error = getpid();
    } else {
	buff.error = 0;
	buff.data.data_len = buff.value;
    }
    return (&buff);
}

vme_rpc_state *vmegetstate_1_svc(int c, int cardtype, struct svc_req *rqstp)
{
  static vme_rpc_state state;
  static int indexlist[512];
  static vme_vmetype values[512];
  int i;
  vme_rc rc;
  vme_register registers[512];
  state.values.values_val = values;
  state.reglist.reglist_val = indexlist;
  rc = vmeGetState(c, cardtype, registers, state.values.values_val);
  if (vmeErrorState() != 0) {
    state.error = getpid();
    state.values.values_len = 0;
    state.reglist.reglist_len = 0;
  } else {
    state.error = 0;
    for (i = 0; i < rc; i++) 
      state.reglist.reglist_val[i] = registers[i]->index;
    state.values.values_len = rc;
    state.reglist.reglist_len = rc;
  }
  return (&state);
}

vme_rpc_return *vmeaddcard_1_svc(int classindex, char* instname, int base, struct svc_req *rqstp)
{
  static int result;
  result = (int)vmeAddCard(VME_Cards[classindex], instname, (vme_vmetype*)base);
  return passback(result);
}

vme_rpc_return *vmeaddcardtype_1_svc(char* name, struct svc_req *rqstp)
{
  static int result = 0;
  vmeAddCardType(name);
  return passback(result);
}

vme_rpc_return *vmewait_1_svc(int card, int rindex, int pole, int seconds,  struct svc_req *rqstp)
{
  static int result;
  result = vmeWait(card,VME_Reg.list[rindex],(vme_vmetype) pole,seconds);
  return passback(result);
}

char **vmefronterror_1_svc(int pid, struct svc_req *rqstp)
{
  static char buff[255];
  static char* str;
  str = buff;
  strcpy(str,vmeFrontError(pid)); /* I guess this is a little silly...still... */
  return &str;
}

vme_rpc_return *vmevmewrite_1_svc(int base, int offset, int value, struct svc_req *rqstp)
{
  static int result;
  result = vmeVmeWrite((vme_vmetype*)base,offset,(vme_vmetype)value);
  return passback(result);
}

vme_rpc_return *vmevmeread_1_svc(int base, int offset, struct svc_req *rqstp)
{
  static int result;
  result = vmeVmeRead((vme_vmetype*)base,offset);
  return passback(result);
}

vme_rpc_return *vmedebug_1_svc(int level, struct svc_req *rqstp)
{
  static int result;
  printf(" Debugging set to %d.\n",level);
  result = vmeDebug(level);
  return passback(result);
}




vme_rpc_return *vmeexeccontrol_1_svc(struct vme_user_params p, struct svc_req *rqstp)
{
  static int result;

  if (debugging) 
    {
      printf("server: vmeExecControl\n");
      printf("events = %d and server = %s\n",p.events,p.server);
    }
  result = vmeExecControl(p);
  return passback(result);
}

vme_rpc_return *vmeaddregister_1_svc(char* name, char* class, int type, int access, 
		 int offset, int mask, int vmask, int shift, int card,  struct svc_req *rqstp)
{
  static int result = 0; /* doesn't return anything */
  
  vme_register dummy;
  
  if (debugging)
    {
      printf("server: adding a register");
    }

  vmeAddRegister(&dummy, name, class, (enum vme_regtype)type, (enum vme_access)access, offset, (vme_vmetype)mask, (vme_vmetype)vmask, shift, VME_Cards[card]);

  return passback(result);
}

/*****************************************************************************
 * Motor control functions.... 
 * ***************************************************************************
 */ 

int * mexec_1_svc(int mbase,char * buff, struct svc_req *rqstp)
{  static int value=0;
   int echo;
   echo = 1;
   if(debugging) {
     printf("%s\n",buff);
     echo = 0;
   }
   if(wrcmd(mbase,buff,strlen(buff),echo)==0){
       value =0;
       return &value;
  }
  value=-1;
  return &value;
}

#define STEP_BASE 0x800000
#define STAT_OFFSET 0x100
#define A24_OFFSET 0xe0000000

int * mstat_1_svc( int axis,struct svc_req *rqstp)
{  static int value=0; 
   short * statp;
   statp = (short *) ( A24_OFFSET + STEP_BASE + (axis-1) * STAT_OFFSET+2);
   value = *(statp);
   return &value;
}

md_ret * iport_1_svc(char * port,int mode,struct svc_req *rqstp)
{ static md_ret dummy;
  int i,fd,len=0,options;
  strcpy(dummy.reply," ");
  fd = open(port,O_RDWR,mode);
  ioctl(fd,FIOBAUDRATE,9600);
  if(fd ==ERROR){
      printf("Can't open serial port\n");
      return 0;
  }
  options=ioctl(fd,FIOGETOPTIONS,0);
  if(debugging) printf("iport-options: %d\n",options);
  ioctl(fd,FIONREAD,(int)&len);
  if(debugging) printf("reply length:%d\n",len);
  if(len !=0)
    len=read(fd,dummy.reply,len);
  dummy.reply[len]='\0';
  for(i=0;i<len;++i){
    if(debugging) printf("%c",dummy.reply[i]);
    if(dummy.reply[i]=='\r') dummy.reply[i]='\0';
  }
  ioctl(fd,FIORFLUSH,0);
  close(fd); 
 return &dummy;
}
 



int * oport_1_svc(char *data,char *port, int mode, struct svc_req *rqstp)
{ 
  int fd;
  static int len=0;
  fd = open(port,O_RDWR,mode);
  if(fd ==ERROR){
      printf("Can't open serial port\n");
      return 0;
  }
  ioctl(fd,FIOBAUDRATE,9600);
  while(data[len]!='\0') len++;
  if(debugging) printf("server:data %s",data);
  len =write(fd,(char *)data,len-1);
 /* ioctl(fd,FIOWFLUSH,0);
  */ 
 close(fd); 
  return &len; 
}

/******************************************************************
 *end of motor control functions
 *******************************************************************
 */

md_ret*  eventseq_1_svc(int c,      int opcode, int nled, int chamsk,
           int wrdcnt, int add, int noop, struct svc_req *rqstp) {
  static md_ret dummy;
  dummy.reply = eventseq(c,opcode,nled,chamsk,wrdcnt,add,noop);
  return &dummy;
}


#ifdef vxworks

/* keep in mind that any special initialization done here will not get done
 * for server not running under vxworks.... a hack around for that can
 * be done in vmeResetFront above.
 */

void vmeServerBegin (){
  vme_rc rc;
  rpcTaskInit();
  /* vmeHandleBusErrors();   /* bus errors should be handled by our error scheme */
  /* bus error stuff doesn't work yet */
  svcBegin();
}

#endif














