/* -*- C -*- */

/* 
 * VME Teststand Software
 *
 * Code revised by Armin Karcher (karcher@design.lbl.gov)
 *
 * Design and Template by:
 *   Eric Gottlieb  (egott@unm.edu)
 *   Tim Thomas     (thomas@hev1.phys.unm.edu)
 *   Micheal Gold   (gold@hepv1.phys.unm.edu)
 *
 */

#include "error.h"
#include "semaphore.h"

#ifdef vxworks

void vmeSemLock(SEM_ID* semaphore) 
{
  static char* whoiam = "semaphore.c : vmeSemLock";

  if (debugging) 
    {
      printf("vmeSemLock: *semaphore = %d\n",(int)*semaphore);
    }
  if (!*semaphore) 
    {
      *semaphore = semMCreate(SEM_Q_FIFO);
      if ((int)*semaphore == ERROR) 
	{
	  vmeFatalError(whoiam,VME_COULDNT_MAKE_SEM);
	}
    }
  if (semTake(*semaphore,120) == ERROR) 
    {
      vmeFatalError(whoiam,VME_COULDNT_TAKE_SEM); 
    }
}

void vmeSemUnlock(SEM_ID* semaphore) {
  static char* whoiam = "semaphore.c : vmeSemUnlock";

  if (debugging) 
    {
      printf("vmeSemUnLock: *semaphore = %d\n",(int)*semaphore);
    }
  if (!*semaphore) 
    {
      vmeFatalError(whoiam,VME_SEMAPHORE_DNE);
    }
  if (semGive(*semaphore) == ERROR) 
    {
      vmeFatalError(whoiam,VME_COULDNT_GIVE_SEM);
    }
}

#else /* not vxworks */

void vmeSemLock(SEM_ID* semaphore) {}
void vmeSemUnlock(SEM_ID* semaphore) {}

#endif
