/* 
 * DAQ Teststand 
 *
 * Code by Armin Karcher (karcher@design.lbl.gov)
 */

#include <stdio.h>
#include <sys/types.h>
#ifdef vxworks
#include <types/vxTypesOld.h>
#endif

#include "unm604.h"
#include "front.h"
#include "low.h"

vme_card mvme604;

vme_register mvme_robn;
vme_register mvme_dwb;
vme_register mvme_lvfair;
vme_register mvme_lvrwd;
vme_register mvme_lvreql;
vme_register mvme_dhal;
vme_register mvme_den;
vme_register mvme_dtbl;
vme_register mvme_dfair;
vme_register mvme_drelm;
vme_register mvme_drelq;
vme_register mvme_inte;
vme_register mvme_vinc;
vme_register mvme_linc;
vme_register mvme_tvme;
vme_register mvme_d16;
vme_register mvme_blk;
vme_register mvme_d64;
vme_register mvme_vme_am;
vme_register mvme_ladrr_ctr;
vme_register mvme_vaddr_ctr;
vme_register mvme_byte_ctr;
vme_register mvme_tbl_ctr;
vme_register mvme_done;
vme_register mvme_arbto;
vme_register mvme_time_off;
vme_register mvme_time_on;
vme_register mvme_vgto;
vme_register mvme_vato;
vme_register mvme_lbto;
vme_register mvme_wdto;
vme_register mvme_prescale;
vme_register mvme_tt1_comp;
vme_register mvme_tt1_ctr;
vme_register mvme_tt2_comp;
vme_register mvme_tt2_ctr;
vme_register mvme_tt1_ovf;
vme_register mvme_tt1_covf;
vme_register mvme_tt1_coc;
vme_register mvme_tt1_en;
vme_register mvme_tt2_ovf;
vme_register mvme_tt2_covf;
vme_register mvme_tt2_coc;
vme_register mvme_tt2_en;



void vmeAdd604Registers()
{

/* these constants are just for conveniance and to make the code below more readable */
  const int FIFO = VME_FIFO;
  const int CONTIG = VME_CONTIG;
  const int WORD = VME_WORD;
  const int BITS = VME_BITS;
  const int BIT = VME_BIT;

  const int RO = VME_R_ACCESS;
  const int WO = VME_W_ACCESS;
  const int RW = VME_RW_ACCESS;
  const int WV = VME_WV_ACCESS;
  const int RWV = VME_RW_ACCESS | VME_V_ACCESS;
  const int RD = VME_RD_ACCESS;

/* vmeAddReg&register, name, class, type, access, offset, mask, vmask, shift, card) */


  vmeAddReg(&mvme_robn,   "robn",    "arbit", BIT ,RW, 12, 0x00008000, 0x0000,  0, mvme604);
  vmeAddReg(&mvme_dwb,    "dwb",     "arbit", BIT ,RW, 12, 0x00002000, 0x0000,  0, mvme604);
  vmeAddReg(&mvme_lvfair, "lvfair",  "arbit", BIT ,RW, 12, 0x00000800, 0x0000,  0, mvme604);
  vmeAddReg(&mvme_lvrwd,  "lvrwd",   "arbit", BIT ,RW, 12, 0x00000400, 0x0000,  0, mvme604);
  vmeAddReg(&mvme_lvreql, "lvreql",  "arbit", BITS,RW, 12, 0x00000300, 0x0000,  8, mvme604);
  vmeAddReg(&mvme_dhal,   "dhal",    "dma_cr1", BIT ,RW, 12, 0x00000080, 0x0000,  0, mvme604);
  vmeAddReg(&mvme_den,    "den",     "dma_cr1", BIT ,RW, 12, 0x00000040, 0x0000,  0, mvme604);
  vmeAddReg(&mvme_dtbl,   "dtbl",    "dma_cr1", BIT ,RW, 12, 0x00000020, 0x0000,  0, mvme604);
  vmeAddReg(&mvme_dfair,  "dfair",   "dma_cr1", BIT ,RW, 12, 0x00000010, 0x0000,  0, mvme604);
  vmeAddReg(&mvme_drelm,  "drelm",   "dma_cr1", BITS,RW, 12, 0x0000000c, 0x0000,  2, mvme604);
  vmeAddReg(&mvme_drelq,  "drelq",   "dma_cr1", BITS,RW, 12, 0x00000003, 0x0000,  0, mvme604);

  vmeAddReg(&mvme_inte, "inte",   "dma_cr2", BIT ,RW, 13, 0x00008000, 0x0000,  0, mvme604);
  vmeAddReg(&mvme_vinc, "vinc",   "dma_cr2", BIT ,RW, 13, 0x00000800, 0x0000,  0, mvme604);
  vmeAddReg(&mvme_linc, "linc",   "dma_cr2", BIT ,RW, 13, 0x00000400, 0x0000,  0, mvme604);
  vmeAddReg(&mvme_tvme, "tvme",   "dma_cr2", BIT ,RW, 13, 0x00000200, 0x0000,  0, mvme604);
  vmeAddReg(&mvme_d16,  "d16",    "dma_cr2", BIT ,RW, 13, 0x00000100, 0x0000,  0, mvme604);
  vmeAddReg(&mvme_blk,  "blk",    "dma_cr2", BIT ,RW, 13, 0x00000080, 0x0000,  0, mvme604);
  vmeAddReg(&mvme_d64,  "d64",    "dma_cr2", BIT ,RW, 13, 0x00000040, 0x0000,  0, mvme604);
  vmeAddReg(&mvme_vme_am,  "vme_am", "dma_cr2", BITS,RW, 13, 0x0000003f, 0x0000,  0, mvme604);

  vmeAddReg(&mvme_ladrr_ctr,"ladrr",  "counters", WORD,RW, 14, 0xffffffff, 0x0000,  0, mvme604);

  vmeAddReg(&mvme_vaddr_ctr,"vaddr",  "counters", WORD,RW, 15, 0xffffffff, 0x0000,  0, mvme604);

  vmeAddReg(&mvme_byte_ctr, "byte",   "counters", WORD,RW, 16, 0xffffffff, 0x0000,  0, mvme604);

  vmeAddReg(&mvme_tbl_ctr,  "tbl addr","counters", WORD,RW, 17, 0xffffffff, 0x0000,  0, mvme604);

  vmeAddReg(&mvme_done,     "done", "dma_status", BIT ,RW, 18, 0x00000001, 0x0000,  0, mvme604);

  vmeAddReg(&mvme_arbto,    "arbto",   "timeout", BIT ,RW, 19, 0x01000000, 0x0000,  0, mvme604);
  vmeAddReg(&mvme_time_off, "time_off","timeout", BITS,RW, 19, 0x00e00000, 0x0000, 21, mvme604);
  vmeAddReg(&mvme_time_on,  "time_on", "timeout", BITS,RW, 19, 0x001c0000, 0x0000, 18, mvme604);
  vmeAddReg(&mvme_vgto,     "vgto",    "timeout", BITS,RW, 19, 0x00030000, 0x0000, 16, mvme604);
  vmeAddReg(&mvme_vato,     "vato",    "timeout", BITS,RW, 19, 0x0000c000, 0x0000, 14, mvme604);
  vmeAddReg(&mvme_lbto,     "lbto",    "timeout", BITS,RW, 19, 0x00003000, 0x0000, 12, mvme604);
  vmeAddReg(&mvme_wdto,     "wdto",    "timeout", BITS,RW, 19, 0x00000f00, 0x0000,  8, mvme604);
  vmeAddReg(&mvme_prescale, "prescale","counters",BITS,RW, 19, 0x000000ff, 0x0000,  0, mvme604);

  vmeAddReg(&mvme_tt1_comp, "tt1_comp","counters", WORD,RW, 20, 0xffffffff, 0x0000,  0, mvme604);

  vmeAddReg(&mvme_tt1_ctr,  "tt1_ctr", "counters", WORD,RW, 21, 0xffffffff, 0x0000,  0, mvme604);

  vmeAddReg(&mvme_tt2_comp, "tt2_comp","counters", WORD,RW, 22, 0xffffffff, 0x0000,  0, mvme604);

  vmeAddReg(&mvme_tt1_ctr,  "tt2_ctr", "counters", WORD,RW, 23, 0xffffffff, 0x0000,  0, mvme604);

  vmeAddReg(&mvme_tt2_ovf, "tt2_ovf", "counters",BITS,RO, 24, 0x0000f000, 0x0000,  0, mvme604);
  vmeAddReg(&mvme_tt2_covf,"tt2_covf","counters",BIT ,WO, 24, 0x00000400, 0x0000,  0, mvme604);
  vmeAddReg(&mvme_tt2_coc, "tt2_coc", "counters",BIT ,RW, 24, 0x00000200, 0x0000,  0, mvme604);
  vmeAddReg(&mvme_tt2_en,  "tt2_en",  "counters",BIT ,RW, 24, 0x00000100, 0x0000,  0, mvme604);
  vmeAddReg(&mvme_tt1_ovf, "tt1_ovf", "counters",BITS,RO, 24, 0x000000f0, 0x0000,  0, mvme604);
  vmeAddReg(&mvme_tt1_covf,"tt1_covf","counters",BIT ,WO, 24, 0x00000004, 0x0000,  0, mvme604);
  vmeAddReg(&mvme_tt1_coc, "tt1_coc", "counters",BIT ,RW, 24, 0x00000002, 0x0000,  0, mvme604);
  vmeAddReg(&mvme_tt1_en,  "tt1_en",  "counters",BIT ,RW, 24, 0x00000001, 0x0000,  0, mvme604);

 }

