/* 
 * Daq-Board Teststand Software
 *
 *
 *Code by:
 *   Armin Karcher (karcher@design.lbl.gov)
 * Design and Template by
 *   Eric Gottlieb  (egott@unm.edu)
 *   Tim Thomas     (thomas@hev1.phys.unm.edu)
 *   Micheal Gold   (gold@hepv1.phys.unm.edu)
 *
 */

#include <stdio.h>
#include <sys/types.h>
#ifdef vxworks
#include <types/vxTypesOld.h>
#endif

#include "unmdaq.h"
#include "front.h"
#include "low.h"


vme_card vme_daq;

vme_register daq_dram;
vme_register daq_vram1;
vme_register daq_vram2;
vme_register daq_board_reset;
vme_register daq_hreset;
vme_register daq_cpu_gbl;
vme_register daq_cpu_smi;
vme_register daq_pm_type;
vme_register daq_int_ppc_en;
vme_register daq_int_vme_en;
vme_register daq_vram_xfer;
vme_register daq_vram_dir;
vme_register daq_busy;
vme_register daq_sclk_enable;
vme_register daq_vme_int;
vme_register daq_inc_qc;
vme_register daq_dec_qc;
vme_register daq_led;
vme_register daq_send_sync;
vme_register daq_en_fast_ctl;
vme_register daq_en_model;
vme_register daq_vram_full_pointer;
vme_register daq_vram_af_level;
vme_register daq_vram_page;
vme_register daq_fifo_af_level;
vme_register daq_fifo_full_level;
vme_register daq_vram_full;
vme_register daq_vram_af;
vme_register daq_vram_empty; 
vme_register daq_pm_full;
vme_register daq_pm_af;
vme_register daq_fifo_full;
vme_register daq_fifo_af;
vme_register daq_fifo_empty; 
vme_register daq_uart_int;
vme_register daq_timer2_int;
vme_register daq_timer1_int;
vme_register daq_timer0_int;
vme_register daq_pm_int;
vme_register daq_xfer_done;
vme_register daq_fast_int;
vme_register daq_pm_full_int;
vme_register daq_vram_full_int;
vme_register daq_fifo_full_int;
vme_register daq_ppc_mail_int;
vme_register daq_msk_uart;  
vme_register daq_msk_timer2; 
vme_register daq_msk_timer1; 
vme_register daq_msk_timer0;
vme_register daq_msk_pm_int;
vme_register daq_msk_xfer_done;
vme_register daq_msk_fast_ctl;
vme_register daq_msk_pm_full;
vme_register daq_msk_vf;
vme_register daq_msk_ff;
vme_register daq_msk_ppc_mail;
vme_register daq_irq_vector;
vme_register daq_irq_level;
vme_register daq_vme_mailbox;
vme_register daq_ppc_mailbox;
vme_register daq_num_pages;
vme_register daq_num_buffs;   
vme_register daq_model_occ;   
vme_register daq_que_ctr; 
vme_register daq_curr_evt; 
vme_register daq_fast_ctl_timebase;
vme_register daq_fifo_read;
vme_register daq_timer0;
vme_register daq_timer1;   
vme_register daq_timer2; 
vme_register daq_timer_ctrl;  
vme_register daq_uart_ctrl_a;  
vme_register daq_uart_data_a;  
vme_register daq_uart_ctrl_b;  
vme_register daq_uart_data_b;  
vme_register pm_end_flag;  
vme_register pm_link_lock;  
vme_register pm_rx_idle;  
vme_register pm_tx_idle;  
vme_register pm_en_fast_ctl;  
vme_register pm_cmd_reg;  
vme_register pm_ch_msk;  
vme_register pm_wrd_cnt;  
vme_register pm_timeout;  
vme_register pm_int_dne; 
vme_register pm_int_timeout; 
vme_register pm_int_link_err; 
vme_register pm_msk_transfer_dne;  
vme_register pm_msk_timeout;  
vme_register pm_msk_link_error;  


#ifdef vxworks   
/*task delay is vxworks only, the rest will work on the backend */
vme_rc vmePmCommand(int c,int opcd,int addr,vme_vmetype chan,vme_vmetype cnt,int dir)
{
  vme_register cmd = pm_cmd_reg;
  vme_register msk = pm_ch_msk;
  vme_register wrd = pm_wrd_cnt;
  vme_register rw = daq_vram_dir;
  vme_register dne = pm_int_dne;
  vme_register tmout = pm_int_timeout;
  vme_rc dummy;
  vme_vmetype v;
  int tst = 0;
  if (c < 0 || c >= cmd->card->bases) return vmeErrorReturn(VME_CARD_DNE);
  if (rw->test(rw->card->base[c],rw) == dir)
   {
    if (dir ==1)  dummy = rw->clear(rw->card->base[c],rw);
    else          dummy = rw->set(rw->card->base[c],rw); 
   }
  taskDelay( (int)(0.001*sysClkRateGet()) );   /* wait for minimum time (0.001sec)???*/
  if (dir ==1)  dummy = rw->set(rw->card->base[c],rw);
  else          dummy = rw->clear(rw->card->base[c],rw); 

  v = (vme_vmetype)((opcd<<4)|(addr)| 0x200);
  dummy = msk->write(msk->card->base[c],msk,chan);
  dummy = wrd->write(wrd->card->base[c],wrd,cnt);
  dummy = cmd->write(cmd->card->base[c],cmd,v);
  while (tst == 0)
  {
   tst = (int) ( dne->test(dne->card->base[c],dne) |  tmout->test(tmout->card->base[c],tmout));
  }
  return dummy;
}
#endif /* vxworks */


void vmeAddDaqRegisters()
{

/* these constants are just for conveniance and to make the code below more readable */
  const int FIFO = VME_FIFO;
  const int CONTIG = VME_CONTIG;
  const int WORD = VME_WORD;
  const int BITS = VME_BITS;
  const int BIT = VME_BIT;

  const int RO = VME_R_ACCESS;
  const int WO = VME_W_ACCESS;
  const int RW = VME_RW_ACCESS;
  const int WV = VME_WV_ACCESS;
  const int RWV = VME_RW_ACCESS | VME_V_ACCESS;
  const int RD = VME_RD_ACCESS;

/* vmeAddReg&register, name, class, type, access, offset, mask, vmask, shift, card) */

/* IMPORTANT -- IMPORTANT -- IMPORTANT -- IMPORTANT -- IMPORTANT -- IMPORTANT ****/
/*                                                                               */
/* fifoinit must come after the fifo and both of its status registers are init'd */
/*                                                                               */
/*********************************************************************************/

  vmeAddReg(&daq_dram,  "dram",  "memory",  CONTIG,  RW, 0x000000, 0xffffffff, 0x0000,  0, vme_daq);
  vmeAddReg(&daq_vram1, "vram1", "memory",  CONTIG,  RW, 0x200000, 0xffffffff, 0x0000,  0, vme_daq);
  vmeAddReg(&daq_vram2, "vram2", "memory",  CONTIG,  RW, 0x280000, 0xffffffff, 0x0000,  0, vme_daq);
  vmeAddReg(&daq_board_reset, "board_reset", "generic",BIT,  RW, 0x340000, 0x0080, 0x0000,  0, vme_daq);
  vmeAddReg(&daq_hreset,      "hreset",  "generic",    BIT,  RW, 0x340000, 0x0040, 0x0000,  0, vme_daq);
  vmeAddReg(&daq_cpu_gbl,     "cpu_gbl", "cpu_mode",   BIT,  RW, 0x340000, 0x0010, 0x0000,  0, vme_daq);
  vmeAddReg(&daq_cpu_smi,     "spu_smi", "cpu_mode",   BIT,  RW, 0x340000, 0x0008, 0x0000,  0, vme_daq);
  vmeAddReg(&daq_pm_type,     "pm_type", "pers_mod",   BIT,  RW, 0x340000, 0x0004, 0x0000,  0, vme_daq);
  vmeAddReg(&daq_int_ppc_en,  "ppc_en",  "int_en",     BIT,  RW, 0x340000, 0x0002, 0x0000,  0, vme_daq);
  vmeAddReg(&daq_int_vme_en,  "vme_en",  "int_en",     BIT,  RW, 0x340000, 0x0001, 0x0000,  0, vme_daq);
 
  vmeAddReg(&daq_send_sync,   "sd_sync", "mode",       BIT,  RW, 0x340002, 0x0400, 0x0000,  0, vme_daq);
  vmeAddReg(&daq_en_fast_ctl, "en_fst_ctl","mode",     BIT,  RW, 0x340002, 0x0200, 0x0000,  0, vme_daq);
  vmeAddReg(&daq_en_model,    "en_model","mode",       BIT,  RW, 0x340002, 0x0100, 0x0000,  0, vme_daq);
  vmeAddReg(&daq_vram_xfer,   "vram_xfer","mode",      BIT,  RW, 0x340002, 0x0080, 0x0000,  0, vme_daq);
  vmeAddReg(&daq_vram_dir,    "vram_dir","mode",       BIT,  RW, 0x340002, 0x0040, 0x0000,  0, vme_daq);
  vmeAddReg(&daq_busy,        "busy",    "mode",       BIT,  RW, 0x340002, 0x0020, 0x0000,  0, vme_daq);
  vmeAddReg(&daq_sclk_enable, "sclk_en", "mode",       BIT,  RW, 0x340002, 0x0010, 0x0000,  0, vme_daq);
  vmeAddReg(&daq_vme_int,     "vme",     "interrupt",  BIT,  RW, 0x340002, 0x0008, 0x0000,  0, vme_daq);
  vmeAddReg(&daq_inc_qc,      "inc_qc",  "intern",     BIT,  RW, 0x340002, 0x0004, 0x0000,  0, vme_daq);
  vmeAddReg(&daq_dec_qc,      "dec_qc",  "intern",     BIT,  RW, 0x340002, 0x0002, 0x0000,  0, vme_daq);
  vmeAddReg(&daq_led,         "led",     "generic",    BIT,  RW, 0x340002, 0x0001, 0x0000,  0, vme_daq);
 
  vmeAddReg(&daq_vram_full_pointer,"vr_full_pointer", "vram", BITS,  RW, 0x340004, 0x01ff, 0x0000,  0, vme_daq);
  vmeAddReg(&daq_vram_af_level,    "vr_af_level",     "vram", BITS,  RW, 0x340006, 0x01ff, 0x0000,  0, vme_daq);
  vmeAddReg(&daq_vram_page,        "vr_page",         "vram", BITS,  RW, 0x340008, 0x01ff, 0x0000,  0, vme_daq);
  vmeAddReg(&daq_fifo_af_level,    "fi_af_level",     "fifo", BITS,  RW, 0x34000a, 0x03ff, 0x0000,  0, vme_daq);
  vmeAddReg(&daq_fifo_full_level,  "fi_full_level",   "fifo", BITS,  RW, 0x34000c, 0x03ff, 0x0000,  0, vme_daq);

  vmeAddReg(&daq_vram_full,   "vr_full",    "vram",       BIT,  RW, 0x34000e, 0x0080, 0x0000,  0, vme_daq);
  vmeAddReg(&daq_vram_af,     "vr_af",      "vram",       BIT,  RW, 0x34000e, 0x0040, 0x0000,  0, vme_daq);
  vmeAddReg(&daq_vram_empty,  "vr_empty",   "vram",       BIT,  RW, 0x34000e, 0x0020, 0x0000,  0, vme_daq);
  vmeAddReg(&daq_pm_full,     "pm_full_reg","pers_mod",   BIT,  RW, 0x34000e, 0x0010, 0x0000,  0, vme_daq);
  vmeAddReg(&daq_pm_af,       "pm_af",      "pers_mod",   BIT,  RW, 0x34000e, 0x0008, 0x0000,  0, vme_daq);
  vmeAddReg(&daq_fifo_full,   "fi_full",    "fifo",       BIT,  RW, 0x34000e, 0x0004, 0x0000,  0, vme_daq);
  vmeAddReg(&daq_fifo_af,     "fi_af",      "fifo",       BIT,  RW, 0x34000e, 0x0002, 0x0000,  0, vme_daq);
  vmeAddReg(&daq_fifo_empty,  "fi_empty",   "fifo",       BIT,  RW, 0x34000e, 0x0001, 0x0000,  0, vme_daq);
 
  vmeAddReg(&daq_uart_int,      "uart",     "interrupt",   BIT,  RO, 0x340010, 0x0400, 0x0000,  0, vme_daq);
  vmeAddReg(&daq_timer2_int,    "timer2",   "interrupt",   BIT,  RO, 0x340010, 0x0200, 0x0000,  0, vme_daq);
  vmeAddReg(&daq_timer1_int,    "timer1",   "interrupt",   BIT,  RO, 0x340010, 0x0100, 0x0000,  0, vme_daq);
  vmeAddReg(&daq_timer0_int,    "timer0",   "interrupt",   BIT,  RO, 0x340010, 0x0080, 0x0000,  0, vme_daq);
  vmeAddReg(&daq_pm_int,        "pm_int",   "interrupt",   BIT,  RO, 0x340010, 0x0040, 0x0000,  0, vme_daq);
  vmeAddReg(&daq_xfer_done,     "xfer_done","interrupt",   BIT,  RO, 0x340010, 0x0020, 0x0000,  0, vme_daq);
  vmeAddReg(&daq_fast_int,      "fast",     "interrupt",   BIT,  RO, 0x340010, 0x0010, 0x0000,  0, vme_daq);
  vmeAddReg(&daq_pm_full_int,   "pm_full",  "interrupt",   BIT,  RO, 0x340010, 0x0008, 0x0000,  0, vme_daq);
  vmeAddReg(&daq_vram_full_int, "vram_full","interrupt",   BIT,  RO, 0x340010, 0x0004, 0x0000,  0, vme_daq);
  vmeAddReg(&daq_fifo_full_int, "fifo_full","interrupt",   BIT,  RO, 0x340010, 0x0002, 0x0000,  0, vme_daq);
  vmeAddReg(&daq_ppc_mail_int,  "ppc_mail", "interrupt",   BIT,  RO, 0x340010, 0x0001, 0x0000,  0, vme_daq);
  
  vmeAddReg(&daq_msk_uart,      "msk_uart",       "mask",   BIT,  RW, 0x340012, 0x0400, 0x0000,  0, vme_daq);
  vmeAddReg(&daq_msk_timer2,    "msk_timer2",     "mask",   BIT,  RW, 0x340012, 0x0200, 0x0000,  0, vme_daq);
  vmeAddReg(&daq_msk_timer1,    "msk_timer1",     "mask",   BIT,  RW, 0x340012, 0x0100, 0x0000,  0, vme_daq);
  vmeAddReg(&daq_msk_timer0,    "msk_timer0",     "mask",   BIT,  RW, 0x340012, 0x0080, 0x0000,  0, vme_daq);
  vmeAddReg(&daq_msk_pm_int,    "msk_pm_int",     "mask",   BIT,  RW, 0x340012, 0x0040, 0x0000,  0, vme_daq);
  vmeAddReg(&daq_msk_xfer_done, "msk_xfer_done",  "mask",   BIT,  RW, 0x340012, 0x0020, 0x0000,  0, vme_daq);
  vmeAddReg(&daq_msk_fast_ctl,  "msk_fast",       "mask",   BIT,  RW, 0x340012, 0x0010, 0x0000,  0, vme_daq);
  vmeAddReg(&daq_msk_pm_full,   "msk_pm_full",    "mask",   BIT,  RW, 0x340012, 0x0008, 0x0000,  0, vme_daq);
  vmeAddReg(&daq_msk_vf,        "msk_vram_full",  "mask",   BIT,  RW, 0x340012, 0x0004, 0x0000,  0, vme_daq);
  vmeAddReg(&daq_msk_ff,        "msk_fifo_full",  "mask",   BIT,  RW, 0x340012, 0x0002, 0x0000,  0, vme_daq);
  vmeAddReg(&daq_msk_ppc_mail,  "msk_ppc_mail",   "mask",   BIT,  RW, 0x340012, 0x0001, 0x0000,  0, vme_daq);
 
  vmeAddReg(&daq_irq_vector,  "irq_vector",  "generic", BITS,  RW, 0x340014, 0x00ff, 0x0000,  0, vme_daq);
  vmeAddReg(&daq_irq_level,   "irq_level",   "generic", BITS,  RW, 0x340014, 0x0300, 0x0000,  8, vme_daq);

  vmeAddReg(&daq_vme_mailbox,  "vme_mailbox",  "generic", BITS,  RW, 0x340016, 0x00ff, 0x0000,  0, vme_daq);
  vmeAddReg(&daq_ppc_mailbox,  "ppc_mailbox",  "generic", BITS,  RW, 0x340018, 0x00ff, 0x0000,  0, vme_daq);

  vmeAddReg(&daq_num_pages,  "num_pages",  "model", BITS,  RW, 0x34001a, 0x0700, 0x0000,  8, vme_daq);
  vmeAddReg(&daq_num_buffs,  "num_buffs",  "model", BITS,  RW, 0x34001a, 0x00f0, 0x0000,  4, vme_daq);
  vmeAddReg(&daq_model_occ,  "model_occ",  "model", BITS,  RW, 0x34001a, 0x000f, 0x0000,  0, vme_daq);

  vmeAddReg(&daq_que_ctr,    "que_ctr",  "generic", BITS,  RW, 0x34001c, 0x00ff, 0x0000,  0, vme_daq);

  vmeAddReg(&daq_curr_evt,  "curr_evt",  "generic", BITS,  RO, 0x34001d, 0x03ff, 0x0000,  0, vme_daq);

  vmeAddReg(&daq_fast_ctl_timebase,"fast_ctl_timebase","generic",BITS,RO,0x340020,0x03ff, 0x0000,0,vme_daq);

  vmeAddReg(&daq_fifo_read,  "fifo_read",  "generic", BITS, RO, 0x340028, 0x3ffff, 0x0000,  0, vme_daq);

  vmeAddReg(&daq_timer0,    "timer0",  "generic", BITS,  RW, 0x34002c, 0xffff, 0x0000,  0, vme_daq);

  vmeAddReg(&daq_timer1,    "timer1",  "generic", BITS,  RW, 0x34002e, 0xffff, 0x0000,  0, vme_daq);

  vmeAddReg(&daq_timer2,    "timer2",  "generic", BITS,  RW, 0x340030, 0xffff, 0x0000,  0, vme_daq);

  vmeAddReg(&daq_timer_ctrl, "timer_ctrl",  "generic", BITS,  RW, 0x340032, 0xffff, 0x0000,  0, vme_daq);

  vmeAddReg(&daq_uart_ctrl_a,    "uart_ctrl_a",  "uart", BITS,  RW, 0x340034, 0xffff, 0x0000,  0, vme_daq);

  vmeAddReg(&daq_uart_data_a,    "uart_data_a",  "uart", BITS,  RW, 0x340036, 0xffff, 0x0000,  0, vme_daq);

  vmeAddReg(&daq_uart_ctrl_b,    "uart_ctrl_b",  "uart", BITS,  RW, 0x340038, 0xffff, 0x0000,  0, vme_daq);

  vmeAddReg(&daq_uart_data_b,    "uart_data_b",  "uart", BITS,  RW, 0x34003a, 0xffff, 0x0000,  0, vme_daq);

  vmeAddReg(&pm_cmd_reg,    "pm_cmd_reg",  "pers_mod", BITS,  RW, 0x300002, 0x3ff, 0x0000,  0, vme_daq);

  vmeAddReg(&pm_ch_msk,     "pm_ch_msk",   "pers_mod", BITS,  RW, 0x300004, 0xffff, 0x0000,  0, vme_daq);

  vmeAddReg(&pm_wrd_cnt,    "pm_wrd_cnt",  "pers_mod", BITS,  RW, 0x300006, 0x0fff, 0x0000,  0, vme_daq);

  vmeAddReg(&pm_timeout,    "pm_timeout",  "pers_mod", BITS,  RW, 0x300008, 0x0fff, 0x0000,  0, vme_daq);

  vmeAddReg(&pm_int_dne,      "pm_int_dne",    "pers_mod",BIT,  RW, 0x300010, 0x0004, 0x0000,  0, vme_daq);
  vmeAddReg(&pm_int_timeout,  "pm_int_timeout","pers_mod",BIT,  RW, 0x300010, 0x0002, 0x0000,  0, vme_daq);
  vmeAddReg(&pm_int_link_err, "int_link_err",  "pers_mod",BIT,  RW, 0x300010, 0x0001, 0x0000,  0, vme_daq);
/*Establish only 20K of memory on each bank to facilitate filewrite*/
  vmeAddContig(daq_dram,vmeLowBuff2Contig,vmeLowContig2Buff,NULL,NULL,0x5000);
  vmeAddContig(daq_vram1,vmeLowBuff2Contig,vmeLowContig2Buff,NULL,NULL,0x5000);
  vmeAddContig(daq_vram2,vmeLowBuff2Contig,vmeLowContig2Buff,NULL,NULL,0x5000);

}

