/* 
 * Daq-Board Teststand Software
 *
 *
 *Code by:
 *   Armin Karcher (karcher@design.lbl.gov)
 * Design and Template by
 *   Eric Gottlieb  (egott@unm.edu)
 *   Tim Thomas     (thomas@hev1.phys.unm.edu)
 *   Micheal Gold   (gold@hepv1.phys.unm.edu)
 *
 */

#include <stdio.h>
#include <sys/types.h>
#ifdef vxworks
#include <types/vxTypesOld.h>
#include "taskLib.h"
#endif

#include "rpc.h"
#include "unmpre.h"
#include "front.h"
#include "low.h"


vme_card vme_pre;

vme_register daq_dram;
vme_register daq_vram1;
vme_register daq_vram2;
vme_register daq_vram_page;
vme_register daq_vram_af_level;
vme_register daq_board_reset;
vme_register daq_led;
vme_register daq_vram_dir;
vme_register daq_assrt_chkstp;
vme_register daq_smi;
vme_register daq_mcp;
vme_register daq_sreset;
vme_register daq_hreset;
vme_register daq_sclk_enable;
vme_register daq_cpu_gbl;
vme_register daq_cpu_smi;
vme_register daq_pm_type;
vme_register daq_fifo_af_level;
vme_register daq_fifo_full_level; 
vme_register daq_pm_error;
vme_register daq_chkstp;
vme_register daq_pm_error_int;
vme_register daq_pm_int;
vme_register daq_xfer_done_int;
vme_register daq_pm_int;
vme_register daq_vf_int;
vme_register daq_ff_int;
vme_register daq_vme_mail_int;
vme_register daq_ppc_mail_int;
vme_register daq_pm_error_reset;
vme_register daq_pm_reset;
vme_register daq_xfer_done_reset;
vme_register daq_pm_reset;
vme_register daq_vf_reset;
vme_register daq_ff_reset;
vme_register daq_vme_mail_reset;
vme_register daq_ppc_mail_reset;
vme_register daq_msk_pm_error;
vme_register daq_msk_pm;
vme_register daq_msk_xfer_done;
vme_register daq_msk_fast_ctl;
vme_register daq_msk_pm_full;
vme_register daq_msk_vf;
vme_register daq_msk_ff;
vme_register daq_msk_vme_mail;
vme_register daq_msk_ppc_mail;
vme_register daq_fifo;
vme_register daq_vme_mailbox;
vme_register daq_ppc_mailbox;
vme_register pm_trailer_en;  
vme_register pm_opto_rdy;  
vme_register pm_rx_idle;  
vme_register pm_tx_idle;  
vme_register pm_en_fast_ctl;  
vme_register pm_rst_link;  
vme_register pm_cmd_reg;  
vme_register pm_ch_msk;  
vme_register pm_rx_wrd_cnt;  
vme_register pm_tx_wrd_cnt;  
vme_register pm_timeout;  
vme_register pm_eot;  
vme_register pm_opto;  
vme_register pm_int_dne; 
vme_register pm_int_timeout; 
vme_register pm_int_link_err; 
vme_register pm_int_rx;  
vme_register pm_int_tx;  
vme_register pm_int_protocol;  
vme_register pm_msk_transfer_dne;  
vme_register pm_msk_timeout;  
vme_register pm_msk_link_error;  
vme_register pm_msk_rx;  
vme_register pm_msk_tx;  
vme_register pm_msk_protocol;  
vme_register pm_clr_transfer_dne;  
vme_register pm_clr_timeout;  
vme_register pm_clr_link_error;  
vme_register pm_clr_rx;  
vme_register pm_clr_tx;  
vme_register pm_clr_protocol;  

#ifdef vxworks   




vme_rc vmePmCommand(int c,int opcd,int addr,int chan,int cnt,int dir,vme_vmetype* data)
{
  vme_register cmd = pm_cmd_reg;
  vme_register msk = pm_ch_msk;
  vme_register rwrd = pm_rx_wrd_cnt; 
  vme_register twrd = pm_tx_wrd_cnt; 
  vme_register rw = daq_vram_dir;
  vme_register vram1 = daq_vram1;
  vme_register vram2 = daq_vram2;
  vme_rc dummy;
  vme_vmetype v,vram_data;
  int i;
  char* fname;

  if (c < 0 || c >= rw->card->bases) return vmeErrorReturn(VME_CARD_DNE);
  v = (vme_vmetype)((opcd<<5)|(addr));
 
  dummy = msk->write(msk->card->base[c],msk,chan); /*setup mask reg*/

  if (dir ==0)   dummy = rwrd->write(rwrd->card->base[c],rwrd,cnt); /*setup wordcount reg*/
  else           dummy = twrd->write(twrd->card->base[c],twrd,cnt);

  if (cnt ==0 ) /*if data-less cmd */
    {           /*set right direction otherwise do nothing*/
      if (dir ==0)  dummy = rw->clear(rw->card->base[c],rw);
      else          dummy = rw->set(rw->card->base[c],rw); 
    }
  else{        /*if data-bearing cmd*/
     if (dir ==0) /*if read */
        {
       if (rw->test(rw->card->base[c],rw) == 1) /*set right direction*/
           {
            dummy = rw->clear(rw->card->base[c],rw);
            vram_wait();
           }
        }
     else{        /*if write*/
            dummy = rw->clear(rw->card->base[c],rw); /*set dir to read*/
            data_2_vram(data,vram1,c);              /*put data*/
           /*data_2_vram(data,vram2,c);  Bank 2 not installed on most boards*/
            dummy = rw->set(rw->card->base[c],rw);  /*set to write*/
            vram_wait();         
         }
      }

  dummy = cmd->write(cmd->card->base[c],cmd,v);  /*execute cmd*/
  return dummy;
}


void data_2_vram(vme_vmetype* data, vme_register vram, int c)
   {
    vme_rc dummy;
    vme_vmetype vram_data;
    int i;
    for(i=0;i<32;i++)
       {
       vram_data = *data;
       vram_data |= (*data <<8);
       vram_data |= (*data <<16);
       vram_data |= (*data <<24);
       dummy = vram->write(vram->card->base[c] +(i*2),vram,vram_data);
       dummy = vram->write(vram->card->base[c] +(i*2)+1,vram,vram_data);
       data++;
        }
  }


vme_rc vmePmCommandAll(int c, int opcd, int nchips, int* addr_out, int* chan_out, int cnt, int dir, vme_vmetype* data_out) 
{
  int i,j,n;
  vme_rc dummy;
  int addr, chan;
  vme_vmetype* data;
  for (i=0; i<nchips; i++) {
    addr  = *(addr_out + i);
    chan  = *(chan_out + i);
    data  = data_out + cnt*i;
/*    printf("chip=%d card=%d opcd=0x%x addr=0x%x chan=0x%x cnt=%d  dir=%d \n",
	   i, c, opcd, addr, chan, cnt, dir);   */
    dummy = vmePmCommand(c, opcd, addr, chan, cnt, dir, data);
/* put in a delay  */
    for (j=0; j<150; j++) n=j*j;
  }
}



#endif

void vram_wait()
    {
    }

void vmeAddPreRegisters()
{

/* these constants are just for conveniance and to make the code below more readable */
  const int FIFO = VME_FIFO;
  const int CONTIG = VME_CONTIG;
  const int WORD = VME_WORD;
  const int BITS = VME_BITS;
  const int BIT = VME_BIT;

  const int RO = VME_R_ACCESS;
  const int WO = VME_W_ACCESS;
  const int RW = VME_RW_ACCESS;
  const int WV = VME_WV_ACCESS;
  const int RWV = VME_RW_ACCESS | VME_V_ACCESS;
  const int RD = VME_RD_ACCESS;

/* vmeAddReg&register, name, class, type, access, offset, mask, vmask, shift, card) */

/* IMPORTANT -- IMPORTANT -- IMPORTANT -- IMPORTANT -- IMPORTANT -- IMPORTANT ****/
/*                                                                               */
/* fifoinit must come after the fifo and both of its status registers are init'd */
/*                                                                               */
/*********************************************************************************/

  vmeAddReg(&daq_vram_page,      "pointer",     "vram",    BITS,  RW, 0x400000, 0x01ff, 0x0000,  0, vme_pre);
  vmeAddReg(&daq_vram_af_level,  "vr_af_level", "vram",    BITS,  RW, 0x400002, 0x01ff, 0x0000,  0, vme_pre);

  vmeAddReg(&daq_vram_dir,       "vram_dir",    "cpu",     BIT,   RW, 0x40000a, 0x0001, 0x0000,  0, vme_pre);
  vmeAddReg(&daq_led,            "led",         "cpu",     BIT,   RW, 0x40000a, 0x0002, 0x0000,  0, vme_pre);
  vmeAddReg(&daq_assrt_chkstp,   "assrt_chkstp","cpu",     BIT,   RW, 0x40000a, 0x0004, 0x0000,  0, vme_pre);
  vmeAddReg(&daq_smi,            "smi",         "cpu",     BIT,   RW, 0x40000a, 0x0008, 0x0000,  0, vme_pre);
  vmeAddReg(&daq_mcp,            "mcp",         "cpu",     BIT,   RW, 0x40000a, 0x0010, 0x0000,  0, vme_pre);
  vmeAddReg(&daq_sreset,         "sreset",      "cpu",     BIT,   RW, 0x40000a, 0x0020, 0x0000,  0, vme_pre);
  vmeAddReg(&daq_hreset,         "hreset",      "cpu",     BIT,   RW, 0x40000a, 0x0040, 0x0000,  0, vme_pre);
  vmeAddReg(&daq_board_reset,    "brd_reset",   "cpu",     BIT,   WO, 0x40000a, 0x0080, 0x0000,  0, vme_pre);
  vmeAddReg(&daq_sclk_enable,    "sclk_enable", "cpu",     BIT,   RW, 0x40000a, 0x0200, 0x0000,  0, vme_pre);
  vmeAddReg(&daq_pm_type,        "pm_id",       "cpu",     BIT,   RW, 0x40000a, 0x0800, 0x0000,  0, vme_pre);
  vmeAddReg(&daq_pm_error,       "pm_error",    "cpu",     BIT,   RO, 0x40000a, 0x0800, 0x0000,  0, vme_pre);
  vmeAddReg(&daq_chkstp,         "chkstp",      "cpu",     BIT,   RO, 0x40000a, 0x0800, 0x0000,  0, vme_pre);
 
  vmeAddReg(&daq_pm_error_int,  "pm_error_in","interrupt",   BIT,  RO, 0x40000e, 0x0080, 0x0000,  0, vme_pre);
  vmeAddReg(&daq_pm_int,        "pm_full_in", "interrupt",   BIT,  RO, 0x40000e, 0x0040, 0x0000,  0, vme_pre);
  vmeAddReg(&daq_xfer_done_int, "xfer_done_in","interrupt",   BIT,  RO, 0x40000e, 0x0020, 0x0000,  0, vme_pre);
  vmeAddReg(&daq_pm_int,        "pm_in",      "interrupt",   BIT,  RO, 0x40000e, 0x0010, 0x0000,  0, vme_pre);
  vmeAddReg(&daq_vf_int,        "vf_in",      "interrupt",   BIT,  RO, 0x40000e, 0x0008, 0x0000,  0, vme_pre);
  vmeAddReg(&daq_ff_int,        "ff_in",      "interrupt",   BIT,  RO, 0x40000e, 0x0004, 0x0000,  0, vme_pre);
  vmeAddReg(&daq_vme_mail_int,  "vme_mail_in","interrupt",   BIT,  RO, 0x40000e, 0x0002, 0x0000,  0, vme_pre);
  vmeAddReg(&daq_ppc_mail_int,  "ppc_mail_in","interrupt",   BIT,  RO, 0x40000e, 0x0001, 0x0000,  0, vme_pre);

  vmeAddReg(&daq_pm_error_reset,  "pm_error_re","interrupt",   BIT,  RW, 0x400010, 0x0080, 0x0000,  0, vme_pre);
  vmeAddReg(&daq_pm_reset,        "pm_full_re", "interrupt",   BIT,  RW, 0x400010, 0x0040, 0x0000,  0, vme_pre);
  vmeAddReg(&daq_xfer_done_reset, "xfer_done_re","interrupt",   BIT,  RW, 0x400010, 0x0020, 0x0000,  0, vme_pre);
  vmeAddReg(&daq_pm_reset,        "pm_re",      "interrupt",   BIT,  RW, 0x400010, 0x0010, 0x0000,  0, vme_pre);
  vmeAddReg(&daq_vf_reset,        "vf_re",      "interrupt",   BIT,  RW, 0x400010, 0x0008, 0x0000,  0, vme_pre);
  vmeAddReg(&daq_ff_reset,        "ff_re",      "interrupt",   BIT,  RW, 0x400010, 0x0004, 0x0000,  0, vme_pre);
  vmeAddReg(&daq_vme_mail_reset,  "vme_mail_re","interrupt",   BIT,  RW, 0x400010, 0x0002, 0x0000,  0, vme_pre);
  vmeAddReg(&daq_ppc_mail_reset,  "ppc_mail_re","interrupt",   BIT,  RW, 0x400010, 0x0001, 0x0000,  0, vme_pre);

  vmeAddReg(&daq_msk_pm_error,  "pm_error_mask","interrupt",   BIT,  RW, 0x400012, 0x0080, 0x0000,  0, vme_pre);
  vmeAddReg(&daq_msk_pm,        "pm_full_mask", "interrupt",   BIT,  RW, 0x400012, 0x0040, 0x0000,  0, vme_pre);
  vmeAddReg(&daq_msk_xfer_done, "xfer_done_mask","interrupt",   BIT,  RW, 0x400012, 0x0020, 0x0000,  0, vme_pre);
  vmeAddReg(&daq_msk_pm,        "pm_mask",      "interrupt",   BIT,  RW, 0x400012, 0x0010, 0x0000,  0, vme_pre);
  vmeAddReg(&daq_msk_vf,        "vf_mask",      "interrupt",   BIT,  RW, 0x400012, 0x0008, 0x0000,  0, vme_pre);
  vmeAddReg(&daq_msk_ff,        "ff_mask",      "interrupt",   BIT,  RW, 0x400012, 0x0004, 0x0000,  0, vme_pre);
  vmeAddReg(&daq_msk_vme_mail,  "vme_mail_mask","interrupt",   BIT,  RW, 0x400012, 0x0002, 0x0000,  0, vme_pre);
  vmeAddReg(&daq_msk_ppc_mail,  "ppc_mail_mask","interrupt",   BIT,  RW, 0x400012, 0x0001, 0x0000,  0, vme_pre);

  vmeAddReg(&daq_fifo_af_level,  "fifo_empty_off", "fifo",   BITS,  RW, 0x400004, 0x03ff, 0x0000,  0, vme_pre);
  vmeAddReg(&daq_fifo_full_level,"fifo_full_off",  "fifo",   BITS,  RW, 0x400006, 0x03ff, 0x0000,  0, vme_pre);

  vmeAddReg(&daq_fifo,           "fifo",        "fifo"   , BITS,  RO, 0x400014, 0x3ffff, 0x0000,  0, vme_pre);
  vmeAddReg(&daq_vme_mailbox,    "vme_mailbox", "mailbox", BITS,  RW, 0x400016, 0x00ff, 0x0000,  0, vme_pre);
  vmeAddReg(&daq_ppc_mailbox,    "ppc_mailbox", "mailbox", BITS,  RW, 0x400018, 0x00ff, 0x0000,  0, vme_pre);

  vmeAddReg(&pm_ch_msk,      "channel_mask",     "pm",    BITS,  RW, 0x800000, 0xffff, 0x0000,  0, vme_pre);
  vmeAddReg(&pm_cmd_reg,     "cmd_register",     "pm",    BITS,  RW, 0x800002, 0x03ff, 0x0000,  0, vme_pre);
  vmeAddReg(&pm_en_fast_ctl, "fctrl_en",         "pm",    BIT,   RW, 0x800004, 0x0001, 0x0000,  0, vme_pre);
  vmeAddReg(&pm_trailer_en,  "trailer_en",       "pm",    BIT,   RW, 0x800004, 0x0002, 0x0000,  0, vme_pre);
  vmeAddReg(&pm_rst_link,    "reset_link",       "pm",    BIT,   RW, 0x800004, 0x0004, 0x0000,  0, vme_pre);
  vmeAddReg(&pm_tx_idle,     "tx_idle",          "pm",    BIT,   RO, 0x800004, 0x0008, 0x0000,  0, vme_pre);
  vmeAddReg(&pm_rx_idle,     "rx_idle",          "pm",    BIT,   RO, 0x800004, 0x0010, 0x0000,  0, vme_pre);
  vmeAddReg(&pm_opto_rdy,    "opto_rdy",         "pm",    BIT,   RO, 0x800004, 0x0011, 0x0000,  0, vme_pre);
  vmeAddReg(&pm_int_dne,     "done_int",         "pm",    BIT,   RO, 0x800006, 0x0001, 0x0000,  0, vme_pre);
  vmeAddReg(&pm_msk_transfer_dne, "done_msk",    "pm",    BIT,   RW, 0x800008, 0x0001, 0x0000,  0, vme_pre);
  vmeAddReg(&pm_clr_transfer_dne, "done_rst",    "pm",    BIT,   WO, 0x80000a, 0x0001, 0x0000,  0, vme_pre);
  vmeAddReg(&pm_int_timeout, "timeout_int",      "pm",    BIT,   RO, 0x800006, 0x0002, 0x0000,  0, vme_pre);
  vmeAddReg(&pm_msk_timeout, "timeout_msk",      "pm",    BIT,   RW, 0x800008, 0x0002, 0x0000,  0, vme_pre);
  vmeAddReg(&pm_clr_timeout, "timeout_rst",      "pm",    BIT,   WO, 0x80000a, 0x0002, 0x0000,  0, vme_pre);
  vmeAddReg(&pm_int_link_err,   "error_int",     "pm",    BIT,   RO, 0x800006, 0x0004, 0x0000,  0, vme_pre);
  vmeAddReg(&pm_msk_link_error, "error_msk",     "pm",    BIT,   RW, 0x800008, 0x0004, 0x0000,  0, vme_pre);
  vmeAddReg(&pm_clr_link_error, "error_rst",     "pm",    BIT,   WO, 0x80000a, 0x0004, 0x0000,  0, vme_pre);
  vmeAddReg(&pm_int_tx,      "tx_int",           "pm",    BIT,   RO, 0x800006, 0x0008, 0x0000,  0, vme_pre);
  vmeAddReg(&pm_msk_tx,      "tx_msk",           "pm",    BIT,   RW, 0x800008, 0x0008, 0x0000,  0, vme_pre);
  vmeAddReg(&pm_clr_tx,      "tx_rst",           "pm",    BIT,   WO, 0x80000a, 0x0008, 0x0000,  0, vme_pre);
  vmeAddReg(&pm_int_rx,      "rx_int",           "pm",    BIT,   RO, 0x800006, 0x0010, 0x0000,  0, vme_pre);
  vmeAddReg(&pm_msk_rx,      "rx_msk",           "pm",    BIT,   RW, 0x800008, 0x0010, 0x0000,  0, vme_pre);
  vmeAddReg(&pm_clr_rx,      "rx_rst",           "pm",    BIT,   WO, 0x80000a, 0x0010, 0x0000,  0, vme_pre);
  vmeAddReg(&pm_int_protocol,"protocol_int",     "pm",    BIT,   RO, 0x800006, 0x0011, 0x0000,  0, vme_pre);
  vmeAddReg(&pm_msk_protocol,"protocol_msk",     "pm",    BIT,   RW, 0x800008, 0x0011, 0x0000,  0, vme_pre);
  vmeAddReg(&pm_clr_protocol,"protocol_rst",     "pm",    BIT,   WO, 0x80000a, 0x0011, 0x0000,  0, vme_pre);
  vmeAddReg(&pm_timeout,     "timeout_register", "pm",    BITS,  RW, 0x80000c, 0x0fff, 0x0000,  0, vme_pre);
  vmeAddReg(&pm_rx_wrd_cnt,  "rx_wordcount",     "pm",    BITS,  RW, 0x80000e, 0x0fff, 0x0000,  0, vme_pre);
  vmeAddReg(&pm_tx_wrd_cnt,  "tx_wordcount",     "pm",    BITS,  RW, 0x800010, 0x0fff, 0x0000,  0, vme_pre);
  vmeAddReg(&pm_eot,         "finished_channels","pm",    BITS,  RO, 0x800012, 0xffff, 0x0000,  0, vme_pre);

  vmeAddReg(&daq_dram,  "dram",  "memory",  CONTIG,  RW, 0x000000, 0xffffffff, 0x0000,  0, vme_pre);
  vmeAddReg(&daq_vram1, "vram1", "memory",  CONTIG,  RW, 0x080000, 0xffffffff, 0x0000,  0, vme_pre);
  vmeAddReg(&daq_vram2, "vram2", "memory",  CONTIG,  RW, 0x100000, 0xffffffff, 0x0000,  0, vme_pre);

/*Establish only 20K of memory on each bank to facilitate filewrite*/
  vmeAddContig(daq_dram,vmeLowBuff2Contig,vmeLowContig2Buff,NULL,NULL,0x5000);
  vmeAddContig(daq_vram1,vmeLowBuff2Contig,vmeLowContig2Buff,NULL,NULL,0x5000);
  vmeAddContig(daq_vram2,vmeLowBuff2Contig,vmeLowContig2Buff,NULL,NULL,0x5000);

}

