// -*- C++ -*-

// 
// VME Teststand Software
//
// Code revised by Armin Karcher (karcher@design.lbl.gov)
//
// Design and Template by:
//   Eric Gottlieb  (egott@unm.edu)
//   Tim Thomas     (thomas@hev1.phys.unm.edu)
//   Micheal Gold   (gold@hepv1.phys.unm.edu)
//

#ifndef ERRORHANDLER_H
#define ERRORHANDLER_H

#include <String.h>

// The ErrorHandler virtual class servers as a superclass to classes that
// want simple error and warning message services provided.  It keeps track
// of the name of the class, and if known at instantiation time, the name
// of the instance and includes this information with any output it generates.
// all warnings and error for now go to cerr which is the c++ version of stderr.
// all routines return except for error() which does a exit(0).

class ErrorHandler
{

public: // ----------------------- public -----------------------
  ErrorHandler(String classname = "unknown", String instname = "unknown");
  ~ErrorHandler();

protected: // -------------------- protected --------------------
  void name(String instname);
  String name() const;
  virtual void inform(String label, String message = "") const;  // for generic messages
  virtual void note(String s = "") const;             // for notes
  virtual void warn(String s = "") const;             // for warnings
  virtual void error(String s = "") const;            // exits program after outputing message

private: // ---------------------- private ----------------------
  String _classname;
  String _instname;

};

#endif
