#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# reads defaults from file cal_default_file
# if the file does not exist ask the user whether
# he/she wants to load hardwired defaults or if he/she wants to load them from
# a special file.........(BUT ONLY IF ask=YES)
# Returns zero if something was loaded
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
proc cal_get_defaults {cal_default_file ask} {

    source cal_global.tcl
    source misc_global.tcl
    global cal_dialog
    global read_runtype
    global read_partname
    global version_number
    global this_date

    set exists [file exist $cal_default_file]
    if {$exists == 1} {
       set f [open  $cal_default_file r]

#--------------------------------------------------------------------
# the following are only used by the "getrun" procedure
# (We set them here for backward compatibility)
#--------------------------------------------------------------------
       set read_runtype  "THRESHOLD OR CHARGE SCAN"
       set read_partname "" 
#--------------------------------------------------------------------
# read first variable, get version number
# For versions 1.0 and later the first enty is a VX.Y, where the
# version number is X.Y   For earlier version, there was no version
# number in the first variable, we'll call all of those version 0.0
#--------------------------------------------------------------------
       gets $f firstvar
       set firstvar  [string trim  $firstvar]
       set firstchar [string index $firstvar 0]
       if {$firstchar == "V"} {
	   set version_number [string trim $firstvar V]
       } else {
	   set version_number 0.0
       }
#--------------------------------------------------------------------
#--------------------------------------------------------------------
# This stuff is for version 1.0 or later
#--------------------------------------------------------------------
#--------------------------------------------------------------------
    if {$version_number > 0.9} {
	if {$version_number == 1.0} {
	    set this_date "UNKNOWN"
	    set Cal_format_check 0x1ff
	} else {
	    gets $f this_date 
	    gets $f Cal_format_check
	}
	gets $f read_misc
	gets $f read_runtype
	gets $f read_partname

#--------------------------------------------------------------------
#- general miscellaneous tests stuff
#--------------------------------------------------------------------
	if {$read_misc == "YES"} {  
	    for {set i 0} {$i < 5} {incr i 1} {
		gets $f  misc_tests_list($i)
		gets $f  misc_nrep($i)
		if {$i != 2} { 
		    gets $f  misc_cal_dac_top($i,0)
		    gets $f  misc_cal_dac_bottom($i,0)
		}
		gets $f  misc_thr_dac($i)
	    }
	    
#- TOT specific 
            gets $f  misc_ntot
            gets $f  misc_tot_dead
            for {set i 0} {$i < $misc_ntot} {incr i 1} {
		gets $f  misc_cal_dac_top(2,$i)
		gets $f  misc_cal_dac_bottom(2,$i)
		gets $f  misc_tot($i)
		gets $f  misc_tot_toll($i)
		gets $f  misc_tot_shaping_time($i)
		gets $f  misc_tot_test_it($i)
	    }
#   programmable skip added in version V1.02
	    if {$version_number < 1.015} {
		for {set i 0} {$i < $misc_ntot} {incr i 1} {
		    if {$misc_tot_shaping_time($i) == "100 nsec"} {
			set misc_tot_skip($i) "skip 0"
		    } elseif {$misc_tot_shaping_time($i) == "200 nsec"} {
			set misc_tot_skip($i) "skip 1"
		    } elseif {$misc_tot_shaping_time($i) == "300 nsec"} {
			set misc_tot_skip($i) "skip 2"
		    } elseif {$misc_tot_shaping_time($i) == "400 nsec"} {
			set misc_tot_skip($i)  "skip 3"
		    }
		}
	    } else {
		for {set i 0} {$i < $misc_ntot} {incr i 1} {
		    gets $f  misc_tot_skip($i)
		}
	    }

#- Time stamp specific
            gets $f  misc_tstamp_shift
            gets $f  misc_tstamp_toll
            gets $f  misc_tstamp_dead

#- here come the test for shorts stuff
            gets $f  misc_tests_list(5)
            gets $f  short_min_chip
            gets $f  short_max_chip
            gets $f  short_thresh
            gets $f  short_charge_top
            gets $f  short_charge_bottom
            gets $f  short_nrep
            for {set i -3} {$i < 4} {incr i 1} {
		gets $f  short_chan($i)
	    }

	}

#- Now also read the 1st variable of the previous format (i.e. pr 1.0)
       gets $f Cal_compressed_format

#-----------------------------------------------------------------
# It is V0.X format....
#-----------------------------------------------------------------
       } else {
           set this_date "UNKNOWN"
           set Cal_format_check 0x1ff
	   set Cal_compressed_format $firstvar
       }


#-----------------------------------------------------------------
# The following is common to all versions with some small exceptions
#-----------------------------------------------------------------
       gets $f Cal_one_at_a_time
	if {$Cal_one_at_a_time == 0} {
	   set Cal_one_at_a_time 128
       }
       gets $f Cal_number_of_chips_top
       gets $f xy_scan
       gets $f xy_which_axis
       gets $f xy_x_initial
       gets $f xy_y_initial
       gets $f xy_step_size
       gets $f xy_nstep
       gets $f xy_start_flag

       gets $f Cal_error_limit
       gets $f Cal_histoscope
       gets $f Cal_fake
       gets $f Cal_write_file
       gets $f Cal_number_of_events
       gets $f Cal_number_of_chips
       gets $f Cal_number_of_buffers
       gets $f Cal_Side_Hybrid
       set i 0
       while {$i <= 19} {
         gets $f Cal_min_threshold($i)
         gets $f Cal_max_threshold($i)
         gets $f Cal_step_threshold($i)
         gets $f Cal_min_charge($i)
         gets $f Cal_max_charge($i)
         gets $f Cal_step_charge($i)
         gets $f Cal_jitter($i)
         gets $f Cal_shaping_time($i)
         gets $f Cal_sampling_rate($i)
	 gets $f Cal_skip($i)
         gets $f Cal_readout_direction($i)
	 gets $f Cal_respond($i)
	 gets $f Cal_clock_through($i)
	 gets $f Cal_test_pattern($i)
	 gets $f Cal_Set_Clock_B($i)

         set j 0
         while {$j <= 7} {
            gets $f Cal_calib_mask($i,$j)
            gets $f Cal_channel_mask($i,$j)
            incr j 1
         }
	 incr i 1
       }

       set i all
       gets $f Cal_min_threshold($i)
       gets $f Cal_max_threshold($i)
       gets $f Cal_step_threshold($i)
       gets $f Cal_min_charge($i)
       gets $f Cal_max_charge($i)
       gets $f Cal_step_charge($i)
       gets $f Cal_jitter($i)
       gets $f Cal_shaping_time($i)
       gets $f Cal_sampling_rate($i)
       gets $f Cal_skip($i)
       gets $f Cal_readout_direction($i)
       gets $f Cal_respond($i)
       gets $f Cal_clock_through($i)
       gets $f Cal_test_pattern($i)
       gets $f Cal_Set_Clock_B($i)
       set j 0
       while {$j <= 7} {
            gets $f Cal_calib_mask($i,$j)
            gets $f Cal_channel_mask($i,$j)
            incr j 1
       }


# Cal_ignore added in V1.10
       for {set i 0} {$i < 20} {incr i 1} {
	   if {$version_number > 1.095} {
	       gets $f Cal_ignore($i)
	   } else {
	       set Cal_ignore($i) 0
	   }
       }

#-------------------------------------------------------------
# Done reading ... check the readout direction and the ignore
#-------------------------------------------------------------
       close $f
       set good [cal_check_direction]
       if {$good != -9999} {
	   cal_set_respond_to_read $good
	   set good2 [cal_check_ignore]
       }



#if the misc_global variables are not defined,
# we will get them from the hardwired defaults
        if {![info exists misc_tests_list(0)]} {
	       prod_tests_defaults 1
	}




       return 0
   }
#
# we get here and set hardwired defaults only if the user so desires
#

      set cal_dialog 0
      toplevel .set_defaults
      grab set .set_defaults
      label .set_defaults.l2 -text "Cannot open file with default settings \n"
      label .set_defaults.l1 -text "Cannot open file with default settings \n $cal_default_file \n Do you want to load hardwired defaults instead ?"
      button .set_defaults.b1 -text "Okay" -command "set cal_dialog 1"
      button .set_defaults.b2 -text "Cancel" -command "set cal_dialog 2"
      if {$ask == "YES"} {
          pack .set_defaults.l1 .set_defaults.b1 .set_defaults.b2 \
	       -side top -in .set_defaults
      } else {
          pack .set_defaults.l2 .set_defaults.b1 \
	       -side top -in .set_defaults
      }
      tkwait variable cal_dialog
      grab release .set_defaults
      destroy .set_defaults


       if {$cal_dialog == 1 && $ask == "YES"} {
	   cal_hard_defaults
	   return 0
       }

       return 1
   }
#############################################################
# here is the procedure to load hardwired defaults 
#############################################################
proc cal_hard_defaults {} {

       source cal_global.tcl
       source misc_global.tcl

       prod_tests_defaults 1
       set Cal_format_check 0x1ff
       set Cal_compressed_format 0
       set Cal_one_at_a_time 128
       set Cal_number_of_chips_top 1
       set xy_scan 0
       set xy_which_axis 1
       set xy_x_initial 1.0
       set xy_y_initial 2.0
       set xy_step_size 0.050
       set xy_nstep 1000
       set xy_start_flag 1


       set  Cal_error_limit 0
       set  Cal_histoscope 0
       set  Cal_fake 1
       set  Cal_write_file 0
       set  Cal_number_of_events 100
       set  Cal_number_of_chips 1
       set  Cal_number_of_buffers 1
       set  Cal_Side_Hybrid 1
       set i 0
       while {$i <= 19} {
         set  Cal_ignore($i) 0
         set  Cal_min_threshold($i) 0
         set  Cal_max_threshold($i) 63
         set  Cal_step_threshold($i) 63
         set  Cal_min_charge($i) 0 
         set  Cal_max_charge($i) 0 
         set  Cal_step_charge($i) 1
         set  Cal_jitter($i) 15
         set  Cal_shaping_time($i) 0
         set  Cal_sampling_rate($i) 2
	 set  Cal_skip($i) 0
         set  Cal_readout_direction($i) 1
	 set Cal_respond($i) 1
	 set Cal_clock_through($i) 0
	 set Cal_test_pattern($i) 0
	 set Cal_Set_Clock_B($i) 0
  
         set j 0
         while {$j <= 7} {
            set  Cal_calib_mask($i,$j)  0xffff
            set  Cal_channel_mask($i,$j) 0xffff
            incr j 1
         }
	 incr i 1
       }

       set i all
       set  Cal_min_threshold($i) 0
       set  Cal_max_threshold($i) 63
       set  Cal_step_threshold($i) 1
       set  Cal_min_charge($i) 0
       set  Cal_max_charge($i) 0
       set  Cal_step_charge($i) 1
       set  Cal_jitter($i) 15
       set  Cal_shaping_time($i) 0
       set  Cal_sampling_rate($i) 2
       set  Cal_skip($i) 0
       set  Cal_readout_direction($i) 1
       set Cal_respond($i) 1
       set Cal_clock_through($i) 0
       set Cal_test_pattern($i) 0
       set Cal_Set_Clock_B($i) 0
       set j 0
       while {$j <= 7} {
            set  Cal_calib_mask($i,$j) 0xffff
            set  Cal_channel_mask($i,$j) 0xffff
            incr j 1
       }
       set good [cal_check_direction]
       if {$good != -9999} {
	   cal_set_respond_to_read $good
           set good2 [cal_check_ignore]
       }

   return
   }


##############################################################
# This procedure is there to choose input file for defaults
##############################################################
proc cal_choose_def {} {

      source cal_global.tcl
      global cal_dialog
      global userfile
      global data_directory
      set cal_dialog -1


      toplevel .choose_def -bg #00f

      frame .choose_def.f1 -relief raised -bd 2 -bg #00f
      frame .choose_def.f2 -relief raised -bd 2 -bg #00f
      frame .choose_def.f3 -relief raised -bd 2 -bg #00f
      pack .choose_def.f1 .choose_def.f2 .choose_def.f3 -side top -pady 8

      grab set .choose_def

      label .choose_def.l1 \
      -text "Here you can choose to load \n defaults from a disk file. \n The default file is cal_defaults.store" \
      -bg #8dd -highlightthickness 0

      label .choose_def.l2 \
      -text "Note: Defaults will be loaded in software, \n NOT in hardware" \
      -bg #8dd -highlightthickness 0

      button .choose_def.b0 \
      -text "Load Hardwired \n defaults" -command "set cal_dialog 0" \
      -bg #8dd -activebackground #7cc -highlightthickness 0
   
      button .choose_def.b1 \
      -text "Load from \n default file" -command "set cal_dialog 1" \
      -bg #8dd -activebackground #7cc -highlightthickness 0

      button .choose_def.b2 \
      -text "Load from \n user file" \
      -command "set cal_dialog 2" \
      -bg #8dd -activebackground #7cc -highlightthickness 0

      button .choose_def.b4 \
      -text "Load from \n previous run" \
      -command "set cal_dialog 4" \
      -bg #8dd -activebackground #7cc -highlightthickness 0
      
      button .choose_def.b3 \
      -text "Cancel \n " \
      -command "set cal_dialog 3" \
      -bg #8dd -activebackground #7cc -highlightthickness 0



      label .choose_def.luser \
      -text "User file or previous runnumber:" \
      -bg #8dd -highlightthickness 0

      entry .choose_def.lentry -relief sunken -width 30 -textvariable userfile 

      pack .choose_def.l1 .choose_def.l2 -fill x -in .choose_def.f1

      pack .choose_def.b0 .choose_def.b1 .choose_def.b2 \
           .choose_def.b4 .choose_def.b3 \
      -side left -in .choose_def.f2 

      pack .choose_def.luser .choose_def.lentry -side left \
      -padx 3 -in .choose_def.f3


      tkwait variable cal_dialog
      grab release .choose_def
      destroy .choose_def

    if {$cal_dialog == 0} {cal_hard_defaults}
    if {$cal_dialog == 1} {cal_get_defaults cal_defaults.store YES}
    if {$cal_dialog == 2} {cal_get_defaults $userfile YES}
    if {$cal_dialog == 3} {return}
    if {$cal_dialog == 4} {
        set mytemp parameter_run
        set Cal_this_run_number $Cal_run_number
	cal_get_defaults $data_directory$mytemp$userfile YES
        set Cal_run_number $Cal_this_run_number
    }
    return
}











