#!cwish -f

# 
# VME Teststand Software
#
# Code revised by Armin Karcher (karcher@design.lbl.gov)
#
# Design and Code by:
#   Eric Gottlieb  (egott@unm.edu)
#   Tim Thomas     (thomas@hev1.phys.unm.edu)
#   Micheal Gold   (gold@hepv1.phys.unm.edu)
#
#

if {! [info exists cwish_exists]} {
    set cwish_exists 1
    rename exit exit.old
    rename source source.old
}

proc source {file} {
    set exts {"" ~/ tcl/ config/ ../tcl/ ../config/ src/tcl/ src/config/}
    foreach i $exts {
	if {[file isfile $i$file]} {return [uplevel 1 source.old "$i$file"]}
    }
    error "could not open \"$file\""
}


proc exit {} {
    puts "exiting cleanly"
    catch vme_reset
    catch vme_unsetserver
    exit.old
}
    
    source utils.tcl
    source vmeIo.tcl
    source shell.tcl
    source lowgui.tcl
    source pmcom.tcl

proc vme_init {} {
    global config config(default_server) current_server
    global already_have_initialized debug_level config(lock_teststand)

    set current_server $config(default_server)

    vme_setserver $config(default_server)

    if {$config(lock_teststand) == 1} {
	vme_resource lock teststand
    }

    if {![info exists debug_level]} { set debug_level 0 }

    vme_reset

    vme_debug $debug_level

    source hardware.tcl

    foreach card [vme_info cards] {
	global config(getstate_$card)
	pack [checkbutton .ts.ct.state.$card -text "$card" -variable config(getstate_$card) -onvalue "$card" -offvalue ""] -side left
    }
    pack [button .ts.ct.state.get -text "Get State" -command {
	foreach card [vme_info cards] {
	    if {$config(getstate_$card) != ""} {vme_getstate $config(getstate_$card)}
	}
    }] -side left -fill x -expand 1
}

proc vme_load_config { {filename config.tcl} } {
    global config
    source $filename
    #source /home/design/grpserv/karcher/v1/config/config.tcl
}

proc vme_save_config { {filename config.tcl} } {
    global config env
    catch {exec cp $filename "$filename\.bkup"} dummy
    set file [open $filename w]
    puts $file "\# configuration file for cwish.tcl"
    set f [open |date]
    puts $file "\# written [gets $f] by $env(USER)"
    puts $file ""
    puts $file "\# list of potential front end servers"
    puts $file "set config(server_hosts) \{$config(server_hosts)\}"
    puts $file ""
    puts $file "\# the front end server that will be used by default"
    puts $file "set config(default_server) $config(default_server)"
    puts $file ""
    puts $file "\# the name of this file"
    puts $file "set config(config_file_name) $filename"
    puts $file ""
    puts $file "\# 1 = do keep the teststand resource locked; 0 = don't"
    puts $file "set config(lock_teststand) $config(lock_teststand)"
    puts $file ""
    puts $file "\# cards to be checked with mainwindow getstate button"
    foreach card [vme_info cards] {
	puts $file "set config(getstate_$card) \"$config(getstate_$card)\""
    }
    puts $file ""

    puts $file "\# list of low level windows"
    puts $file "set config(windowlist) \{$config(windowlist)\}"
    foreach base $config(windowlist) {
	puts $file ""
	puts $file "  \# specific information (if necessary) regarding the low level window: $base"
	if {[info exists config($base\_snaplist)]} {
	    puts $file "  set config($base\_snaplist) \{$config($base\_snaplist)\}"
	}
	if {[info exists config($base\_state)]} {
	    puts $file "  set config($base\_state) $config($base\_state)"
	}
	if {[info exists config($base\_title)]} {
	    puts $file "  set config($base\_title) \"$config($base\_title)\""
	}
    }
    puts $file ""

    close $file
}

#main window definition
proc mainwindow {} {
    global config
    vme_load_config

    catch {destroy .ts} dummy
    toplevel .ts
    wm minsize .ts 0 0
    wm title .ts "Low-Level Object Interface."
    wm withdraw .
    
    set source_file_name cwish.tcl
    set other_server ""
    
    #there are three parts to the window: menu bar (.ts.mb) , controls (.ts.ct), and text shell (.ts.sh)
    
    #control's def's
    
    frame .ts.ct
    
    pack [frame .ts.ct.info] -side top -pady 1 -fill x -expand 1
    pack [label .ts.ct.info.dl -text "default:"] -side left -padx 1
    pack [label .ts.ct.info.ds -textvariable config(default_server) -relief groove -bd 2] -side left -padx 1 -fill x -expand 1
    pack [label .ts.ct.info.cl -text "server:"] -side left -padx 1
    pack [label .ts.ct.info.cs -textvariable current_server -relief groove -bd 2] -side left -padx 1 -fill x -expand 1
    
    button .ts.ct.init -text "Initialize" -command {vme_init}
    button .ts.ct.vme -text "VME Window" -command {vme_VmeIoWindow}
    button .ts.ct.low -text "Teststand Window" -command {vme_LowWindow}
    button .ts.ct.event -text "send command" -command {command_window}
    checkbutton .ts.ct.debug -text "debugging" -command {catch {vme_debug $debug_level}} -variable debug_level -onvalue 1 -offvalue 0;
    frame .ts.ct.state
    pack .ts.ct.init .ts.ct.state .ts.ct.vme .ts.ct.low .ts.ct.event .ts.ct.debug -side top -pady 1 -fill x -expand 1
        
    set mess "hello world"
    pack [frame .ts.ct.mess] -side top -pady 1 -fill x -expand 1 
    pack [button .ts.ct.mess.b -text "display" -command {vme_frontdisplay $mess}] -side left -padx 1
    pack [entry .ts.ct.mess.e -textvariable mess -relief sunken] -side left -padx 1 -fill x -expand 1
    bind .ts.ct.mess.e <Return> {evoke .ts.ct.mess.b} 
    
    pack [frame .ts.ct.cmd -relief ridge -bd 2] -side top -pady 1 -fill x -expand 1
    button .ts.ct.cmd.reload -text "reload" -command {
       source cwish.tcl
       source utils.tcl
       source vmeIo.tcl
       source shell.tcl
       source lowgui.tcl
       source pmcom.tcl}
    button .ts.ct.cmd.exit -text "exit" -command {exit}
    button .ts.ct.cmd.vxreboot -text "vxreboot" -command {exec vxreboot}
    button .ts.ct.cmd.reinstall -text "reinstall" -command {make install}
    pack .ts.ct.cmd.reload .ts.ct.cmd.exit .ts.ct.cmd.vxreboot .ts.ct.cmd.reinstall -side left -padx 1 -fill x -expand 1
    
    # menu definition
    
    set menu ts.mb
    
    frame .$menu -relief raised -bd 2
    
    menubutton .$menu.file -text File -underline 0 -menu .$menu.file.menu
    menubutton .$menu.opt -text Config -underline 0 -menu .$menu.opt.menu
    menubutton .$menu.win -text Windows -underline 0 -menu .$menu.win.menu
    menubutton .$menu.help -text Help -underline 0 -menu .$menu.help.menu
    
    pack .$menu.file .$menu.opt .$menu.win -side left
    pack .$menu.help -side right
    
    set file $menu.file.menu
    
    menu .$file
    .$file add command -label "Source" -underline 0 -accelerator " C s" -command {
	if {![info exists source_file_name]} {set source_file_name ""}
	set source_file_name [FSBox "enter file name" $source_file_name]
	if {$source_file_name != ""} {source $source_file_name}
    }
    .$file add command -label "Reload" -underline 0 -accelerator " C r" -command {source cwish.tcl}
    .$file add separator
    .$file add command -label "About" -command {error "Not implemented yet"}
    .$file add command -label "Exit" -underline 0 -accelerator " C e" -command exit
    
    set opt $menu.opt.menu
    
    menu .$opt
    .$opt add cascade -label "Server Host" -menu .$opt.s
    menu .$opt.s
    
    foreach svc $config(server_hosts) {
	.$opt.s add radiobutton -label "$svc" \
	    -variable config(default_server) -value $svc
    }
    .$opt.s add separator
    .$opt.s add command -label "other ..." -command {
	set config(default_server) [textreq "enter server host name" $config(default_server)]
    }
    
    .$opt add command -label "Unset Host" -underline 0 -accelerator " C u" -command vme_unsetserver
    .$opt add command -label "Initialize" -underline 0 -accelerator " C i" -command vme_initialize
    .$opt add checkbutton -label "Lock Teststand Resource" -variable config(lock_teststand)  -onvalue 1 -offvalue 0 -command {
	if ($config(lock_teststand)) {
	    catch {vme_resource lock teststand}
	} else {
	    catch {vme_resource unlock teststand}
	}
    }
    
    .$opt add separator
    .$opt add command -label "Save Config" -command {vme_save_config config.tcl}
    .$opt add command -label "Save Config As ..." -command {
	set config(config_file_name) [FSBox "enter config file name" $config(config_file_name)]
	if {$config(config_file_name) != ""} {vme_save_config $config(config_file_name)}
    }
    .$opt add command -label "Restore Config" -command {vme_load_config config.tcl}
    .$opt add command -label "Restore Config From ..." -command {
	global config
	set config(config_file_name) [FSBox "enter config file name" $config(config_file_name)]
	if {$config(config_file_name) != ""} {vme_load_config $config(config_file_name)}
    }
    
    set windows $menu.win.menu
    
    menu .$windows
    .$windows add command -label "Add low level interface" \
	-underline 0 -accelerator " C l" -command "vme_LowWindow"
    .$windows add command -label "vme interface" -command "vme_VmeIoWindow"
    .$windows add command -label "add shell (alpha)" -command {attach_shell ts}
    .$windows add command -label "Snapshot Low Level Windows" -command {vme_SnapWindows}
    .$windows add separator
    
    set help $menu.help.menu
    
    menu .$help
    .$help add command -label "Big help" \
	-underline 0 -accelerator " C b" -command {
	    toplevel .help
	    message .help.m -text "No help is avaiable at this time"
	    pack .help.m
	}
    
    tk_menuBar .ts.mb .ts.mb.file .ts.mb.opt .ts.mb.win .ts.mb.help
    focus .ts.mb
    
    pack .ts.mb .ts.ct -side top -pady 1 -fill x

#     foreach base $config(windowlist) {
# 	if {[winfo exists .$base]} {destroy .$base}
# 	if { $base == "vmeIo" } {
# 	    vme_VmeIoWindow
# 	} elseif {[info exists config($base\_snaplist)]} {
# 	    vme_LowWindow $base $config($base\_state) "$config($base\_title)"
# 	}
#     }

}

proc vme_AddWindow {base name} {
    set windows ts.mb.win.menu
    .$windows add command -label "$name" -command "wm deiconify .$base"
}

proc vme_DelWindow {base name} {
    global config
    set windows ts.mb.win.menu
    .$windows delete "$name"
    if {[info exists config($base\_snaplist)]} {
	unset config($base\_snaplist)
    }
    if {[info exists config($base\_state)]} {
	unset config($base\_state)
    }
    if {[info exists config($base\_title)]} {
	unset config($base\_title)
    }
    set i [lsearch $config(windowlist) $base]
    if {$i != -1} {
	set config(windowlist) [lreplace $config(windowlist) $i $i]
    }
    destroy .$base
}

proc vme_ModWindow {base oldname newname} {
    set windows ts.mb.win.menu
    .$windows entryconfigure $oldname -label $newname
}

proc vme_SnapWindows {} {
    global config
    set config(windowlist) [list]
    foreach w [winfo children .] {
	scan $w ".%s.%s" base leaf
	if {[info exists config($base\_snaplist)] || $base == "vmeIo" } {
	    lappend config(windowlist) $base
	    set config($base\_state) [wm state $w]
	    set config($base\_title) [wm title $w]
	}
    }
}

mainwindow

