#===========================================================
# this procedure takes a histoscope file and turns it
# into a hbook file, but puts the histoscope file
# in the same directory as the original histoscope file
# If widget_name is defined, it also outputs the results
# to the text widget called widget_name
# Returns "OK" or an error string 
#
# Mar 24 1998 - Also run quickfit at the end
#===========================================================
proc histo_to_hbook { histofile {widget_name ""}} {

source cal_global.tcl
global part_name  
#----------------------------------------------------
# if the file does not exist, complain and return
#----------------------------------------------------
  set file_exists [file exists $histofile]
  if {$file_exists != 1} {
      set ermsg \ 
         "PROCEDURE histo_to_hbook:\n  FILE\n  $histofile \n  DOES NOT EXIST"
      cal_display_error_message $ermsg
      return $ermsg
  }
#--------------------------------------------------------------
# if we are asked to put the results in a text
# widget, and this widget does not exist, complain and return
#--------------------------------------------------------------
  if {$widget_name != ""} {
      if {![winfo exists $widget_name]} {
	  set ermsg \
           "PROCEDURE histo_to_hbook:\n  WIDGET \n$widget_name\nDOES NOT EXIST"
	  cal_display_error_message $ermsg
	  return $ermsg
      }
      set dummy ".text"
      set w $widget_name$dummy
  }
#----------------------------------
# some information for the user
#----------------------------------
  if {$widget_name != ""} {
      $w insert end "\n\n_______________________________________________\n\n"
      $w insert end \
	      "Now trying to turn the histoscope file into an hbook file...\n"
      $w insert end \
	      "For this to work, \"tohbook\" must be in your PATH... \n\n"
      $w yview -pickplace end
  }
#----------------------------------
# this is the current directory
#----------------------------------
  set result [catch "set currentdir [pwd]" ermsg]
  if {$result != 0} {
      if {$widget_name != ""} {
	  $w insert end "$ermsg \n"
	  $w yview -pickplace end
      }
      return $ermsg
  }
#------------------------------------------------
# this is the directory of the histscope file 
#------------------------------------------------
  set histodir [file dirname $histofile]
#------------------------------------------------
# move to histodir
#------------------------------------------------
  if {$widget_name != ""} {
      $w insert end "\n cd $histodir \n"
      $w yview -pickplace end
  }
  set result [catch "cd $histodir" ermsg]
  if {$result != 0} {
      if {$widget_name != ""} {
	  $w insert end "$ermsg \n"
	  $w yview -pickplace end
      }
      return $ermsg
  }
#-----------------------------------------------------------------
# run tohbook...if it fails, restore the directory before exiting
#------------------------------------------------------------------
  set result [catch "exec tohbook $histofile > junkjunkfile" ermsg]
  if {$widget_name != ""} {
      $w insert end "\n tohbook $histofile \n"
      $w yview -pickplace end

#      if {[file exist junkjunkfile] == 1} {
#	  set f [open junkjunkfile]
#	  while {![eof $f]} {
#	      $w insert end [read $f 1000]
#	      $w yview -pickplace end
#	  }
	  catch "rm junkjunkfile"
#      }   
  }
  if {$result != 0} {
      if {$widget_name != ""} {
	  $w insert end "$ermsg \n"
          $w insert end "cd $currentdir \n"
	  $w yview -pickplace end
      }
      cd $currentdir
      return $ermsg
  }
#------------------------------------------------
# go back to the original directory
#------------------------------------------------
  if {$widget_name != ""} {
      $w insert end "\n cd $currentdir \n"
      $w yview -pickplace end
  }
  set result [catch "cd $currentdir" ermsg]
  if {$result != 0} {
      if {$widget_name != ""} {
	  $w insert end "$ermsg \n"
	  $w yview -pickplace end
      }
  }
#------------------------------------------------
# Tell the user that the Hbook file was made
#------------------------------------------------
  if {$widget_name != ""} {
      $w insert end "\n....Hbook file made succesfully.... \n"
      $w yview -pickplace end
  }
#------------------------------------------------
# Run quickfit, perhaps ?
#------------------------------------------------
  set answer [cal_yes_no "Do you want to run quickfit ?\n"]
  if {$widget_name != ""} {grab set $widget_name}
  if {$answer == "YES"} {
      if {$widget_name != ""} {
	  $w insert end "\nWe will run quickfit....\n"
          $w insert end "quickfit must be in your path...\n"
          $w yview -pickplace end
      }
      set this_part [string trim $part_name]
      set n [expr $Cal_number_of_events * $Cal_number_of_buffers]
      set fquick [open quick w]
      puts $fquick y
      puts $fquick $Cal_run_number
      puts $fquick $this_part
      puts $fquick $n
      puts $fquick y
      close $fquick
      set result [catch "exec quickfit $histofile < quick" ermsg]
      if {$result != 0} {
	  if {$widget_name != ""} {
	      $w insert end "$ermsg \n"
	      $w yview -pickplace end
	  }
	  return $ermsg
      } else {
	  if {$widget_name != ""} {
	      $w insert end "\n....quickfit completed..... \n"
	      $w yview -pickplace end
	  }
      }
  }
#------------------------------------------------
# done
#------------------------------------------------
  return "OK"
}
