##########################################################
# This drives the writing of registers
# flag = 1 --> write CntrlReg to given chip
# flag = 2 --> write Masks to given chip
# flag = 3 --> read all from given chip
# flag = 4 --> write default CntrlReg to ALL chips
# flag = 5 --> write default Masks to ALL chips
##########################################################
proc read_write_atom_registers {flag} {
source cal_global.tcl
global creg_read
global creg_write
global chamsk_write
global calmsk_write
global chamsk_read
global calmsk_read
global which_chip_to_test
global check_registers
global check_masks
global error_limit
global ignore_last_mask
global from_channel
#-----------------------------------------------
# check that we have a legal number of chips
#-----------------------------------------------
  set okeydokey [cal_check_chip_number]
  if {$okeydokey != 0} {return}
#-----------------------------------------------
# check that what we are writing is legal
#-----------------------------------------------
  if {$flag == 1} {
      set creg_write [fixup_hex $creg_write]
      set good [test_value creg_write $creg_write 0x0 0xffffffff]
      if {$good != 0} {return}
  } elseif {$flag == 2} {
      set j 0
      while {$j <= 3} {
         set chamsk_write($j) [fixup_hex $chamsk_write($j)]
         set calmsk_write($j) [fixup_hex $calmsk_write($j)]
	 set good [test_value chamsk_write($j) $chamsk_write($j) 0x0 0xffffffff]
	 if {$good != 0} {return}
	 set good [test_value calmsk_write($j) $calmsk_write($j) 0x0 0xffffffff]
	 if {$good != 0} {return}
	 incr j 1
     }
  } elseif {$flag == 3} {
       set creg_read "-"
       set from_channel "-"
       set j 0
       while {$j <= 3} {
          set chamsk_read($j) "-"
          set calmsk_read($j) "-"
          incr j 1
       }
   } elseif {$flag == 4} {
     set good [cal_check_direction]
       if {$good == -9999} {
	   return
       } else {
	   cal_set_respond_to_read $good
       }
       set good [cal_check_ignore]
       if {$good != 0} {return}
   } elseif {$flag > 5} {
      return
  }
#----------------------------------------------------
# check that we have been asked to test a legal chip
#----------------------------------------------------
  set temp [expr $Cal_number_of_chips-1]
  if {$flag < 4}  {
     set okeydokey [test_value which_chip_to_test $which_chip_to_test 0 $temp]
  } else {
     set okeydokey [test_value which_chip_to_test $which_chip_to_test -1 $temp]
  }
  if {$okeydokey != 0} {return}
#---------------------------------------------------------
# warn the user if toggling a dangerous bit
#---------------------------------------------------------
  if {$flag == 1} {
      set okeydokey [bit_toggle $creg_write]
      if {$okeydokey != 0} {return}
  }
#---------------------------------------------------------
# okay, setup is correct. Call C code that does the work
#----------------------------------------------------------
  rw_atom_registers $flag $which_chip_to_test
#------------------------------------------
# done
#------------------------------------------
return
}


##################################################################
# this procedure makes the widget used by the looped tests code
##################################################################
proc make_pltw_widget {} {
  if {![winfo exists .pltw]} {
       toplevel .pltw -bg #00f
       frame .pltw.f1 -bg #00f
       frame .pltw.f2 -bg #00f
       pack .pltw.f1 .pltw.f2 -side top -pady 3 -in .pltw
       button .pltw.close -text "Close This Window" \
		 -command {destroy .pltw} \
                 -bg #8dd  \
                 -activebackground #7cc  \
                 -highlightthickness 0        
       pack .pltw.close -in .pltw.f1 -fill x
       text .pltw.text \
	    -relief raised -bd 2 \
	    -yscrollcommand ".pltw.scroll set" \
	    -bg #aff -highlightthickness 0 \
	    -selectbackground #0ff

       scrollbar .pltw.scroll \
	         -command ".pltw.text yview" \
		 -highlightthickness 0 -relief ridge -bd 4 \
		 -bg #8dd -troughcolor #00f \
		 -activebackground #7cc
       pack .pltw.text .pltw.scroll -in .pltw.f2 -side left -fill y
       tkwait visibility .pltw.scroll
  }
}
##################################################################
# This procedure calls the C code that performs the looped tests
###################################################################
proc perform_looped_tests {} {
source cal_global.tcl
global which_chip_to_test
global check_registers
global check_masks
global error_limit
global ignore_last_mask
global creg_write
global creg_read
global from_channel
global all_or_some
global chip_test_list
global direction_choice


#-----------------------------------------------
# check that we have a legal number of chips
#-----------------------------------------------
  set okeydokey [cal_check_chip_number]
  if {$okeydokey != 0} {return}
#----------------------------------------------------
# check that we have been asked to test a legal chip
# OBSOLETE OBSOLETE OSOLETE OBSOLETE OBSOLETE
#----------------------------------------------------
#  set temp [expr $Cal_number_of_chips-1]
#  set okeydokey [test_value which_chip_to_test $which_chip_to_test -1 $temp]
#  if {$okeydokey != 0} {return}
#--------------
# make the text widget
#--------------
  make_pltw_widget
#------------------------------------------
# set the grab while we are doing the test
#------------------------------------------
  grab set .pltw
#---------------------------------------------------------------
# check the parameters... is there anything we need to test ?
#----------------------------------------------------------------
  set okeydokey [check_looped_tests_parameters]
  if {$okeydokey != 0} {return} 
#---------------------------------------------------------------------------
# make sure that the direction makes sense 
#---------------------------------------------------------------------------
  set good [cal_check_direction]
  if {$good == -9999} {
	return
  } else {
      cal_set_respond_to_read $good
  }
#----------------------------------------------------------------------------
# store the default readout direction so that we can restore it
#----------------------------------------------------------------------------
  for {set i 0} {$i < $Cal_number_of_chips} {incr i 1} {
      set Cal_readout_direction_store($i) $Cal_readout_direction($i) 
      set Cal_respond_store($i)           $Cal_respond($i)
      set Cal_Set_Clock_B_store($i)       $Cal_Set_Clock_B($i)
  } 
#--------------------------------------------------------------
# some shorthand variables to avoid lots of typing
#--------------------------------------------------------------
  set p2 $check_registers
  set p3 $check_masks
  set p4 $error_limit
  set p5 $ignore_last_mask
#--------------------------------------------------------------
# Testing with the existing readout direction setup
#--------------------------------------------------------------
  if {$direction_choice(0) == 1} {
      .pltw.text insert end "\n \n^^^^^^^^^^^^^^^^^^^^^^^^^^^^\n"
      .pltw.text insert end "TESTING USING EXISTING SETTINGS \n"
      .pltw.text insert end "^^^^^^^^^^^^^^^^^^^^^^^^^^^^\n"
      .pltw.text yview -pickplace end
      if {$all_or_some == -1} {
	  set good [loop_test -1 $p2 $p3 $p4 $p5]
          if {$good != "OK"} {
	      .pltw.text insert end "\n \n^^^^^^^^^^^^^^^^^^^^^^^^^^^^\n"
	      .pltw.text insert end "TESTING NOT COMPLETED \n"
	      .pltw.text insert end "^^^^^^^^^^^^^^^^^^^^^^^^^^^^\n"
              .pltw.text yview -pickplace end
	      grab release .pltw
	      return
	  }
      } else {
	  for {set i 0} {$i < $Cal_number_of_chips} {incr i 1} {
	      if {$chip_test_list($i) == 1} {
		  set good [loop_test $i $p2 $p3 $p4 $p5]
		  if {$good != "OK"} {
		      .pltw.text insert end "\n\n^^^^^^^^^^^^^^^^^^^^^^^^^^^^\n"
		      .pltw.text insert end "TESTING NOT COMPLETED \n"
		      .pltw.text insert end "^^^^^^^^^^^^^^^^^^^^^^^^^^^^\n"
		      .pltw.text yview -pickplace end
		      grab release .pltw
		      return
		  }
	      }
	  }
      }
  }
	  

#-----------------------------------------------------------------
# Testing from the left and the right, with clock A and clock B 
#-----------------------------------------------------------------
  set dir(0) "LEFT"
  set dir(1) "RIGHT"
  set clk(0) "CLK/CMD A"
  set clk(1) "CLK/CMD B"
  for {set iclk 0} {$iclk < 2} {incr iclk 1} {
   for {set i 0} {$i < 2} {incr i 1} {
      if {$i == 0} {
	  set jtop [expr $iclk*4 + 1]
      } else {
	  set jtop [expr $iclk*4 + 3]
      }
      set jbot [expr $jtop+1]

#- are we even trying to test with this direction ???
      if {$direction_choice($jtop) == 1 || $direction_choice($jbot) == 1} {
	  .pltw.text insert end "\n \n^^^^^^^^^^^^^^^^^^^^^^^^^^^^\n"
	  .pltw.text insert end "TESTING FROM THE $dir($i) USING $clk($iclk)\n"
	  .pltw.text insert end "^^^^^^^^^^^^^^^^^^^^^^^^^^^^\n"
          .pltw.text yview -pickplace end


#- find the first and last chip that we are potentially testing
	  if {$direction_choice($jtop) == 1 && $direction_choice($jbot) == 1} {
	      set chip1 0
	      set chip2 $Cal_number_of_chips
	      set p1 -1
	  } elseif {$direction_choice($jtop) == 1} {
	      set chip1 0
	      set chip2 [expr $Cal_number_of_chips_top - 1]
	      set p1 -2
	  } else {
	      set chip1 $Cal_number_of_chips_top
	      set chip2 $Cal_number_of_chips
	      set p1 -3
	  }



#- now set the readout direction... good should NEVER be returned as -9999 !!!
#- Also, set the clock (bug fix 9 jun 98)
	  for {set j $chip1} {$j < $chip2} {incr j 1} {
	      set Cal_readout_direction($j) $i
	      set Cal_Set_Clock_B($j) $iclk
	  }
          set good [cal_check_direction]
          if {$good == -9999} {
	      for {set k 0} {$k < $Cal_number_of_chips} {incr k 1} {
		set Cal_readout_direction($k) $Cal_readout_direction_store($k) 
                set Cal_respond($k)           $Cal_respond_store($k)
		set Cal_Set_Clock_B($k)       $Cal_Set_Clock_B_store($k)
	      } 
	      grab release .pltw
	      return
          } else {
            cal_set_respond_to_read $good
          }

#- now we are ready to test it...
          if {$all_or_some == -1} {
	      set good [loop_test $p1 $p2 $p3 $p4 $p5]
	  } else {
	      set good "OK"
	      for {set j $chip1} {$j <= $chip2 && $good == "OK"} {incr j 1} {
		  if {$chip_test_list($j) == 1} {
		      set good [loop_test $j $p2 $p3 $p4 $p5]
		  }
	      }
	  }
	      
#- if there was an error restore the original direction, and return
          if {$good != "OK"} {
	      for {set k 0} {$k < $Cal_number_of_chips} {incr k 1} {
		set Cal_readout_direction($k) $Cal_readout_direction_store($k) 
                set Cal_respond($k)           $Cal_respond_store($k)
		set Cal_Set_Clock_B($k)       $Cal_Set_Clock_B_store($k)

	      } 
	      .pltw.text insert end "\n \n^^^^^^^^^^^^^^^^^^^^^^^^^^^^\n"
	      .pltw.text insert end "TESTING NOT COMPLETED \n"
	      .pltw.text insert end "^^^^^^^^^^^^^^^^^^^^^^^^^^^^\n"
	      .pltw.text yview -pickplace end
	      grab release .pltw
	      return
	  }
#- close the first "if direction_choice...." block
      }

#- close the for loop over LEFT and right
   }	      
#- close the loop over clk/cmd A and B
  }


#-----------------------------------------
# Restore the original readout direction
#----------------------------------------
  for {set k 0} {$k < $Cal_number_of_chips} {incr k 1} {
      set Cal_readout_direction($k) $Cal_readout_direction_store($k) 
      set Cal_respond($k)           $Cal_respond_store($k)
      set Cal_Set_Clock_B($k)       $Cal_Set_Clock_B_store($k)

  }   
#-----------------------------------
# done
#-----------------------------------
  .pltw.text insert end "\n \n^^^^^^^^^^^^^^^^^^^^^^^^^^^^\n"
  .pltw.text insert end "ALL TESTING COMPLETED \n"
  .pltw.text insert end "^^^^^^^^^^^^^^^^^^^^^^^^^^^^\n"
  .pltw.text yview -pickplace end
  grab release .pltw
  return
}
##########################################################
# This procedure defines the window for the looped tests
# and for the reading/writing to registers
##########################################################
proc looped_tests_window {} {
source cal_global.tcl
global creg_read
global creg_write
global chamsk_write
global calmsk_write
global chamsk_read
global calmsk_read
global which_chip_to_test
global check_registers
global check_masks
global error_limit
global from_channel
global ignore_last_mask
global all_or_some
global chip_test_list
global direction_choice

#-- set variables to default values
   set creg_read "-"
   set from_channel "-"
   set creg_write 0x29908814
   set which_chip_to_test 0
      set j 0
      while {$j <= 3} {
	  set chamsk_read($j) "-"
	  set calmsk_read($j) "-"
	  set chamsk_write($j) 0xffffffff
	  set calmsk_write($j) 0xffffffff
	  incr j 1
      }

   if {![info exists all_or_some]} {
       prod_tests_defaults 0
   }
#-------
# if the cal_global variables are not filled, fill them.
#-------
     if {![info exists Cal_number_of_chips]} {
	 cal_get_defaults cal_defaults.store YES
     }
#--------------------------------------------
# probably better if these are defined 
#--------------------------------------------
  set Cal_run "stop"
  cal_get_run_number
#-------
# if the window does not exists, make it
#-------
     if {![winfo exists .ltw]} {
	 toplevel .ltw -bg #00f

# frames
         frame .ltw.f -bg #00f
	 frame .ltw.f0 -bg #00f
	 frame .ltw.f100 -bg #00f
	 frame .ltw.f1 -bg #00f
	 frame .ltw.f2 -bg #00f
	 frame .ltw.f3 -bg #00f
	 frame .ltw.f4 -bg #00f
	 frame .ltw.f10 -bg #00f
	 frame .ltw.f200 -bg #00f
         frame .ltw.f1.0 -bg #00f
         frame .ltw.f1.2 -bg #00f
         frame .ltw.f1.1 -bg #00f
         frame .ltw.f10.0 -bg #00f
         frame .ltw.f10.1 -bg #00f
         frame .ltw.f11 -bg #00f
         frame .ltw.f12 -bg #00f
         frame .ltw.f12.0 -bg #00f
         frame .ltw.f12.1 -bg #00f
         frame .ltw.f12.2 -bg #00f
         frame .ltw.f12.3 -bg #00f
         frame .ltw.f12.4 -bg #00f
         frame .ltw.f12.5 -bg #00f
         frame .ltw.f12.6 -bg #00f
         frame .ltw.f12.7 -bg #00f
         frame .ltw.f3.0 -bg #00f
         frame .ltw.f3.1 -bg #00f
         frame .ltw.f5 -bg #00f
	 pack .ltw.f .ltw.f0 .ltw.f100 .ltw.f1 .ltw.f10 .ltw.f11 .ltw.f12 \
              .ltw.f2 .ltw.f3 .ltw.f200 .ltw.f4 .ltw.f5 -side top -pady 3 
         pack .ltw.f1.0 .ltw.f1.2 .ltw.f1.1 -side left -padx 3 -in .ltw.f1
         pack .ltw.f10.0 .ltw.f10.1 -side left -padx 3 -in .ltw.f10
         pack .ltw.f3.0 .ltw.f3.1 -side left -padx 3 -in .ltw.f3
         pack .ltw.f12.0 .ltw.f12.1 .ltw.f12.2 \
	      .ltw.f12.3 .ltw.f12.4 .ltw.f12.5 .ltw.f12.6 \
	      .ltw.f12.7 -side top -padx 3 -in .ltw.f12

# message
         label .ltw.label \
	       -text "Test reading and writing of \n control registers and masks" \
               -bg #ff0 \
	       -highlightthickness 0 
         pack .ltw.label -fill x -in .ltw.f


# load defaults, close, help
         button .ltw.exit -text Close \
		 -command {destroy .ltw} \
                 -bg #8dd  \
                 -activebackground #7cc  \
                 -highlightthickness 0        
         button .ltw.help -text Help \
		 -command {cal_help .register_help register_helpfile} \
                 -bg #8dd  \
                 -activebackground #7cc  \
                 -highlightthickness 0        
         button .ltw.rd1 \
               -text "Set defaults\nin software" \
               -command "cal_choose_def" \
               -bg #8dd  \
               -activebackground #7cc  \
               -highlightthickness 0
         button .ltw.rd100 \
               -text "Default CRs\nto hybrid" \
               -command "read_write_atom_register 4" \
               -bg #8dd  \
               -activebackground #7cc  \
               -highlightthickness 0
         button .ltw.rd101 \
               -text "Default MSKs\nto hybrid" \
               -command "read_write_atom_register 5" \
               -bg #8dd  \
               -activebackground #7cc  \
               -highlightthickness 0

         pack  .ltw.exit .ltw.help \
	      -fill x -padx 5 -side left -in .ltw.f0
         pack .ltw.rd1 .ltw.rd100 .ltw.rd101 -side left -in .ltw.f100


# number of chips
         label .ltw.num_chips_label \
                -text "No. of chips\n (total)" \
                -bg #8dd \
                -highlightthickness 0
         entry .ltw.num_chips_entry \
                -relief sunken \
                -width 3 \
                -textvariable Cal_number_of_chips  \
                -bg #aff   \
                -highlightcolor #f00 \
                -highlightbackground #8dd \
                -selectbackground #0ff

         label .ltw.num_chips_label2 \
                -text "No. of chips\n (top side)" \
                -bg #8dd \
                -highlightthickness 0
         entry .ltw.num_chips_entry2 \
                -relief sunken \
                -width 3 \
                -textvariable Cal_number_of_chips_top  \
                -bg #aff   \
                -highlightcolor #f00 \
                -highlightbackground #8dd \
                -selectbackground #0ff

# one-sided or two sided hybrid
        radiobutton .ltw.oneside \
                -text "One-Sided Hybrid" \
                -variable Cal_Side_Hybrid \
                -value 1 \
                -anchor w\
                -bg #8dd  \
                -activebackground #7cc \
                -highlightthickness 0\
                -relief raised
        radiobutton .ltw.twoside \
                -text "Two-Sided Hybrid" \
                -variable Cal_Side_Hybrid \
                -value 2 \
                -anchor w \
                -bg #8dd  \
                -activebackground #7cc \
                -highlightthickness 0\
                -relief raised

# pack number of chips, hybrid
        pack .ltw.num_chips_label .ltw.num_chips_entry -fill x -in .ltw.f1.0 
        pack .ltw.num_chips_label2 .ltw.num_chips_entry2 -fill x -in .ltw.f1.2 
	pack  .ltw.oneside .ltw.twoside -side top -fill x -in .ltw.f1.1 

# Write control registers
         button .ltw.wcr \
                -text "Write Control \n Registers (hex)" \
		-command {read_write_atom_registers 1} \
                -bg #f0d \
                -highlightthickness 0
         entry .ltw.wcr_entry \
                -relief sunken \
                -width 3 \
                -textvariable creg_write  \
                -bg #f0f   \
                -highlightcolor #f00 \
                -highlightbackground #8dd \
                -selectbackground #0ff

# Read control registers
         button .ltw.rcr \
                -text "Read Control \n Registers (hex)" \
		-command {read_write_atom_registers 3} \
                -bg #f00 \
                -highlightthickness 0
         entry .ltw.rcr_entry \
                -relief sunken \
                -width 3 \
                -textvariable creg_read  \
                -bg #f00   \
                -highlightcolor #f00 \
                -highlightbackground #8dd \
                -selectbackground #0ff

# pack read/write control registers
         pack .ltw.wcr .ltw.wcr_entry -fill x -in .ltw.f10.0 
	 pack  .ltw.rcr .ltw.rcr_entry -side top -fill x -in .ltw.f10.1 



# Write masks
         button .ltw.wrmsk \
                -text "Write Masks (hex)" \
		-command {read_write_atom_registers 2} \
                -bg #f0d \
                -highlightthickness 0
         set j 0
         while {$j <=3} {
	     set k1 [expr $j*32]
	     set k2 [expr $k1+31]
	     set temp "ChaMsk"
 	     label .ltw.camlw$j \
	             -text "$temp  $k1-$k2" -bg #8dd -highlightthickness 0 \
		   -anchor center -width 15
	     entry .ltw.chmew$j -relief sunken -width 11 \
		   -textvariable chamsk_write($j) -bg #f0f \
		   -highlightcolor #f00 -highlightbackground #8dd \
		   -selectbackground #0ff
	     incr j 1
	 }
	 set j 0
         while {$j <=3} {
	     set k1 [expr $j*32]
	     set k2 [expr $k1+31]
	     set temp "CalMsk"
	     label .ltw.callw$j \
	           -text "$temp $k1-$k2" -bg #8dd -highlightthickness 0 \
		   -anchor center -width 15
	     entry .ltw.calew$j -relief sunken -width 11 \
		   -textvariable calmsk_write($j) -bg #f0f \
		   -highlightcolor #f00 -highlightbackground #8dd \
		   -selectbackground #0ff
	     incr j 1
	 }

# Read masks
         button .ltw.rdmsk \
                -text "Read Masks (hex)" \
		-command {read_write_atom_registers 3} \
                -bg #f00 \
                -highlightthickness 0
         set j 0
         while {$j <=3} {
	     set k1 [expr $j*32]
	     set k2 [expr $j+31]
	     entry .ltw.chmer$j -relief sunken -width 11 \
		   -textvariable chamsk_read($j) -bg #f00 \
		   -highlightcolor #f00 -highlightbackground #8dd \
		   -selectbackground #0ff
	     incr j 1
	 }
	 set j 0
         while {$j <=3} {
	     set k1 [expr $j*32]
	     set k2 [expr $j+31]
	     entry .ltw.caler$j -relief sunken -width 11 \
		   -textvariable calmsk_read($j) -bg #f00 \
		   -highlightcolor #f00 -highlightbackground #8dd \
		   -selectbackground #0ff
	     incr j 1
	 }

# pack read/write masks
	 pack .ltw.wrmsk .ltw.rdmsk -fill x -padx 3 -side left -in .ltw.f11
         set j 0
         while {$j <= 3} {
	     pack .ltw.camlw$j .ltw.chmew$j .ltw.chmer$j -fill x -side left \
		  -padx 3 -in .ltw.f12.$j
	     incr j 1
	 }
	 set j 0
	 while {$j <= 3} {
	     set k [expr $j+4]
	     pack .ltw.callw$j .ltw.calew$j .ltw.caler$j -fill x -side left \
		  -padx 3 -in .ltw.f12.$k
	     incr j 1
	 }


# which chip to test ?
         label .ltw.which_chip_label \
                -text "Enter the chip number \n for Read/Write CR and MSK" \
                -bg #8dd \
                -highlightthickness 0
         entry .ltw.which_chip_entry \
                -relief sunken \
                -width 3 \
                -textvariable which_chip_to_test  \
                -bg #aff   \
                -highlightcolor #f00 \
                -highlightbackground #8dd \
                -selectbackground #0ff
         

         pack .ltw.which_chip_label \
	      .ltw.which_chip_entry \
	      -side left -in .ltw.f2 -padx 3 -fill x


# test it
         button .ltw.test_button \
               -text "Perform \n loop tests" \
               -command "perform_looped_tests" \
               -bg #8dd  \
               -activebackground #7cc  \
               -highlightthickness 0
         button .ltw.setup \
               -text "Setup \n loop tests" \
               -command "looped_tests_setup" \
               -bg #8dd  \
               -activebackground #7cc  \
               -highlightthickness 0
         pack .ltw.test_button .ltw.setup -side left -in .ltw.f3




# where we read from
        label .ltw.from1 \
                -text "Registers read from PM chan " \
                -bg #8dd \
                -highlightthickness 0
        label .ltw.from2 \
               -textvariable from_channel \
                -bg #8dd \
                -highlightthickness 0
         pack .ltw.from1 .ltw.from2 -in .ltw.f5 -side left -fill x

}
}

#########################################################
# check for toggling of SetClockB and ClockThrough bits
#########################################################
proc bit_toggle {value} {
  set good 0
  global cal_dialog
  set junk [expr $value & 0xbdffffff]
  if {$value != $junk} {
      set cal_dialog -1
      toplevel .dialog
      grab set .dialog
      label .dialog.label \
	    -text "You are trying to set \n the clock through bit \n or the bit that changes \n the command line. \n Are you sure ?" \
	    -bd 2 -bg #aff -highlightthickness 0
      button .dialog.yes -text "Yes" \
	     -command {set cal_dialog 0} -relief raised -bd 2 -bg #8dd \
	     -activebackground #7cc -highlightthickness 0
      button .dialog.cancel -text "Cancel" \
	     -command {set cal_dialog 1} -relief raised -bd 2 -bg #8dd \
	     -activebackground #7cc -highlightthickness 0
      pack .dialog.label .dialog.yes .dialog.cancel -side top -in .dialog
      tkwait variable cal_dialog
      grab release .dialog
      destroy .dialog
      set good $cal_dialog
  }
  return $good
}

#########################################################
# Looped tests setup window
#########################################################
proc looped_tests_setup { } { 
source cal_global.tcl
global creg_read
global creg_write
global chamsk_write
global calmsk_write
global chamsk_read
global calmsk_read
global which_chip_to_test
global check_registers
global check_masks
global error_limit
global ignore_last_mask
global from_channel
global all_or_some
global chip_test_list
global direction_choice


#-----------------------------------------------
# check that we have a legal number of chips
#-----------------------------------------------
  set okeydokey [cal_check_chip_number]
  if {$okeydokey != 0} {return}

#-------
# if the window does not exists, make it
#-------
     if {![winfo exists .ltws]} {
	 toplevel .ltws 
     
# frames
	 frame .ltws.f0 -bg #00f
         frame .ltws.f1 -bg #8dd
	 frame .ltws.f2 -bg #00f
	 frame .ltws.f3 -bg #8dd
	 frame .ltws.f4 -bg #00f
	 pack .ltws.f0 .ltws.f1 \
              .ltws.f2 .ltws.f3 .ltws.f4 -side top -pady 5

# frame 0 : close window, help
         button .ltws.f0.exit -text Close \
		 -command {destroy .ltws} \
                 -bg #8dd  \
                 -activebackground #7cc  \
                 -highlightthickness 0
         button .ltws.f0.help -text Help \
		 -command {cal_help .register_setup_help register_setup_helpfile} \
                 -bg #8dd  \
                 -activebackground #7cc  \
                 -highlightthickness 0        
         button .ltws.f0.restore -text "Restore Defaults" \
		 -command {prod_tests_defaults 0} \
                 -bg #8dd  \
                 -activebackground #7cc  \
                 -highlightthickness 0
          pack .ltws.f0.exit .ltws.f0.help .ltws.f0.restore \
		  -fill x -in .ltws.f0 -side left -padx 5

# frame 1 : test all at once or one by one
         radiobutton .ltws.f1.rb00 -text "Test all chips at once" \
		 -variable all_or_some -value -1  -bg #8dd \
                 -activebackground #7cc -highlightthickness 0
         radiobutton .ltws.f1.rb01 -text "Only test selected chips" \
		 -variable all_or_some -value -2  -bg #8dd \
                 -activebackground #7cc -highlightthickness 0
         pack .ltws.f1.rb00 .ltws.f1.rb01 -side top -in .ltws.f1 \
		 -anchor w

# frame 2 : list of chips to set up
         for {set i 0} {$i < $Cal_number_of_chips} {incr i 1} {
	     checkbutton .ltws.f2.$i -text "Chip $i" \
		-variable chip_test_list($i)  -bg #8dd \
                 -activebackground #7cc -highlightthickness 0
	     pack .ltws.f2.$i  -in .ltws.f2 -anchor w -fill x
	 }

# frame 3 : direction
         set txt(0) "Use existing setting"
         set txt(1) "Read top of HDI from the left..clk/cmd A"
         set txt(2) "Read bottom HDI from the left..clk/cmd A"
         set txt(3) "Read top of HDI from the right..clk/cmd A"
         set txt(4) "Read bottom of HDI from the right..clk/cmd A"
         set txt(5) "Read top of HDI from the left..clk/cmd B"
         set txt(6) "Read bottom HDI from the left..clk/cmd B"
         set txt(7) "Read top of HDI from the right..clk/cmd B"
         set txt(8) "Read bottom of HDI from the right..clk/cmd B"
         label .ltws.f3.label \
                -text "Readout Direction. Command lines\nMore than one can be checked" \
                -bg #8dd \
                -highlightthickness 0
	 pack .ltws.f3.label -in .ltws.f3
         for {set i 0} {$i < 9} {incr i 1} {
	     checkbutton .ltws.f3.$i -text $txt($i) \
		-variable direction_choice($i)  -bg #8dd \
                 -activebackground #7cc -highlightthickness 0
	     pack .ltws.f3.$i -in .ltws.f3 -anchor w
	 }

# frame 4: error_limit, skip checking last mask
         checkbutton .ltws.skip -text \
		     "Do not check bit 127 of ChaMsk" \
		     -variable ignore_last_mask \
                     -anchor w \
                     -bg #fff  \
                     -activebackground #8aa \
                     -highlightthickness 0\
                     -relief raised
         label .ltws.err_label \
                -text "Quit after this many errors" \
                -bg #fff \
                -highlightthickness 0
         entry .ltws.err_entry \
                -relief sunken \
                -width 6 \
                -textvariable error_limit  \
                -bg #fff   \
                -highlightcolor #f00 \
                -highlightbackground #fff \
                -selectbackground #0ff
         checkbutton .ltws.cr -text \
		     "Check Control Registers" \
		     -variable check_registers \
                     -anchor w \
                     -bg #fff  \
                     -activebackground #8aa \
                     -highlightthickness 0\
                     -relief raised
         checkbutton .ltws.msk -text \
		     "Check Masks" \
		     -variable check_masks \
                     -anchor w \
                     -bg #fff  \
                     -activebackground #8aa \
                     -highlightthickness 0\
                     -relief raised


         pack .ltws.cr   -in .ltws.f4 -fill x
         pack .ltws.msk  -in .ltws.f4 -fill x
         pack .ltws.skip -in .ltws.f4 -fill x
         pack .ltws.err_label .ltws.err_entry -in .ltws.f4 -side left \
		 -fill x -anchor w -pady 3



}
}


#########################################################################
# procedure called at beginning of looped tests to make sure that the 
# parameters are OK.  Return 0 if OK.
#########################################################################
proc check_looped_tests_parameters { } {

source cal_global.tcl
global creg_read
global creg_write
global chamsk_write
global calmsk_write
global chamsk_read
global calmsk_read
global which_chip_to_test
global check_registers
global check_masks
global error_limit
global ignore_last_mask
global from_channel
global all_or_some
global chip_test_list
global direction_choice



# check hybrid and chip numbers
    set good [cal_check_chip_number]
    if {$good == 1} {
      .pltw.text insert end "Illegal number of chips \n"
      .pltw.text yview -pickplace end
      grab release .pltw
      return $good
    }


# check that we are asked to check CR or MSK (or both)
  if {$check_registers == 0 && $check_masks == 0} {
      .pltw.text insert end "There is nothing to test !!! \n"
      .pltw.text insert end "Both CR and MSK tests are disabled !!! \n"
      .pltw.text yview -pickplace end
      grab release .pltw
      set good 1
      return $good
  }

# check that at least one direction has been selected
  set direction_selected_top    "NO"
  set direction_selected_bottom "NO"
  for {set i 0} {$i < $Cal_number_of_chips} {incr i 1} {
      set chip_selected($i) "NO"
      set this_side "BOTTOM"
      if {$i < $Cal_number_of_chips_top} {
	  set this_side "TOP"
      }

      if {$all_or_some == -1 || $chip_test_list($i) == 1} {
	  if {$direction_choice(0) == 1} {
	      set chip_selected($i) "YES"
	  }

	  
	  for {set k 1} {$k < 9} {incr k 2} {
	      if {$direction_choice($k) == 1} {
		  if {$this_side == "TOP"} {
		      set chip_selected($i) "YES"
		  }
	      }
	  }
          for {set k 2} {$k < 9} {incr k 2} {
	      if {$direction_choice($k) == 1} {
		  if {$this_side == "BOTTOM"} {
		      set chip_selected($i) "YES"
		  }
	      }
	  }

      }


  }
  set at_least_one "NO"
  for {set i 0} {$i <  $Cal_number_of_chips} {incr i 1} {
      if {$chip_selected($i) == "YES"} {
	  set at_least_one "YES"
      }
  }
  if {$at_least_one == "NO"} {
      .pltw.text insert end "There is nothing to test !!! \n"
      .pltw.text insert end "The combination of readout direction and chips\n"
      .pltw.text insert end "to test is such that no chip is to be tested\n"
      .pltw.text yview -pickplace end
      grab release .pltw
      set good 1 
      return $good
  }




set good 0
return $good
}
	





