# 
# VME Teststand Software
#
# Code revised by Armin Karcher (karcher@design.lbl.gov)
#
# Design and Code by:
#   Eric Gottlieb  (egott@unm.edu)
#   Tim Thomas     (thomas@hev1.phys.unm.edu)
#   Micheal Gold   (gold@hepv1.phys.unm.edu)
#
#  Updated 11/95 by Armin Karcher

proc Regx {c base {offset ""}} {
    eval global [vme_info cards]
    if {$offset != ""} {
	return "$base$c\($offset\)"
    } else {
	return "[vme_info cardinst $base]$c\([vme_info name $base]\)"
    }
}

proc Reg   {base {offset ""}} { return [Regx "" $base $offset] } 
# register name
proc Regv  {base {offset ""}} { return [Regx v  $base $offset] } 
# global (read) value of register 
proc Regw  {base {offset ""}} { return [Regx w  $base $offset] } 
# global (write) value of register 
proc Regwf {base {offset ""}} { return [Regx wf $base $offset] } 
# global write file for register 
proc Regrf {base {offset ""}} { return [Regx rf $base $offset] } 
# gobbal read file for register 
proc Regra {base {offset ""}} { return [Regx ra $base $offset] } 
# gobbal read file address for register 
proc Regrl {base {offset ""}} { return [Regx rl $base $offset] } 
# gobbal read file length for register 
proc Regh  {base {offset ""}} { return [Regx h  $base $offset] } 
# global hex value of register 
proc hextrace_update {name element op} {
     foreach card [vme_info cards] {
 	global $card\h
     }
    if {$element != ""} { set name ${name}($element) }
    upvar #0 $name reg
    regexp {(.+)(v)(\()(.+)(\))} $name regh base v popen offset pclose
    set [Regh $base $offset] [format "0x%4.4x" $reg]
}

proc addreg {base card reg} {
    eval global [vme_info cards] config
    upvar #0 $base\_snaplist snaplist

    set r $card\($reg\)
    if { [lsearch -exact $snaplist $r] == -1 } {
	lappend snaplist $r
	dispreg $base $card $reg
    }
}

proc delreg {base card reg} {
    eval global [vme_info cards]
    upvar #0 $base\_snaplist snaplist

    set r $card\($reg\)
    set top $base.$card\_$reg
    set n [lsearch -exact $snaplist $r]
    set snaplist [lreplace $snaplist $n $n]
    destroy .$top
}

proc dispinfo {reg} {
    eval global [vme_info cards] 
    set infolist { card name class type access offset mask vmask shift size fullreg emptyreg address}
    set t .reginfo
    if {![winfo exists $t]} {
	toplevel $t \
		-bg #00f
	foreach i $infolist {
	    pack [frame $t.$i	\
		-bg #00f \
		-highlightthickness 0] 	\
			-fill x 	\
			-expand 1
	    pack [label $t.$i.name \
		-text "$i:" 	\
		-anchor w 	\
		-width 9	\
		-bg #aff	\
		-highlightthickness 0] 		\
			-side left
	    pack [label $t.$i.value 		\
		-text "[eval [list vme_info $i $reg]]"\
		-bg #aff	\
		-highlightthickness 0] 		\
			-side left 	\
			-fill x 	\
			-expand 1
	}
	pack [button $t.close 		\
		-text Close  		\
		-relief raised  	\
		-bd 2  			\
		-bg #8dd  		\
		-activebackground #7cc  \
		-highlightthickness 0	\
		-command "destroy $t"]
    } else {
	foreach i $infolist {
	    $t.$i.value configure -text "[eval [list vme_info $i $reg]]" 
	}
    }
}

proc mkbtn {enabled root leaf text cmd pad} {
    set len [string length $text]
    if {[eval lsearch -exact $enabled $leaf] != -1} {
	pack [button .$root.$leaf 		\
		-text $text 			\
		-width $len 			\
		-command "$cmd"			\
		-relief raised  		\
		-bd 2  				\
		-bg #8dd  			\
		-activebackground #7cc  	\
		-highlightthickness 0] 		\
			-side left 		\
			-fill both 		\
			-padx $pad
    }	    
}

proc mklbl {enabled root leaf text variable cmd} {
    global $variable
    if {[eval lsearch -exact $enabled $leaf] != -1} {
    pack [frame .$root.$leaf \
	-bg #00f\
	-highlightthickness 0 ] 		\
		-side left 		\
		-fill x 		\
		-padx 2 		
		
    mkbtn $enabled $root.$leaf $leaf $text $cmd 0
	pack [label .$root.$leaf.t 	\
		-width 11 		\
		-textvariable $variable \
		-bg #aff	\
		-highlightthickness 0] 		\
			-side left \
			-fill both
    }
}

proc flipbool {root variable} {
    upvar #0 $variable var
    if {$var != 0} {
	$root configure -text "1"
    } else {
	$root configure -text "0"    }
}

proc mkbool {enabled root leaf text variable cmd} {
    global $variable
    pack [frame .$root.$leaf \
		-bg #00f 	\
		-highlightthickness 0] \
		-side left \
		-fill both \
		-padx 2    
    mkbtn $enabled $root.$leaf $leaf $text "$cmd ; flipbool .$root.$leaf.t $variable" 0
    proc tracebool.$root {name element op} "flipbool .$root.$leaf.t $variable"
    if {[eval lsearch -exact $enabled $leaf] != -1}  {
	pack [label .$root.$leaf.t -width 1 \
		-text "0" \
		-bd 2 \
		-bg #aff	\
		-highlightthickness 0] \
			-side left \
			-padx 2
	uplevel #0 trace variable $variable w tracebool.$root
    }
}

proc mkent {enabled root leaf ente text variable cmd} {
    global $variable
  if {[eval lsearch -exact "$enabled" $ente] != -1 } {
    pack [frame .$root.$leaf \
		-bg #00f	\
		-highlightthickness 0] \
		-side left \
		-fill both \
		-padx 2 
    mkbtn $enabled $root.$leaf $ente $text $cmd 0
	pack [entry .$root.$leaf.t \
		-width 11 \
		-textvariable $variable \
		-relief sunken \
		-bg #aff  \
		-highlightcolor #f00 \
		-highlightbackground #8dd \
		-selectbackground #0ff] \
			-side left \
			-fill both
        bind .$root.$leaf.t <Return> [list evoke .$root.$leaf.$ente]
    }
}

proc mkfile {enabled root leaf text variable cmd} {
    global $variable
    if {[eval lsearch -exact "$enabled" $leaf] != -1} {
	pack [frame .$root.$leaf\
		-bg #00f	\
		-highlightthickness 0] \
			-fill both \
			-pady 1\
			-padx 2
	mkbtn $enabled $root.$leaf $leaf $text $cmd 0
	pack [entry .$root.$leaf.t \
		-width 25 \
		-textvariable $variable \
		-bd 2\
		-relief sunken \
		-bg #aff  \
		-highlightcolor #f00 \
		-highlightbackground #8dd \
		-selectbackground #0ff] \
			-side left \
			-fill both \
			-pady 1
	pack [button .$root.$leaf.b \
		-width 1 \
		-text "f" \
		-relief raised  \
		-bd 2  \
		-bg #8dd  \
		-activebackground #7cc  \
		-highlightthickness 0 \
		-command "set $variable \[FSBox \"Select a File\" \$$variable\]"] \
			-side left \
			-fill both \
			-pady 1 \
			-padx 1
	bind .$root.$leaf.t <Return> [list evoke .$root.$leaf.$leaf]
    }
}

proc dispreg {base card reg} {
    eval global [vme_info cards]
    foreach c [vme_info cards] {
	global $c\v $c\h
    }
    set r $card\($reg\)
    set top $base.$card\_$reg
    pack [frame .$top \
		-bg #00f \
		-bd 0 	\
		-highlightthickness 0] \
			-fill both \
			-expand 1 \
			-pady 1
    set e $top.cont
    pack [frame .$e \
		-bg #00f \
		-highlightthickness 0] \
		-side left \
		-padx 2 \
		-anchor n \
		-pady 1
    pack [button .$e.rem \
		-text "r" \
		-command "delreg $base $card $reg"\
		-relief raised  \
		-bd 2  \
		-bg #8dd  \
		-activebackground #7cc  \
		-highlightthickness 0] \
			-padx 4 \
			-side left \
			-pady 1
    pack [button .$e.$reg \
		-text $r \
		-width 23 \
		-command "dispinfo $r"\
		-relief raised  \
		-bd 2  \
		-bg #8dd  \
		-activebackground #7cc  \
		-highlightthickness 0] \
			-side left \
			-pady 1
    set c $top.cmd
    pack [frame .$c \
		-bg #00f \
		-highlightthickness 0] \
		-side left \
		-padx 2 \
		-fill both \
		-expand 1 \
		-pady 1 

    set enabledlist [vme_info enabled $r]
    
    if {[vme_info type $r] == "bit"} {
	mkbtn  $enabledlist $c set set  "vme_set       $r"  2
	mkbtn  $enabledlist $c clear clear "vme_clear     $r"  2
	mkbool $enabledlist $c test test  [Regv $r]  "set [Regv $r] \[vme_test $r\]"
    } else {
	pack [frame .$c.rw \
		-bg #00f \
		-highlightthickness 0] -side top -fill both -expand 1
	mkent  $enabledlist $c.rw wrt write write  [Regw $r]  "vme_write     $r \$[Regw $r]"
	mklbl  $enabledlist $c.rw read read   [Regh $r]  "set [Regv $r] \[vme_read $r\];"
	uplevel #0 trace variable [Regv $r] w hextrace_update 
	mkfile $enabledlist $c filewrite "file write" [Regwf $r] "vme_filewrite $r \$[Regra $r] \$[Regwf $r]" 
	mkfile $enabledlist $c fileread  "file read " [Regrf $r] "vme_fileread $r \$[Regra $r] $[Regrl $r] \$[Regrf $r]"
	mkent  $enabledlist $c.rw adr fileread  "address" [Regra $r]  "expr 1"
	mkent  $enabledlist $c.rw len fileread  "length (words)" [Regrl $r]  "expr 1"
    }
}


proc vme_NewWindowName {{base vme_teststand}} {
    set i 2
    while {[winfo exists .$base$i]} {incr i}
    return $base$i
}

proc vme_RenameWindow {base} {
    set name [wm title .$base]
    set oldname $name
    set name [textreq "Rename Window" $name]
    wm title .$base $name
    wm iconname .$base $name
    vme_ModWindow $base $oldname $name
}

proc vme_LowWindow {{base {vme_teststand}} {state {normal}} {name {}} } {
    if {[winfo exists .$base]} {set base [vme_NewWindowName $base]}
    if {$name == {}} {set name $base}
    
    eval global [vme_info cards] config $base\_hidden
    upvar #0 $base\_snaplist snaplist

    set snaplist [list]

    toplevel .$base -bg #00f
    if {$state == "withdrawn"} {wm withdraw .$base}
    if {$state == "iconic"} {wm iconify .$base}
    wm title .$base $name
    wm iconname .$base $name
    vme_AddWindow $base $name

    pack [frame .$base.mb \
		-relief raised \
		-bd 2 \
		-bg #8dd] \
			-fill x
    
    pack [menubutton .$base.mb.file \
	-text "Window" \
	-menu .$base.mb.file.menu\
	-background  #8dd\
	-activebackground #7cc] \
		-side left \
		-padx 2
    pack [menubutton .$base.mb.filler \
	-text "----" 	\
	-background  #8dd\
	-activebackground #7cc] \
		-side left \
		-padx 2
    set m .$base.mb.file.menu
    menu $m 	\
	-background  #bbc  

    $m add command \
		-label "Snapshot" \
		-command "set config($base\_snaplist) \$$base\_snaplist" \
		-activebackground #7cc  
    $m add command \
		-label "Rename" \
		-command "vme_RenameWindow $base" \
		-activebackground #7cc  
    $m add command \
		-label "Hide" \
		-command "wm withdraw .$base"  \
		-activebackground #7cc  
    $m add command \
		-label "Delete" \
		-command "vme_DelWindow \"$base\" \"$name\""  \
		-activebackground #7cc  
    
    foreach card [vme_info cards] {
	if { [info exists reglist] } { unset reglist }
	foreach i [lsort [array names $card]] {
	    set reg $card\($i\)
	    set class [string trim [vme_info class $reg] " "]
	    lappend reglist($class) $i
	}
	pack [menubutton .$base.mb.$card \
		-text $card \
		-menu .$base.mb.$card.menu\
		-background  #8dd\
		-activebackground #7cc] \
			-side left
	set m .$base.mb.$card.menu
	menu $m -background  #bbc  
	foreach class [array names reglist] {
	    $m add cascade \
		-label $class \
		-menu $m.$class\
		-background  #bbc\
		-activebackground #7cc
	    set mc $m.$class
	    menu $mc -background #bbc
	    foreach reg $reglist($class) {
		$mc add command \
			-label $reg \
			-command "addreg $base $card $reg" \
			-background  #bbc\
			-activebackground #7cc
	    }
	}
    }
    
    if {![info exists config($base\_snaplist)]} {
	set config($base\_snaplist) [list]
    }

    foreach reg $config($base\_snaplist) {
	set r [vme_info name $reg]
	set c [vme_info cardinst $reg]
	addreg $base $c $r
    }
}



