##########################################################
# Tests of :
#            channel masks
#            on chip buffers
#            TOT
#            time stamp
#            jitter time
#            shorts
##########################################################

###############################################################
# This checks that the parameters are for these tests are okay
# returns zero if okay
##############################################################
proc misc_check_parameters { } {
source cal_global.tcl
source misc_global.tcl

  set tb(0) "Channel Masks" 
  set tb(1) "Chip Buffering" 
  set tb(2) "TOT" 
  set tb(3) "Time Stamp" 
  set tb(4) "Jitter Time"
#----------------------------------------------------------
# loop over tests
# (All except the test for shorts which is a special case)
#----------------------------------------------------------
  for {set i 0} {$i < 5} {incr i 1} {
    if {$misc_tests_list($i) == 1} {
	set good \
	    [test_value "\n\"No. of repetitions for $tb($i) test\"\n" \
	    $misc_nrep($i)  1 999999]
	if {$good != 0} {return $good}

	set good \
	    [test_value "\n\"Thr DAC value for $tb($i) test\"\n" \
	    $misc_thr_dac($i) 0 63]
	if {$good != 0} {return $good}

	set temp 1
	if {$i == 2} {set temp $misc_ntot}
	for {set j 0} {$j < $temp} {incr j 1} {
	    set good \
		[test_value "\n\"Cal DAC bottom for $tb($i) test\"\n" \
		$misc_cal_dac_bottom($i,$j) 0 63]
	    if {$good != 0} {return $good}
	    set good \
		[test_value "\n\"Cal DAC top for $tb($i) test\"\n" \
		$misc_cal_dac_top($i,$j) 0 63]
	    if {$good != 0} {return $good}
	}

#- TOT specific tests
        if {$i == 2} {
	    for {set j 0} {$j < $misc_ntot} {incr j 1} {
		set good [check_real $misc_tot($j)]
		if {$good != 0} {
		    cal_display_error_message \
    "Expected <TOT> for setting no. $j is $misc_tot($j)  \n Should be real"
		    return $good
		}
		set good [check_real $misc_tot($j)]
		if {$good != 0} {
		    cal_display_error_message \
    "Tollerance on <TOT> for setting no. $j is $misc_tot_toll($j)  \n Should be real"
		    return $good
		}
	    }
	}

#- time stamp specific tests
	if {$i == 3} {
	    set good [check_real $misc_tstamp_shift]
	    if {$good != 0} {
		cal_display_error_message \
     "Expected time stamp shift is $misc_tstamp_shift \n Should be real"
		return $good
	    }
	    set good [check_real $misc_tstamp_toll]
	    if {$good != 0} {
		cal_display_error_message \
     "Time stamp shift tollerance is $misc_tstamp_toll \n Should be real"
		return $good
	    }

	}
    }
}
#----------------------------------------------------------
# Now check the test for shorts if necessary
#----------------------------------------------------------
      if {$misc_tests_list(5) == 1} {
#------
# check that min and max are within range
#------
            set cmax $Cal_number_of_chips
            incr cmax -1
            set good [test_value short_min_chip $short_min_chip 0 $cmax]
            if {$good != 0} {return $good}
	    set good \
	      [test_value short_max_chip $short_max_chip $short_min_chip $cmax]
	    if {$good != 0} {return $good}
#------
# check that thresholds and charge injections are within range
#------
            set good [test_value short_thresh $short_thresh 0 63]
            if {$good != 0} {return $good}
            set good [test_value short_charge_top $short_charge_top 0 63]
            if {$good != 0} {return $good}
            set good [test_value short_charge_bottom $short_charge_bottom 0 63]
            if {$good != 0} {return $good}
	}
#----------------------------------------------------------
# Done
#----------------------------------------------------------
  return 0
}

##########################################################
# This actually performs the tests
##########################################################
proc misc_start_testing { } {
source site.tcl
source cal_global.tcl
source misc_global.tcl

#------------------------------------------------------------------
# if this is UCSB, we have to make sure that the DC level is OFF
# (also no delay select)
#------------------------------------------------------------------
  if {$site == "UCSB"} {
      set good [delay_select_off]
      if {$good != 0} {
          cal_display_error_message "Error turning Delay Select OFF"
          return
      }
      set good [dc_level_off]
      if {$good != 0} {
          cal_display_error_message "Error trying to turn DC level OFF"
          return
      }
  }
#----------------------------------------------
# check that we have a legal number of chips
#-----------------------------------------------
  set okeydokey [cal_check_chip_number]
  if {$okeydokey != 0} {return}
#-------------------------------------------------------------
#- anything to test ? (also set the bitmask for later)
#-------------------------------------------------------------
   set bit_mask_flag 0
   set something_to_test "no"
   set something_to_test_except_shorts "no"
   for {set i 0} {$i < 6} {incr i 1} {
       if {$i == 0} {
	   set dummy 1
       } elseif {$i == 1} {
	   set dummy 2
       } else {
	   set dummy [expr $dummy*2]
       }
       if {$misc_tests_list($i) == 1} {
	   set something_to_test "yes"
           incr bit_mask_flag $dummy
	   if {$i != 5} {set something_to_test_except_shorts "yes"}
       }
   }
   if {$something_to_test == "no"} {
       cal_display_error_message "All tests disabled !"
       return
   }
#-----------------------------------------------
# make sure all the relevant entries are kosher
#-----------------------------------------------
   set okeydokey [misc_check_parameters]
   if {$okeydokey != 0} {return}
#-------------------------------------------------------------
# make sure that the part name is OK
#-------------------------------------------------------------
   get_part_name
#-------------------------------------------------------------
# before running, write out a file with the parameters..
# if the file exists, abort
#-------------------------------------------------------------
   set mytemp parameter_run
   set logfile $data_directory$mytemp$Cal_run_number
   set exist [file exist $logfile]
   if {$exist == 1} {
       cal_display_error_message "Run $Cal_run_number already performed"
       return
   }
   cal_write_defaults $logfile YES "MISCELLANEOUS TESTS"
#-----------------------------------------------
# make the text widget, set the grab to it
#-----------------------------------------------
   make_pltw_widget
   grab set .pltw
#-----------------------------------------------
# now execute the tests (shorts are special case)
#-----------------------------------------------
   set good "OK"
   if {$something_to_test_except_shorts == "yes"} {
       set good [misc_tests $bit_mask_flag]
   }
   if {$misc_tests_list(5) == 1 && $good == "OK"} {start_shorts_test}
#-----------------------------------------------
# done
#-----------------------------------------------
   cal_bump_runnumber
   grab release .pltw
   return
}

##########################################################
# This sets up the window to drive the tests
##########################################################
proc misc_tests_window { } {
source cal_global.tcl
source misc_global.tcl

#-----------------------------------------------------------
# if the global variables are not filled, fill them.
# Now, this will also fill the misc_global.tcl variables
#-------------------------------------------------------------
     if {![info exists Cal_number_of_chips]} {
         cal_get_defaults cal_defaults.store YES
     }
#--------------------------------------------
# probably better if these are defined
#--------------------------------------------
  set Cal_run "stop"
  cal_get_run_number
#------------------------------------------------------------
# The output file, in case it is not defined yet.
# (Not needed for this code, but needed by cal_bump_sunnumber
#------------------------------------------------------------
  set Cal_output_file [std_output_file run]
#---------------------------------------------------------
# if the window does not exists, make it
#---------------------------------------------------------
    if {![winfo exists .misc_tests]} {

       toplevel .misc_tests -bg #00f
       frame .misc_tests.f0 -bg #00f
       frame .misc_tests.f1 -bg #00f
       frame .misc_tests.f2 -bg #00f
       frame .misc_tests.f3 -bg #00f 
       frame .misc_tests.f10 -bg #00f 
       frame .misc_tests.f11 -bg #00f 
       frame .misc_tests.f12 -bg #00f 
       frame .misc_tests.f13 -bg #00f 
       frame .misc_tests.f14 -bg #00f 
       frame .misc_tests.f15 -bg #00f 

       frame .misc_tests.f20 -bg #00f 
       frame .misc_tests.f3.0 -bg #00f
       frame .misc_tests.f3.2 -bg #00f
       frame .misc_tests.f3.1 -bg #00f


       pack  .misc_tests.f0 \
	     .misc_tests.f1 \
	     .misc_tests.f2 \
	     .misc_tests.f3 \
	     .misc_tests.f10 \
	     .misc_tests.f11 \
	     .misc_tests.f12 \
	     .misc_tests.f13 \
	     .misc_tests.f14 \
	     .misc_tests.f15 \
	     .misc_tests.f20 \
	       -side top -pady 3 -in .misc_tests 
       pack .misc_tests.f3.0 .misc_tests.f3.2 .misc_tests.f3.1 \
	       -side left -padx 3 -in .misc_tests.f3


# message
         label .misc_tests.label \
               -text "Miscellaneous tests" \
               -bg #ff0 \
               -highlightthickness 0
         pack .misc_tests.label -fill x -in .misc_tests.f0

# runnumber
        label .misc_tests.run_number_label \
                -text "Run Number"   \
                -bg #8dd \
                -highlightthickness 0
        entry .misc_tests.run_number_entry \
                -textvariable Cal_run_number  \
                -width 8 \
                -relief sunken \
                -bg #aff   \
                -highlightcolor #f00 \
                -highlightbackground #8dd \
                -selectbackground #0ff
         pack .misc_tests.run_number_label .misc_tests.run_number_entry \
		 -side left -padx 4 -fill both -in .misc_tests.f1


# load defaults, close, help
         button .misc_tests.exit -text "Close \n" \
                 -command {destroy .misc_tests} \
                 -bg #8dd  \
                 -activebackground #7cc  \
                 -highlightthickness 0
         button .misc_tests.help -text "Help \n" \
                 -command {cal_help .misc_tests_help misc_tests_helpfile} \
                 -bg #8dd  \
                 -activebackground #7cc  \
                 -highlightthickness 0
         button .misc_tests.rd1 \
               -text "Set HDI defaults\nin software" \
               -command "cal_choose_def" \
               -bg #8dd  \
               -activebackground #7cc  \
               -highlightthickness 0

         button .misc_tests.rd2 \
               -text "Restore default\nlist of tests" \
               -command "prod_tests_defaults 1" \
               -bg #8dd  \
               -activebackground #7cc  \
               -highlightthickness 0

         pack  .misc_tests.exit .misc_tests.help \
		 .misc_tests.rd2 .misc_tests.rd1 \
              -fill x -padx 5 -side left -in .misc_tests.f2

# number of chips
         label .misc_tests.num_chips_label \
                -text "No. of chips\n (total)" \
                -bg #8dd \
                -highlightthickness 0
         entry .misc_tests.num_chips_entry \
                -relief sunken \
                -width 3 \
                -textvariable Cal_number_of_chips  \
                -bg #aff   \
                -highlightcolor #f00 \
                -highlightbackground #8dd \
                -selectbackground #0ff

         label .misc_tests.num_chips_label2 \
                -text "No. of chips\n (top side)" \
                -bg #8dd \
                -highlightthickness 0
         entry .misc_tests.num_chips_entry2 \
                -relief sunken \
                -width 3 \
                -textvariable Cal_number_of_chips_top  \
                -bg #aff   \
                -highlightcolor #f00 \
                -highlightbackground #8dd \
                -selectbackground #0ff

# one-sided or two sided hybrid
        radiobutton .misc_tests.oneside \
                -text "One-Sided Hybrid" \
                -variable Cal_Side_Hybrid \
                -value 1 \
                -anchor w\
                -bg #8dd  \
                -activebackground #7cc \
                -highlightthickness 0\
                -relief raised
        radiobutton .misc_tests.twoside \
                -text "Two-Sided Hybrid" \
                -variable Cal_Side_Hybrid \
                -value 2 \
                -anchor w \
                -bg #8dd  \
                -activebackground #7cc \
                -highlightthickness 0\
                -relief raised

# pack number of chips, hybrid
        pack .misc_tests.num_chips_label .misc_tests.num_chips_entry -fill x -in .misc_tests.f3.0
        pack .misc_tests.num_chips_label2 .misc_tests.num_chips_entry2 -fill x -in .misc_tests.f3.2
        pack  .misc_tests.oneside .misc_tests.twoside -side top -fill x -in .misc_tests.f3.1


# list of tests, with their setup buttons
# The test for shorts is special for historical reasons
       set cb(0) "Test Channel Masks " 
       set cb(1) "Test Chip Buffering" 
       set cb(2) "Test TOT           " 
       set cb(3) "Test Time Stamp    " 
       set cb(4) "Test Jitter Time   "
       set cb(5) "Test for Shorts    "
       for {set i 0} {$i < 6}  {incr i 1 } {
           
	   checkbutton .misc_tests.cb$i -text "$cb($i)" \
		       -variable misc_tests_list($i)     \
		       -anchor w \
		       -bg #8dd  \
		       -activebackground #8dd \
		       -highlightthickness 0\
		       -relief raised
	   if {$i != 5} {
	       button .misc_tests.b$i -text "Setup" \
                 -command "misc_test_setup $i" \
                 -bg #8dd  \
                 -activebackground #7cc  \
                 -highlightthickness 0
	   } else {
	       button .misc_tests.b$i -text "Setup" \
                 -command "shorts_window" \
                 -bg #8dd  \
                 -activebackground #7cc  \
                 -highlightthickness 0
	   }
	   set j [expr 10 + $i]
           pack .misc_tests.cb$i .misc_tests.b$i -side left \
		   -in .misc_tests.f$j -padx 5 -fill both
       }
           



# do it
       button .misc_tests.doit -text "Start Testing Now" \
	       -command misc_start_testing \
               -bg #8dd \
                 -activebackground #7cc  \
                 -highlightthickness 0
       pack .misc_tests.doit -in .misc_tests.f20

   }
}

####################################################
# procedure for setting up the miscellaneous tests
# test 0 = masks
# test 1 = buffers
# test 2 = TOT
# test 3 = time stamp
# test 4 = jitter time
################################
proc misc_test_setup { test } { 
source cal_global.tcl
source misc_global.tcl

  set txt(0) "channel mask"
  set txt(1) "on chip buffering"
  set txt(2) "TOT"
  set txt(3) "time stamp"
  set txt(4) "jitter time"
#--  puts "Setting up test number $test"
  if {![winfo exists .mts$test]} {
      toplevel .mts$test
      for {set i 0} {$i < 10} {incr i 1} {
          set j .f$i
	  frame .mts$test$j -bg #00f
	  pack .mts$test$j -side top -in .mts$test -pady 3
      }
  

#- message
      set this_frame 0
      set junk .label
      label .mts$test$junk -text "Setup tests of $txt($test)" \
               -bg #ff0 \
               -highlightthickness 0
      set j .f$this_frame
      pack .mts$test$junk -fill x -in .mts$test$j

# frame 1 : close window, help
         incr this_frame 1 
         set b1 .b1
         set b2 .b2
         set b3 .b3
         set flag [expr 10 + $test]
         button .mts$test$b1 -text Close \
                 -command "destroy .mts$test" \
                 -bg #8dd  \
                 -activebackground #7cc  \
                 -highlightthickness 0
         button .mts$test$b2 -text Help \
                 -command "cal_help .misc_help$test misc_setup_helpfile$test" \
                 -bg #8dd  \
                 -activebackground #7cc  \
                 -highlightthickness 0
         button .mts$test$b3 -text "Restore Defaults" \
                 -command "prod_tests_defaults $flag" \
                 -bg #8dd  \
                 -activebackground #7cc  \
                 -highlightthickness 0
          set j .f$this_frame
          pack .mts$test$b1 .mts$test$b2 .mts$test$b3 \
                  -fill x -in .mts$test$j -side left -padx 5

# frame 2 : repetitions, threshold DAC
          incr this_frame 1
          set l1 .l1$this_frame
          set l2 .l2$this_frame
          set e1 .e1$this_frame
          set e2 .e2$this_frame

          label .mts$test$l1 -text "Number of repetitions" \
		  -bg #8dd -highlightthickness 0
          label .mts$test$l2 -text "   Threshold DAC" \
		  -bg #8dd -highlightthickness 0
         entry .mts$test$e1 \
                -relief sunken \
                -width 3 \
                -textvariable "misc_nrep($test)" \
                -bg #aff   \
                -highlightcolor #f00 \
                -highlightbackground #8dd \
                -selectbackground #0ff
         entry .mts$test$e2 \
                -relief sunken \
                -width 3 \
                -textvariable "misc_thr_dac($test)" \
                -bg #aff   \
                -highlightcolor #f00 \
                -highlightbackground #8dd \
                -selectbackground #0ff
          set j .f$this_frame
          pack .mts$test$l1 .mts$test$e1 .mts$test$l2 .mts$test$e2 \
                  -fill x -in .mts$test$j -side left -padx 5
          
# next frame : calibration DAC... for the TOT a totally different setup
          if {$test != 2} {
	      incr this_frame 1
	      set l1 .l1$this_frame
	      set l2 .l2$this_frame
	      set l3 .l3$this_frame
	      set e1 .e1$this_frame
	      set e2 .e2$this_frame
              set op .op$this_frame
	      set b1 .b1$this_frame
	      set l10 .l10$this_frame
	      set l20 .l20$this_frame
	      set e10 .e10$$this_frame
	      set e20 .e20$this_frame
	      label .mts$test$l1 -text "CAL DAC top side" \
		  -bg #8dd -highlightthickness 0
	      label .mts$test$l2 -text "CAL DAC bottom side" \
		  -bg #8dd -highlightthickness 0
	      entry .mts$test$e1 \
                -relief sunken \
                -width 3 \
                -textvariable "misc_cal_dac_top($test,0)" \
                -bg #aff   \
                -highlightcolor #f00 \
                -highlightbackground #8dd \
                -selectbackground #0ff
	      entry .mts$test$e2 \
                -relief sunken \
                -width 3 \
                -textvariable "misc_cal_dac_bottom($test,0)" \
                -bg #aff   \
                -highlightcolor #f00 \
                -highlightbackground #8dd \
                -selectbackground #0ff


	      set j .f$this_frame
	      pack .mts$test$l1 .mts$test$e1 .mts$test$l2 .mts$test$e2 \
              -fill x -in .mts$test$j -side left -padx 5
	  }

# calibration DAC etc for the TOT test
          if {$test == 2} {
	      incr this_frame 1
	      set nsubframe [expr $misc_ntot+1]
	      set j .f$this_frame
	      for {set isub 0} {$isub < $nsubframe} {incr isub 1} {
		  set temp .sf$isub
		  set subframe($isub) .mts$test$temp
		  set temp $subframe($isub)
		  frame $temp -bg #8dd
	      	  pack $temp -in .mts$test$j -side top -fill x
	      }
	      set ll(0) "                  \n  "
	      set ll(1) "Cal DAC\nTop Side"
	      set ll(2) "Cal DAC\nBottom Side"
	      set ll(3) "  Shaping\n    Time  "
              set ll(4) "            Skip  "
	      set ll(5) "    Expected\n      <TOT>"
	      set ll(6) " Tollerance\n  on <TOT>  "
	      set ll(7) "           \n  "
	      for {set i 0} {$i < 8} {incr i 1} {
		  set this_subframe $subframe(0)
		  set l1 .ll1$this_frame$i
		  label .mts$test$l1 -text "$ll($i)" \
			  -bg #8dd -highlightthickness 0
		  pack .mts$test$l1 -in $this_subframe -side left
	      }
	      for {set ii 1} {$ii < $nsubframe} {incr ii 1} {
		  set this_subframe $subframe($ii)
		  set l2 .ll2$this_frame$ii
		  set i [expr $ii-1]
		  label .mts$test$l2 -text "Setting No. $i" \
			  -bg #8dd -highlightthickness 0
		  set e1 .ee1$this_frame$i
		  entry .mts$test$e1 -relief sunken -width 8 \
			  -textvariable "misc_cal_dac_top(2,$i)" \
			  -bg #aff -highlightcolor #f00  \
		          -highlightbackground #8dd \
                           -selectbackground #0ff
		  set e2 .ee2$this_frame$i
		  entry .mts$test$e2 -relief sunken -width 8 \
			  -textvariable "misc_cal_dac_bottom(2,$i)" \
			  -bg #aff -highlightcolor #f00 \
		          -highlightbackground #8dd \
                           -selectbackground #0ff
		  set op3 .op3$this_frame$i
		  tk_optionMenu .mts$test$op3 misc_tot_shaping_time($i) \
			  "100 nsec" "200 nsec" "300 nsec" "400 nsec"
		  set op4 .op4$this_frame$i
		  tk_optionMenu .mts$test$op4 misc_tot_skip($i) \
			  "skip 0" "skip 1" "skip 2" "skip 3"
		  set e4 .ee4$this_frame$i
		  entry .mts$test$e4 -relief sunken -width 8 \
			  -textvariable "misc_tot($i)" \
			  -bg #aff -highlightcolor #f00 \
		          -highlightbackground #8dd \
                           -selectbackground #0ff
		  set e5 .ee5$this_frame$i
		  entry .mts$test$e5 -relief sunken -width 8 \
			  -textvariable "misc_tot_toll($i)" \
			  -bg #aff -highlightcolor #f00 \
		          -highlightbackground #8dd \
                           -selectbackground #0ff
		  set cb6 .cb6$this_frame$i
		  checkbutton .mts$test$cb6 -text "Test it " \
                       -variable misc_tot_test_it($i) -anchor w -bg #8dd \
                       -activebackground #8dd \
                       -highlightthickness 0 -relief raised


		  pack .mts$test$l2 \
			  .mts$test$e1 \
			  .mts$test$e2 \
			  .mts$test$op3 \
			  .mts$test$op4 \
			  .mts$test$e4 \
			  .mts$test$e5 \
			  .mts$test$cb6 \
			  -in $this_subframe -side left
	      }
	  
	  }

# next three frames are for time stamp tests only
          if {$test == 3} {
	      incr this_frame 1
	      set l1 .l1$this_frame
	      set e1 .e2$this_frame
	      label .mts$test$l1 -text \
	       "Expected change in TimeStamp\n(2nd settings - 1st settings)" \
		  -bg #8dd -highlightthickness 0
	      entry .mts$test$e1 \
                -relief sunken \
                -width 8 \
                -textvariable misc_tstamp_shift \
                -bg #aff   \
                -highlightcolor #f00 \
                -highlightbackground #8dd \
                -selectbackground #0ff
	      set j .f$this_frame
	      pack .mts$test$l1 .mts$test$e1  \
                   -in .mts$test$j -side top

	      incr this_frame 1
	      set l1 .l1$this_frame
	      set e1 .e2$this_frame
	      label .mts$test$l1 -text \
	       "Tollerance for error reporting\n(enter a real number)" \
		  -bg #8dd -highlightthickness 0
	      entry .mts$test$e1 \
                -relief sunken \
                -width 8 \
                -textvariable misc_tstamp_toll \
                -bg #aff   \
                -highlightcolor #f00 \
                -highlightbackground #8dd \
                -selectbackground #0ff
	      set j .f$this_frame
	      pack .mts$test$l1 .mts$test$e1  \
                  -in .mts$test$j -side top

	      incr this_frame 1
	      set b1 .b1$this_frame
	      checkbutton .mts$test$b1 -text "Ignore dead channels" \
		       -variable misc_tstamp_dead     \
		       -anchor w \
		       -bg #8dd  \
		       -activebackground #8dd \
		       -highlightthickness 0\
		       -relief raised
	      set j .f$this_frame
	      pack .mts$test$b1 \
                  -in .mts$test$j -side top

	  }


# next frames are for TOT test only
          if {$test == 2} {
	   incr this_frame 1
	   set b1 .b1$this_frame
	   checkbutton .mts$test$b1 -text "Ignore dead channels" \
		       -variable misc_tot_dead     \
		       -anchor w \
		       -bg #8dd  \
		       -activebackground #8dd \
		       -highlightthickness 0\
		       -relief raised
	      set j .f$this_frame
	      pack .mts$test$b1 \
                  -in .mts$test$j -side top
	  }
   }
}
#------------------------------------------------------------------
# Dump routine for misc tests parameters
#------------------------------------------------------------------
proc dump_misc_parameters {test f} {
  source cal_global.tcl
  source misc_global.tcl

    if {$test > 5} {return}

    set tb(0) "Channel Masks" 
    set tb(1) "Chip Buffering" 
    set tb(2) "TOT" 
    set tb(3) "Time Stamp" 
    set tb(4) "Jitter Time"
    set tb(5) "Shorts"
    set yn(0) "NO"
    set yn(1) "YES"
    set one_blank " "
    set blank(0) ""
    for {set i 1} {$i < 20} {incr i 1} {
	set j [expr $i-1]
        set tmp $blank($j)
	set blank($i) $tmp$one_blank
    }


    puts $f "   "
    puts $f "   "
    puts $f "Parameters for $tb($test) Test"
    puts $f "--------------------------------------------"

    if {$test != 2 && $test != 5} {
	puts $f "Number of repetitions     = $misc_nrep($test)"
        puts $f "Threshold DAC             = $misc_thr_dac($test)"
	puts $f "Cal DAC top side          = $misc_cal_dac_top($test,0)"
	puts $f "Cal DAC bottom side       = $misc_cal_dac_bottom($test,0)"
        if {$test == 3} {
	    puts $f "Exp. change in time stamp = $misc_tstamp_shift"
	    puts $f "Tollerance                = $misc_tstamp_toll"
            puts $f "Ignore dead channels        $yn($misc_tstamp_dead)"
	}
    } elseif {$test == 2} {
	puts $f "Number of repetitions     = $misc_nrep($test)"
        puts $f "Threshold DAC             = $misc_thr_dac($test)"
	puts $f "   "
        puts $f \
"         Cal DAC   Cal DAC   Shaping   Skip   Expected  Tollerance   Test it"
        puts $f \
"           top      bottom                      <TOT>     on <TOT>           "
        puts $f \
"------------------------------------------------------------------------"
        for {set i 0} {$i < $misc_ntot} {incr i 1} {
	    set temp "Setting $i  "
	    set temp $temp$misc_cal_dac_top(2,$i)$blank(8)
	    set temp $temp$misc_cal_dac_bottom(2,$i)$blank(6)
	    set temp $temp$misc_tot_shaping_time($i)$blank(2)$misc_tot_skip($i)
            set temp $temp$blank(3)$misc_tot($i)
            set temp $temp$blank(7)$misc_tot_toll($i)$blank(10)
	    set temp $temp$yn($misc_tot_test_it($i))
	    puts $f $temp
	}
        puts $f " "
        puts $f "Ignore dead channels        $yn($misc_tot_dead)"
    } elseif {$test == 5} {
        puts $f "Test chips from $short_min_chip to $short_max_chip"
	puts $f "Number of repetitions           = $short_nrep"
        puts $f "Threshold DAC                   = $short_thresh"
	puts $f "Cal DAC top side                = $short_charge_top"
	puts $f "Cal DAC bottom side             = $short_charge_bottom"
	for {set i -3} {$i < 4} {incr i 1} {
	    puts $f "Examine response for channel $i $yn($short_chan($i))"
	}
    }
}
##########################################################################
# This is procedure used to write the output file for miscellaneous tests.
# Returns OK or an error string
# Input: test = test number
#        flag = 0    open the file
#        flag = 1    close the file
#        flag = 2    dump the parameters to the file
#        flag = 3    save the position of the initial character from the
#                    .pltw.text window that will be dumped to the file
#        flag = 4    save the position of the final character from the
#                    .pltw.text window that will be dumped to the file
#        flag = 5    dump the characters to the file
##########################################################################
proc misc_file_handler {test flag} {

  global Cal_run_number
  global part_name
#------------------------------------------------
# the following are variables that are local to 
# this file but which we want to "remember" from
# one call to the next ---> we have to make them 
# global, since "static" is not supported in tcl
#-------------------------------------------------
  global misc_file_locator
  global misc_initial_character
  global misc_final_character
#------------------------------------------------
# these are needed to build the output file name
#------------------------------------------------
  set fname(0) "chmsk"
  set fname(1) "buffer"
  set fname(2) "tot"
  set fname(3) "tstamp"
  set fname(4) "jitter"
  set fname(5) "shorts"
#------------------------------------------------
# verify that the test number is OK 
#------------------------------------------------
  set good [test_value test $test 0 5]
  if {$good != 0} {
      return "Invalid test no. $test inputted to misc_file_handler"
  }
#------------------------------------------------
# verify that the directory for this part exists
#------------------------------------------------
  set good [make_part_directory]
  if {$good != "OK"} {return $good}
#------------------------------------------------
# build the actual file name
#------------------------------------------------
  set filename [std_output_file $fname($test)] 
#------------------------------------------------
# open file  
#------------------------------------------------
  if {$flag == 0} {
      set r [catch {set misc_file_locator($test) [open $filename w]}]
      if {$r != 0} {
	  return "Cannot open new file $filename"
      }
      set junk [exec date]
      puts $misc_file_locator($test) "$junk Run = $Cal_run_number   Part = $part_name"
      puts $misc_file_locator($test) "  "
      return "OK"
  }
#------------------------------------------------
# a bunch of action require the file to be open
#------------------------------------------------
  if {$flag == 1 || $flag == 2 || $flag == 5} {
      if {![info exists misc_file_locator($test)]} {
	  return "File $filename was never opened"
      }
  }
#------------------------------------------------------------
# a bunch of actions require the .pltw.text widget to exist
# (remember to close the file on error if it is opened)
# (also remember to clear the initial/final characters)
#------------------------------------------------------------
  set t ".pltw.text"
  if {$flag == 3 || $flag == 4 || $flag == 5} {
      if {![winfo exists $t]} {

	  if {[info exists misc_file_locator($test)]} {
	      close $misc_file_locator($test)
	      unset  misc_file_locator($test)
	  }
	  if {[info exists misc_initial_character($test)]} {
	      unset  misc_initial_character($test)
	  }
	  if {[info exists misc_final_character($test)]} {
	      unset  misc_initial_character($test)
	  }
	  return "Widget $t does not exist"
      }
  }
#------------------------------------------------
# close file
#------------------------------------------------
  if {$flag == 1} {
      close $misc_file_locator($test)
      unset  misc_file_locator($test)
      if {[info exists misc_initial_character($test)]} {
	  unset  misc_initial_character($test)
      }
      if {[info exists misc_final_character($test)]} {
	  unset  misc_final_character($test)
      }
      return "OK"
  }
#------------------------------------------------
# dump the parameters to the file
#------------------------------------------------
  if {$flag == 2} {
      dump_misc_parameters $test $misc_file_locator($test)
      puts $misc_file_locator($test) " "
      return "OK"
  }
#------------------------------------------------
# save the initial/final characters
#------------------------------------------------
  if {$flag == 3} {
      set misc_initial_character($test) [$t index end]
      return "OK"
  }
  if {$flag == 4} {
      set misc_final_character($test) [$t index end]
      return "OK"
  }
#------------------------------------------------
# dump the characters to the file
#------------------------------------------------
  if {$flag == 5} {
      if {![info exists misc_final_character($test)]} {
	  return "The final character in $t has not been indexed"
      }
      if {![info exists misc_initial_character($test)]} {
	  return "The initial character in $t has not been indexed"
      }
      set f  $misc_file_locator($test)  
      set a1 $misc_initial_character($test)
      set a2 $misc_final_character($test)
      puts $f [$t get $a1 $a2]
      return "OK"
  }
#------------------------------------------------
# Done
#------------------------------------------------
}