source osvt_utils.tcl

proc osvt_chipmenu { {hybrid q0-s0} {chiplist {c0}} } {

  global osvt

  set base "chip"
  set mn osvt_chipmenu

  if {[winfo exists .$mn]} {
    destroy .$mn
  }
  
  # initialize from first chip
  set chip $hybrid-[lindex $chiplist 0]

#      osvt($chip-peaking-label) osvt($chip-peaking) \
      osvt($chip-sampling-label) osvt($chip-sampling) \
      osvt($chip-totskip-label) osvt($chip-totskip) \
      osvt($chip-rodir-label) osvt($chip-rodir) \
      osvt($chip-rdresp-label) osvt($chip-rdresp) \
      osvt($chip-clkthru-label) osvt($chip-clkthru) \
       osvt($chip-selclkb-label) osvt($chip-selclkb) \
      osvt($chip-selclka-label) osvt($chip-selclka) \

  # +++
  osvt_fixglob \
      osvt($chip-ctrlreg)

  # --- split control register
  osvt_splitctrlreg osvt($chip-ctrlreg) \
      osvt($base-thrdac) \
      osvt($base-caldac) \
      osvt($base-jitter) \
      osvt($base-peaking) \
      osvt($base-sampling) \
      osvt($base-totskip) \
      osvt($base-rodir) \
      osvt($base-rdresp) \
      osvt($base-clkthru) \
      osvt($base-digpatt) \
      osvt($base-selclkb) \
      osvt($base-selclka)

  if ![info exists osvt($chip-chamask)] {
    set osvt($chip-chamask) {0x12345678 0x12345678 0x12345678 0x12345678}
  }
  osvt_masktoentry $osvt($chip-chamask) \
      osvt($base-chamask0) \
      osvt($base-chamask1) \
      osvt($base-chamask2) \
      osvt($base-chamask3) \
      osvt($base-chamask4) \
      osvt($base-chamask5) \
      osvt($base-chamask6) \
      osvt($base-chamask7)
  
  if ![info exists osvt($chip-calmask)] {
    set osvt($chip-calmask) {0x12345678 0x12345678 0x12345678 0x12345678}
  }
  osvt_masktoentry $osvt($chip-calmask) \
      osvt($base-calmask0) \
      osvt($base-calmask1) \
      osvt($base-calmask2) \
      osvt($base-calmask3) \
      osvt($base-calmask4) \
      osvt($base-calmask5) \
      osvt($base-calmask6) \
      osvt($base-calmask7)

  # --- re-build ctrlreg: this will check the fields and correct them
  osvt_buildctrlreg osvt($base-ctrlreg) \
      osvt($base-thrdac) \
      osvt($base-caldac) \
      osvt($base-jitter) \
      osvt($base-peaking) \
      osvt($base-sampling) \
      osvt($base-totskip) \
      osvt($base-rodir) \
      osvt($base-rdresp) \
      osvt($base-clkthru) \
      osvt($base-digpatt) \
      osvt($base-selclkb) \
      osvt($base-selclka)

  toplevel .$mn
  grab set .$mn

#  osvt_daqmenu $mn

  set labfmt "%24s"

  # --- threshold
  osvt_entrylinepack .$mn.thre [format $labfmt "Threshold"] \
      osvt($base-thrdac) 4

  # --- calibration DAC
  osvt_entrylinepack .$mn.cald [format $labfmt "Calibration DAC"] \
      osvt($base-caldac) 4

  # --- Max jitter time
  osvt_entrylinepack .$mn.jitt [format $labfmt "Max jitter time"] \
      osvt($base-jitter) 4

  # --- peaking time
  osvt_optlinepack .$mn.shap [format $labfmt "Peaking time" ] \
      osvt($base-peaking-label) osvt($base-peaking) \
      {"100 nsec" 0} \
      {"200 nsec" 1} \
      {"300 nsec" 2} \
      {"400 nsec" 3}

  # --- sampling rate
  osvt_optlinepack .$mn.samp [format $labfmt "Sampling rate"] \
      osvt($base-sampling-label) osvt($base-sampling) \
      {"clock /2" 0} \
      {"clock /3" 1} \
      {"clock /4" 2} \
      {"no write" 3}

  # --- skip control
  osvt_optlinepack .$mn.skco [format $labfmt "Skip Control"] \
      osvt($base-totskip-label) osvt($base-totskip) \
      {"no skip  " 0} \
      {"every 2nd" 1} \
      {"every 3rd" 2} \
      {"every 4th" 3}

  # --- readout direction
  osvt_optlinepack .$mn.rodir [format $labfmt "Readout direction"] \
      osvt($base-rodir-label) osvt($base-rodir) \
      {"left " 0} \
      {"right" 1}

  # --- respond to read event command
  osvt_optlinepack .$mn.rdresp [format $labfmt "Respond to read event"] \
      osvt($base-rdresp-label) osvt($base-rdresp) \
      {"yes" 1} \
      {"no " 0}

  # --- clock through mode
  osvt_optlinepack .$mn.clkt [format $labfmt "Clock through mode"] \
      osvt($base-clkthru-label) osvt($base-clkthru) \
      {"no " 0} \
      {"yes" 1}

  # --- digital test pattern
  osvt_entrylinepack .$mn.digp [format $labfmt "Test pattern"] \
      osvt($base-digpatt) 4

  # --- select clock&cmd B
  osvt_optlinepack .$mn.clkb [format $labfmt "Select clock B"] \
      osvt($base-selclkb-label) osvt($base-selclkb) \
      {"no " 0} \
      {"yes" 1}

  # --- select clock&cmd A
  osvt_optlinepack .$mn.clka [format $labfmt "Select clock A"] \
      osvt($base-selclka-label) osvt($base-selclka) \
      {"yes" 1} \
      {"no " 0}
  
  # --- active channel mask
  osvt_masklab .$mn.chamask [format $labfmt "      Channel mask      "] \
      osvt($base-chamask0) \
      osvt($base-chamask1) \
      osvt($base-chamask2) \
      osvt($base-chamask3) \
      osvt($base-chamask4) \
      osvt($base-chamask5) \
      osvt($base-chamask6) \
      osvt($base-chamask7)
  pack .$mn.chamask -side top -anchor w -pady 2
  
  # --- active calibration mask
  osvt_masklab .$mn.calmask [format $labfmt "    Calibration mask    "] \
      osvt($base-calmask0) \
      osvt($base-calmask1) \
      osvt($base-calmask2) \
      osvt($base-calmask3) \
      osvt($base-calmask4) \
      osvt($base-calmask5) \
      osvt($base-calmask6) \
      osvt($base-calmask7)
  pack .$mn.calmask -side top -anchor w -pady 2
  
  frame .$mn.ctrl
  button .$mn.ctrl.butt -text "Assemble control register" \
      -command "osvt_buildctrlreg osvt($base-ctrlreg) \
      osvt($base-thrdac) \
      osvt($base-caldac) \
      osvt($base-jitter) \
      osvt($base-peaking) \
      osvt($base-sampling) \
      osvt($base-totskip) \
      osvt($base-rodir) \
      osvt($base-rdresp) \
      osvt($base-clkthru) \
      osvt($base-digpatt) \
      osvt($base-selclkb) \
      osvt($base-selclka)"
  label .$mn.ctrl.lab -textvariable osvt($base-ctrlreg)
  pack .$mn.ctrl.butt .$mn.ctrl.lab \
      -side left -padx 2
  pack .$mn.ctrl -side top -anchor w -pady 2

  set osvt(chipmenu_talk) 0
  frame .$mn.end
  frame .$mn.end.fok
  message .$mn.end.msg \
      -width 320 \
      -justify left \
      -text "Apply parameters to hybrid/side: $hybrid\nchip(s): $chiplist"
  button .$mn.end.ok -text "OK" \
      -command "set osvt(chipmenu_talk) 1"

  button .$mn.end.cancel -text "CANCEL" \
      -command "set osvt(chipmenu_talk) 2"
  pack .$mn.end.msg .$mn.end.ok .$mn.end.cancel -side left -padx 2
  pack .$mn.end -side top -anchor w -pady 2

#  grab release .$mn
#  return

# --- wait for OK or CANCEL
  while {$osvt(chipmenu_talk) == 0} {
    
    tkwait variable osvt(chipmenu_talk)
    if {$osvt(chipmenu_talk) == 1} { 

      # --- build control register
      osvt_buildctrlreg cr \
	  osvt($base-thrdac) \
	  osvt($base-caldac) \
	  osvt($base-jitter) \
	  osvt($base-peaking) \
	  osvt($base-sampling) \
	  osvt($base-totskip) \
	  osvt($base-rodir) \
	  osvt($base-rdresp) \
	  osvt($base-clkthru) \
	  osvt($base-digpatt) \
	  osvt($base-selclkb) \
	  osvt($base-selclka)

      foreach chip $chiplist {
	set pre $hybrid-$chip
	
	set osvt($pre-ctrlreg) $cr
	set osvt($pre-rodir) $osvt($base-rodir)
	set osvt($pre-rdresp) $osvt($base-rdresp)
	
	osvt_entrytomask osvt($pre-chamask) \
	    $osvt($base-chamask0) \
	    $osvt($base-chamask1) \
	    $osvt($base-chamask2) \
	    $osvt($base-chamask3) \
	    $osvt($base-chamask4) \
	    $osvt($base-chamask5) \
	    $osvt($base-chamask6) \
	    $osvt($base-chamask7)

	osvt_entrytomask osvt($pre-calmask) \
	    $osvt($base-calmask0) \
	    $osvt($base-calmask1) \
	    $osvt($base-calmask2) \
	    $osvt($base-calmask3) \
	    $osvt($base-calmask4) \
	    $osvt($base-calmask5) \
	    $osvt($base-calmask6) \
	    $osvt($base-calmask7)
      }
    }
    if {$osvt(chipmenu_talk) == 2} { break }
  }

  grab release .$mn
  destroy .$mn
  return
}
