source osvt_utils.tcl
source osvt_chipmenu.tcl

if { [winfo exists .setup] } { destroy .setup }

if { ![info exists osvt(debug)] } then {
  set osvt(debug) 0
} 

set osvt(HDI-typelist) { h1 h2 h3 }
set osvt(HDI-h1-sidelist) { s0 s1 }
set osvt(HDI-h2-sidelist) { s0 s1 }
set osvt(HDI-h3-sidelist) { s0 s1 }
set osvt(HDI-side_names) { phi z }
set osvt(HDI-s0-name) phi
set osvt(HDI-s1-name) z
set osvt(HDI-h1-s0-nlocations)  7
set osvt(HDI-h1-s0-locationlist) { c0 c1 c2 c3 c4 c5 c6 }
set osvt(HDI-h1-s1-nlocations)  7
set osvt(HDI-h1-s1-locationlist) { c0 c1 c2 c3 c4 c5 c6 }
set osvt(HDI-h2-s0-nlocations)  10
set osvt(HDI-h2-s0-locationlist) { c0 c1 c2 c3 c4 c5 c6 c7 c8 c9 }
set osvt(HDI-h2-s1-nlocations)  10
set osvt(HDI-h2-s1-locationlist) { c0 c1 c2 c3 c4 c5 c6 c7 c8 c9 }
set osvt(HDI-h3-s0-nlocations)  4
set osvt(HDI-h3-s0-locationlist) { c0 c1 c2 c3 }
set osvt(HDI-h3-s1-nlocations)  5
set osvt(HDI-h3-s1-locationlist) { c0 c1 c2 c3 c4 }

#------------------------------------------------------------------------------
proc osvt_hdisetup {} {
#------------------------------------------------------------------------------
  global osvt
  
  # --- name of frame
  set nfr setup
  
  if [winfo exists .$nfr] then {
    wm deiconify $nfr
    focus $nfr
    return
  }

  toplevel .$nfr
  osvt_daqmenu $nfr
  wm title .$nfr "OSVT -- Hybrid Setup"
  if [file readable "/homes/babar/oasis_new/tcl/babar1.xbm"] {
    wm iconbitmap .$nfr @/homes/babar/oasis_new/tcl/babar1.xbm
  }
  
  frame .$nfr.hdi
  pack .$nfr.hdi -side top -anchor w
  
  # --- babar logo
  if [file readable "/homes/babar/oasis_new/tcl/babar1.xbm"] {
    label .$nfr.hdi.logo -bitmap "@/homes/babar/oasis_new/tcl/babar1.xbm"
  }

  # --- default 1 hybrid
  set osvt(hdilist_length) 1
  
  frame .$nfr.hdi.nhyb
  label .$nfr.hdi.nhyb.lab -text "Select no. of hybrids"
  scale .$nfr.hdi.nhyb.sca -to 10 -from 1 -orien horizontal \
      -variable osvt(hdilist_length) -length 240
  bind .$nfr.hdi.nhyb.sca <ButtonRelease-1> "+osvt_hdibuttmenu $nfr"
  pack .$nfr.hdi.nhyb.lab .$nfr.hdi.nhyb.sca -side top -fill x -expand 1
  
  pack .$nfr.hdi.logo -side left -padx 2
  pack .$nfr.hdi.nhyb -side left -padx 2 -fill x -expand 1 
  osvt_hdibuttmenu $nfr

  return
}

#------------------------------------------------------------------------------
proc osvt_hdibuttmenu {nfr} {
#------------------------------------------------------------------------------
    global osvt

    if [winfo exists .$nfr.which] {destroy .$nfr.which}
    frame .$nfr.which
    pack  .$nfr.which -side top -pady 2

# --- Build the HDI list
    set osvt(hdilist) {}
    if { $osvt(hdilist_length) < 1 } then {set osvt(hdilist_length) 0}
    if { $osvt(hdilist_length) > 20 } then {set osvt(hdilist_length) 20}
    for { set i 0 } { $i < $osvt(hdilist_length) } { incr i 1 } {
	lappend osvt(hdilist) "q$i"
    }

    if { $osvt(debug) } then {
       puts -nonewline "HDIs to configure: "
       puts $osvt(hdilist)
       puts "\n"
    }     

# --- init if needed, build hybrid widgets
    foreach hyb $osvt(hdilist) {
      # --- if hybrid was never initialized, set to default
      if ![info exists osvt($hyb-initialized)] {
	set osvt($hyb-type) {h3}
	osvt_hdisettodefault $hyb
      }
      osvt_hyblinesel $nfr.which $hyb
    }

#    if { $osvt(hdilist_length) > 1 } then {
#	button .$nfr.which.all  -text "all" -command "osvt_hdi_configureall"
#	pack .$nfr.which.all -side top -fill x -in .$nfr.which
#    }

}

# --- setup hybrid selection line made of hybrid label, type, edit
#------------------------------------------------------------------------------
proc osvt_hyblinesel {nfr hyb} {
#------------------------------------------------------------------------------
  global osvt

  set tfr $nfr.$hyb

  frame .$tfr
  pack .$tfr -side top

  label .$tfr.lab -text [format "%-16s" "Hybrid $hyb" ]

  osvt_optbutton .$tfr.typ osvt($hyb-type-label) osvt($hyb-type) \
      [list "type h1" "h1" \
      "osvt_hdi_reconfigurehdi $hyb"] \
      [list "type h2" "h2" \
      "osvt_hdi_reconfigurehdi $hyb"] \
      [list "type h3" "h3" \
      "osvt_hdi_reconfigurehdi $hyb"]
  
  button .$tfr.edit -text "edit" -command "osvt_hdi_configurehdi $hyb"
  button .$tfr.clr -text "set to default" \
      -command "osvt_hdisettodefault $hyb"
  pack .$tfr.lab .$tfr.typ .$tfr.edit .$tfr.clr -side left

}

#------------------------------------------------------------------------------
proc osvt_hdi_reconfigurehdi { hybrid } {
#------------------------------------------------------------------------------
  global osvt

  set w .$hybrid
  
  # --- set to default each time the hybrid type is changed
  osvt_hdisettodefault $hybrid
  if ![winfo exists $w] then {return}
  osvt_hdi_configurehdi $hybrid
}

#------------------------------------------------------------------------------
proc osvt_hdi_configureall { } {
#------------------------------------------------------------------------------
    global osvt

    if { $osvt(debug) } then {
       puts -nonewline "You select to configure all the hdi"
    }
    error "Option all still not implemented"
}

#------------------------------------------------------------------------------
proc osvt_hdi_configurehdi { hybrid } {
#------------------------------------------------------------------------------
  global osvt

  set w .setup-$hybrid

  set loctype $osvt($hybrid-type)

  if ![winfo exists $w] then {
    toplevel $w
    if [file readable "/homes/babar/oasis_new/tcl/babar1.xbm"] {
      wm iconbitmap $w @/homes/babar/oasis_new/tcl/babar1.xbm
    }
#   grab $w
    wm title $w "Setup hybrid $hybrid type $loctype"
    wm iconname $w $hybrid-setup
  }

  if ![winfo exists $w.side] {frame $w.side}
  osvt_hdi_configuresides $hybrid $loctype

  if ![winfo exists $w.cmd] then {
    frame $w.cmd
    pack $w.side $w.cmd  -side top

#--- Frame for commands:
#    1) Cancel -> close the window
#    2) Help   -> (not implemented)

    button $w.cmd.cancel -text "Close window" -command "destroy $w"
    button $w.cmd.help -text "Help" \
	-command "error \" Help still not implemented\" "
    pack $w.cmd.cancel $w.cmd.help -side left -padx 2
  }
  wm deiconify $w
  focus $w
}

#------------------------------------------------------------------------------
proc osvt_hdi_configuresides {hybrid htype} {
#------------------------------------------------------------------------------
    global osvt

#--- Local variables 
    set w .setup-$hybrid.side
    if { $osvt(debug) } then {
	puts "You selected type $htype for hybrid $hybrid" 
    }

#--- Build two frames (one per side)
    if { [ winfo exists $w.s0 ] } then {
      destroy $w.s0
    }
    if { [ winfo exists $w.s1 ] } then {
      destroy $w.s1
    }
    frame $w.s0 -relief raised  -borderwidth 4
    frame $w.s1 -relief sunken  -borderwidth 4 
    pack $w.s0 $w.s1 -side top -anchor w -fill x

#--- Each side contains a chiplist and a 
    foreach side $osvt(HDI-$htype-sidelist) {
      label $w.$side.lab -text "$osvt(HDI-$side-name) side" \
	  -relief ridge -bd 4
      osvt_setupsideromch $w.$side.rom $hybrid $htype $side
      frame $w.$side.vert
      pack  $w.$side.lab $w.$side.rom $w.$side.vert \
	  -side top -fill x -expand 1 -pady 2
      frame $w.$side.vert.act
      pack  $w.$side.vert.act -side left
      osvt_side_configureside $w.$side.vert.act $hybrid $htype $side
    }
}

proc osvt_setupsideromch {romfr hybrid htype side} {
  global osvt

  #--- ROM SETTING frame (left and right)
  #    1) a frame for setting the link readout-direction to the ROM channel 
  
  frame $romfr

  set base $hybrid-$side
  osvt_optline $romfr.left "Left readout on ROM channel:" \
      osvt($base-l-romch-label) osvt($base-l-romch) \
      { " 0 " 0 } \
      { " 1 " 1 }\
      { " 2 " 2 }\
      { " 3 " 3 }
  
  osvt_optline $romfr.right "Right readout on ROM channel:" \
      osvt($base-r-romch-label) osvt($base-r-romch) \
      { " 0 " 0 } \
      { " 1 " 1 }\
      { " 2 " 2 } \
      { " 3 " 3 } 
  
  pack $romfr.left  -side left -fill x
  pack $romfr.right -side right -fill x
}

#------------------------------------------------------------------------------
proc osvt_side_configureside {w hybrid htype side} {
#------------------------------------------------------------------------------
    global osvt

#---  
    if { $osvt(debug) } then {
	puts "Active window is $w"
    }

    if { $osvt(debug) } then {
	puts "Active window is $w"
    }

    frame $w.lab
    frame $w.setall
    frame $w.chipframe
    pack  $w.chipframe $w.lab $w.setall -side left -padx 6 -anchor nw

    # --- labels
    set window $w.lab
    label $window.chip -text "Description"
    label $window.existing -text "Exists"
    label $window.respond  -text "Respond to read event"
    label $window.direction -text "Readout direction"
    label $window.setup -text "Detailed chip setup"
    pack  $window.chip -side top -fill x -pady 4
    pack  $window.existing -side top -fill x -pady 3
    pack  $window.respond $window.direction \
	-side top -fill x -pady 3

#----------------------------
#--- GLOBAL SETTING sub-frame

    set window $w.setall
    label $window.chip -text "set ALL chips"
    frame $window.existing
    frame $window.respond
    frame $window.direction
    frame $window.setup

    pack $window.chip -side top -fill x -pady 4
    pack $window.existing $window.respond \
	$window.direction \
	-side top -fill x -pady 2
    pack $window.setup -side top -fill x -pady 4

    # --- Sub-frame for setting all the IC to existing mode (yes/no)
    button $window.existing.no      -text "destroy" \
	-command "osvt_hdi_setallhdi $hybrid $side exists 0 ; osvt_hdibuildchipl $hybrid $side"
    button $window.existing.yes     -text "create " \
	-command "osvt_hdi_setallhdi $hybrid $side exists 1 ; osvt_hdibuildchipl $hybrid $side"
    pack $window.existing.no $window.existing.yes -side left -fill x

    # --- Sub-frame for setting all the IC to respond mode (yes/no)
    button $window.respond.no       -text "  no   " \
	-command "osvt_hdi_setallhdi $hybrid $side rdresp 0"
    button $window.respond.yes      -text "  yes  " \
	-command "osvt_hdi_setallhdi $hybrid $side rdresp 1"
    pack $window.respond.no $window.respond.yes -side left -fill x

    # --- Sub-frame for setting all the IC to direction mode (left/right)
    button $window.direction.left   -text " left  " \
	-command "osvt_hdi_setallhdi $hybrid $side rodir 0"
    button $window.direction.right  -text " right " \
	-command "osvt_hdi_setallhdi $hybrid $side rodir 1"
    pack $window.direction.left $window.direction.right -side left -fill x

#   --- Button for changing setup to all the ICs

    set osvt($hybrid-$side-chiplist) {}
    button $window.setup.cmd -text " setup " \
	-command "osvt_hdibuildchipl $hybrid $side; osvt_hdi_chiplistsetup $hybrid $side"
    pack $window.setup.cmd -side top -fill x

#--------------------------------
#--- CHIPFRAME.$chip sub-frame
#    Now the loop on all the chip_locations

    set window $w.chipframe
    foreach chip $osvt(HDI-$htype-$side-locationlist) { 
	osvt_hdi_chipframe $window.$chip $hybrid $side $chip
	pack $window.$chip -side left -fill x -fill y -padx 2
    }
}
#------------------------------------------------------------------------------
proc osvt_hdi_chipframe { window hybrid side chip } {
#------------------------------------------------------------------------------
    global osvt

    if { $osvt(debug) } then {
	puts "New window $window - $hybrid $side $chip"
    }

#--- Define a frame for any chip location (i.e. id-label, enable,direction buttons, 
#    chip-setup button)

    frame $window
    label $window.id -text "$chip"
    pack $window.id \
	-side top -pady 4 -fill x

#   --- Just check that variables are set 
#
    set base $hybrid-$side-$chip

#   --- Sub-frame for setting the $chip IC to existing mode (yes/no)
    osvt_optbuttonpack $window.existing  \
    osvt($base-exists-label) osvt($base-exists) \
	[list "yes  " 1  "osvt_hdibuildchipl $hybrid $side" ] \
	[list "no   " 0  "osvt_hdibuildchipl $hybrid $side" ]

#   --- Sub-frame for setting the $chip IC to responding mode (yes/no)

    osvt_optbuttonpack $window.respond   \
    osvt($base-rdresp-label) osvt($base-rdresp) \
	[list "yes  " 1 "osvt_updctrlreg osvt($base-ctrlreg) rdresp 1" ] \
	[list "no   " 0 "osvt_updctrlreg osvt($base-ctrlreg) rdresp 0" ]

#   --- Sub-frame for setting to the $chip IC the direction mode (left/right)

    osvt_optbuttonpack $window.direction \
    osvt($base-rodir-label) osvt($base-rodir) \
	[list "left " 0 "osvt_updctrlreg osvt($base-ctrlreg) rodir 0 " ] \
	[list "right" 1 "osvt_updctrlreg osvt($base-ctrlreg) rodir 1 " ]

    button $window.chipsetup -text "setup" -command "osvt_hdi_chipsetup $hybrid $side $chip"
    pack $window.chipsetup \
	-side top -pady 4

}

#------------------------------------------------------------------------------
proc osvt_hdisettodefault {hybrid} {
#------------------------------------------------------------------------------
  global osvt

  set htype $osvt($hybrid-type)

  foreach side $osvt(HDI-$htype-sidelist) {
    # --- set readout info
    set osvt($hybrid-$side-l-romch) 0
    set osvt($hybrid-$side-r-romch) 1
    # --- set chip info
    foreach chip $osvt(HDI-$htype-$side-locationlist) {
      set base $hybrid-$side-$chip
      set osvt($base-addr)    \
	  [lsearch $osvt(HDI-$htype-$side-locationlist) $chip]
      set osvt($base-exists)  0
      set osvt($base-rdresp)  0
      set osvt($base-rodir)   0
      set osvt($base-ctrlreg) 0x81910e80
      set osvt($base-chamask) { 0xffffffff 0xffffffff  0xffffffff 0xffffffff }
      set osvt($base-calmask) { 0x00000000 0x00000000  0x00000000 0x00000000 }
      if { $osvt(debug) } then {
	puts "Base address : $base -> $osvt($base-addr) "
      }
      osvt_updctrlreg  osvt($base-ctrlreg)  rdresp $osvt($base-rdresp)
      osvt_updctrlreg  osvt($base-ctrlreg)  rodir  $osvt($base-rodir)
    }
    # --- build list of actually existing chips per side
    osvt_hdibuildchipl $hybrid $side
  }
  
  set osvt($hybrid-initialized) {}
}

#------------------------------------------------------------------------------
proc osvt_hdi_clearhdi { { hybrid q0 } } {
#------------------------------------------------------------------------------
    global osvt

    set type $osvt($hybrid-type)
    foreach side $osvt(HDI-$type-sidelist) {
	if { [ info exists osvt($hybrid-$side-chiplist) ] } then {
	    if { [ llength $osvt($hybrid-$side-chiplist) ] } then {
		foreach chip $osvt($hybrid-$side-chiplist) {
		    unset osvt($hybrid-$side-$chip-exists) 
		    unset osvt($hybrid-$side-$chip-rdresp) 
		    unset osvt($hybrid-$side-$chip-rodir) 
		}
	    }
	    unset osvt($hybrid-$side-chiplist) 
	}
    }
    unset osvt($hybrid-type)
}

#------------------------------------------------------------------------------
proc osvt_hdi_setallhdi { { hybrid q0 } { side s0 } { cosa exists } { mode 0 } } {
#------------------------------------------------------------------------------
    global osvt

    set tmp(exists-0)     "no   "
    set tmp(exists-1)     "yes  "
    set tmp(rdresp-0)     "no   "
    set tmp(rdresp-1)     "yes  "
    set tmp(rodir-0)      "left "
    set tmp(rodir-1)      "right"

    set htype $osvt($hybrid-type)
    foreach chip $osvt(HDI-$htype-$side-locationlist) {
	set base $hybrid-$side-$chip
	set osvt($base-$cosa) $mode
	set osvt($base-$cosa-label) $tmp($cosa-$mode)
	switch -- $cosa {
	    exists  { }
	    default {  osvt_updctrlreg osvt($base-ctrlreg) $cosa $osvt($base-$cosa) }
	}
	if { $osvt(debug) } then {
	    puts " osvt $base-$cosa         is :  $osvt($base-$cosa) "
	    puts " osvt $base-$cosa-label   is :  $osvt($base-$cosa-label) "
	    puts " osvt $base-ctrlreg is :  $osvt($base-ctrlreg) "
	}
    }
    update idletasks
}

#------------------------------------------------------------------------------
proc osvt_hdi_chipsetup { { hybrid q0 } { side s0 } { chip c0 } } {
#------------------------------------------------------------------------------
    global osvt

    if { $osvt($hybrid-$side-$chip-exists) } then {
	if { $osvt(debug) } then {
	    puts "Configuring $hybrid->$side: $chip "
	}
	osvt_chipmenu $hybrid-$side $chip
    }	
}

#------------------------------------------------------------------------------
proc osvt_hdi_chiplistsetup { { hybrid q0 } { side s0 } } {
#------------------------------------------------------------------------------
    global osvt

    if { $osvt(debug) } then {
	puts "Configuring $hybrid->$side: $osvt($hybrid-$side-chiplist) "
    }
    if {$osvt($hybrid-$side-chiplist) != {}} {
      osvt_chipmenu $hybrid-$side $osvt($hybrid-$side-chiplist)
    }
}

#------------------------------------------------------------------------------
proc osvt_hdibuildchipl { { hybrid q0 } { side s0 } } {
#------------------------------------------------------------------------------
    global osvt

    set htype $osvt($hybrid-type)

    # --- setup list of actually existing chips
    set osvt($hybrid-$side-chiplist) {} 
    foreach chip $osvt(HDI-$htype-$side-locationlist) {	     
      if { $osvt($hybrid-$side-$chip-exists) } then {
	lappend osvt($hybrid-$side-chiplist) $chip
      }
    }

    if { $osvt(debug) } then {
	puts "Built $hybrid->$side-chiplist: $osvt($hybrid-$side-chiplist) "
    }
}

#------------------------------------------------------------------------------
proc osvt_hdi_delfrom_chiplist { hybrid  side chip } {
#------------------------------------------------------------------------------
    global osvt

    if { $osvt(debug) } then {
	puts "Before... --- $hybrid->$side-chiplist: $osvt($hybrid-$side-chiplist) "
    }

    set tmp [ lsearch $osvt($hybrid-$side-chiplist)  $chip ]
    if { $tmp != -1 } then {
	set osvt($hybrid-$side-chiplist) [ lreplace $osvt($hybrid-$side-chiplist) $tmp $tmp  ]
    }
    if { $osvt(debug) } then {
	puts "After... --- $hybrid->$side-chiplist: $osvt($hybrid-$side-chiplist) "
    }
}

#------------------------------------------------------------------------------
proc osvt_hdi_addto_chiplist { hybrid  side chip } {
#------------------------------------------------------------------------------
    global osvt

    if { $osvt(debug) } then {
	puts "Before... --- $hybrid->$side-chiplist: $osvt($hybrid-$side-chiplist) "
    }

    set tmp [ lsearch $osvt($hybrid-$side-chiplist)  $chip ]
    if { $tmp == -1 } then {
	lappend osvt($hybrid-$side-chiplist) $chip
    }
    if { $osvt(debug) } then {
	puts "After... --- $hybrid->$side-chiplist: $osvt($hybrid-$side-chiplist) "
    }
}

#------------------------------------------------------------------------------
proc osvt_save_config { filename } {
#------------------------------------------------------------------------------
  global osvt
  
  catch {exec cp $filename "$filename\.bkup"} dummy
  
  set idch [ open $filename w ]
  foreach name [ lsort [ array names osvt ] ] {
    puts $idch [format "%-32s %s" "set osvt($name)" "{$osvt($name)}"]
  }
  close $idch
}

#------------------------------------------------------------------------------
proc osvt_load_config { filename } {
#------------------------------------------------------------------------------
  global osvt
  
  if { ! [ file exists $filename ] } then {
    error "File $filename can't be accessed"
  }
  
  source $filename
}

# --- menu for DAQ window
#------------------------------------------------------------------------------
proc osvt_daqmenu {topl} {
#------------------------------------------------------------------------------
  global osvt  
  set menu $topl.mb
  frame .$menu -relief raised -bd 2 -bg #8dd 
  
  menubutton .$menu.file  \
      -text File  \
      -underline 0  \
      -menu .$menu.file.menu
  menubutton .$menu.opt  \
	-text Config  \
	-underline 0  \
	-menu .$menu.opt.menu
  menubutton .$menu.win  \
      -text Windows  \
      -underline 0  \
      -menu .$menu.win.menu
  menubutton .$menu.help  \
      -text Help  \
      -underline 0  \
      -menu .$menu.help.menu  
  
  pack .$menu.file .$menu.opt .$menu.win  \
      -side left
  pack .$menu.help  \
      -side right
  
  set file $menu.file.menu
  
  menu .$file

  .$file add command  \
      -label "Source"  \
      -underline 0  \
      -accelerator " C s"   \
      -command {
    if {![info exists source_file_name]} {set source_file_name ""}
    set source_file_name [FSBox "enter file name" $source_file_name]
    if {$source_file_name != ""} {source $source_file_name}
  }

  .$file add command  \
      -label "Reload"  \
      -underline 0  \
      -accelerator " C r"   \
      -command {source osvt_hdisetup.tcl ; osvt_hdisetup}

  .$file add separator
  .$file add command  \
      -label "About"   \
      -command {error "Not implemented yet"}
  .$file add command  \
      -label "Exit"  \
      -underline 0  \
      -accelerator " C e"   \
      -command "destroy .setup"
  
  set opt $menu.opt.menu
  
  set osvt(config_fn) "osvt_config.tcl"
  set osvt(config_last) $osvt(config_fn)
  menu .$opt
  .$opt add command \
      -label "Save Config"  \
      -command {osvt_save_config $osvt(config_fn)}
  .$opt add command  \
      -label "Save Config As ..."  \
      -command {
    set osvt(config_last) [FSBox "Enter config file name" $osvt(config_last)]
    if {$osvt(config_last) != ""} {
      osvt_save_config $osvt(config_last)
    }
  }
  .$opt add command  \
      -label "Restore Config"  \
      -command {osvt_load_config $osvt(config_fn)}
  .$opt add command -label "Restore Config From ..."  \
      -command {
    set osvt(config_last) [FSBox "Enter config file name" $osvt(config_last)]
    if {$osvt(config_last) != ""} {
      osvt_load_config $osvt(config_last)
    }
  }
  
  set windows $menu.win.menu
  menu .$windows
  
  set help $menu.help.menu
  
  menu .$help
  .$help add command -label "Big help" \
      -underline 0  \
      -accelerator " C b"  \
      -command {
    toplevel .help
    message .help.m  \
	-text "No help is avaiable at this time"
    pack .help.m
  }
  
  tk_menuBar .$menu .$menu.file .$menu.opt .$menu.win .$menu.help
  focus .$menu
  
  pack .$menu  \
      -side top  \
      -pady 1  \
      -fill x
}
