set osvt(HDI_DIR_WRITE)     1
set osvt(HDI_DIR_READ)      0

set osvt(HDI_TAG_NONE)      0x00
set osvt(HDI_TAG_ALL)       0x0f

set osvt(HDI_OP_NOOP)       0x00
set osvt(HDI_OP_CLEAR)      0x10
set osvt(HDI_OP_SYNCH)      0x8
set osvt(HDI_OP_TRIGGER)    0x18
set osvt(HDI_OP_READEV)     0x4
set osvt(HDI_OP_CALSTROBE)  0x14
set osvt(HDI_OP_WRCTRL)     0x1b
set osvt(HDI_OP_WRMSK)      0x7
set osvt(HDI_OP_RDREG)      0x17
set osvt(HDI_OP_RESET)      0xf

set osvt(HDI_CHIPDATA_ZERO) {}
for { set i 0 } {$i < 32} {incr i} {
  lappend osvt(HDI_CHIPDATA_ZERO) 0x00
}
  
proc osvt_setxdefaults {} {
  option clear
  
  option add *background                  #00f    user
  option add *foreground                  black   user
  option add *activeBackground            #7cc    user
  option add *activeForeground            black   user
  option add *selectColor                 black   user
 
  option add *Frame*background            #00f    user

  option add *Label*background            #aff    user
  option add *Label*highlightThickness    0       user
  option add *Label*borderWidth           2       user
  option add *Label*padY                  5       user
  option add *Label*padX                  4       user
 
  option add *Entry*background            #aff    user
  option add *Entry*highlightColor        #f00    user
  option add *Entry*highlightBackground   #8dd    user
  option add *Entry*selectBackground      #0ff    user
  option add *Entry*relief                sunken  user
 
  option add *Button*background           #8dd    user
  option add *Button*highlightThickness   0       user
  option add *Button*borderWidth          2       user
  option add *Button*padY                 5       user
  option add *Button*padX                 4       user
 
  option add *Menubutton*background               #8dd    user
  option add *Menubutton*highlightThickness       0       user
  option add *Menubutton*borderWidth              2       user
  option add *Menubutton*padY                     6       user
  option add *Menubutton*padX                     4       user
#  option add *Menubutton*activeBackground         #7cc    user
#  option add *Menubutton*relief                   raised       user

  option add *Radiobutton*background              #8dd    user
  option add *Radiobutton*highlightThickness      0       user
 
  option add *Menu*tearOff                        0       user
  option add *Menu*background                     #8dd    user
  option add *Menu*borderWidth                    2       user
#  option add *Menu*activeBackground               #7cc    user
 
  option add *Message*background          #aff    user

  option add *Scale*highlightBackground   #8dd    user
  option add *Scale*background            #8dd    user
  option add *Scale*borderWidth           2       user
  option add *Scale*relief                raised  user
#  option add *Scale*foreground            #000    user
#  option add *Scale*activeBackground      #000    user

#  option add *troughColor                 #dfdfdf user
#  option add *selectBackground            #bfdfff user
#  option add *disabledForeground          #7f7f7f user
}

osvt_setxdefaults

# --- create global variables if they do not exist
proc osvt_fixglob args {
  foreach arg $args {
    upvar $arg var
    if ![info exists var] {
      set var {}
    }
  }
}

# --- reverse bits in a word
proc osvt_bitrev { word {nbits 8} } {
  set tmp 0
  for { set i 0 } {$i < $nbits} {incr i 1} {
    set tmp [expr ($tmp<<1) | ($word & 1)]
    set word [expr $word>>1]
  }
  return $tmp
}
 
# --- transform a 32 bits longword to 4 bytes for vram
# --- reordering bits for LBL ROM send command
proc osvt_lwtovram {word} {
  set cd {}
  set rw [osvt_bitrev $word 32]
  lappend cd [format "0x%02x" [expr ($rw>>24) & 0xff]]
  lappend cd [format "0x%02x" [expr ($rw>>16) & 0xff]]
  lappend cd [format "0x%02x" [expr ($rw>>8)  & 0xff]]
  lappend cd [format "0x%02x" [expr  $rw      & 0xff]]
  return $cd
}

# --- transform 128 bit mask (4 32bits words) in byte list for vram
proc osvt_masktovram {vm} {
  foreach wo $vm {
    eval lappend cd [osvt_lwtovram $wo]
  }
  return $cd
}

# --- invert all bits in a mask (4 32-bits words)
proc osvt_maskinvert {vm} {
  set im {}
  foreach mw $vm {
    set im [concat [format "0x%08x" [osvt_bitrev $mw 32]] $im]
  }
  return $im
}

proc osvt_masktoentry {ml m0 m1 m2 m3 m4 m5 m6 m7} {
  upvar $m0 vm0
  upvar $m1 vm1
  upvar $m2 vm2
  upvar $m3 vm3
  upvar $m4 vm4
  upvar $m5 vm5
  upvar $m6 vm6
  upvar $m7 vm7

  set i 0
  foreach mw $ml {
    set vm$i [format "0x%04x" [expr $mw & 0xffff]]
    incr i
    set vm$i [format "0x%04x" [expr ($mw>>16) & 0xffff]]
    incr i
  }
}

proc osvt_entrytomask {ml m0 m1 m2 m3 m4 m5 m6 m7} {
  upvar $ml vml

  set vml {} 
  for {set i 0} {$i < 8} {incr i} {
    eval set t1 \$m$i
    incr i
    eval set t2 \$m$i
    lappend vml [format "0x%08x" [expr $t1 | ($t2<<16)]]
  }
}

# MakeMenu {m List repeat Path options}
#       This function makes a menu $m with entries from list. All menu
#       entries will have the same options.  each item will have the
#       name fronm the list. If list contains a sublist, a cascade menu
#       will be created, and so on. repeat tells if the menu entry
#       should be repeated in the cascade that follows:
#       List: {1 {2 2a {2b {2b1 2b2} 2c}}
#       repeat = 0                      repeat = 1
#       1                               1
#       2 - 2a                          2 - 2
#           2b - 2b1                        2a 
#                2b2                        2b - 2b
#           2c                                   2b1
#                                                2b2
#                                           2c
#
#       If you want to use the name of the entry, use %Entry, for example: 
#         MakeMenu .m $List 0 -command {puts stdout %Entry}
#       will generate "puts stdout 2b2" for the command
#       If you want the entry with the path put in %EntryPath
#         MakeMenu .m $List 0 -command {puts stdout %EntryPath}
#       will generate "puts stdout 2/2b/2b2" for the command.
#       This will be prepended by Path, usually that entry is empty. it's 
#       only there for recursive calling.

proc osvt_makemenu {m List repeat Path args} {
  if {![winfo exists $m]} {menu $m}
  foreach Entry $List {
    if {$Path != {}} {set EntryPath $Path/[lindex $Entry 0]} \
	else {set EntryPath [lindex $Entry 0]}
    if {[llength $Entry] == 1} {
      regsub -all "%EntryPath" $args "$EntryPath" locargs
      regsub -all "%Entry" $locargs "$Entry" locargs
      eval "$m add command -label $Entry $locargs"
    } else {
      set Sub [lindex $Entry 0]
      $m add cascade -label $Sub -menu $m.m$Sub
      if $repeat {
	if {![winfo exists $m.m$Sub]} {menu $m.m$Sub}
	regsub -all "%EntryPath" $args "$EntryPath" locargs
	regsub -all "%Entry" $locargs "$Sub" locargs
	eval "$m.m$Sub add command -label $Sub $locargs"
      }
      set Rest [lindex $Entry 1]
      eval MakeMenu $m.m$Sub {$Rest} $repeat $EntryPath $args
    }
  }
}

proc osvt_killmenu {m} {
  if {![winfo exists $m]} {return}
  set Subs [winfo children $m]
  foreach Sub $Subs {KillMenu $Sub}
  $m delete 0 last
  update
}

# ---
# does osvt_optline and pack frame
#
proc osvt_optlinepack {nfr label tvar nvar args} {
  eval {osvt_optline $nfr $label $tvar $nvar} $args
  pack $nfr -side top -anchor w -pady 2 
}

# ---
# creates a frame containing a label and a menubutton with choices
# osvt_optline 
#   nfr        frame name
#   label      text for label
#   ...        --> see osvt_optbutton
#
proc osvt_optline {nfr label tvar nvar args} {
  frame $nfr
  label $nfr.lab -text $label
  eval {osvt_optbutton $nfr.mb $tvar $nvar} $args
  pack $nfr.lab $nfr.mb -side left -padx 2
}

# ---
# does osvt_optbutton and pack frame
#
proc osvt_optbuttonpack {nfr tvar nvar args} {
  eval {osvt_optbutton $nfr $tvar $nvar} $args
  pack $nfr -side top -anchor w -pady 2 
}

# ---
# creates a menubutton with options
# osvt_optbutton 
#   m           name for menu option button
#   tvar        global text variable for button label
#   nvar        global variable to be updated by button
#   args        series of lists containing:
#     <tv>      text for menu entry
#     <nv>      corresponding value to be put into nvar
#     [<nv>]    optional command
#
proc osvt_optbutton {m tvar nvar args} {
  upvar #0 $tvar tt
  upvar #0 $nvar nn

  menubutton $m -menu $m.menu -textvariable $tvar -relief raised
  menu $m.menu

  set vv "<INCORRECT>"
  set nopts [llength $args]

  set body "\nupvar #0 $tvar tt\n"
  append body "upvar #0 $nvar nn\n"
  append body "switch -- \$nn \{\n"
    
  foreach arg $args {
    set text [lindex $arg 0]
    set num  [lindex $arg 1]
    set cmd  [lindex $arg 2]
    # --- init button label with text corresponding to 'nvar' argument
    if {$num == $nn} {set vv $text}

    set cs "set $nvar $num"
    if {$cmd != {}} {
      append cs " \; $cmd"
    }
    # --- create menu entry
    $m.menu add command -label $text \
	-command $cs
#    puts "command '$cs'"
    append body "{$num} {set tt {$text}}\n"
  }
  set tt $vv

  append body "default {set tt {<INCORRECT>}}\n"
  append body "\}\n"
  regsub -all {[\(\)]} $nvar {_} tag
  proc osvt_trace_$tag {n1 n2 op} $body
  trace vdelete  nn w osvt_trace_$tag
  trace variable nn w osvt_trace_$tag
}

#proc osvt_trace { n1 n2 op } {
#  puts "n1 $n1 n2 $n2 op $op"
#  if {$n2 == {}} {
#    eval upvar \#0 $n1 vv
#  } else {
#    eval upvar \#0 $n1\($n2\) vv
#  }
#  puts "vv $vv"
#}

# ---
# creates a menubutton with options
# osvt_menubutton
#   m           name for menu option button
#   var         global variable to be updated by button
#   args        series of lists containing:
#     <tv>      text for menu entry (will go into var)
#     [<nv>]    optional command
#
proc osvt_menubutton {m var args} {
  upvar #0 $var vv
 
  menubutton $m -menu $m.menu -textvariable $var -relief raised
  menu $m.menu
 
  foreach arg $args {
    set text [lindex $arg 0]
    set cmd  [lindex $arg 1]

    set cs "set $var $text"
    if {$cmd != {}} {
      append cs " \; $cmd"
    }
 
    # --- create menu entry
    $m.menu add command -label $text -command $cs
#    puts "command '$cs'"
  }
}

# ---
# does osvt_entryline and pack frame
#
proc osvt_entrylinepack {nfr label nvar {width 0}} {
  eval [list osvt_entryline $nfr $label $nvar $width]
  pack $nfr -side top -anchor w -pady 2 
}

# ---
# creates a frame containing a label and an entry for a variable
# osvt_entryline 
#   nfr        frame name
#   label      text for label
#   nvar       variable for entry
#   width      width of entry field
#
proc osvt_entryline {nfr label nvar {width 0}} {
  frame $nfr
  label $nfr.lab -text $label
  entry $nfr.ent -textvariable $nvar -width $width
  pack $nfr.lab $nfr.ent -side left -padx 2
}

# ---
# routine for entry of 128 bit masks
#
proc osvt_mask16pack {nfr lab nvar} {
  frame $nfr
  pack $nfr -side left -padx 2
  label $nfr.lab -text $lab
  entry $nfr.ent -textvariable $nvar -width 6
  pack $nfr.lab $nfr.ent -side top -pady 0
}

# ---
# routine for entry of 128 bit masks
#
proc osvt_mask2line {nfr v1 v2 v3 v4 v5 v6 v7 v8} {
  frame $nfr

  frame $nfr.l1
  frame $nfr.l2
  pack $nfr.l1 $nfr.l2 -side top -pady 2

  osvt_mask16pack $nfr.l1.w1 "0x00->" $v1
  osvt_mask16pack $nfr.l1.w2 "0x10->" $v2
  osvt_mask16pack $nfr.l1.w3 "0x20->" $v3
  osvt_mask16pack $nfr.l1.w4 "0x30->" $v4

  osvt_mask16pack $nfr.l2.w1 "0x40->" $v5
  osvt_mask16pack $nfr.l2.w2 "0x50->" $v6
  osvt_mask16pack $nfr.l2.w3 "0x60->" $v7
  osvt_mask16pack $nfr.l2.w4 "0x70->" $v8

}

# ---
# routine for entry of 128 bit masks
#
proc osvt_masklab {nfr label v1 v2 v3 v4 v5 v6 v7 v8} {
  frame $nfr

  frame $nfr.labset
  label $nfr.labset.lab -text $label
  frame $nfr.labset.set  
  pack $nfr.labset.lab $nfr.labset.set -side top -pady 2

  label $nfr.labset.set.set -text "Set all channels"
  frame $nfr.labset.set.at
  pack $nfr.labset.set.set $nfr.labset.set.at \
      -side top -pady 0

  button $nfr.labset.set.at.on -text "on" \
      -command "set $v1 0xffff; set $v2 0xffff; set $v3 0xffff; set $v4 0xffff; set $v5 0xffff; set $v6 0xffff; set $v7 0xffff; set $v8 0xffff" 

  button $nfr.labset.set.at.off -text "off" \
      -command "set $v1 0x0000; set $v2 0x0000; set $v3 0x0000; set $v4 0x0000; set $v5 0x0000; set $v6 0x0000; set $v7 0x0000; set $v8 0x0000" 

  pack $nfr.labset.set.at.on $nfr.labset.set.at.off \
      -side left -padx 2

  osvt_mask2line $nfr.val $v1 $v2 $v3 $v4 $v5 $v6 $v7 $v8
  pack $nfr.labset $nfr.val -side left -padx 2
}

# --- fix a value that must fit in a bit field
proc osvt_fixbitfval {var {mask 1} {def 0}} {
  upvar $var vv

  if [catch {expr $vv} rs] {
    set vv $def
  }
  set vv [expr $vv & $mask]
}

proc osvt_dectogray {val} {
  set val [expr $val]
  switch -- $val {
    0 { return 0}
    1 { return 1}
    2 { return 3}
    3 { return 2}
    4 { return 6}
    5 { return 7}
    6 { return 5}
    7 { return 4}
    8 { return 12}
    9 { return 13}
    10 { return 15}
    11 { return 14}
    12 { return 10}
    13 { return 11}
    14 { return 9}
    15 { return 8}
    16 { return 24}
    17 { return 25}
    18 { return 27}
    19 { return 26}
    20 { return 30}
    21 { return 31}
    22 { return 29}
    23 { return 28}
    24 { return 20}
    25 { return 21}
    26 { return 23}
    27 { return 22}
    28 { return 18}
    29 { return 19}
    30 { return 17}
    31 { return 16}
    default {return -1}
  }
}

proc osvt_graytodec {val} {
  set val [expr $val]
  switch -- $val {
    0 { return 0}
    1 { return 1}
    3 { return 2}
    2 { return 3}
    6 { return 4}
    7 { return 5}
    5 { return 6}
    4 { return 7}
    12 { return 8}
    13 { return 9}
    15 { return 10}
    14 { return 11}
    10 { return 12}
    11 { return 13}
    9 { return 14}
    8 { return 15}
    24 { return 16}
    25 { return 17}
    27 { return 18}
    26 { return 19}
    30 { return 20}
    31 { return 21}
    29 { return 22}
    28 { return 23}
    20 { return 24}
    21 { return 25}
    23 { return 26}
    22 { return 27}
    18 { return 28}
    19 { return 29}
    17 { return 30}
    16 { return 31}
    default {return -1}
  }
}

proc osvt_buildctrlreg {creg th cl ji pk sa to di re ct dp cb ca} {
  upvar $creg vcreg
  upvar $th vth
  upvar $cl vcl
  upvar $ji vji
  upvar $pk vpk
  upvar $sa vsa
  upvar $to vto
  upvar $di vdi
  upvar $re vre
  upvar $ct vct
  upvar $dp vdp
  upvar $cb vcb
  upvar $ca vca

  osvt_fixbitfval vth 0x3f
  osvt_fixbitfval vcl 0x3f

  osvt_fixbitfval vji 0x1f 8
  if {$vji == 0} {set vji 1}

  osvt_fixbitfval vpk 0x03
  osvt_fixbitfval vsa 0x03
  osvt_fixbitfval vto 0x03
  osvt_fixbitfval vdi 0x01
  osvt_fixbitfval vre 0x01
  osvt_fixbitfval vct 0x01
  osvt_fixbitfval vdp 0x0f
  osvt_fixbitfval vcb 0x01
  osvt_fixbitfval vca 0x01
  
  # --- build control register
  set cr 0
  set cr [expr $cr | ($vth<<0)]
  set cr [expr $cr | ($vcl<<6)]
  set cr [expr $cr | ([osvt_dectogray $vji]<<12)]
  set cr [expr $cr | ($vpk<<17)]
  set cr [expr $cr | ($vsa<<19)]
  set cr [expr $cr | ($vto<<21)]
  set cr [expr $cr | ($vdi<<23)]
  set cr [expr $cr | ($vre<<24)]
  set cr [expr $cr | ($vct<<25)]
  set cr [expr $cr | ($vdp<<26)]
  set cr [expr $cr | ($vcb<<30)]
  set cr [expr $cr | ($vca<<31)]

  set vcreg [format "0x%08x" $cr]
}

proc osvt_splitctrlreg { creg th cl ji pk sa to di re ct dp cb ca } {
  upvar $creg vcreg
  upvar $th vth
  upvar $cl vcl
  upvar $ji vji
  upvar $pk vpk
  upvar $sa vsa
  upvar $to vto
  upvar $di vdi
  upvar $re vre
  upvar $ct vct
  upvar $dp vdp
  upvar $cb vcb
  upvar $ca vca

  # --- split control register
  set cr $vcreg
  if [catch {expr $cr}] {set cr 0}
  set vth [expr $cr & 0x3f] ; set cr [expr $cr>>6]
  set vcl [expr $cr & 0x3f] ; set cr [expr $cr>>6]
  set vji [osvt_graytodec [expr $cr & 0x1f]] ; set cr [expr $cr>>5]
  set vpk [expr $cr & 0x03] ; set cr [expr $cr>>2]
  set vsa [expr $cr & 0x03] ; set cr [expr $cr>>2]
  set vto [expr $cr & 0x03] ; set cr [expr $cr>>2]
  set vdi [expr $cr & 0x01] ; set cr [expr $cr>>1]
  set vre [expr $cr & 0x01] ; set cr [expr $cr>>1]
  set vct [expr $cr & 0x01] ; set cr [expr $cr>>1]
  set vdp [expr $cr & 0x0f] ; set cr [expr $cr>>4]
  set vcb [expr $cr & 0x01] ; set cr [expr $cr>>1]
  set vca [expr $cr & 0x01] ; set cr [expr $cr>>1]
}

# --- trace changes tyo rodir and apply them to ctrlreg
# --- caution: must access rodir via osvt(xxx-rodir)
proc osvt_trace_rodir { n1 rodir op } {
  eval upvar \#0 $n1\($rodir\) vrodir
  regsub {rodir} $rodir {ctrlreg} ctrl
  eval upvar \#0 $n1\($ctrl\) vctrl

  set vctrl [expr ($vctrl & ~(1<<23)) | (($vrodir & 1)<<23)]
}

# --- trace changes tyo rdresp and apply them to ctrlreg
# --- caution: must access rdresp via osvt(xxx-rdresp)
proc osvt_trace_rdresp { n1 rdresp op } {
  eval upvar \#0 $n1\($rdresp\) vrdresp
  regsub {rdresp} $rdresp {ctrlreg} ctrl
  eval upvar \#0 $n1\($ctrl\) vctrl

  set vctrl [expr ($vctrl & ~(1<<24)) | (($vrdresp & 1)<<24)]
}

# --- update control register bit field (rodir and rdresp defined)
# --- to update rodir bit to 1 do: osvt_updctrlreg rodir 1
proc osvt_updctrlreg {ctrl tag val} {
  upvar $ctrl vctrl

  switch -- $tag {
    rodir  {set nb 23 ; set ma 0x1}
    rdresp {set nb 24 ; set ma 0x1}
    default {return -code error \
	-errorinfo "Unforeseen option '$tag' in osvt_updctrlreg" \
	-errorcode NONE "Unforeseen option '$tag'"}
  }
  set vctrl [format "0x%08x" \
      [expr ($vctrl & ~($ma<<$nb)) | (($val & $ma)<<$nb)]]
}

# --- read vram buffer and convert it to hex
proc osvt_buffread {reg nw} {
  foreach wo [vme_buffread $reg $nw] {
    lappend out [format "0x%08x " $wo]
  }
  return $out
}

