#--------------------------------------------------------
# procedure to set the defaults for production testing
#  flag = 0  : set defaults for looped tests
#  flag = 1  : set defaults for miscellaneous tests (general)
#  flag = 10 : defaults for channel mask tests
#  flag = 11 : defaults for chip buffer tests
#  flag = 12 : defaults for TOT tests
#  flag = 13 : defaults for time stamp test
#  flag = 14 : defaults for jitter time tests
#  flag = 15 : defaults for test for shorts
#--------------------------------------------------------
proc prod_tests_defaults { flag } {
source cal_global.tcl
source misc_global.tcl
source site.tcl
global direction_choice
global all_or_some
global check_registers
global check_masks
global error_limit
global ignore_last_mask
global chip_test_list

#---------------------------------
#- defaults for looped tests
#---------------------------------
  if {$flag == 0} {
      for {set i 0} {$i < 20} {incr i 1} {
	   set chip_test_list($i) 0
      }
      for {set i 1} {$i < 7} {incr i 1} {
       set direction_choice($i) 1
      }
      set direction_choice(0) 0
      set direction_choice(7) 0
      set direction_choice(8) 0
      set all_or_some -1
      set check_registers 1
      set check_masks 1
      set error_limit 10
      set ignore_last_mask 0
  }
#--------------------------------------------------------
# defaults for misc tests (general)
# Jitter Time turned off because it is not implemented
#---------------------------------------------------------
  if {$flag == 1} {
      for {set i 0} {$i < 6} {incr i 1} {
	  set misc_tests_list($i) 1
      }
  }
  set misc_tests_list(4) 0
#---------------------------------
# defaults for mask tests
#---------------------------------
  if {$flag == 1 || $flag == 10} {
      set misc_nrep(0)             10
      set misc_cal_dac_top(0,0)    10
      set misc_cal_dac_bottom(0,0) 53
      set misc_thr_dac(0)          32
  }
#---------------------------------
# defaults for buffer tests
#---------------------------------
  if {$flag == 1 || $flag == 11} {
      set misc_nrep(1)             1
      set misc_cal_dac_top(1,0)    10
      set misc_cal_dac_bottom(1,0) 53
      set misc_thr_dac(1)          32
  }
#-------------------------------------------------------
# defaults for TOT tests
# Note : for the shaping time, valid values 
# are "100 nsec" "200 nsec" "300 nsec" and "400 nsec"
# The variable "misc_ntot" contains the number of different settings
# to be tested 
#--------------------------------------------------------
  if {$flag == 1 || $flag == 12} {
      set misc_nrep(2) 10
      set misc_thr_dac(2)          32
      set misc_tot_dead             1
      set misc_ntot                 8

#- values for setting no. 0
      set misc_cal_dac_top(2,0)     8
      set misc_cal_dac_bottom(2,0)  55
      set misc_tot(0)               2.5
      set misc_tot_toll(0)          1.6
      set misc_tot_shaping_time(0)  "100 nsec"
      set misc_tot_test_it(0)       1
      set misc_tot_skip(0)          "skip 0"

#- values for setting no. 1
      set misc_cal_dac_top(2,1)    45
      set misc_cal_dac_bottom(2,1) 18
      set misc_tot(1)              4.0
      set misc_tot_toll(1)         1.6
      set misc_tot_shaping_time(1)  "100 nsec"
      set misc_tot_test_it(1)       1
      set misc_tot_skip(1)          "skip 0"

#- values for setting no. 2
      set misc_cal_dac_top(2,2)     8
      set misc_cal_dac_bottom(2,2)  55
      set misc_tot(2)               4.5
      set misc_tot_toll(2)          2.0
      set misc_tot_shaping_time(2)  "200 nsec"
      set misc_tot_test_it(2)       1
      set misc_tot_skip(2)          "skip 0"

#- values for setting no. 3
      set misc_cal_dac_top(2,3)     45
      set misc_cal_dac_bottom(2,3)  18
      set misc_tot(3)               6.5
      set misc_tot_toll(3)          2.0
      set misc_tot_shaping_time(3)  "200 nsec"
      set misc_tot_test_it(3)       1
      set misc_tot_skip(3)          "skip 0"

#- values for setting no. 4
      set misc_cal_dac_top(2,4)     13
      set misc_cal_dac_bottom(2,4)  50
      set misc_tot(4)               6.5
      set misc_tot_toll(4)          2.0
      set misc_tot_shaping_time(4)  "300 nsec"
      set misc_tot_test_it(4)       1
      set misc_tot_skip(4)          "skip 0"

#- values for setting no. 5
      set misc_cal_dac_top(2,5)     43
      set misc_cal_dac_bottom(2,5)  20
      set misc_tot(5)               8.5
      set misc_tot_toll(5)          2.1
      set misc_tot_shaping_time(5)  "300 nsec"
      set misc_tot_test_it(5)       1
      set misc_tot_skip(5)          "skip 0"

#- values for setting no. 6
      set misc_cal_dac_top(2,6)     9
      set misc_cal_dac_bottom(2,6)  54
      set misc_tot(6)               7.5
      set misc_tot_toll(6)          2.5
      set misc_tot_shaping_time(6)  "400 nsec"
      set misc_tot_test_it(6)       1
      set misc_tot_skip(6)          "skip 0"

#- values for setting no. 7
      set misc_cal_dac_top(2,7)     52
      set misc_cal_dac_bottom(2,7)  11
      set misc_tot(7)               12
      set misc_tot_toll(7)          2.5
      set misc_tot_shaping_time(7)  "400 nsec"
      set misc_tot_test_it(7)       1
      set misc_tot_skip(7)          "skip 0"

  }
#--------------------------------------------
# defaults for time stamp tests... 
# for some of this, use the "site" variable
#-------------------------------------------
  if {$flag == 1 || $flag == 13} {
      set misc_nrep(3)             10
      set misc_cal_dac_top(3,0)    10
      set misc_cal_dac_bottom(3,0) 53
      set misc_thr_dac(3)          32
      set misc_tstamp_shift -4.7
      set misc_tstamp_toll 2.5
      set misc_tstamp_dead 1
      if {$site == "UCSB"} {
	  set misc_tstamp_shift -4.7
	  set misc_tstamp_toll 2.5
	  set misc_tstamp_dead 1
      }	  
  }
#---------------------------------
# defaults for jitter time tests
#---------------------------------
  if {$flag == 1 || $flag == 14} {
      set misc_nrep(4)             10
      set misc_cal_dac_top(4,0)    10
      set misc_cal_dac_bottom(4,0) 53
      set misc_thr_dac(4)          32
  }
#---------------------------------
# defaults for test for shorts
#---------------------------------
  if {$flag == 1 ||  $flag == 15} {
     set  short_min_chip 0
     set  short_max_chip $Cal_number_of_chips
     incr short_max_chip -1
     set  short_thresh 20
     set  short_charge_top 15
     set  short_charge_bottom [expr 63-$short_charge_top]
     set  short_nrep 10
     for {set i -3} {$i < 4} {incr i 1} {
	 set short_chan($i) 1
     } 
  }
#---------------------------------
#- done
#---------------------------------
   return
}