#############################################################################
#  This procedure prompts for the part name
#############################################################################
proc get_part_name {} {
 global part_name 

#- if "part_name" is not defined, read it from the partname file
#- if the file does not exist, set it to an empty string
    if {![info exists part_name]} {
	set exists [file exists partname]
	if {$exists == 1} {
	    set f [open partname r]
            gets $f part_name
	    set part_name [string trim $part_name]
	} else {
	    set part_name ""
	}
    }

#- Prompt the user to verify the part_name
    set part_name \
	    [textreq "Is this the part you are working with ?" $part_name]
    set part_name [string trim $part_name]
    set f [open partname w]
	puts $f $part_name
	close $f
}
#############################################################################
# This procedure checks that the directory for this part exists.
# If it does not exist, it will try to make it.
# The procedure returns "OK" or an error message 
#############################################################################
proc make_part_directory {} {
 global part_name
 global data_directory

#- trim whitespaces for safety
    set part_name         [string trim $part_name]
    set data_directory    [string trim $data_directory]

#- if data_directory does not end with a "slash", add it
    set mytemp1 "/"
    set junk    [string trimright $data_directory "/"]
    if {$junk == $data_directory} {
        set data_directory $junk$mytemp1
    }
#- this is the directory we want to make
    set part_directory $data_directory$part_name
    set part_directory [string trimright $part_directory "/"]

#- does this file exist (1=yes)?
    set file_exists [file exists $part_directory]

#- if it exists, is it a directory ?
    if {$file_exists == 1} { 
	set file_is_a_dir [file isdirectory $part_directory]
	if {$file_is_a_dir != 1} {
	    return "ERROR.  $part_directory IS NOT A DIRECTORY"
	}
	return "OK"
    }
#- The directory does not exist.  Make it.
   set flag [catch {exec mkdir $part_directory} msg]
   if {$flag == 0} {    
       return "OK"
   } else {
       return "ERROR TRYNG TO MAKE $part_directory \n $msg"
   }
}

#############################################################################
#  This procedure starts a run.  It also pops up the run window
#############################################################################
proc start_running {} {

   global part_name 

# include the site
    source site.tcl

# all global variables
    source cal_global.tcl

# for UCSB, the global variables with the status of the motors
    if {$site == "UCSB"} {
	global stat1
	global stat2
    }

# do not allow for multiple starts
    if {$Cal_run != "stop"} {
	.run_menu.text insert end "Run is in progress \n"
        .run_menu.text yview -pickplace end
	return
    }

# check for valid error limit... must be integer
    set good [test_int Cal_error_limit $Cal_error_limit]
    if {$good == 1} {return}

# check for one_at_a_time value...must be between 1 and 128
    set good [test_value Cal_one_at_a_time $Cal_one_at_a_time 1 128]
    if {$good == 1} return

# if the one_at_a_time value is not a power of 2, warn the user
# (BTW, there should be a more elegant way of doing this)
   set nice_value "no" 
   if {$Cal_one_at_a_time == 1} {set nice_value "yes"}
   set value 2
   for {set i 1} {$i < 9} {incr i 1} {
       if {$Cal_one_at_a_time == $value} {set nice_value "yes"}
       set value [expr $value*2]
   }
   if {$nice_value == "no"} {
       set answer [cal_yes_no "The number of channels to be fired together\nis not a power of 2.  This is OK, but not very nice.\nDo you want to continue ?\n"]
       if {$answer == "NO"} {return}
   }


# check hybrid and chip numbers
    set good [cal_check_chip_number]
    if {$good == 1} {return}

# check readout direction, set respond_to_read
    set good [cal_check_direction]
    if {$good == -9999} {
	return
    } else {
	cal_set_respond_to_read $good
    }

# check ignore
    set good [cal_check_ignore]
    if {$good != 0} {return}

# if it is an LED scan, check for appropriate settings
   if {$xy_scan == 1} {
       set good [xy_good_site $site]
       if {$good != 0} {
	   cal_display_error_message "Site $site is not setup for XY scans"
	   return
       }
       set good [xy_check_parameters]
       if {$good != "OK"} {
	   cal_display_error_message $good
	   return
       }
       if {$Cal_one_at_a_time != 128} {
	   cal_display_error_message "You cannot set the calibration to \n one channel at a time \n for an XY detector scan"
	   return
       }
   }

# if this is UCSB, set the DC level that allows us to toggle between 
# ordinary calibration runs and the XY scan...Also the delay select
   if {$site == "UCSB"} {
       set good [delay_select_off]
       if {$good != 0} {
	   cal_display_error_message "Error turning Delay Select off"
	   return
       }
       if {$xy_scan == 1} {
	   set good [dc_level_on]
       } else {
	   set good [dc_level_off]
       }
       if {$good != 0} {
	   cal_display_error_message "Error trying to set the DC level"
	   return
       }
   }

# if this is UCSB, and it is an LED scan, set the motor parameters.  
# Note that if these were set interactively to something, 
# they WILL NOT be changed by "set_default_motor_parameters".
# Also FORCE the encoders to measure in an absolute coordinate system
  if {$site == "UCSB" && $xy_scan == 1} {
      set_default_motor_parameters
      set_default_encoder_parameters
  }

# if this is UCSB, and it is an LED scan, turn the motors on.
  if {$site == "UCSB" && $xy_scan == 1} {
      set cmd(1) 1mn
      set cmd(2) 2mn
      set i 1
      while {$i < 3} {
	  mstatus $i
	  set dummy(1)  $stat1
	  set dummy(2)  $stat2
	  if {$dummy($i) != "ON"} {
	      set good [mxc $cmd($i)]
	      mstatus $i
	      set dummy(1)  $stat1
	      set dummy(2)  $stat2
	      if {$good != 0 || $dummy($i) != "ON"} {
	         cal_display_error_message "Error trying to turn motor $i on"
		 return
	      }
	  }
	  incr i 1
      }
  }


# verify the part name
   get_part_name

# make the directory for this part, if necessary
   set good [make_part_directory]
   if {$good != "OK"} {
       cal_display_error_message $good
       return
   }

# double check the name of the output file
  if {$Cal_write_file == 1} {
      set std_file [std_output_file run]
      if {$std_file != $Cal_output_file} {
	  set msg1 "The Runnumber is $Cal_run_number.\n"
	  set msg2 "The Part Name is $part_name.\n"
	  set msg3 "The Output File is \n $Cal_output_file.\n\n"
	  set msg4 "The Standard Output File should be\n $std_file.\n\n"
	  set msg5 "Are you sure that you want to continue ??? \n\n" 
	  set msg $msg1$msg2$msg3$msg4$msg5
	  if {[cal_yes_no $msg] != "YES"} {return}
      }
  }
	     

# before running, write out a file with the parameters..
# if the file exists, abort
   set mytemp parameter_run
   set logfile $data_directory$mytemp$Cal_run_number
   set exist [file exist $logfile]
   if {$exist == 1} {
       cal_display_error_message "Run $Cal_run_number already performed"
       return
   }
   cal_write_defaults $logfile NO "THRESHOLD OR CHARGE SCAN"

# we are now running.....
    set Cal_run start

# do not try to make the buttons more than once
    if {![winfo exists .run_menu]} {
      toplevel .run_menu -bg #00f
      frame .run_menu.f1 \
		-bg #00f \
	 	-highlightthickness 0
      frame .run_menu.f2 \
		-bg #00f \
	 	-highlightthickness 0
      pack .run_menu.f1 .run_menu.f2 -side top

      button .run_menu.suspend \
		-text "Suspend\nThe Run" \
		-command "suspend_running"\
		-relief raised  \
		-bd 2  \
		-bg #8dd  \
		-activebackground #7cc  \
		-highlightthickness 0
      button .run_menu.end \
		-text "  End  \nThe Run" \
		-command "stop_running"\
		-relief raised  \
		-bd 2  \
		-bg #8dd  \
		-activebackground #7cc  \
		-highlightthickness 0
      button .run_menu.resume \
		-text "Resume \nThe Run" \
		-command "resume_running"\
		-relief raised  \
		-bd 2  \
		-bg #8dd  \
		-activebackground #7cc  \
		-highlightthickness 0
      button .run_menu.histoscope \
		-text "  Talk to \nhistoscope" \
		-command "talk_histoscope"\
		-relief raised  \
		-bd 2  \
		-bg #8dd  \
		-activebackground #7cc  \
		-highlightthickness 0
      button .run_menu.close \
		-text "Close \nWindow" \
		-command "check_and_close" \
		-relief raised  \
		-bd 2  \
		-bg #8dd  \
		-activebackground #7cc  \
		-highlightthickness 0
      button .run_menu.move \
	        -text "Move \nLED" \
		-command "goto_motor_window" \
	        -relief raised \
		-bd 2 \
		-bg #8dd  \
		-activebackground #7cc  \
		-highlightthickness 0
      if {$site == "UCSB"} {
	  pack .run_menu.suspend .run_menu.resume .run_menu.end \
	        .run_menu.histoscope .run_menu.close .run_menu.move \
		-side left \
		-fill x \
		-padx 4 \
		-in .run_menu.f1
      } else {
	  pack .run_menu.suspend .run_menu.resume .run_menu.end \
	        .run_menu.histoscope .run_menu.close \
		-side left \
		-fill x \
		-padx 4 \
		-in .run_menu.f1
      }
      text .run_menu.text \
		-relief raised \
		-bd 2 \
		-yscrollcommand ".run_menu.scroll set"\
		-bg #aff  \
		-highlightthickness 0\
		-selectbackground #0ff

      scrollbar .run_menu.scroll \
		-command ".run_menu.text yview" \
		-highlightthickness 0 \
		-relief ridge \
		-bd 4 \
		-bg #8dd \
		-troughcolor #00f \
		-activebackground #7cc
      pack .run_menu.scroll -side left -fill y -in .run_menu.f2
      pack .run_menu.text -side left -in .run_menu.f2
    }


# set the grab
grab set .run_menu

# do the infinite loop
cal_loop

# cal_loop exits if it is completed or if it failed or if it is suspended
# NB unless it is suspended, it should have set Cal_run to stop.  In
#    that case, we need to relase the grab.  Chances are this has already
#    been done in cal_loop, but it does not hurt
       if {$Cal_run == "stop"} {
	   grab release .run_menu
       }

       if {$Cal_run == "stop"} {
	   cal_bump_runnumber
      }

}


#####################################################
# reset procedure...not implementes yet
######################################################
# proc cal_reset {} {
#      return
#  }


##############################################################
# Suspend run procedure
##############################################################
proc suspend_running {} {
    global part_name
    source site.tcl
    source cal_global.tcl
    if {$Cal_run != "start" && $Cal_run != "resume"} {
	.run_menu.text insert end "You can suspend only when you are running \n"
        .run_menu.text yview -pickplace end
	return
    }
    set Cal_run suspend
    return
}
###################################################################
# End Run Procedure...Deal with suspended runs...
###################################################################
proc stop_running {} {
    global part_name
    source cal_global.tcl
    source site.tcl

    if {$Cal_run == "stop"} {
	.run_menu.text insert end "There is no run in progress \n"
        .run_menu.text yview -pickplace end
	return
    }
    if {$Cal_run == "suspend"} {
	set Cal_run "stop"
	cal_loop
	cal_bump_runnumber
    }
    set Cal_run stop
###    if {$Cal_write_file == 1 || $Cal_histoscope == 1} {
###	cal_bump_runnumber
###    }
###    grab release .run_menu
    return
}
##############################################################
# Bump the run number procedure
##############################################################
proc cal_bump_runnumber {} {
        source cal_global.tcl
        source site.tcl
	incr Cal_run_number 1
	set Cal_output_file [std_output_file run]
	set f [open runnumber w]
	puts $f $Cal_run_number
	close $f
    return
}
##############################################################
# Resume Run Procedure....
##############################################################
proc resume_running {} {
    global part_name
    source cal_global.tcl
    source site.tcl
    if {$Cal_run != "suspend"} {
	.run_menu.text insert end "You can resume only when you are suspended \n"
        .run_menu.text yview -pickplace end
	return
    }
    set Cal_run resume
    cal_loop

# cal_loop exits if it is completed or if it failed or if it is suspended
# NB unless it is suspended, it should have set Cal_run to stop.  In
#    that case, we need to relase the grab.  Chances are this has already
#    been done in cal_loop, but it does not hurt
       if {$Cal_run == "stop"} {
           cal_bump_runnumber
	   grab release .run_menu
       }


    return
}
#############################################################################
# this is the error handling menu
#############################################################################
proc cal_error_handler { input } {

    global error_dialog
    toplevel .error -bg #00f
    set error_dialog junk

    label .error.label \
	  -text "$input" -bd 2 -bg #aff -highlightthickness 0 
    button .error.continue \
           -text "Continue the run" \
	   -relief raised -bd 2 -bg #8dd \
	   -activebackground #7cc \
           -highlightthickness 0 \
	   -command "set error_dialog continue"
    button .error.abort \
           -text "Abort the run" \
	   -relief raised -bd 2 -bg #8dd \
	   -activebackground #7cc \
           -highlightthickness 0 \
	   -command "set error_dialog abort"
    pack .error.label .error.continue .error.abort -side top -in .error
    grab set .error


    update
    tkwait variable error_dialog
    destroy .error
    grab set .run_menu
    return $error_dialog
}
#############################################################################
# close the .run_menu window, but only if we are not running
#############################################################################
proc check_and_close {} {
    global part_name
    source cal_global.tcl
    if {$Cal_run == "stop"} {
	destroy .run_menu
    }
}

#############################################################################
# procedure to move the LED while running, useful for debugging             #
#############################################################################
proc goto_motor_window {} {
    global part_name
    source site.tcl
    source cal_global.tcl 

#- check that the site is UCSB
    if {$site != "UCSB"} {
	.run_menu.text insert end "\n"
	.run_menu.text insert end "*****************************************\n"
	.run_menu.text insert end "\"Move LED\" allowed only at UCSB        \n"
	.run_menu.text insert end "*****************************************\n"
        .run_menu.text yview -pickplace end
	return
    }

#- makes sense to do this only for xy scans
    if {$xy_scan != 1} {
	.run_menu.text insert end "\n"
	.run_menu.text insert end "***************************************\n"
	.run_menu.text insert end "\"Move LED\" allowed only for XY scans \n"
	.run_menu.text insert end "***************************************\n"
        .run_menu.text yview -pickplace end
	return
    }

#- we should be suspended    
    if {$Cal_run != "suspend"} {
	.run_menu.text insert end "\n"
	.run_menu.text insert end "****************************************\n"
	.run_menu.text insert end "\"Move LED\" allowed only if suspended  \n"
        .run_menu.text insert end "****************************************\n"
        .run_menu.text yview -pickplace end
	return
    }

#- give the user some information
   .run_menu.text insert end "\n"
   .run_menu.text insert end "**********************************************\n"
   .run_menu.text insert end "You can move the LED from the motor window.   \n"
   .run_menu.text insert end "You will have to close the motor window       \n"
   .run_menu.text insert end "before you will be allowed to resume the run. \n"
   .run_menu.text insert end "**********************************************\n"
   .run_menu.text yview -pickplace end


#- open the motor window, set the grab to it
   if {![winfo exists .motor]} {
      motor
      tkwait visibility .motor
   }
   grab set .motor

#- when the motor window is destroyed:
#-              (1) set the grab back to .run_menu
#-              (2) return
   tkwait window .motor
   grab set .run_menu
   return
}



