########################################################################
# This procedure pops up a dialog box and warns the user               #
# if the variable is out of range OR if the variable is not            #
# an integer (decimal OR hex formats are allowed)                      #
# hex format must be preceeded by 0x                                   #
#                                                                      #
# call it with                                                         #
#     set good [test_value thisone $try_value $try_min $try_max]       #
# returns a zero if everything okay                                    #
#                                                                      #
# Careful : for hex numbers, we do not accept negative numbers, rather #
# we turn them into double-precision integers                          #
# Assumption : in single precision 0xffffffff=-1, 0xfffffffe=-2, etc   #
########################################################################
proc test_value {varname value min max} {

    set good 0
    global cal_dialog

# Only accept hex or decimal
     if ![regexp {^((0x)+[0-9a-fA-F]+|(-)?[0-9]+)$} $value] {
	set good 1
    }
     if ![regexp {^((0x)+[0-9a-fA-F]+|(-)?[0-9]+)$} $min] {
	set good 1
    }
     if ![regexp {^((0x)+[0-9a-fA-F]+|(-)?[0-9]+)$} $max] {
	set good 1
    }

    if {$good == 0} {
# Are the numbers hex or decimal ?
       set hex_value 0
       set hex_min   0
       set hex_max   0
       if [regexp {^(0x)+[0-9a-fA-F]+$} $value] {set hex_value 1}
       if [regexp {^(0x)+[0-9a-fA-F]+$} $max]   {set hex_max 1}
       if [regexp {^(0x)+[0-9a-fA-F]+$} $min]   {set hex_min 1}

# If they are hex, and they are positive, then they behave like decimal
       if {$hex_value == 1 && $value >= 0} {set hex_value 0}
       if {$hex_max   == 1 && $max   >= 0} {set hex_max 0}
       if {$hex_min   == 1 && $min   >= 0} {set hex_min 0}

# Start with the straightforward tests.........
       if {$hex_value == 0 && $hex_min == 0} {
	   if {$value < $min} {set good 1}
       }
       if {$hex_value == 0 && $hex_max == 0} {  
           if {$value > $max} {set good 1}
       }

# Now the more complicated tests.............
       if {$hex_value == 0 && $hex_min == 1} {set good 1}
       if {$hex_value == 1 && $hex_min == 1} {
	   if {$value < $min} {set good 1}
       }
       if {$hex_value == 1 && $hex_max == 0} {set good 1}
       if {$hex_value == 0 && $hex_max == 0} {  
           if {$value > $max} {set good 1}
       }
    }

# Okay, done.  Pop up menu if it is not good......... 
    if {$good == 1} {
       set dialog 0
       toplevel .dialog -bg #00f

       grab set .dialog
       label .dialog.label \
		-text "Variable $varname out of range \n value is $value...minimum is $min ...maximum is $max \n Variable, minimum and maximum \n must be integer (decimal or hex)" \
		-bd 2  \
		-bg #aff\
		-highlightthickness 0

       button .dialog.done \
		-text "Okay" \
		-command "set cal_dialog 1"\
		-relief raised  \
		-bd 2  \
		-bg #8dd  \
		-activebackground #7cc  \
		-highlightthickness 0
       pack .dialog.label .dialog.done \
		-side top \
		-in .dialog
       tkwait variable cal_dialog
       grab release .dialog
       destroy .dialog
    }
    return $good
}
###########################################################
# procedure to "fix up" a hex number :                    #
# (1) remove white spaces etc                             #
# (2) turn into lower case (not necessary, but why not ?) #
# (3) Add a "0x" if it is not there already               #
###########################################################
proc fixup_hex {input} {
     set output [string trim $input]
     set output [string tolower $output]     
     if [regexp {^((0x)?[0-9a-fA-F]+$)} $output] {
	 if ![regexp {^((0x)+[0-9a-fA-F]+$)} $output] {
	     set output 0x$output
	 }
     }
 return $output
 }


# procedure to check taht number is integer (hex is okay)
proc test_int {inputname input} {
     set good 0
     global cal_dialog
     if ![regexp {^((0x)+[0-9a-fA-F]+|(-)?[0-9]+)$} $input] {
	set good 1
    }
# Okay, done.  Pop up menu if it is not good......... 
    if {$good == 1} {
       set dialog 0
       toplevel .dialog -bg #00f

       grab set .dialog
       label .dialog.label \
		-text "Variable $inputname not integer \n value is $input" \
		-bd 2  \
		-bg #aff\
		-highlightthickness 0

       button .dialog.done \
		-text "Okay" \
		-command "set cal_dialog 1"\
		-relief raised  \
		-bd 2  \
		-bg #8dd  \
		-activebackground #7cc  \
		-highlightthickness 0
       pack .dialog.label .dialog.done \
		-side top \
		-in .dialog
       tkwait variable cal_dialog
       grab release .dialog
       destroy .dialog
    }
return $good
}




##############################################################
# check whether value is a valid real number
##############################################################
proc check_real { value } {

    set okeydokey 1
    if [regexp {^(-)?[0-9.]+$} $value] {
       set okeydokey 0
    }
    if [regexp {^(\+)?[0-9.]+$} $value] {
       set okeydokey 0
    }
#- so far we have not checked that there is at most one "."
    if {$okeydokey == 0} {
	set ndots 0
        set string_length [string length $value]
        set i 0
        while {$i < $string_length} {
	    set this_char [string index $value $i]
	    if {$this_char == "."} {
		incr ndots 1
	    }
	    incr i 1
	}
       if {$ndots > 1} {set okeydokey 1}
   }
    return $okeydokey
}

