#==========================================================
# this procedure returns zero if the this_site is
# on the list of allowed sites for an XY scan
#==========================================================
proc xy_good_site { this_site } {
  source xy_allowed_sites.tcl
  set good 1
  set i 1
  while {$i <= $number_allowed_sites} {
      if {$this_site == $allowed_sites($i)} {
	  set good 0
      }
      incr i 1
  }
  return $good
}
#==========================================================
# this procedure returns "OK" if all the parameters for
# an XY scan are legal.  Otherwise, returns error message
#==========================================================
proc xy_check_parameters {} {
  source cal_global.tcl
  source site.tcl
#---------------------------------------------------------
# first test the site
#---------------------------------------------------------
  set good [xy_good_site $site]
  if {$good != 0} {
     return "XY scans not allowed at Site = $site"
  } 
#---------------------------------------------------------
# Now check the values of the actual parameters.  Should
# not need to do that, but let's double chack
#---------------------------------------------------------
  set good [test_value xy_nstep $xy_nstep 0 9999999]
  if {$good != 0} {
      return "Illegal no. of steps = $xy_nstep for XY scan"
  } 
  set good [test_value xy_which_axis $xy_which_axis 1 2]
  if {$good != 0} {
      return "Illegal axis number = $xy_which_axis for XY scan"
  } 
  set good [check_real $xy_x_initial]
  if {$good != 0} {
      return "Illegal initial X value = $xy_x_initial for XY scan"
  } 
  set good [check_real $xy_y_initial]
  if {$good != 0} {
      return "Illegal initial Y value = $xy_y_initial for XY scan"
  } 
  set good [check_real $xy_step_size]
  if {$good != 0} {
      return "Illegal step size = $xy_step_size for XY scan"
  } 
  set good [test_value xy_start_flag $xy_start_flag 0 1]
  if {$good != 0} {
      return "Illegal xy_start_flag = $xy_start_flag for XY scan"
  } 
#---------------------------------------------------------
# This is more serious.  
# No charge injection allowed. No threshold steps allowed
#---------------------------------------------------------
  set i 0
  while {$i < $Cal_number_of_chips} {
      set ok 0
      if {$i <  $Cal_number_of_chips_top && $Cal_min_charge($i) != 0} {
	  set ok 1
      }
      if {$i >= $Cal_number_of_chips_top && $Cal_min_charge($i) != 63} {
	  set ok 1
      }
      set temp [expr $Cal_min_charge($i) + $Cal_step_charge($i)]
      if {$temp <= $Cal_max_charge($i)} {
	  set ok 1
      }
      if {$ok != 0} {
	  return "No charg inj on XY scans allowed.\nCheck parameters for chip $i"
      }
      set temp [expr $Cal_min_threshold($i) + $Cal_step_threshold($i)]
      if {$temp <= $Cal_max_threshold($i)} {
	  return "Threshold on XY scans must be fixed.\nCheck parameters for chip $i"
      }
      incr i 1
  }
#---------------------------------------------------------
# if we get here, everything must be OKAY
#---------------------------------------------------------
  return "OK"
}
