####################################################
# This procedure sets up the XY stage parameters
####################################################
proc xy_stage_window {} {

  source cal_global.tcl
  source site.tcl

  global temp_xy_which_axis
  global temp_xy_x_initial
  global temp_xy_y_initial
  global temp_xy_step_size
  global temp_xy_nstep
  global temp_xy_start_flag 
  global xy_setup_action 

#- quit if the window exists already
  if {[winfo exists .xystage]} {return}
  toplevel .xystage -bg #00f
  grab set .xystage

#- copy the "real" variables into a set of "temporary" variables
  set temp_xy_which_axis $xy_which_axis
  set temp_xy_x_initial  $xy_x_initial
  set temp_xy_y_initial  $xy_y_initial
  set temp_xy_step_size  $xy_step_size
  set temp_xy_nstep      $xy_nstep
  set temp_xy_start_flag $xy_start_flag

#- if it is UCSB, set the encoder coordinate system to "absolute"
   if {$site == "UCSB"} {
       set_default_encoder_parameters
   }
  
#- define the frames, and pack them
   frame .xystage.f0 -bg #00f
   frame .xystage.f1 -bg #00f
   frame .xystage.f2 -bg #00f
   frame .xystage.f3 -bg #00f
   frame .xystage.f3.1 -bg #00f
   frame .xystage.f3.1.1 -bg #00f
   frame .xystage.f3.1.2 -bg #00f
   frame .xystage.f3.2 -bg #00f
   frame .xystage.f4 -bg #00f
   pack .xystage.f0 .xystage.f1 .xystage.f2 .xystage.f3 .xystage.f4 \
        -in .xystage -side top -pady 2 -fill x
   pack .xystage.f3.1 .xystage.f3.2 -in .xystage.f3 -side left \
        -padx 2  -fill x
   pack .xystage.f3.1.1 .xystage.f3.1.2 -in .xystage.f3.1 -side top -fill x


#- label
   message .xystage.msg  -width 8c  \
                -justify center  \
                -relief raised  \
                -bg #ff0  \
                -bd 2  \
                -text "Setup the XY scan parameters"
   pack .xystage.msg -fill x -in .xystage.f0


#- UCSB warning
   if {$site == "UCSB"} {
      message .xystage.ucsb -width 8c \
                -justify center  \
                -relief raised  \
                -bg #8dd  \
                -bd 2  \
                -text "Coordinates are ABSOLUTE\nfrom UCSB encoders "
      pack .xystage.ucsb -fill x -in .xystage.f0
   }
#- starting positions
   radiobutton .xystage.rb1 -text "Start at X = " \
               -variable temp_xy_start_flag \
               -value 0 \
	       -bg #8dd -activebackground #7cc \
	       -highlightthickness 0 -anchor w
   radiobutton .xystage.rb2 -text "Start at current position " \
               -variable temp_xy_start_flag \
               -value 1 \
	       -bg #8dd -activebackground #7cc \
	       -highlightthickness 0 -anchor w
   entry .xystage.xinit \
               -width 10 \
               -textvariable temp_xy_x_initial  \
               -relief sunken  \
               -bg #aff   \
               -highlightcolor #f00  \
               -highlightbackground #8dd
   entry .xystage.yinit \
               -width 10 \
               -textvariable temp_xy_y_initial  \
               -relief sunken  \
               -bg #aff   \
               -highlightcolor #f00  \
               -highlightbackground #8dd
   label .xystage.l1 \
               -text "  and Y = "  \
               -bg #8dd  \
               -highlightthickness 0

   pack .xystage.rb1 .xystage.xinit .xystage.l1 .xystage.yinit \
	-in .xystage.f1 -side left
   pack .xystage.rb2 -fill x -in .xystage.f2 

#- step size, number of steps
   label .xystage.l2 \
               -text "Step size (mm)   "  \
               -bg #8dd  \
	       -width 17 \
               -highlightthickness 0 
   entry .xystage.stepsize \
               -width 10 \
               -textvariable temp_xy_step_size  \
               -relief sunken  \
               -bg #aff   \
               -highlightcolor #f00  \
               -highlightbackground #8dd
   pack .xystage.l2 .xystage.stepsize -side left -in .xystage.f3.1.1 
   label .xystage.l3 \
               -text "Number of steps "  \
               -bg #8dd  \
	       -width 17 \
               -highlightthickness 0
   entry .xystage.nstep \
               -width 10 \
               -textvariable temp_xy_nstep  \
               -relief sunken  \
               -bg #aff   \
               -highlightcolor #f00  \
               -highlightbackground #8dd
   pack .xystage.l3 .xystage.nstep -side left -in .xystage.f3.1.2


#- X or Y scan ?
   radiobutton .xystage.rb3 -text "X-scan" \
               -variable temp_xy_which_axis \
               -value 1 \
	       -bg #8dd -activebackground #7cc \
	       -highlightthickness 0 
   radiobutton .xystage.rb4 -text "Y-scan" \
               -variable temp_xy_which_axis \
               -value 2 \
	       -bg #8dd -activebackground #7cc \
	       -highlightthickness 0 
   pack .xystage.rb3 .xystage.rb4  -anchor e -pady 2 \
	   -in .xystage.f3.2  
      

#- Okay, Cancel
    set xy_setup_action 0
    button .xystage.ok  \
                -text "OK"  \
                -command "set xy_setup_action 1"  \
                -bg #8dd  \
                -activebackground #7cc  \
                -highlightthickness 0
    button .xystage.cancel  \
                -text "CANCEL"  \
                -command "set xy_setup_action 2"  \
                -bg #8dd  \
                -activebackground #7cc  \
                -highlightthickness 0
    pack        .xystage.ok  .xystage.cancel  -side left  \
                -padx 40  -fill x -in .xystage.f4

#- now wait for the user to hit one of the two buttons
   while {$xy_setup_action == 0} {
       tkwait variable xy_setup_action

#- User hit CANCEL
       if {$xy_setup_action == 2} {
	   destroy .xystage
	   return
       }

#- User it OK...we must now check all variables
       set good [test_value temp_xy_nstep $temp_xy_nstep 0 9999999]
       if {$good != 0} {
	   grab set .xystage
	   set xy_setup_action 0
	   continue
       }
       set good [test_value temp_xy_which_axis $temp_xy_which_axis 1 2]
       if {$good != 0} {
	   grab set .xystage
	   set xy_setup_action 0
	   continue
       }
       set good [check_real $temp_xy_x_initial]
       if {$good != 0} {
           cal_display_error_message "Variable temp_xy_x_initial = $temp_xy_x_initial \n should be a real number"
	   grab set .xystage
	   set xy_setup_action 0
	   continue
       }
       set good [check_real $temp_xy_y_initial]
       if {$good != 0} {
           cal_display_error_mesage "Variable temp_xy_y_initial = $temp_xy_y_initial \n should be a real number"
	   grab set .xystage
	   set xy_setup_action 0
	   continue
       }
       set good [check_real $temp_xy_step_size]
       if {$good != 0} {
           cal_display_error_mesage "Variable temp_xy_step_size = $temp_xy_step_size \n should be a real number"
	   grab set .xystage
	   set xy_setup_action 0
	   continue
       }

#- OK, good, store final variables, destroy window, get out of here
       set  xy_which_axis $temp_xy_which_axis
       set  xy_x_initial  $temp_xy_x_initial
       set  xy_y_initial  $temp_xy_y_initial
       set  xy_step_size  $temp_xy_step_size
       set  xy_nstep      $temp_xy_nstep
       set  xy_start_flag $temp_xy_start_flag
       destroy .xystage
       return
   }

}

